@@ -497,6 +497,13 @@ static int can_replace(object_handle_t source_handle,
 			src_rc = EINVAL;
 		}
 
+		/* Can't move "must-be-top" objects (e.g. snapshots and snap-origins). */
+		if (source->flags & SOFLAG_MUST_BE_TOP) {
+			LOG(log_level, "Object %s must be a top object - cannot be replaced.\n",
+			    source->name);
+			src_rc = EINVAL;
+		}
+
 		if (!engine_can_online_copy()) {
 			if (!engine_is_offline(source, &vol)) {
 				LOG(log_level, "Object %s is part of volume %s which is currently opened.  Replace must be done off-line.\n",
@@ -716,8 +723,7 @@ int evms_can_set_info(object_handle_t thing_handle) {
 	int rc = 0;
 	object_type_t type;
 	void  * thing = NULL;
-	int count = -1;
-	task_context_t * task;
+	task_handle_t task_handle;
 
 	LOG_PROC_ENTRY();
 
@@ -733,95 +739,64 @@ int evms_can_set_info(object_handle_t thing_handle) {
 		return rc;
 	}
 
-	task = engine_alloc(sizeof(task_context_t));
+	rc = translate_handle(thing_handle, &thing, &type);
 
-	if (task != NULL) {
-		rc = translate_handle(thing_handle, &thing, &type);
+	if (rc == HANDLE_MANAGER_NO_ERROR) {
 
-		if (rc == HANDLE_MANAGER_NO_ERROR) {
+		/*
+		 * Check if the plug-in supports setting of object information
+		 * by getting the option count for EVMS_Task_Set_Info.  If the
+		 * option count is 0 or less, then the plug-in does not support
+		 * the setting of object information.
+		 */
+		switch (type) {
+			case DISK:
+			case SEGMENT:
+			case REGION:
+			case EVMS_OBJECT:
+				rc = evms_create_task(thing_handle, EVMS_Task_Set_Info, &task_handle);
+				if (rc == 0) {
+					evms_destroy_task(task_handle);
+				}
+				break;
 
-			/*
-			 * Check if the plug-in supports setting of object information
-			 * by getting the option count for EVMS_Task_Set_Info.  If the
-			 * option count is 0 or less, then the plug-in does not support
-			 * the setting of object information.
-			 */
-			switch (type) {
-				case DISK:
-				case SEGMENT:
-				case REGION:
-				case EVMS_OBJECT:
-					{
-						storage_object_t * obj = (storage_object_t *) thing;
-						task->object = obj;
-						task->action = EVMS_Task_Set_Info;
+			case CONTAINER:
+				rc = evms_create_task(thing_handle, EVMS_Task_Set_Container_Info, &task_handle);
+				if (rc == 0) {
+					evms_destroy_task(task_handle);
+				}
+				break;
 
-						count = obj->plugin->functions.plugin->get_option_count(task);
+			case VOLUME:
+				{
+					logical_volume_t * vol = (logical_volume_t *) thing;
 
-						if (count <= 0) {
-							LOG_DETAILS("Plug-in %s does not have any descriptors for setting object info.\n", obj->plugin->short_name);
+					if (vol->flags & VOLFLAG_COMPATIBILITY) {
+						if (is_kernel_volume_mounted(vol, DETAILS)) {
+							rc = EBUSY;
+						}
+					}
+
+					if (rc == 0) {
+						if (vol->file_system_manager != NULL) {
+							rc = evms_create_task(thing_handle,EVMS_Task_Set_Info,&task_handle);
+							if (rc == 0) {
+								evms_destroy_task(task_handle);
+							}
+
+						} else {
+							LOG_DETAILS("Volume %s does not have a file system interface module.\n", vol->name);
 							rc = ENOSYS;
 						}
 					}
-					break;
+				}
+				break;
 
-				case CONTAINER:
-					{
-						storage_container_t * con = (storage_container_t *) thing;
-						task->container = con;
-						task->action = EVMS_Task_Set_Container_Info;
-
-						count = con->plugin->functions.plugin->get_option_count(task);
-
-						if (count <= 0) {
-							LOG_DETAILS("Plug-in %s does not have any descriptors for setting container info.\n", con->plugin->short_name);
-							rc = ENOSYS;
-						}
-					}
-					break;
-
-				case VOLUME:
-					{
-						logical_volume_t * vol = (logical_volume_t *) thing;
-
-						if (vol->flags & VOLFLAG_COMPATIBILITY) {
-							if (is_kernel_volume_mounted(vol, DETAILS)) {
-								rc = EBUSY;
-							}
-						}
-						
-						if (rc == 0) {
-							if (vol->file_system_manager != NULL) {
-								task->volume = vol;
-								task->action = EVMS_Task_Set_Info;
-
-								rc = vol->file_system_manager->functions.fsim->get_option_count(task);
-
-								if (count <= 0) {
-									LOG_DETAILS("Plug-in %s does not have any descriptors for setting volume info.\n", vol->file_system_manager->short_name);
-									rc = ENOSYS;
-								}
-
-							} else {
-								LOG_DETAILS("Volume %s does not have a file system interface module.\n", vol->name);
-								rc = ENOSYS;
-							}
-						}
-					}
-					break;
-
-				default:
-					LOG_ERROR("Cannot set info on a thing of type %#x.\n", type);
-					rc = EINVAL;
-					break;
-			}
+			default:
+				LOG_ERROR("Cannot set info on a thing of type %#x.\n", type);
+				rc = EINVAL;
+				break;
 		}
-
-		engine_free(task);
-
-	} else {
-		LOG_CRITICAL("Error allocating memory for a task context.\n");
-		rc = ENOMEM;
 	}
 
 	LOG_PROC_EXIT_INT(rc);
@@ -864,7 +839,7 @@ static inline int get_plugin_extended_info(plugin_record_t         * plugin,
 }
 
 
-static void free_extended_info_object_contents(void * object) {
+void free_extended_info_array_contents(void * object) {
 
 	int i, j;
 	extended_info_array_t * eia = (extended_info_array_t *) object;
@@ -1030,9 +1005,9 @@ int evms_get_extended_info(object_handle_t thing, char * descriptor_name, extend
 
 			} else {
 				/* The extended info array has at least one entry. */
-				uint size = sizeof(extended_info_array_t) + (sizeof(extended_info_t) * (info->count - 1));
+				uint size = sizeof(extended_info_array_t) + (sizeof(extended_info_t) * info->count);
 
-				*user_info = alloc_app_struct(size, free_extended_info_object_contents);
+				*user_info = alloc_app_struct(size, free_extended_info_array_contents);
 
 				if (*user_info != NULL) {
 					/* Fill in the structure returned to the app. */
@@ -1378,8 +1353,8 @@ static int do_volume_delete_warnings(logical_volume_t * vol) {
 				int answer = 0;	    /* Default is "Continue". */
 
 				engine_user_message(&answer, choices,
-						    "Volume \"%s\" has the %s file system on it.  "
-						    "The file system will be removed along with the volume.\n",
+						    _("Volume \"%s\" has the %s file system on it.  "
+						    "The file system will be removed along with the volume.\n"),
 						    vol->name, vol->file_system_manager->short_name);
 				switch (answer) {
 				case 0:
@@ -1411,12 +1386,20 @@ static int do_volume_delete_warnings(logical_volume_t * vol) {
 
 			sector_count_t one_meg_in_sectors = (1024 * 1024) >> EVMS_VSECTOR_SIZE_SHIFT;
 
-			engine_user_message(&answer, choices,
-					    "Volume \"%s\" does not have a File System Interface Module (FSIM) assigned to it.  "
-					    "The volume may have a file system on it, but none of the installed FSIM plug-ins recognizes it.  "
-					    "Do you want to write zeros to the %sdisable any file system that may be on the volume?\n",
-					    vol->name,
-					    (vol->object->size > one_meg_in_sectors) ? "first 1MB of the volume to potentially " : "volume to ");
+			if (vol->object->size > one_meg_in_sectors) {
+				engine_user_message(&answer, choices,
+						    _("Volume \"%s\" does not have a File System Interface Module (FSIM) assigned to it.  "
+						    "The volume may have a file system on it, but none of the installed FSIM plug-ins recognizes it.  "
+						    "Do you want to write zeros to the first 1MB of the volume to potentially disable any file system "
+						    "that may be on the volume?\n"),
+						    vol->name);
+			} else {
+				engine_user_message(&answer, choices,
+						    _("Volume \"%s\" does not have a File System Interface Module (FSIM) assigned to it.  "
+						    "The volume may have a file system on it, but none of the installed FSIM plug-ins recognizes it.  "
+						    "Do you want to write zeros to the volume to disable any file system that may be on the volume?\n"),
+						    vol->name);
+			}
 			switch (answer) {
 			case 0:
 				rc = vol->object->plugin->functions.plugin->add_sectors_to_kill_list(vol->object, 0, min(vol->vol_size, (1024 * 1024) >> EVMS_VSECTOR_SIZE_SHIFT));
@@ -1447,10 +1430,10 @@ static int do_volume_delete_warnings(logical_volume_t * vol) {
 			int answer = 0;	    /* Default is "Continue". */
 
 			engine_user_message(&answer, choices,
-					    "Volume \"%s\" is not an EVMS volume.  "
+					    _("Volume \"%s\" is not an EVMS volume.  "
 					    "Removing a non-EVMS volume requires writing %zd bytes of metadata at the end of the volume. "
 					    "The metadata will overwrite any data that may be at the end of the volume.  "
-					    "Do you want to continue with the delete?\n",
+					    "Do you want to continue with the delete?\n"),
 					    vol->name, (EVMS_FEATURE_HEADER_SECTORS * 2) << EVMS_VSECTOR_SIZE_SHIFT);
 
 			if (answer != 0) {
@@ -1559,15 +1542,6 @@ static int delete_volume(logical_volume_t * vol) {
 		/* Mark that this volume has no FSIM. */
 		vol->file_system_manager = NULL;
 
-		/*
-		 * If the volume's object is associated with another object, put
-		 * the other object's volume into this volume's
-		 * associated_volume field.
-		 */
-		if (obj->associated_object != NULL) {
-			vol->associated_volume = obj->associated_object->volume;
-		}
-
 		/* If the volume is active, it needs to be deactivated. */
 		if (vol->flags & VOLFLAG_ACTIVE) {
 			vol->flags |= VOLFLAG_NEEDS_DEACTIVATE;
@@ -1629,7 +1603,7 @@ storage_object_t * get_working_top_object(storage_object_t * obj) {
 
 	while ((working_top_obj != NULL) &&
 	       (GetPluginType(working_top_obj->plugin->id) == EVMS_ASSOCIATIVE_FEATURE) &&
-	       (working_top_obj->associated_object == NULL)) {
+	       (list_empty(working_top_obj->associated_children))) {
 
 		working_top_obj = first_thing(working_top_obj->child_objects, NULL);
 	}
@@ -1669,7 +1643,7 @@ void sync_volumes(void) {
 				 */
 				if ((strcmp(vol->name + EVMS_DEV_NODE_PATH_LEN, working_top_obj->name) != 0)) {
 					engine_user_message(NULL, NULL,
-							    "Volume name %s has shifted to %s%s.\n",
+							    _("Volume name %s has shifted to %s%s.\n"),
 							    vol->name, EVMS_DEV_NODE_PATH, working_top_obj->name);
 
 					engine_unregister_name(vol->name);
@@ -1679,10 +1653,6 @@ void sync_volumes(void) {
 					strcpy(vol->name, EVMS_DEV_NODE_PATH);
 					strcat(vol->name, working_top_obj->name);
 					engine_register_name(vol->name);
-
-					if (!(vol->flags & VOLFLAG_NEW)) {
-						vol->flags |= VOLFLAG_NEEDS_ACTIVATE;
-					}
 				}
 			}
 
@@ -2408,11 +2378,7 @@ int make_handle_array(list_anchor_t      list,
 	count = list_count(list);
 
 		LOG_DEBUG("Number of objects in the list:  %d\n", count);
-		if (count > 1) {
-			size = sizeof(handle_array_t) + ((count -1) * sizeof(object_handle_t));
-		} else {
-			size = sizeof(handle_array_t);
-		}
+		size = sizeof(handle_array_t) + (count * sizeof(object_handle_t));
 
 		*ha = engine_alloc(size);
 		if (*ha != NULL) {
@@ -2673,17 +2639,6 @@ int evms_get_info(object_handle_t thing, handle_object_info_t * * user_info) {
 							}
 						}
 
-						if (rc == 0) {
-							if (vol->associated_volume != NULL) {
-								rc = ensure_app_handle(vol->associated_volume);
-								if (rc == 0) {
-									info->info.volume.associated_volume = vol->associated_volume->app_handle;
-								}
-							} else {
-								info->info.volume.associated_volume = 0;
-							}
-						}
-
 						if (rc == 0) {
 							if (vol->disk_group != NULL) {
 
@@ -2965,30 +2920,56 @@ int evms_get_object_handle_for_name(object_type_t type, char * name, object_hand
 		return rc;
 	}
 
+	LOG_DEBUG("Lookup handle for thing of type %#x and name \"%s\".\n", type, name);
+
 	rc = ENOENT;
 
 	if (type & DISK) {
 		rc = find_object_handle_by_name(&disks_list, name, object_handle);
+		if (rc == 0) {
+			LOG_DEBUG("Found %s in the disks list.  It has handle %d.\n",
+				  name, *object_handle);
+		}
 	}
 
 	if ((rc == ENOENT) && (type & SEGMENT)) {
 		rc = find_object_handle_by_name(&segments_list, name, object_handle);
+		if (rc == 0) {
+			LOG_DEBUG("Found %s in the segments list.  It has handle %d.\n",
+				  name, *object_handle);
+		}
 	}
 
 	if ((rc == ENOENT) && (type & REGION)) {
 		rc = find_object_handle_by_name(&regions_list, name, object_handle);
+		if (rc == 0) {
+			LOG_DEBUG("Found %s in the regions list.  It has handle %d.\n",
+				  name, *object_handle);
+		}
 	}
 
 	if ((rc == ENOENT) && (type & EVMS_OBJECT)) {
 		rc = find_object_handle_by_name(&EVMS_objects_list, name, object_handle);
+		if (rc == 0) {
+			LOG_DEBUG("Found %s in the EVMS objects list.  It has handle %d.\n",
+				  name, *object_handle);
+		}
 	}
 
 	if ((rc == ENOENT) && (type & CONTAINER)) {
 		rc = find_container_handle_by_name(name, object_handle);
+		if (rc == 0) {
+			LOG_DEBUG("Found %s in the containers list.  It has handle %d.\n",
+				  name, *object_handle);
+		}
 	}
 
 	if ((rc == ENOENT) && (type & VOLUME)) {
 		rc = find_volume_handle_by_name(name, object_handle);
+		if (rc == 0) {
+			LOG_DEBUG("Found %s in the volumes list.  It has handle %d.\n",
+				  name, *object_handle);
+		}
 	}
 
 	LOG_PROC_EXIT_INT(rc);
