@@ -26,11 +26,6 @@
 
 /* External APIs provided by container.c */
 
-extern int evms_can_add_to_container(object_handle_t object,
-                                     object_handle_t container);
-
-extern int evms_can_remove_from_container(object_handle_t object);
-
 extern int evms_create_container(plugin_handle_t   plugin,
                                  handle_array_t  * input_objects,
                                  option_array_t  * options,
