@@ -423,7 +423,7 @@ static int receive_response_for_command(ece_msg_t * response_msg) {
 				       (response_msg->cmd & COMMAND_MASK), (required_cmd & COMMAND_MASK));
 			}
 			if (!(response_msg->cmd & COMMAND_RESPONSE)) {
-				LOG_DEBUG("Recived command %u is not a response.\n", response_msg->cmd);
+				LOG_DEBUG("Received command %u is not a response.\n", response_msg->cmd);
 			}
 
 			LOG_DEBUG("Sending the message back to node %s.\n", nodeid_to_string(&response_msg->node));
@@ -553,9 +553,10 @@ static void process_open_engine(const ece_msg_t * msg) {
 	worker = engine_alloc(sizeof(worker_context_t));
 
 	if (worker == NULL) {
+		LOG_CRITICAL("Failed to allocate memory for a worker context.\n");
 		response_msg = get_msg(msg);
 
-		evms_host_to_net(&net_rc, int_f, errno);
+		evms_host_to_net(&net_rc, int_f, ENOMEM);
 
 		response_msg->cmd |= COMMAND_RESPONSE;
 		response_msg->size = sizeof(net_rc);
@@ -575,6 +576,9 @@ static void process_open_engine(const ece_msg_t * msg) {
 	pthread_mutex_init(&worker->output_pipe_mutex, NULL);
 
 	if (pipe(worker->input_pipe)) {
+		LOG_SERIOUS("Failed to create an input pipe for the worker.  Error code is %d: %s\n",
+			    errno, strerror(errno));
+
 		response_msg = get_msg(msg);
 
 		evms_host_to_net(&net_rc, int_f, errno);
@@ -597,6 +601,9 @@ static void process_open_engine(const ece_msg_t * msg) {
 	LOG_DEBUG("worker->input_pipe handles are %d (read) and %d (write).\n", worker->input_pipe[0], worker->input_pipe[1]);
 
 	if (pipe(worker->output_pipe)) {
+		LOG_SERIOUS("Failed to create an output pipe for the worker.  Error code is %d: %s\n",
+			    errno, strerror(errno));
+
 		response_msg = get_msg(msg);
 
 		evms_host_to_net(&net_rc, int_f, errno);
@@ -626,6 +633,9 @@ static void process_open_engine(const ece_msg_t * msg) {
 	switch (worker->pid) {
 		case -1:
 			/* Fork failed.  Send back an error code. */
+			LOG_SERIOUS("Failed to fork to create the worker process.  Error code is %d: %s\n",
+				    errno, strerror(errno));
+
 			response_msg = get_msg(msg);
 
 			evms_host_to_net(&net_rc, int_f, errno);
@@ -791,7 +801,7 @@ void daemon_router(ece_msg_t * msg) {
 		 */
 		if ((cmd != EVMS_OPEN_ENGINE) && !worker_running) {
 			ece_msg_t * response = get_msg(msg);
-			flat_int_ret_t flat_int = HOST_TO_NET32((u_int32_t) EINVAL);
+			u_int32_t flat_int = HOST_TO_NET32((u_int32_t) EINVAL);
 
 			LOG_DEBUG("The Engine is not open.  The worker thread is not running.\n");
 			response->cmd |= COMMAND_RESPONSE;
