@@ -2584,6 +2584,7 @@ static void cleanup_empty_dirs(char * dir_name) {
 	struct dirent * dir_ent;
 	char name_buf[PATH_MAX];
 	int dir_path_len;
+	boolean dir_empty = TRUE;
 
 	LOG_PROC_ENTRY();
 
@@ -2627,8 +2628,8 @@ static void cleanup_empty_dirs(char * dir_name) {
 				cleanup_empty_dirs(name_buf);
 
 				/*
-				 * If the sub directory was empty it will have
-				 * benn deleted. Check if it is still there.
+				 * If the subdirectory was empty it will have
+				 * been deleted. Check if it is still there.
 				 */
 				if (stat(name_buf, &stat_buf) != 0) {
 
@@ -2644,23 +2645,27 @@ static void cleanup_empty_dirs(char * dir_name) {
 					/*
 					 * The subdirectory is still there.
 					 * This directory is not empty.
-					 * We're finished.
 					 */
-					break;
+					dir_empty = FALSE;
 				}
-			}
 
-		} else {
-			/*
-			 * It's not a subdirectory.  Something else is in this
-			 * directory.  It's not empty.  We're finished.
-			 */
-			break;
+			} else {
+				/*
+				 * It's not a subdirectory.  Something else is
+				 * in this directory.  It's not empty.
+				 */
+				dir_empty = FALSE;
+			}
 		}
 	}
 
 	closedir(dir);
 
+	if (dir_empty) {
+		LOG_DEBUG("Removing empty directory %s.\n", dir_name);
+		rmdir(dir_name);
+	}
+
 	LOG_PROC_EXIT_VOID();
 	return;
 }
@@ -2730,7 +2735,7 @@ static void cleanup_dev_evms_tree(void) {
 
 	engine_free(dev_nodes);
 
-	cleanup_empty_dirs(EVMS_DIR_NAME);
+	cleanup_empty_dirs(EVMS_DEV_NODE_DIR);
 
 	status_message(_("Finished cleaning up the /dev/evms tree.\n"));
 
