@@ -182,17 +182,18 @@ static boolean isa_duplicate_plugin( plugin_record_t * pPlugRec, list_anchor_t p
 		
 		if (pr->id == pPlugRec->id) {
 
-			engine_user_message(NULL, NULL, "Duplicate Plug-ins detected\n"
-					    "Already have loaded:\n"
-					    "  long name: %-40s\n"
-					    "  oem name: %-40s\n"
-					    "  ID: %#08X\n"
-					    "  version: %d.%d.%d\n"
-					    "Currently loading:\n"
-					    "  long name: %-40s\n"
-					    "  oem name: %-40s\n"
-					    "  ID: %#08X\n"
-					    "  version: %d.%d.%d\n",
+			engine_user_message(NULL, NULL,
+					    _("Duplicate Plug-ins detected\n"
+					      "Already have loaded:\n"
+					      "  long name: %-40s\n"
+					      "  oem name: %-40s\n"
+					      "  ID: %#08X\n"
+					      "  version: %d.%d.%d\n"
+					      "Currently loading:\n"
+					      "  long name: %-40s\n"
+					      "  oem name: %-40s\n"
+					      "  ID: %#08X\n"
+					      "  version: %d.%d.%d\n"),
 					    pr->long_name,
 					    pr->oem_name,
 					    pr->id,
@@ -229,13 +230,15 @@ static boolean isa_duplicate_plugin( plugin_record_t * pPlugRec, list_anchor_t p
 				 * loaded.
 				 */
 
-				engine_user_message(NULL, NULL, "Unloading version %d.%d.%d of the plug-in.\n",
+				engine_user_message(NULL, NULL,
+						    _("Unloading version %d.%d.%d of the plug-in.\n"),
 						    pr->version.major, pr->version.minor, pr->version.patchlevel);
 				delete_element(iter1);
 				release_plugin(pr);
 
 			} else {
-				engine_user_message(NULL, NULL, "Keeping version %d.%d.%d of the plug-in.\n",
+				engine_user_message(NULL, NULL,
+						    _("Keeping version %d.%d.%d of the plug-in.\n"),
 						    pr->version.major, pr->version.minor, pr->version.patchlevel);
 			}
 
@@ -263,13 +266,15 @@ static boolean isa_valid_plugin_record(plugin_record_t * pPlugRec) {
 	if (rc != 0) {
 
 		if (rc < 0) {
-			engine_user_message(NULL, NULL, "The plug-in %s in module %s requires Engine services API version (%d.%d.%d) which is greater than this Engine's services API version (%d.%d.%d).\n",
+			engine_user_message(NULL, NULL,
+					    _("The plug-in %s in module %s requires Engine services API version (%d.%d.%d) which is greater than this Engine's services API version (%d.%d.%d).\n"),
 					    pPlugRec->short_name, pPlugRec->so_record->name,
 					    pPlugRec->required_engine_api_version.major, pPlugRec->required_engine_api_version.minor, pPlugRec->required_engine_api_version.patchlevel,
 					    engine_services_api_version.major, engine_services_api_version.minor, engine_services_api_version.patchlevel);
 
 		} else {
-			engine_user_message(NULL, NULL, "The plug-in %s in module %s requires Engine services API version (%d.%d.%d) which is less than this Engine's services API version (%d.%d.%d).\n",
+			engine_user_message(NULL, NULL,
+					    _("The plug-in %s in module %s requires Engine services API version (%d.%d.%d) which is less than this Engine's services API version (%d.%d.%d).\n"),
 					    pPlugRec->short_name, pPlugRec->so_record->name,
 					    pPlugRec->required_engine_api_version.major, pPlugRec->required_engine_api_version.minor, pPlugRec->required_engine_api_version.patchlevel,
 					    engine_services_api_version.major, engine_services_api_version.minor, engine_services_api_version.patchlevel);
@@ -288,13 +293,15 @@ static boolean isa_valid_plugin_record(plugin_record_t * pPlugRec) {
 			rc = check_version(pPlugRec->required_plugin_api_version.fsim, engine_fsim_api_version);
 			if (rc != 0) {
 				if (rc < 0) {
-					engine_user_message(NULL, NULL, "The plug-in %s in module %s requires Engine FSIM API version (%d.%d.%d) which is greater than this Engine's FSIM API version (%d.%d.%d).\n",
+					engine_user_message(NULL, NULL,
+							    _("The plug-in %s in module %s requires Engine FSIM API version (%d.%d.%d) which is greater than this Engine's FSIM API version (%d.%d.%d).\n"),
 							    pPlugRec->short_name, pPlugRec->so_record->name,
 							    pPlugRec->required_plugin_api_version.fsim.major, pPlugRec->required_plugin_api_version.fsim.minor, pPlugRec->required_plugin_api_version.fsim.patchlevel,
 							    engine_fsim_api_version.major, engine_fsim_api_version.minor, engine_fsim_api_version.patchlevel);
 
 				} else {
-					engine_user_message(NULL, NULL, "The plug-in %s in module %s requires Engine FSIM API version (%d.%d.%d) which is less than this Engine's FSIM API version (%d.%d.%d).\n",
+					engine_user_message(NULL, NULL,
+							    _("The plug-in %s in module %s requires Engine FSIM API version (%d.%d.%d) which is less than this Engine's FSIM API version (%d.%d.%d).\n"),
 							    pPlugRec->short_name, pPlugRec->so_record->name,
 							    pPlugRec->required_plugin_api_version.fsim.major, pPlugRec->required_plugin_api_version.fsim.minor, pPlugRec->required_plugin_api_version.fsim.patchlevel,
 							    engine_fsim_api_version.major, engine_fsim_api_version.minor, engine_fsim_api_version.patchlevel);
@@ -308,13 +315,15 @@ static boolean isa_valid_plugin_record(plugin_record_t * pPlugRec) {
 			rc = check_version(pPlugRec->required_plugin_api_version.cluster, engine_cluster_api_version);
 			if (rc != 0) {
 				if (rc < 0) {
-					engine_user_message(NULL, NULL, "The plug-in %s in module %s requires Engine cluster manager API version (%d.%d.%d) which is greater than this Engine's cluster manager API version (%d.%d.%d).\n",
+					engine_user_message(NULL, NULL,
+							    _("The plug-in %s in module %s requires Engine cluster manager API version (%d.%d.%d) which is greater than this Engine's cluster manager API version (%d.%d.%d).\n"),
 							    pPlugRec->short_name, pPlugRec->so_record->name,
 							    pPlugRec->required_plugin_api_version.cluster.major, pPlugRec->required_plugin_api_version.cluster.minor, pPlugRec->required_plugin_api_version.cluster.patchlevel,
 							    engine_cluster_api_version.major, engine_cluster_api_version.minor, engine_cluster_api_version.patchlevel);
 
 				} else {
-					engine_user_message(NULL, NULL, "The plug-in %s in module %s requires Engine cluster manager API version (%d.%d.%d) which is less than this Engine's cluster manager API version (%d.%d.%d).\n",
+					engine_user_message(NULL, NULL,
+							    _("The plug-in %s in module %s requires Engine cluster manager API version (%d.%d.%d) which is less than this Engine's cluster manager API version (%d.%d.%d).\n"),
 							    pPlugRec->short_name, pPlugRec->so_record->name,
 							    pPlugRec->required_plugin_api_version.cluster.major, pPlugRec->required_plugin_api_version.cluster.minor, pPlugRec->required_plugin_api_version.cluster.patchlevel,
 							    engine_cluster_api_version.major, engine_cluster_api_version.minor, engine_cluster_api_version.patchlevel);
@@ -328,13 +337,15 @@ static boolean isa_valid_plugin_record(plugin_record_t * pPlugRec) {
 			rc = check_version(pPlugRec->required_plugin_api_version.plugin, engine_plugin_api_version);
 			if (rc != 0) {
 				if (rc < 0) {
-					engine_user_message(NULL, NULL, "The plug-in %s in module %s requires Engine plug-in API version (%d.%d.%d) which is greater than this Engine's plug-in API version (%d.%d.%d).\n",
+					engine_user_message(NULL, NULL,
+							    _("The plug-in %s in module %s requires Engine plug-in API version (%d.%d.%d) which is greater than this Engine's plug-in API version (%d.%d.%d).\n"),
 							    pPlugRec->short_name, pPlugRec->so_record->name,
 							    pPlugRec->required_plugin_api_version.plugin.major, pPlugRec->required_plugin_api_version.plugin.minor, pPlugRec->required_plugin_api_version.plugin.patchlevel,
 							    engine_plugin_api_version.major, engine_plugin_api_version.minor, engine_plugin_api_version.patchlevel);
 
 				} else {
-					engine_user_message(NULL, NULL, "The plug-in %s in module %s requires Engine plug-in API version (%d.%d.%d) which is less than this Engine's plug-in API version (%d.%d.%d).\n",
+					engine_user_message(NULL, NULL,
+							    _("The plug-in %s in module %s requires Engine plug-in API version (%d.%d.%d) which is less than this Engine's plug-in API version (%d.%d.%d).\n"),
 							    pPlugRec->short_name, pPlugRec->so_record->name,
 							    pPlugRec->required_plugin_api_version.plugin.major, pPlugRec->required_plugin_api_version.plugin.minor, pPlugRec->required_plugin_api_version.plugin.patchlevel,
 							    engine_plugin_api_version.major, engine_plugin_api_version.minor, engine_plugin_api_version.patchlevel);
@@ -350,13 +361,15 @@ static boolean isa_valid_plugin_record(plugin_record_t * pPlugRec) {
 			rc = check_version(pPlugRec->required_container_api_version, engine_container_api_version);
 			if (rc != 0) {
 				if (rc < 0) {
-					engine_user_message(NULL, NULL, "The plug-in %s in module %s requires Engine container API version (%d.%d.%d) which is greater than this Engine's container API version (%d.%d.%d).\n",
+					engine_user_message(NULL, NULL,
+							    _("The plug-in %s in module %s requires Engine container API version (%d.%d.%d) which is greater than this Engine's container API version (%d.%d.%d).\n"),
 							    pPlugRec->short_name, pPlugRec->so_record->name,
 							    pPlugRec->required_container_api_version.major, pPlugRec->required_container_api_version.minor, pPlugRec->required_container_api_version.patchlevel,
 							    engine_container_api_version.major, engine_container_api_version.minor, engine_container_api_version.patchlevel);
 
 				} else {
-					engine_user_message(NULL, NULL, "The plug-in %s in module %s requires Engine container API version (%d.%d.%d) which is less than this Engine's container API version (%d.%d.%d).\n",
+					engine_user_message(NULL, NULL,
+							    _("The plug-in %s in module %s requires Engine container API version (%d.%d.%d) which is less than this Engine's container API version (%d.%d.%d).\n"),
 							    pPlugRec->short_name, pPlugRec->so_record->name,
 							    pPlugRec->required_container_api_version.major, pPlugRec->required_container_api_version.minor, pPlugRec->required_container_api_version.patchlevel,
 							    engine_container_api_version.major, engine_container_api_version.minor, engine_container_api_version.patchlevel);
@@ -369,19 +382,25 @@ static boolean isa_valid_plugin_record(plugin_record_t * pPlugRec) {
 		if (pPlugRec->short_name == NULL) {
 			if (pPlugRec->long_name == NULL) {
 				if (pPlugRec->oem_name == NULL) {
-					engine_user_message(NULL, NULL, "A plug-in in module %s did not provide a short name, long name, nor an oem name.\n",
+					engine_user_message(NULL, NULL,
+							    _("A plug-in in module %s did not provide a short name, long name, nor an oem name.\n"),
 							    pPlugRec->so_record->name);
 				} else {
-					engine_user_message(NULL, NULL, "A plug-in in module %s did not provide a short name nor a long name.  The oem name is \"%s\".\n",
+					engine_user_message(NULL, NULL,
+							    _("A plug-in in module %s did not provide a short name nor a long name.  "
+							      "The oem name is \"%s\".\n"),
 							    pPlugRec->so_record->name, pPlugRec->oem_name);
 				}
 
 			} else {
 				if (pPlugRec->oem_name == NULL) {
-					engine_user_message(NULL, NULL, "A plug-in in module %s did not provide a short name nor an oem name.  The long name is \"%s\".\n",
+					engine_user_message(NULL, NULL,
+							    _("A plug-in in module %s did not provide a short name nor an oem name.  The long name is \"%s\".\n"),
 							    pPlugRec->so_record->name, pPlugRec->long_name);
 				} else {
-					engine_user_message(NULL, NULL, "A plug-in in module %s did not provide a short name.  The long name is \"%s\".  The oem name is \"%s\".\n",
+					engine_user_message(NULL, NULL,
+							    _("A plug-in in module %s did not provide a short name.  The long name is \"%s\".  "
+							      "The oem name is \"%s\".\n"),
 							    pPlugRec->so_record->name, pPlugRec->long_name, pPlugRec->oem_name);
 				}
 			}
@@ -391,10 +410,12 @@ static boolean isa_valid_plugin_record(plugin_record_t * pPlugRec) {
 		} else {
 			if (pPlugRec->long_name == NULL) {
 				if (pPlugRec->oem_name == NULL) {
-					engine_user_message(NULL, NULL, "A plug-in in module %s did not provide a long name, nor an oem name.  The short name is \"%s\".\n",
+					engine_user_message(NULL, NULL,
+							    _("A plug-in in module %s did not provide a long name, nor an oem name.  The short name is \"%s\".\n"),
 							    pPlugRec->so_record->name, pPlugRec->short_name);
 				} else {
-					engine_user_message(NULL, NULL, "A plug-in in module %s did not provide a long name.  The short name is \"%s\".  The oem name is \"%s\".\n",
+					engine_user_message(NULL, NULL,
+							    _("A plug-in in module %s did not provide a long name.  The short name is \"%s\".  The oem name is \"%s\".\n"),
 							    pPlugRec->so_record->name, pPlugRec->short_name, pPlugRec->oem_name);
 				}
 
@@ -402,7 +423,8 @@ static boolean isa_valid_plugin_record(plugin_record_t * pPlugRec) {
 
 			} else {
 				if (pPlugRec->oem_name == NULL) {
-					engine_user_message(NULL, NULL, "A plug-in in module %s did not provide an oem name.  The short name is \"%s\".  The long name is \"%s\".\n",
+					engine_user_message(NULL, NULL,
+							    _("A plug-in in module %s did not provide an oem name.  The short name is \"%s\".  The long name is \"%s\".\n"),
 							    pPlugRec->so_record->name, pPlugRec->short_name, pPlugRec->long_name);
 					result = FALSE;
 
@@ -413,7 +435,8 @@ static boolean isa_valid_plugin_record(plugin_record_t * pPlugRec) {
 		}
 
 		if (pPlugRec->functions.plugin == NULL) {
-			engine_user_message(NULL, NULL, "The plug-in %s in module %s did not provide a function table.\n",
+			engine_user_message(NULL, NULL,
+					    _("The plug-in %s in module %s did not provide a function table.\n"),
 					    pPlugRec->short_name, pPlugRec->so_record->name);
 			result = FALSE;
 		}
@@ -421,7 +444,7 @@ static boolean isa_valid_plugin_record(plugin_record_t * pPlugRec) {
 	}
 
 	if (!result) {
-		engine_user_message(NULL, NULL, "The plug-in failed to load.\n");
+		engine_user_message(NULL, NULL, _("The plug-in failed to load.\n"));
 	}
 
 	LOG_PROC_EXIT_BOOLEAN(result);
@@ -445,7 +468,7 @@ static int load_module(char * soname, so_record_t * * pso_record) {
 
 	if (so_record != NULL) {
 
-		status_message("Loading module: %s\n", soname);
+		status_message(_("Loading module: %s\n"), soname);
 		dlerror();
 		so_record->handle = dlopen(soname, RTLD_NOW);
 
@@ -465,7 +488,7 @@ static int load_module(char * soname, so_record_t * * pso_record) {
 			}
 
 		} else {
-			engine_user_message(NULL, NULL, "Error loading %s: %s\n", soname, error);
+			engine_user_message(NULL, NULL, _("Error loading %s: %s\n"), soname, error);
 			engine_free(so_record);
 			rc = ELIBBAD;
 		}
@@ -523,8 +546,8 @@ static int setup_clustering() {
 
 				if (current_nodeid == &no_nodeid) {
 					engine_user_message(NULL, NULL,
-							    "%s is not the name of a node in this cluster. "
-							    "The Engine cannot be opened on the node named %s.\n",
+							    _("%s is not the name of a node in this cluster. "
+							      "The Engine cannot be opened on the node named %s.\n"),
 							    current_node_name, current_node_name);
 					rc = ENOENT;
 				}
@@ -537,7 +560,7 @@ static int setup_clustering() {
 
 			} else {
 				engine_user_message(NULL, NULL,
-						    "The node_name parameter is not valid when starting the daemon.\n");
+						    _("The node_name parameter is not valid when starting the daemon.\n"));
 				rc = EINVAL;
 			}
 
@@ -554,14 +577,14 @@ static int setup_clustering() {
 			 * Make sure the EVMS daemons in the cluster are
 			 * at the correct protocol version.
 			 */
-			status_message("Verifying communication protocol versions...\n");
+			status_message(_("Verifying communication protocol versions...\n"));
 
 			rc = remote_verify_version();
 			if (rc != 0) {
 				engine_user_message(NULL, NULL,
-						    "There was an error when validating the version of the daemon protocol on the other nodes in the cluster.  "
-						    "The Engine will run with clustering support disabled.  "
-						    "EVMS will only manage local devices on this system.\n");
+						    _("There was an error when validating the version of the daemon protocol on the other nodes in the cluster.  "
+						      "The Engine will run with clustering support disabled.  "
+						      "EVMS will only manage local devices on this system.\n"));
 				disconnect_from_ece();
 				local_focus = TRUE;
 				rc = 0;
@@ -576,16 +599,16 @@ static int setup_clustering() {
 	} else {
 		if (current_node_name != NULL) {
 			engine_user_message(NULL, NULL,
-					    "There was an error when connecting to %s.  "
-					    "The error code was %d: %s  "
-					    "This machine is not running in a clustered environment.\n",
+					    _("There was an error when connecting to %s.  "
+					      "The error code was %d: %s  "
+					      "This machine is not running in a clustered environment.\n"),
 					    cluster_manager->short_name, rc, evms_strerror(rc));
 
 		} else {
 			engine_user_message(NULL, NULL,
-					    "There was an error when connecting to %s.  "
-					    "The error code was %d: %s  "
-					    "EVMS will only manage local devices on this system.\n",
+					    _("There was an error when connecting to %s.  "
+					      "The error code was %d: %s  "
+					      "EVMS will only manage local devices on this system.\n"),
 					    cluster_manager->short_name, rc, evms_strerror(rc));
 			rc = 0;
 		}
@@ -688,14 +711,18 @@ static int validate_plugin_functions(plugin_record_t * pPlugRec) {
 	if (pPlugRec->functions.plugin->replace_child == NULL) {
 		pPlugRec->functions.plugin->replace_child = (void *) return_ENOSYS;
 	}
-
+	if (pPlugRec->functions.plugin->can_activate == NULL) {
+		pPlugRec->functions.plugin->can_activate = (void *) return_ENOSYS;
+	}
 	if (pPlugRec->functions.plugin->activate == NULL) {
-		pPlugRec->functions.plugin->activate = (void *) return_0;
+		pPlugRec->functions.plugin->activate = (void *) return_ENOSYS;
+	}
+	if (pPlugRec->functions.plugin->can_deactivate == NULL) {
+		pPlugRec->functions.plugin->can_deactivate = (void *) return_ENOSYS;
 	}
 	if (pPlugRec->functions.plugin->deactivate == NULL) {
-		pPlugRec->functions.plugin->deactivate = (void *) return_0;
+		pPlugRec->functions.plugin->deactivate = (void *) return_ENOSYS;
 	}
-
 	if (pPlugRec->functions.plugin->get_option_count == NULL) {
 		pPlugRec->functions.plugin->get_option_count = (void *) return_0;
 	}
@@ -723,6 +750,9 @@ static int validate_plugin_functions(plugin_record_t * pPlugRec) {
 	if (pPlugRec->functions.plugin->plugin_function == NULL) {
 		pPlugRec->functions.plugin->plugin_function = (void *) return_ENOSYS;
 	}
+	if (pPlugRec->functions.plugin->backup_metadata == NULL) {
+		pPlugRec->functions.plugin->backup_metadata = (void *) return_ENOSYS;
+	}
 
 	LOG_PROC_EXIT_INT(rc);
 	return rc;
@@ -762,9 +792,6 @@ static int validate_fsim_functions(plugin_record_t * pPlugRec) {
 	if (pPlugRec->functions.fsim->can_fsck == NULL) {
 		pPlugRec->functions.fsim->can_fsck = (void *) return_ENOSYS;
 	}
-	if (pPlugRec->functions.fsim->can_defrag == NULL) {
-		pPlugRec->functions.fsim->can_defrag = (void *) return_ENOSYS;
-	}
 	if (pPlugRec->functions.fsim->can_expand_by == NULL) {
 		pPlugRec->functions.fsim->can_expand_by = (void *) return_ENOSYS;
 	}
@@ -786,15 +813,15 @@ static int validate_fsim_functions(plugin_record_t * pPlugRec) {
 	if (pPlugRec->functions.fsim->fsck == NULL) {
 		pPlugRec->functions.fsim->fsck = (void *) return_ENOSYS;
 	}
-	if (pPlugRec->functions.fsim->defrag == NULL) {
-		pPlugRec->functions.fsim->defrag = (void *) return_ENOSYS;
-	}
 	if (pPlugRec->functions.fsim->expand == NULL) {
 		pPlugRec->functions.fsim->expand = (void *) return_ENOSYS;
 	}
 	if (pPlugRec->functions.fsim->shrink == NULL) {
 		pPlugRec->functions.fsim->shrink = (void *) return_ENOSYS;
 	}
+	if (pPlugRec->functions.fsim->commit_changes == NULL) {
+		pPlugRec->functions.fsim->commit_changes = (void *) return_0;
+	}
 	if (pPlugRec->functions.fsim->get_option_count == NULL) {
 		pPlugRec->functions.fsim->get_option_count = (void *) return_0;
 	}
@@ -836,15 +863,9 @@ static int validate_container_functions(plugin_record_t * pPlugRec) {
 	 * functions.
 	 */
 
-	if ((pPlugRec->container_functions->can_create_container     == NULL) ||
-	    (pPlugRec->container_functions->can_delete_container     == NULL) ||
-	    (pPlugRec->container_functions->can_add_object           == NULL) ||
-	    (pPlugRec->container_functions->can_remove_object        == NULL) ||
+	if ((pPlugRec->container_functions->can_delete_container     == NULL) ||
 	    (pPlugRec->container_functions->create_container         == NULL) ||
-	    (pPlugRec->container_functions->add_object               == NULL) ||
-	    (pPlugRec->container_functions->transfer_object          == NULL) ||
 	    (pPlugRec->container_functions->discard_container        == NULL) ||
-	    (pPlugRec->container_functions->remove_object            == NULL) ||
 	    (pPlugRec->container_functions->delete_container         == NULL) ||
 	    (pPlugRec->container_functions->commit_container_changes == NULL) ||
 	    (pPlugRec->container_functions->get_container_info       == NULL)) {
@@ -856,12 +877,27 @@ static int validate_container_functions(plugin_record_t * pPlugRec) {
 	 * The remaining functions are not absolutely necessary for a plug-in to
 	 * function.  If the plug-in doesn't specify them, we supply a default.
 	 */
+	if (pPlugRec->container_functions->can_expand_container == NULL) {
+		pPlugRec->container_functions->can_expand_container = (void *) return_ENOSYS;
+	}
+	if (pPlugRec->container_functions->can_shrink_container == NULL) {
+		pPlugRec->container_functions->can_shrink_container = (void *) return_ENOSYS;
+	}
+	if (pPlugRec->container_functions->expand_container == NULL) {
+		pPlugRec->container_functions->expand_container = (void *) return_ENOSYS;
+	}
+	if (pPlugRec->container_functions->shrink_container == NULL) {
+		pPlugRec->container_functions->shrink_container = (void *) return_ENOSYS;
+	}
 	if (pPlugRec->container_functions->get_plugin_functions == NULL) {
 		pPlugRec->container_functions->get_plugin_functions = (void *) return_ENOSYS;
 	}
 	if (pPlugRec->container_functions->plugin_function == NULL) {
 		pPlugRec->container_functions->plugin_function = (void *) return_ENOSYS;
 	}
+	if (pPlugRec->container_functions->backup_container_metadata == NULL) {
+		pPlugRec->container_functions->backup_container_metadata = (void *) return_ENOSYS;
+	}
 
 	LOG_PROC_EXIT_INT(rc);
 	return rc;
@@ -900,7 +936,8 @@ static int check_plugin(list_anchor_t plugin_list, plugin_record_t * pPlugRec) {
 
 	LOG_PROC_ENTRY();
 
-	status_message("Validating plug-in: %s\n", pPlugRec->long_name);
+	status_message(_("Validating plug-in: %s\n"),
+		       pPlugRec->long_name);
 
 	if (isa_valid_plugin_record( pPlugRec ) == TRUE) {
 
@@ -929,12 +966,16 @@ static int check_plugin(list_anchor_t plugin_list, plugin_record_t * pPlugRec) {
 						rc = validate_container_functions(pPlugRec);
 
 						if (rc != 0) {
-							engine_user_message(NULL, NULL, "The %s plug-in in module %s failed to load.  It does not have a valid container functions table.\n", pPlugRec->short_name, pPlugRec->so_record->name);
+							engine_user_message(NULL, NULL,
+									    _("The %s plug-in in module %s failed to load.  It does not have a valid container functions table.\n"),
+									    pPlugRec->short_name, pPlugRec->so_record->name);
 						}
 					}
 
 				} else {
-					engine_user_message(NULL, NULL, "The %s plug-in in module %s failed to load.  It does not have a valid function table.\n", pPlugRec->short_name, pPlugRec->so_record->name);
+					engine_user_message(NULL, NULL,
+							    _("The %s plug-in in module %s failed to load.  It does not have a valid function table.\n"),
+							    pPlugRec->short_name, pPlugRec->so_record->name);
 				}
 				break;
 
@@ -947,7 +988,9 @@ static int check_plugin(list_anchor_t plugin_list, plugin_record_t * pPlugRec) {
 				} else {
 					rc = validate_fsim_functions(pPlugRec);
 					if (rc != 0) {
-						engine_user_message(NULL, NULL, "The %s plug-in in module %s failed to load.  It does not have a valid function table.\n", pPlugRec->short_name, pPlugRec->so_record->name);
+						engine_user_message(NULL, NULL,
+								    _("The %s plug-in in module %s failed to load.  It does not have a valid function table.\n"),
+								    pPlugRec->short_name, pPlugRec->so_record->name);
 					}
 
 				}
@@ -961,7 +1004,9 @@ static int check_plugin(list_anchor_t plugin_list, plugin_record_t * pPlugRec) {
 				if (cluster_manager == NULL) {
 					rc = validate_cluster_functions(pPlugRec);
 					if (rc != 0) {
-						engine_user_message(NULL, NULL, "The %s plug-in in module %s failed to load.  It does not have a valid function table.\n", pPlugRec->short_name, pPlugRec->so_record->name);
+						engine_user_message(NULL, NULL,
+								    _("The %s plug-in in module %s failed to load.  It does not have a valid function table.\n"),
+								    pPlugRec->short_name, pPlugRec->so_record->name);
 					}
 
 				} else {
@@ -970,7 +1015,9 @@ static int check_plugin(list_anchor_t plugin_list, plugin_record_t * pPlugRec) {
 				break;
 
 			default:
-				engine_user_message(NULL, NULL, "The %s plug-in in module %s failed to load.  It is of type of %#x which is not valid.\n", pPlugRec->short_name, pPlugRec->so_record->name, plugin_type);
+				engine_user_message(NULL, NULL,
+						    _("The %s plug-in in module %s failed to load.  It is of type of %#x which is not valid.\n"),
+						    pPlugRec->short_name, pPlugRec->so_record->name, plugin_type);
 				rc = ENOSYS;
 			}
 
@@ -1081,7 +1128,7 @@ static int setup_plugins() {
 
 	LIST_FOR_EACH(plugin_set, iter, plug_rec) {
 		if (cluster_manager == NULL) {
-			status_message("Setup plug-in: %s\n", plug_rec->long_name);
+			status_message(_("Setup plug-in: %s\n"), plug_rec->long_name);
 			rc = plug_rec->functions.cluster->setup_evms_plugin(&engine_functions);
 
 			if (rc == 0) {
@@ -1101,7 +1148,9 @@ static int setup_plugins() {
 
 			} else {
 				if (rc != E_NOLOAD) {
-					engine_user_message(NULL, NULL, "The plug-in %s in module %s failed to load.  The plug-in's setup_evms_plugin() function failed with error code %d: %s.\n", plug_rec->short_name, plug_rec->so_record->name, rc, strerror(rc));
+					engine_user_message(NULL, NULL,
+							    _("The plug-in %s in module %s failed to load.  The plug-in's setup_evms_plugin() function failed with error code %d: %s.\n"),
+							    plug_rec->short_name, plug_rec->so_record->name, rc, strerror(rc));
 				}
 			}
 
@@ -1126,7 +1175,7 @@ static int setup_plugins() {
 	}
 
 	LIST_FOR_EACH(plugin_set, iter, plug_rec) {
-		status_message("Setup plug-in: %s\n", plug_rec->long_name);
+		status_message(_("Setup plug-in: %s\n"), plug_rec->long_name);
 		rc = plug_rec->functions.plugin->setup_evms_plugin(&engine_functions);
 		if (rc == 0) {
 			/* Handle special plug-ins. */
@@ -1155,7 +1204,9 @@ static int setup_plugins() {
 
 		} else {
 			if (rc != E_NOLOAD) {
-				engine_user_message(NULL, NULL, "The plug-in %s in module %s failed to load.  The plug-in's setup_evms_plugin() function failed with error code %d: %s.\n", plug_rec->short_name, plug_rec->so_record->name, rc, strerror(rc));
+				engine_user_message(NULL, NULL,
+						    _("The plug-in %s in module %s failed to load.  The plug-in's setup_evms_plugin() function failed with error code %d: %s.\n"),
+						    plug_rec->short_name, plug_rec->so_record->name, rc, strerror(rc));
 			}
 
 			release_plugin(plug_rec);
@@ -1173,7 +1224,7 @@ static int setup_plugins() {
 	}
 
 	LIST_FOR_EACH(plugin_set, iter, plug_rec) {
-		status_message("Setup plug-in: %s\n", plug_rec->long_name);
+		status_message(_("Setup plug-in: %s\n"), plug_rec->long_name);
 		rc = plug_rec->functions.plugin->setup_evms_plugin(&engine_functions);
 		if (rc == 0) {
 			/* Handle special plug-ins. */
@@ -1188,7 +1239,9 @@ static int setup_plugins() {
 
 		} else {
 			if (rc != E_NOLOAD) {
-				engine_user_message(NULL, NULL, "The plug-in %s in module %s failed to load.  The plug-in's setup_evms_plugin() function failed with error code %d: %s.\n", plug_rec->short_name, plug_rec->so_record->name, rc, strerror(rc));
+				engine_user_message(NULL, NULL,
+						    _("The plug-in %s in module %s failed to load.  The plug-in's setup_evms_plugin() function failed with error code %d: %s.\n"),
+						    plug_rec->short_name, plug_rec->so_record->name, rc, strerror(rc));
 			}
 
 			release_plugin(plug_rec);
@@ -1206,11 +1259,13 @@ static int setup_plugins() {
 	}
 
 	LIST_FOR_EACH(plugin_set, iter, plug_rec) {
-		status_message("Setup plug-in: %s\n", plug_rec->long_name);
+		status_message(_("Setup plug-in: %s\n"), plug_rec->long_name);
 		rc = plug_rec->functions.plugin->setup_evms_plugin(&engine_functions);
 		if (rc != 0) {
 			if (rc != E_NOLOAD) {
-				engine_user_message(NULL, NULL, "The plug-in %s in module %s failed to load.  The plug-in's setup_evms_plugin() function failed with error code %d: %s.\n", plug_rec->short_name, plug_rec->so_record->name, rc, strerror(rc));
+				engine_user_message(NULL, NULL,
+						    _("The plug-in %s in module %s failed to load.  The plug-in's setup_evms_plugin() function failed with error code %d: %s.\n"),
+						    plug_rec->short_name, plug_rec->so_record->name, rc, strerror(rc));
 			}
 
 			release_plugin(plug_rec);
@@ -1228,11 +1283,13 @@ static int setup_plugins() {
 	}
 
 	LIST_FOR_EACH(plugin_set, iter, plug_rec) {
-		status_message("Setup plug-in: %s\n", plug_rec->long_name);
+		status_message(_("Setup plug-in: %s\n"), plug_rec->long_name);
 		rc = plug_rec->functions.plugin->setup_evms_plugin(&engine_functions);
 		if (rc != 0) {
 			if (rc != E_NOLOAD) {
-				engine_user_message(NULL, NULL, "The plug-in %s in module %s failed to load.  The plug-in's setup_evms_plugin() function failed with error code %d: %s.\n", plug_rec->short_name, plug_rec->so_record->name, rc, strerror(rc));
+				engine_user_message(NULL, NULL,
+						    _("The plug-in %s in module %s failed to load.  The plug-in's setup_evms_plugin() function failed with error code %d: %s.\n"),
+						    plug_rec->short_name, plug_rec->so_record->name, rc, strerror(rc));
 			}
 
 			release_plugin(plug_rec);
@@ -1250,11 +1307,13 @@ static int setup_plugins() {
 	}
 
 	LIST_FOR_EACH(plugin_set, iter, plug_rec) {
-		status_message("Setup plug-in: %s\n", plug_rec->long_name);
+		status_message(_("Setup plug-in: %s\n"), plug_rec->long_name);
 		rc = plug_rec->functions.plugin->setup_evms_plugin(&engine_functions);
 		if (rc != 0) {
 			if (rc != E_NOLOAD) {
-				engine_user_message(NULL, NULL, "The plug-in %s in module %s failed to load.  The plug-in's setup_evms_plugin() function failed with error code %d: %s.\n", plug_rec->short_name, plug_rec->so_record->name, rc, strerror(rc));
+				engine_user_message(NULL, NULL,
+						    _("The plug-in %s in module %s failed to load.  The plug-in's setup_evms_plugin() function failed with error code %d: %s.\n"),
+						    plug_rec->short_name, plug_rec->so_record->name, rc, strerror(rc));
 			}
 
 			release_plugin(plug_rec);
@@ -1272,11 +1331,13 @@ static int setup_plugins() {
 	}
 
 	LIST_FOR_EACH(plugin_set, iter, plug_rec) {
-		status_message("Setup plug-in: %s\n", plug_rec->long_name);
+		status_message(_("Setup plug-in: %s\n"), plug_rec->long_name);
 		rc = plug_rec->functions.fsim->setup_evms_plugin(&engine_functions);
 		if (rc != 0) {
 			if (rc != E_NOLOAD) {
-				engine_user_message(NULL, NULL, "The plug-in %s in module %s failed to load.  The plug-in's setup_evms_plugin() function failed with error code %d: %s.\n", plug_rec->short_name, plug_rec->so_record->name, rc, strerror(rc));
+				engine_user_message(NULL, NULL,
+						    _("The plug-in %s in module %s failed to load.  The plug-in's setup_evms_plugin() function failed with error code %d: %s.\n"),
+						    plug_rec->short_name, plug_rec->so_record->name, rc, strerror(rc));
 			}
 
 			release_plugin(plug_rec);
@@ -1351,7 +1412,9 @@ int load_plugins() {
 								 * for a plug-in record table.  It must not be
 								 * an Engine plug-in.  Unload the module.
 								 */
-								engine_user_message(NULL, NULL, "Failed to load module %s.  It does not export an \"evms_plugin_records\" variable.\n", so_record->name);
+								engine_user_message(NULL, NULL,
+										    _("Failed to load module %s.  It does not export an \"evms_plugin_records\" variable.\n"),
+										    so_record->name);
 								unload_module(so_record);
 							}
 
