@@ -260,6 +260,55 @@ out:
 	return device_list;
 }
 
+/**
+ * build_module_list
+ *
+ * @dmi: Pointer to ioctl packet.
+ *
+ * Convert an ioctl packet to a list of DM target-modules.
+ **/
+static dm_module_list_t *build_module_list(dm_ioctl_t *dmi)
+{
+	dm_target_versions_t *version, *old_version;
+	dm_module_list_t *module, *module_list = NULL;
+	char *data_start = (char*)dmi + dmi->data_start;
+
+	LOG_PROC_ENTRY();
+
+	version = (dm_target_versions_t*)data_start;
+	
+	if (version->name[0] == '\0') {
+		goto out;
+	}
+
+	do {
+		module = engine_alloc(sizeof(*module));
+		if (!module) {
+			dm_deallocate_module_list(module_list);
+			module_list = NULL;
+			goto out;
+		}
+
+		module->version.major = version->version[0];
+		module->version.minor = version->version[1];
+		module->version.patchlevel = version->version[2];
+		strncpy(module->name, version->name, EVMS_NAME_SIZE);
+		module->next = module_list;
+		module_list = module;
+
+		old_version = version;
+		version = (dm_target_versions_t*)((char*)version + version->next);
+
+		LOG_DEBUG("Found module %s (%u.%u.%u)\n", module->name,
+			  module->version.major, module->version.minor,
+			  module->version.patchlevel);
+	} while (old_version->next);
+
+out:
+	LOG_PROC_EXIT_PTR(module_list);
+	return module_list;
+}
+
 /**
  * get_ioctl_packet
  *
@@ -1015,3 +1064,47 @@ out:
 	return rc;
 }
 
+/**
+ * dm_get_modules_v4
+ *
+ * @module_list: Return pointer to list of currently loaded DM modules.
+ *
+ * Ask Device-Mapper for a list of all currently loaded target modules. The
+ *  name and version number of each module will be returned.
+ **/
+int dm_get_modules_v4(dm_module_list_t **module_list)
+{
+	dm_ioctl_t *dmi;
+	dm_module_list_t *list = NULL;
+	int rc;
+
+	LOG_PROC_ENTRY();
+
+	/* Build the ioctl packet. */
+	dmi = build_ioctl_packet(NULL, NULL, NULL);
+	if (!dmi) {
+		rc = ENOMEM;
+		goto out;
+	}
+
+	/* The LIST_VERSIONS command is an addition to DM, so the ioctl
+	 * version for this command must be 4.1.x instead of 4.0.x.
+	 */
+	dmi->version[1] = 1;
+
+	rc = run_command_v4(dmi, DM_LIST_VERSIONS);
+	if (rc) {
+		goto out;
+	}
+
+	list = build_module_list(dmi);
+
+out:
+	if (module_list) {
+		*module_list = list;
+	}
+	put_ioctl_packet(dmi);
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
