@@ -28,6 +28,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <dirent.h>
+#include <glob.h>
 #include <pthread.h>
 #include <signal.h>
 #include <time.h>
@@ -171,6 +172,9 @@ boolean is_2_4_kernel;
 
 pthread_attr_t pthread_attr_detached;
 
+char metadata_db_name[PATH_MAX];
+int metadata_db_fd = 0;
+
 
 /*
  * The following functions are used to prevent potential deadlock in a child
@@ -1019,9 +1023,9 @@ start:
 				 * Log a generic message.
 				 */
 				if (mode & ENGINE_DAEMON) {
-					sprintf(lock_msg, "The EVMS Daemon is already running.\n");
+					sprintf(lock_msg, _("The EVMS Daemon is already running.\n"));
 				} else {
-					sprintf(lock_msg, "The EVMS Engine is currently in use by another process.\n");
+					sprintf(lock_msg, _("The EVMS Engine is currently in use by another process.\n"));
 				}
 			}
 
@@ -1038,9 +1042,9 @@ start:
 		}
 
 	} else {
-		sprintf(lock_msg, "Unable to open the Engine lock file %s: %s.", ENGINE_LOCK_FILE, strerror(rc));
+		sprintf(lock_msg, _("Unable to open the Engine lock file %s: %s."), ENGINE_LOCK_FILE, strerror(rc));
 		if (rc == EROFS) {
-			strcat(lock_msg, "  The Engine is not protected against other instances of the Engine being opened at the same time.\n");
+			strcat(lock_msg, _("  The Engine is not protected against other instances of the Engine being opened at the same time.\n"));
 			rc = 0;
 		} else {
 			strcat(lock_msg, "\n");
@@ -1357,7 +1361,7 @@ static void shutdown_engine() {
 		pthread_create(&msg_tid,
 			       &pthread_attr_detached,
 			       thread_engine_user_message,
-			       "Self destruct sequence initiated.\n");
+			       _("Self destruct sequence initiated.\n"));
 
 
 	}
@@ -1605,14 +1609,26 @@ static void * shutdown_thread(void * arg) {
 	sem_t * shutdown_sem = (sem_t *) arg;
 	int rc;
 
+	LOG_PROC_ENTRY();
+
+	LOG_DEBUG("Register SIGUSR1 handler.\n");
 	signal(SIGUSR1, sigusr1_handler);
 
 	sem_init(shutdown_sem, 0, 0);
 
 	do {
-                rc = sem_wait(shutdown_sem);
+		LOG_DEBUG("Wait for SIGUSR1 shutdown signal.\n");
+		rc = sem_wait(shutdown_sem);
+		LOG_DEBUG("sem_wait returned %d.\n", rc);
+
+		if (rc != 0) {
+			LOG_DEBUG("errno is %d: %s\n", errno, strerror(errno));
+		}
+
 	} while ((rc != 0) && (errno == EINTR));
 					
+	LOG_DEBUG("Shutdown signal received.\n");
+
 	if (rc == 0) {
 		shutdown_engine();
 
@@ -1621,6 +1637,8 @@ static void * shutdown_thread(void * arg) {
 		LOG_CRITICAL("The shutdown thread is disabled.\n");
 	}
 
+	LOG_PROC_EXIT_VOID();
+
 	return NULL;
 }
 
@@ -1646,7 +1664,7 @@ int evms_open_engine(char           * node_name,
 	if (geteuid() != 0) {
 		if (callbacks != NULL) {
 			if (callbacks->user_message != NULL) {
-				callbacks->user_message("You must have root privilege to open the EVMS Engine.", NULL, NULL);
+				callbacks->user_message(_("You must have root privilege to open the EVMS Engine."), NULL, NULL);
 			}
 		}
 
@@ -1728,7 +1746,29 @@ int evms_open_engine(char           * node_name,
 	LOG_DEFAULT("Cluster API version:     %2d.%d.%d\n", VERSION_FIELDS(engine_cluster_api_version));
 	LOG_DEFAULT("Daemon protocol version: %2d.%d.%d\n", VERSION_FIELDS(engine_daemon_msg_version));
 
-	LOG_DEBUG("Requested open mode is %#x.\n", mode);
+	sprintf(message_buffer, "Requested open mode is %#x.", mode);
+	if (mode == -1) {
+		strcat(message_buffer, " (default)\n");
+
+	} else {
+		if (mode & ENGINE_READ) {
+			strcat(message_buffer, " ENGINE_READ");
+		}
+		if (mode & ENGINE_WRITE) {
+			strcat(message_buffer, " ENGINE_WRITE");
+		}
+		if (mode & ENGINE_CRITICAL) {
+			strcat(message_buffer, " ENGINE_CRITICAL");
+		}
+		if (mode & ENGINE_DAEMON) {
+			strcat(message_buffer, " ENGINE_DAEMON");
+		}
+		if (mode & ENGINE_WORKER) {
+			strcat(message_buffer, " ENGINE_WORKER");
+		}
+		strcat(message_buffer, "\n");
+	}
+	LOG_DEFAULT("%s", message_buffer);
 
 	if (!(open_mode & (ENGINE_READ | ENGINE_DAEMON))) {
 		LOG_ERROR("Open mode of %d is not valid.\n", open_mode);
@@ -1757,16 +1797,34 @@ int evms_open_engine(char           * node_name,
 	/* Check if we can access the EVMS kernel code. */
 	rc = check_for_evms_kernel();
 	if (rc != 0) {
-		engine_user_message(NULL, NULL, "This system is running the EVMS kernel.  "
-				    "EVMS Engine versions 1.9.0 and above do not run with the EVMS kernel.  "
-				    "Use EVMS Engine version 1.2 to configure with the EVMS kernel.\n");
+		engine_user_message(NULL, NULL,
+				    _("This system is running the EVMS kernel.  "
+				      "EVMS Engine versions 1.9.0 and above do not run with the EVMS kernel.  "
+				      "Use EVMS Engine version 1.2 to configure with the EVMS kernel.\n"));
 		evms_free_config();
 		LOG_PROC_EXIT_INT(rc);
 		return rc;
 	}
 
 	engine_mode = open_mode;
-	LOG_DEBUG("Open mode is %#x.\n", engine_mode);
+	sprintf(message_buffer, "Open mode is %#x.", engine_mode);
+	if (engine_mode & ENGINE_READ) {
+		strcat(message_buffer, " ENGINE_READ");
+	}
+	if (engine_mode & ENGINE_WRITE) {
+		strcat(message_buffer, " ENGINE_WRITE");
+	}
+	if (engine_mode & ENGINE_CRITICAL) {
+		strcat(message_buffer, " ENGINE_CRITICAL");
+	}
+	if (engine_mode & ENGINE_DAEMON) {
+		strcat(message_buffer, " ENGINE_DAEMON");
+	}
+	if (engine_mode & ENGINE_WORKER) {
+		strcat(message_buffer, " ENGINE_WORKER");
+	}
+	strcat(message_buffer, "\n");
+	LOG_DEFAULT("%s", message_buffer);
 
 	check_for_2_4_kernel();
 
@@ -1778,8 +1836,8 @@ int evms_open_engine(char           * node_name,
 
 	if (open_dm_control_node() != 0) {
 		engine_user_message(NULL, NULL,
-				    "Unable to open the control node for Device-Mapper.  "
-				    "The Engine will run without Device-Mapper support.\n");
+				    _("Unable to open the control node for Device-Mapper.  "
+				      "The Engine will run without Device-Mapper support.\n"));
 	}
 
 	/* Setup the pthread attribute for starting threads detached. */
@@ -1798,7 +1856,7 @@ int evms_open_engine(char           * node_name,
 				/* Daemon can't run without a cluster manager. */
 				if (cluster_manager == NULL) {
 					engine_user_message(NULL, NULL,
-							    "There is no cluster manager plug-in loaded on this system.\n");
+							    _("There is no cluster manager plug-in loaded on this system.\n"));
 					rc = ENODEV;
 				}
 
@@ -1823,9 +1881,9 @@ int evms_open_engine(char           * node_name,
 							local_focus = TRUE;
 
 							engine_user_message(NULL, NULL,
-									    "There was an error when starting EVMS on the other nodes in the cluster.  "
-									    "The error code was %d: %s.  "
-									    "EVMS will only manage local devices on this system.\n",
+									    _("There was an error when starting EVMS on the other nodes in the cluster.  "
+									      "The error code was %d: %s.  "
+									      "EVMS will only manage local devices on this system.\n"),
 									    rc, evms_strerror(rc));
 							rc = 0;
 						}
@@ -1834,8 +1892,8 @@ int evms_open_engine(char           * node_name,
 				} else {
 					if (current_node_name != NULL) {
 						engine_user_message(NULL, NULL,
-								    "There is no cluster manager plug-in loaded on this system.  "
-								    "The node_name parameter \"%s\" is ignored.\n",
+								    _("There is no cluster manager plug-in loaded on this system.  "
+								      "The node_name parameter \"%s\" is ignored.\n"),
 								    current_node_name);
 					}
 				}
@@ -2067,7 +2125,7 @@ int evms_write_log_entry(debug_level_t level,
 			pthread_mutex_lock(&log_mutex);
 			timestamp(log_buf, LOG_BUF_SIZE, level);
 			strcat(log_buf, module_name);
-			strcat(log_buf, ":  ");
+			strcat(log_buf, ": ");
 			len = strlen(log_buf);
 
 			va_start(args, fmt);
@@ -2160,11 +2218,7 @@ int make_user_handle_array(list_anchor_t      list,
 
 	count = list_count(list);
 
-	if (count > 1) {
-		size = sizeof(handle_array_t) + ((count -1) * sizeof(object_handle_t));
-	} else {
-		size = sizeof(handle_array_t);
-	}
+	size = sizeof(handle_array_t) + (count * sizeof(object_handle_t));
 
 	*ha = alloc_app_struct(size, NULL);
 	if (*ha != NULL) {
@@ -2311,19 +2365,27 @@ int evms_get_plugin_by_name(char            * plugin_name,
  */
 int make_directory(char * dir_name, mode_t mode) {
 
+	int rc = 0;
+	int status;
 	char name_buf[EVMS_VOLUME_NAME_SIZE + 1];
 	char * tmp_ptr = name_buf;
 	struct stat statbuf;
-	int rc = 0;
 
 	LOG_PROC_ENTRY();
 
 	/* See if this directory exists */
-	rc = stat(dir_name, &statbuf);
-	if (rc != 0) {
+	status = stat(dir_name, &statbuf);
+	if (status != 0) {
 		if (errno == ENOENT) {
 			/* This directory doesn't exist. */
 			strcpy(name_buf, dir_name);
+
+			/* Strip of any trailing '/'. */
+			if (name_buf[strlen(name_buf) - 1] == '/') {
+				name_buf[strlen(name_buf) - 1] = '\0';
+			}
+
+			/* Find the parent directory of this one. */
 			tmp_ptr = strrchr(name_buf, '/');
 
 			if ((tmp_ptr != NULL) && (tmp_ptr != name_buf)) {
@@ -2335,17 +2397,20 @@ int make_directory(char * dir_name, mode_t mode) {
 				*tmp_ptr = 0;
 
 				rc = make_directory(name_buf, mode);
-
-			} else {
-				rc = 0;
 			}
 
 			if (rc == 0) {
-				rc = mkdir(dir_name, mode);
+				LOG_DEBUG("Make directory \"%s\".\n", dir_name);
+				status = mkdir(dir_name, mode);
 
-				if (rc != 0) {
-					rc = errno;
-					LOG_WARNING("mkdir(%s) failed with error code %d.\n", dir_name, rc);
+				if (status != 0) {
+					if (errno == EEXIST) {
+						rc = 0;
+
+					} else {
+						rc = errno;
+						LOG_WARNING("mkdir(%s) failed with error code %d.\n", dir_name, rc);
+					}
 				}
 			}
 
@@ -2357,8 +2422,10 @@ int make_directory(char * dir_name, mode_t mode) {
 	} else {
 		/* Make sure this is a directory */
 		if (!(statbuf.st_mode & S_IFDIR)) {
-			LOG_ERROR("%s is a non-directory file\n", dir_name);
+			LOG_ERROR("%s is not a directory.\n", dir_name);
 			rc = EINVAL;
+		} else {
+			LOG_DEBUG("Directory %s already exists.\n", dir_name);
 		}
 	}
 
@@ -2413,3 +2480,466 @@ const char * evms_strerror(int err_num) {
 	return "Unknown error code";
 }
 
+
+static int backup_fh(char             * parent_name,
+		     storage_object_t * child) {
+
+	int rc = 0;
+
+	LOG_PROC_ENTRY();
+
+	feature_header_cpu_to_disk(child->feature_header);
+	child->feature_header->crc = 0;
+	child->feature_header->crc = CPU_TO_DISK32(evms_calculate_crc(EVMS_INITIAL_CRC,
+								       child->feature_header,
+								       sizeof(evms_feature_header_t)));
+
+	rc = engine_save_metadata(parent_name,
+				  child->name,
+				  child->size - (EVMS_FEATURE_HEADER_SECTORS * 2),
+				  EVMS_FEATURE_HEADER_SECTORS,
+				  child->feature_header);
+	if (rc == 0) {
+		rc = engine_save_metadata(parent_name,
+					  child->name,
+					  child->size - EVMS_FEATURE_HEADER_SECTORS,
+					  EVMS_FEATURE_HEADER_SECTORS,
+					  child->feature_header);
+	}
+
+	if (rc != 0) {
+		LOG_SERIOUS("Error backing up feature header for %s.  Error code is %d: %s\n",
+			    parent_name, rc, evms_strerror(rc));
+	}
+
+	/* Convert the feature header back to CPU endian format. */
+	feature_header_disk_to_cpu(child->feature_header);
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+
+/*
+ * Backup the feature headers for an object.  The feature headers reside on the
+ * object's child objects.
+ */
+static int backup_feature_headers(storage_object_t * obj,
+				  char           * * ancestors,
+				  int                depth) {
+
+	int rc = 0;
+	list_element_t iter;
+	storage_object_t * child;
+	int i;
+
+	LOG_PROC_ENTRY();
+
+	LIST_FOR_EACH(obj->child_objects, iter, child) {
+		if (child->feature_header != NULL) {
+
+			/* This child may have feature headers to build it. */
+			ancestors[depth] = obj->name;
+			rc = backup_feature_headers(child, ancestors, depth + 1);
+
+			if (rc == 0) {
+				if (obj->volume != NULL) {
+					rc = backup_fh(obj->volume->name, child);
+				}
+			}
+
+			for (i = 0; (rc == 0) && (i < depth); i++) {
+				rc = backup_fh(ancestors[i], child);
+			}
+
+			if (rc == 0) {
+				rc = backup_fh(obj->name, child);
+			}
+
+			if (rc != 0) {
+				break;
+			}
+		}
+	}
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+
+int evms_metadata_backup(const char * directory) {
+
+	int rc = 0;
+	int i;
+	change_record_array_t * changes;
+	time_t time_now;
+	struct tm * tm_now;
+
+	list_anchor_t object_lists[] = {&disks_list,
+					&segments_list,
+					&regions_list,
+					&EVMS_objects_list,
+					NULL};
+	list_element_t iter;
+	storage_object_t * obj;
+	storage_container_t * con;
+	logical_volume_t * vol;
+	list_anchor_t top_objects;
+
+	LOG_PROC_ENTRY();
+
+	rc = check_engine_read_access();
+	if (rc != 0) {
+		LOG_PROC_EXIT_INT(rc);
+		return rc;
+	}
+
+	if (!local_focus) {
+		rc = remote_metadata_backup(directory);
+		LOG_PROC_EXIT_INT(rc);
+		return rc;
+	}
+
+	/*
+	 * Make sure nothing is dirty.  We can't do a backup of a system
+	 * that is currently being configured.  All changes must be on disk.
+	 * Needing activation/deactivation is OK since it doesn't involve
+	 * writing metadata.
+	 */
+	if (are_changes_pending(&changes)) {
+
+		int i;
+		boolean nothing_dirty = TRUE;
+
+		for (i = 0; nothing_dirty && (i < changes->count); i++) {
+			if (changes->changes_pending[i].changes & ~(CHANGE_ACTIVATE |
+								    CHANGE_REACTIVATE |
+								    CHANGE_DEACTIVATE)) {
+				nothing_dirty = FALSE;
+			}
+		}
+
+		engine_free(changes);
+
+		if (!nothing_dirty) {
+			engine_user_message(NULL,NULL,
+					    _("EVMS cannot backup metadata while changes are pending.\n"));
+
+			LOG_PROC_EXIT_INT(EPERM);
+			return EPERM;
+		}
+	}
+
+	/* Build the metadata database file name. */
+	if (directory == NULL) {
+		directory = DEFAULT_METADATA_DIR;
+		evms_get_config_string("engine.metadata_backup_dir", &directory);
+	}
+
+	strcpy(metadata_db_name, directory);
+	if (metadata_db_name[strlen(metadata_db_name) -1] != '/') {
+		strcat(metadata_db_name, "/");
+	}
+
+	rc = make_directory(metadata_db_name, (S_IFDIR | S_IRWXU |
+					       S_IRGRP | S_IXGRP |
+					       S_IROTH | S_IXOTH));
+	if (rc != 0) {
+		engine_user_message(NULL, NULL,
+				    "Error creating metadata backup directory %s.  Error code is %d: %s\n",
+				    metadata_db_name, rc, evms_strerror(rc));
+		
+		LOG_PROC_EXIT_INT(rc);
+		return rc;
+	}
+
+	strcat(metadata_db_name, EVMS_METADATA_FILE_NAME_PREFIX);
+
+	time_now = time(NULL);
+	tm_now = localtime(&time_now);
+
+	sprintf(metadata_db_name + strlen(metadata_db_name), "%04u-%02u-%02u-%02u.%02u.%02u",
+		tm_now->tm_year + 1900,
+		tm_now->tm_mon + 1,
+		tm_now->tm_mday,
+		tm_now->tm_hour,
+		tm_now->tm_min,
+		tm_now->tm_sec);
+
+	LOG_DEBUG("Metadata database file name is %s.\n", metadata_db_name);
+	metadata_db_fd = open(metadata_db_name, O_CREAT | O_TRUNC | O_WRONLY, 0664);
+	if (metadata_db_fd < 0) {
+
+		rc = errno;
+		
+		engine_user_message(NULL, NULL,
+				    "Error opening metadata backup file %s.  open() returned error %d: %s\n",
+				    metadata_db_name, rc, strerror(rc));
+		
+		LOG_PROC_EXIT_INT(rc);
+		return rc;
+	}
+
+	/* Call the plug-ins for all objects and have them save their metadata. */
+	for (i = 0; object_lists[i] != NULL; i++) {
+		LIST_FOR_EACH(object_lists[i], iter, obj) {
+			rc = obj->plugin->functions.plugin->backup_metadata(obj);
+			if (rc != 0) {
+				if (rc == ENOSYS) {
+					engine_user_message(NULL, NULL,
+							    "Warning: The %s plug-in does not support backing up metadata for object %s.  "
+							    "The metadata backup may be incomplete.\n",
+							    obj->plugin->short_name, obj->name);
+					rc = 0;
+
+				} else {
+					engine_user_message(NULL, NULL,
+							    _("The %s plug-in returned error %d (%s) when asked to backup the metadata for object %s.\n"),
+							    obj->plugin->short_name, rc, evms_strerror(rc), obj->name);
+					goto error_out;
+				}
+			}
+		}
+	}
+
+	/*
+	 * Call the plug-ins for all containers and have them save their
+	 * metadata.
+	 */
+	LIST_FOR_EACH(&containers_list, iter, con) {
+		rc = con->plugin->container_functions->backup_container_metadata(con);
+		if (rc != 0) {
+			if (rc == ENOSYS) {
+				LOG_WARNING("The %s plug-in does not support backing up metadata for container %s.\n",
+					    con->plugin->short_name, con->name);
+				rc = 0;
+
+			} else {
+				engine_user_message(NULL, NULL,
+						    _("The %s plug-in returned error %d (%s) when asked to backup the metadata for container %s.\n"),
+						    con->plugin->short_name, rc, evms_strerror(rc), con->name);
+				goto error_out;
+			}
+		}
+	}
+
+	/*
+	 * Save dummy entries for the parents of top objects.
+	 * They need an entry in the metadata database in order to save
+	 * the stop data that is put on the object.
+	 */
+	rc = engine_get_object_list(DISK | SEGMENT | REGION | EVMS_OBJECT,
+					DATA_TYPE,
+					NULL,
+					NULL,
+					TOPMOST,
+					&top_objects);
+
+	if (rc != 0) {
+		LOG_SERIOUS("Error getting a list of top objects.  Return code is %d: %s\n",
+			    rc, evms_strerror(rc));
+		goto error_out;
+	}
+
+	if (stop_data.crc == 0) {
+		feature_header_cpu_to_disk(&stop_data);
+		stop_data.crc = CPU_TO_DISK32(evms_calculate_crc(EVMS_INITIAL_CRC, &stop_data, sizeof(evms_feature_header_t)));
+	}
+
+	LIST_FOR_EACH(top_objects, iter, obj) {
+		if (obj->object_type == EVMS_OBJECT) {
+			char * ancestors[8];
+
+			rc = backup_feature_headers(obj, ancestors, 0);
+			if (rc != 0) {
+				goto error_out;
+			}
+
+		} else {
+			rc = engine_save_metadata(NULL,
+						  obj->name,
+						  obj->size - (EVMS_FEATURE_HEADER_SECTORS * 2),
+						  EVMS_FEATURE_HEADER_SECTORS,
+						  &stop_data);
+			if (rc != 0) {
+				goto error_out;
+			}
+
+			rc = engine_save_metadata(NULL,
+						  obj->name,
+						  obj->size - EVMS_FEATURE_HEADER_SECTORS,
+						  EVMS_FEATURE_HEADER_SECTORS,
+						  &stop_data);
+			if (rc != 0) {
+				goto error_out;
+			}
+		}
+	}
+
+	destroy_list(top_objects);
+	top_objects = NULL;
+
+	LIST_FOR_EACH(&volumes_list, iter, vol) {
+
+		if (vol->flags & VOLFLAG_HAS_OWN_DEVICE) {
+			/*
+			 * Entries for EVMS volumes that have their own
+			 * device contain the feature headers that go on
+			 * the object.
+			 */
+			rc = backup_fh(vol->name, get_working_top_object(vol->object));
+
+		} else {
+			rc = engine_save_metadata(vol->name,
+						  vol->object->name,
+						  0,
+						  0,
+						  NULL);
+
+			if ((rc == 0) && !(vol->flags & VOLFLAG_COMPATIBILITY)){
+				char * ancestors[8];
+
+				rc = backup_feature_headers(vol->object, ancestors, 0);
+			}
+		}
+
+		if (rc != 0) {
+			break;
+		}
+
+		/*
+		 * Save a dummy entry for a "parent" of the volume.
+		 * The dummy parent is used for terminating a parent walk up the
+		 * stack.  It is also used as a starting point for a recursive
+		 * "restore all children" walk down the stack.
+		 */
+		rc = engine_save_metadata(NULL,
+					  vol->name,
+					  0,
+					  0,
+					  NULL);
+		if (rc != 0){
+			break;
+		}
+	}
+
+error_out:
+	close(metadata_db_fd);
+	metadata_db_fd = 0;
+	if (rc != 0) {
+		unlink(metadata_db_name);
+	}
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+
+int evms_engine_read(object_handle_t handle,
+		     lsn_t           lsn,
+		     sector_count_t  length,
+		     void          * buffer) {
+
+	int rc = 0;
+	object_type_t type;
+	storage_object_t * obj;
+
+	LOG_PROC_ENTRY();
+
+	rc = check_engine_read_access();
+	if (rc != 0) {
+		LOG_PROC_EXIT_INT(rc);
+		return rc;
+	}
+
+	if (!local_focus) {
+		rc = remote_engine_read(handle, lsn, length, buffer);
+		LOG_PROC_EXIT_INT(rc);
+		return rc;
+	}
+
+	rc = translate_handle(handle, &obj, &type);
+	if (rc == 0) {
+		switch (type) {
+			case DISK:
+			case SEGMENT:
+			case REGION:
+			case EVMS_OBJECT:
+				rc = obj->plugin->functions.plugin->read(obj, lsn, length, buffer);
+				break;
+
+			case CONTAINER:
+				LOG_ERROR("Cannot read from container %s.\n", ((storage_container_t *) obj)->name);
+				rc = EINVAL;
+				break;
+
+			case VOLUME:
+				LOG_ERROR("Cannot read from container %s.\n", ((logical_volume_t *) obj)->name);
+				rc = EINVAL;
+				break;
+
+			default:
+				LOG_ERROR("I don't know how to read from a thing of type %d (%#x).\n", type, type);
+				rc = EINVAL;
+				break;
+		}
+	}
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+
+int evms_engine_write(object_handle_t handle,
+		      lsn_t           lsn,
+		      sector_count_t  length,
+                      void          * buffer) {
+
+	int rc = 0;
+	object_type_t type;
+	storage_object_t * obj;
+
+	LOG_PROC_ENTRY();
+
+	rc = check_engine_write_access();
+	if (rc != 0) {
+		LOG_PROC_EXIT_INT(rc);
+		return rc;
+	}
+
+	if (!local_focus) {
+		rc = remote_engine_write(handle, lsn, length, buffer);
+		LOG_PROC_EXIT_INT(rc);
+		return rc;
+	}
+
+	rc = translate_handle(handle, &obj, &type);
+	if (rc == 0) {
+		switch (type) {
+			case DISK:
+			case SEGMENT:
+			case REGION:
+			case EVMS_OBJECT:
+				rc = obj->plugin->functions.plugin->write(obj, lsn, length, buffer);
+				break;
+
+			case CONTAINER:
+				LOG_ERROR("Cannot write to container %s.\n", ((storage_container_t *) obj)->name);
+				rc = EINVAL;
+				break;
+
+			case VOLUME:
+				LOG_ERROR("Cannot write to container %s.\n", ((logical_volume_t *) obj)->name);
+				rc = EINVAL;
+				break;
+
+			default:
+				LOG_ERROR("I don't know how to write to a thing of type %d (%#x).\n", type, type);
+				rc = EINVAL;
+				break;
+		}
+	}
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
