@@ -25,6 +25,7 @@
 #include <string.h>
 #include <errno.h>
 #include <unistd.h>
+#include <csm_funcs.h>
 
 #include "fullengine.h"
 #include "expand.h"
@@ -42,7 +43,9 @@
 #include "volume.h"
 
 
-int get_object_expand_points(storage_object_t * obj, u_int64_t * max_delta_size, list_anchor_t expand_points) {
+int get_object_expand_points(storage_object_t * obj,
+			     u_int64_t          max_delta_size,
+			     list_anchor_t      expand_points) {
 
 	int rc = 0;
 
@@ -55,10 +58,11 @@ int get_object_expand_points(storage_object_t * obj, u_int64_t * max_delta_size,
 }
 
 
-int get_volume_expand_points(logical_volume_t * volume, list_anchor_t expand_points) {
+int get_volume_expand_points(logical_volume_t * volume,
+			     list_anchor_t      expand_points) {
 
 	int rc = 0;
-	sector_count_t max_delta_size = -1;
+	sector_count_t max_delta_size = UINT64_MAX - volume->vol_size;
 
 	LOG_PROC_ENTRY();
 
@@ -80,54 +84,11 @@ int get_volume_expand_points(logical_volume_t * volume, list_anchor_t expand_poi
 	if ((volume->original_fsim != NULL) &&
 	    !(volume->flags & VOLFLAG_UNMKFS)) {
 
-		/*
-		 * Reget the current size and limits because the file system could be
-		 * mounted and be active.
-		 */
-		rc = get_volume_sizes_and_limits(volume);
-
-		if (rc == 0) {
-			max_delta_size = volume->max_vol_size - volume->vol_size;
-		}
+		rc = volume->original_fsim->functions.fsim->can_expand_by(volume, &max_delta_size);
 	}
 
 	if (rc == 0) {
-		rc = get_object_expand_points(volume->object, &max_delta_size, expand_points);
-	}
-
-	if (rc == 0) {
-		/*
-		 * If the volume has an FSIM and the FSIM is not scheduled to be removed
-		 * and there are expand points, check if the FSIM can handle the expand.
-		 *
-		 */
-		if ((volume->original_fsim != NULL) &&
-		    !(volume->flags & VOLFLAG_UNMKFS) &&
-		    !list_empty(expand_points)) {
-
-			max_delta_size = round_down_to_hard_sector(max_delta_size, volume->object);
-
-			rc = volume->original_fsim->functions.fsim->can_expand_by(volume, &max_delta_size);
-
-			/*
-			 * The FSIM cannot handle the expand.  Delete all the expand
-			 * points from the list.
-			 */
-			if (rc != 0) {
-				/*
-				 * Can't use delete_all_elements(expand_points)
-				 * because we need to free the expand points.
-				 */
-				list_element_t iter1;
-				list_element_t iter2;
-				void * thing;
-
-				LIST_FOR_EACH_SAFE(expand_points, iter1, iter2, thing) {
-					delete_element(iter1);
-					engine_free(thing);
-				}
-			}
-		}
+		rc = get_object_expand_points(volume->object, max_delta_size, expand_points);
 	}
 
 	LOG_PROC_EXIT_INT(rc);
@@ -135,8 +96,22 @@ int get_volume_expand_points(logical_volume_t * volume, list_anchor_t expand_poi
 }
 
 
+int get_container_expand_points(storage_container_t * con,
+				list_anchor_t         expand_points) {
+
+	int rc = 0;
+
+	LOG_PROC_ENTRY();
+
+	rc = con->plugin->container_functions->can_expand_container(con, expand_points);
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+
 static int can_expand_fs_on_volume(logical_volume_t * vol,
-				   u_int64_t * max_expand_size) {
+				   u_int64_t        * max_expand_size) {
 
 	int rc = 0;
 	u_int64_t usable_object_size;
@@ -165,7 +140,8 @@ static int can_expand_fs_on_volume(logical_volume_t * vol,
 		usable_object_size = vol->object->size - EVMS_FEATURE_HEADER_SECTORS * 2;
 	}
 
-	if (vol->vol_size < usable_object_size) {
+	if (vol->vol_size < usable_object_size &&
+	    vol->file_system_manager) {
 		expand_size = usable_object_size - vol->fs_size;
 		rc = vol->file_system_manager->functions.fsim->can_expand_by(vol, &expand_size);
 
@@ -184,9 +160,9 @@ static int can_expand_fs_on_volume(logical_volume_t * vol,
 
 
 /*
- * Find out if an object can be expanded.  The "thing" must be either a volume
- * or a top level object.  What we'll do is call the internal
- * get_expand_points() and see if any expand candidates show up.  If we get
+ * Find out if an object can be expanded.  The "thing" must be either a volume,
+ * top level object, or container.  What we'll do is call the internal
+ * get_*_expand_points() and see if any expand candidates show up.  If we get
  * expand points then return 0 to say that the object can be expanded.  Else,
  * return an error.
  */
@@ -229,9 +205,8 @@ int evms_can_expand(object_handle_t thing) {
 					 */
 					if (is_top_object(obj)) {
 						STATIC_LIST_DECL(expand_points);
-						u_int64_t max_delta_size = -1;
 
-						rc = get_object_expand_points(obj, &max_delta_size, &expand_points);
+						rc = get_object_expand_points(obj, UINT64_MAX - obj->size, &expand_points);
 
 						if (rc == 0) {
 							if (list_empty(&expand_points)) {
@@ -255,7 +230,7 @@ int evms_can_expand(object_handle_t thing) {
 							rc = EBUSY;
 						}
 					}
-					
+
 					if (rc == 0) {
 						if (volume->flags & (VOLFLAG_ACTIVE | VOLFLAG_NEW)) {
 							STATIC_LIST_DECL(expand_points);
@@ -276,6 +251,22 @@ int evms_can_expand(object_handle_t thing) {
 				}
 				break;
 
+			case CONTAINER:
+				{
+					storage_container_t * con = object;
+
+					STATIC_LIST_DECL(expand_points);
+
+					rc = get_container_expand_points(con, &expand_points);
+
+					if (rc == 0) {
+						if (list_empty(&expand_points)) {
+							rc = ENOENT;
+						}
+					}
+				}
+				break;
+
 			default:
 				LOG_ERROR("Cannot expand object of type %#x.\n", type);
 				rc = EINVAL;
@@ -292,7 +283,8 @@ int evms_can_expand(object_handle_t thing) {
  * Make an array of expand handles (expand_handle_array_t) for the objects in
  * a list_anchor_t.
  */
-static int make_expand_handle_array(list_anchor_t list, expand_handle_array_t * * peha) {
+static int make_expand_handle_array(list_anchor_t             list,
+				    expand_handle_array_t * * peha) {
 
 	int rc = 0;
 	uint count;
@@ -310,18 +302,16 @@ static int make_expand_handle_array(list_anchor_t list, expand_handle_array_t *
 
 	object_info = first_thing(list, NULL);
 	if (object_info != NULL) {
-		expand_vol = object_info->object->volume;
+		if (object_info->object != NULL) {
+			expand_vol = object_info->object->volume;
+		}
 	}
 
 	if (can_expand_fs_on_volume(expand_vol, &max_vol_expand_size) == 0) {
 		count++;
 	}
 
-	if (count > 1) {
-		size = sizeof(expand_handle_array_t) + ((count -1) * sizeof(expand_handle_t));
-	} else {
-		size = sizeof(expand_handle_array_t);
-	}
+	size = sizeof(expand_handle_array_t) + (count * sizeof(expand_handle_t));
 
 	eha = alloc_app_struct(size, NULL);
 	if (eha != NULL) {
@@ -340,11 +330,20 @@ static int make_expand_handle_array(list_anchor_t list, expand_handle_array_t *
 
 		LIST_FOR_EACH(list, iter, object_info) {
 
-			LOG_DEBUG("Add entry for storage object %s.\n", object_info->object->name);
+			if (object_info->object != NULL) {
+				LOG_DEBUG("Add entry for storage object %s.\n", object_info->object->name);
+				rc = ensure_app_handle(object_info->object);
+			} else if (object_info->container != NULL) {
+				LOG_DEBUG("Add entry for container %s.\n", object_info->container->name);
+				rc = ensure_app_handle(object_info->container);
+			}
 
-			rc = ensure_app_handle(object_info->object);
 			if (rc == 0) {
-				eha->expand_point[eha->count].object = object_info->object->app_handle;
+				if (object_info->object != NULL) {
+					eha->expand_point[eha->count].object = object_info->object->app_handle;
+				} else if (object_info->container != NULL) {
+					eha->expand_point[eha->count].object = object_info->container->app_handle;
+				}
 				eha->expand_point[eha->count].max_expand_size = object_info->max_expand_size;
 				eha->count++;
 			}
@@ -368,10 +367,11 @@ static int make_expand_handle_array(list_anchor_t list, expand_handle_array_t *
  * list.  The list is then converted into an expand_handle_array_t for return
  * to the user.
  */
-int evms_get_expand_points(object_handle_t thing, expand_handle_array_t * * expand_points) {
+int evms_get_expand_points(object_handle_t           thing_handle,
+			   expand_handle_array_t * * expand_points) {
 
 	int rc = 0;
-	void * object;
+	void * thing;
 	object_type_t type;
 	STATIC_LIST_DECL(expand_point_list);
 
@@ -384,13 +384,13 @@ int evms_get_expand_points(object_handle_t thing, expand_handle_array_t * * expa
 	}
 
 	if (!local_focus) {
-		rc = remote_get_expand_points(thing, expand_points);
+		rc = remote_get_expand_points(thing_handle, expand_points);
 		LOG_PROC_EXIT_INT(rc);
 		return rc;
 	}
 
-	rc = translate_handle(thing,
-			      &object,
+	rc = translate_handle(thing_handle,
+			      &thing,
 			      &type);
 
 	if (rc != HANDLE_MANAGER_NO_ERROR) {
@@ -407,14 +407,14 @@ int evms_get_expand_points(object_handle_t thing, expand_handle_array_t * * expa
 	switch (type) {
 		case VOLUME:
 			{
-				logical_volume_t * volume = (logical_volume_t *) object;
+				logical_volume_t * volume = (logical_volume_t *) thing;
 
 				if (volume->flags & VOLFLAG_COMPATIBILITY) {
 					if (is_kernel_volume_mounted(volume, ERROR)) {
 						rc = EBUSY;
-                                        }
+					}
 				}
-				
+
 				if (rc == 0) {
 					if (volume->flags & (VOLFLAG_ACTIVE | VOLFLAG_NEW)) {
 						rc = get_volume_expand_points(volume, &expand_point_list);
@@ -432,14 +432,12 @@ int evms_get_expand_points(object_handle_t thing, expand_handle_array_t * * expa
 		case SEGMENT:
 		case DISK:
 			{
-				storage_object_t * obj = (storage_object_t *) object;
+				storage_object_t * obj = (storage_object_t *) thing;
 				/*
 				 * Only top level objects can be expanded.
 				 */
 				if (is_top_object(obj)) {
-					u_int64_t max_delta_size = -1;
-
-					rc = get_object_expand_points(obj, &max_delta_size, &expand_point_list);
+					rc = get_object_expand_points(obj, UINT64_MAX - obj->size, &expand_point_list);
 
 				} else {
 					LOG_ERROR("Object %s is not a top level object.  Only volumes and top level objects can be expanded.\n", obj->name);
@@ -448,8 +446,16 @@ int evms_get_expand_points(object_handle_t thing, expand_handle_array_t * * expa
 			}
 			break;
 
+		case CONTAINER:
+			{
+				storage_container_t * con = (storage_container_t *) thing;
+				
+				rc = get_container_expand_points(con, &expand_point_list);
+			}
+			break;
+
 		default:
-			LOG_ERROR("An object of type %d cannot be expanded.\n", type);
+			LOG_ERROR("A thing of type %d cannot be expanded.\n", type);
 			rc = EINVAL;
 			break;
 	}
@@ -510,10 +516,10 @@ static int warn_if_no_fsim(storage_object_t * obj) {
 			int answer = 0;	    /* Initialize to "Continue" */
 
 			engine_user_message(&answer, choices,
-					    "WARNING: Volume \"%s\" does not have an associated File System Interface Module which could coordinate the expansion of the file system on the volume.  "
-					    "The file system will not be expanded.  "
-					    "You may need to run a separate utility to expand the file system after this operation completes.  "
-					    "Expanding the volume may make the file system unusable.\n",
+					    _("WARNING: Volume \"%s\" does not have an associated File System Interface Module which could coordinate the expansion of the file system on the volume.  "
+					      "The file system will not be expanded.  "
+					      "You may need to run a separate utility to expand the file system after this operation completes.  "
+					      "Expanding the volume may make the file system unusable.\n"),
 					    obj->volume->name);
 
 			if (answer != 0) {
@@ -568,7 +574,7 @@ static int isa_valid_expand_input_object(storage_object_t    * obj,
 
 static void post_expand_cleanup(storage_object_t * top_object,
 				storage_object_t * obj) {
-	
+
 	list_element_t iter;
 	storage_object_t * child_obj;
 
@@ -587,27 +593,26 @@ static void post_expand_cleanup(storage_object_t * top_object,
 		child_obj->flags &= ~SOFLAG_HAS_STOP_DATA;
 	}
 
-	if (obj->object_type == EVMS_OBJECT) {
-		/*
-		 * Mark all the child feature headers dirty to ensure that they
-		 * get the correct depth and get written in the right place.
-		 */
-		LIST_FOR_EACH(obj->child_objects, iter, child_obj) {
-			mark_feature_headers_dirty(child_obj);
-		}
-	}
-
 	/* If the object is part of a volume... */
 	if (obj->volume != NULL) {
 
-		/*
-		 * Mark it dirty so that the feature headers get written to
-		 * their new locations.
-		 */
-		obj->volume->flags |= VOLFLAG_DIRTY;
-
+		/* An EVMS volume needs some adjusting after expanding. */
 		if (!(obj->volume->flags & VOLFLAG_COMPATIBILITY)) {
-			obj->volume->flags |= VOLFLAG_NEEDS_ACTIVATE;
+
+			/*
+			 * Mark the feature headers dirty so that they get
+			 * written to their new locations.
+			 */
+			obj->volume->flags |= VOLFLAG_FEATURE_HEADER_DIRTY;
+
+			if (obj->volume->flags & VOLFLAG_HAS_OWN_DEVICE) {
+
+				/*
+				 * The volume device needs to be reactivated
+				 * with the new size.
+				 */
+				obj->volume->flags |= VOLFLAG_NEEDS_ACTIVATE;
+			}
 		}
 
 		/* Update the volume size. */
@@ -626,7 +631,9 @@ static void post_expand_cleanup(storage_object_t * top_object,
 }
 
 
-static int do_object_expand(storage_object_t * obj, handle_array_t * input_objects, option_array_t * options) {
+static int do_object_expand(storage_object_t * obj,
+			    handle_array_t   * input_objects,
+			    option_array_t   * options) {
 
 	int rc = 0;
 	STATIC_LIST_DECL(input_object_list);
@@ -674,55 +681,134 @@ static int do_object_expand(storage_object_t * obj, handle_array_t * input_objec
 
 			parent_count = list_count(top_object->parent_objects);
 
-			while (parent_count == 1) {
+			while ((parent_count == 1) && (top_object->consuming_container == NULL)) {
 				top_object = first_thing(top_object->parent_objects, NULL);
 				parent_count = list_count(top_object->parent_objects);
 			}
 
-			if (parent_count <= 1) {
+			if (top_object->consuming_container != NULL) {
+				storage_container_t * con = top_object->consuming_container;
 
-				/*
-				 * Warn the user if there is no FSIM to handle
-				 * the expand.
-				 */
-				rc = warn_if_no_fsim(obj);
-
-				if (rc == 0) {
-					/*
-					 * If the object is not part of a
-					 * volume and it has stop data,
-					 * remove the stop data from its
-					 * current location.  It will be
-					 * reapplied during commit.
-					 */
-					if ((top_object->volume == NULL) &&
-					    (top_object->flags & SOFLAG_HAS_STOP_DATA)) {
-						top_object->plugin->functions.plugin->add_sectors_to_kill_list(top_object, top_object->size - (EVMS_FEATURE_HEADER_SECTORS * 2), EVMS_FEATURE_HEADER_SECTORS * 2);
-						top_object->flags &= ~SOFLAG_HAS_STOP_DATA;
-					}
-
-					/*
-					 * Start the expand at the top level
-					 * object.
-					 */
-					rc = top_object->plugin->functions.plugin->expand(top_object, obj, &input_object_list, options);
-
-					if (rc == 0) {
-						post_expand_cleanup(top_object, obj);
-					}
-				}
+				rc = con->plugin->container_functions->expand_container(con,
+											top_object,
+											obj,
+											&input_object_list,
+											options);
 
 			} else {
-				if (parent_count > 1) {
-					LOG_ERROR("Object %s cannot be expanded because it has multiple parents.\n", obj->name);
-					rc = EINVAL;
+				if (parent_count <= 1) {
+
+					/*
+					 * Warn the user if there is no FSIM to handle
+					 * the expand.
+					 */
+					rc = warn_if_no_fsim(obj);
+
+					if (rc == 0) {
+						/*
+						 * If the object is not part of a
+						 * volume and it has stop data,
+						 * remove the stop data from its
+						 * current location.  It will be
+						 * reapplied during commit.
+						 */
+						if ((top_object->volume == NULL) &&
+						    (top_object->flags & SOFLAG_HAS_STOP_DATA)) {
+							top_object->plugin->functions.plugin->add_sectors_to_kill_list(top_object,
+														       top_object->size - (EVMS_FEATURE_HEADER_SECTORS * 2),
+														       EVMS_FEATURE_HEADER_SECTORS * 2);
+							top_object->flags &= ~SOFLAG_HAS_STOP_DATA;
+						}
+
+						/*
+						 * Start the expand at the top level
+						 * object.
+						 */
+						rc = top_object->plugin->functions.plugin->expand(top_object,
+												  obj,
+												  &input_object_list,
+												  options);
+
+						if (rc == 0) {
+							post_expand_cleanup(top_object, obj);
+						}
+					}
+
 				} else {
-					LOG_WARNING("Error code %d encountered when trying to find the top level object.\n", rc);
+					if (parent_count > 1) {
+						LOG_ERROR("Object %s cannot be expanded because it has multiple parents.\n", obj->name);
+						rc = EINVAL;
+					} else {
+						LOG_WARNING("Error code %d encountered when trying to find the top level object.\n", rc);
+					}
 				}
 			}
 
+
 		} else {
-			LOG_ERROR("One or more items in the input object list is not a storage object.\n");
+			LOG_ERROR("One or more items in the input object list is not a valid input object.\n");
+			rc = EINVAL;
+		}
+
+	} else {
+		LOG_ERROR("Error code %d when making a list from the input_objects handle array.\n", rc);
+		rc = EINVAL;
+	}
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+
+static int do_container_expand(storage_container_t * con,
+			       handle_array_t      * input_objects,
+			       option_array_t      * options) {
+
+	int rc = 0;
+	STATIC_LIST_DECL(input_object_list);
+
+	LOG_PROC_ENTRY();
+
+	rc = make_list(input_objects, &input_object_list);
+
+	if (rc == 0) {
+		list_element_t iter;
+		storage_object_t * input_obj;
+		storage_container_t * disk_group;
+
+		/*
+		 * The CSM expands a container by taking in objects (disks)
+		 * that are not in a disk group.  Other container plug-ins
+		 * must obey the rule of having everything in the same
+		 * disk group.
+		 */
+		if (con->plugin->id == CSM_PLUGIN_ID) {
+			disk_group = NULL;
+		} else {
+			disk_group = con->disk_group;
+		}
+
+		/*
+		 * Make sure all of the input objects are storage
+		 * objects.  (The caller could have supplied handles
+		 * for bogus items such as plug-ins or volumes.)
+		 */
+		LIST_FOR_EACH(&input_object_list, iter, input_obj) {
+			rc = isa_valid_expand_input_object(input_obj, disk_group);
+			if (rc != 0) {
+				break;
+			}
+		}
+
+		if (rc == 0) {
+       			rc = con->plugin->container_functions->expand_container(con,
+       										NULL,
+       										NULL,
+       										&input_object_list,
+       										options);
+
+		} else {
+			LOG_ERROR("One or more items in the input object list is not a valid input object.\n");
 			rc = EINVAL;
 		}
 
@@ -741,10 +827,12 @@ static int do_object_expand(storage_object_t * obj, handle_array_t * input_objec
  * the feature stack.  The plug-ins pass it on down until it hits the object
  * that is to do the real expansion.
  */
-int evms_expand(object_handle_t thing, handle_array_t * input_objects, option_array_t * options) {
+int evms_expand(object_handle_t  thing_handle,
+		handle_array_t * input_objects,
+		option_array_t * options) {
 
 	int rc = 0;
-	void * object;
+	void * thing;
 	object_type_t type;
 
 	LOG_PROC_ENTRY();
@@ -756,8 +844,14 @@ int evms_expand(object_handle_t thing, handle_array_t * input_objects, option_ar
 		return rc;
 	}
 
-	rc = translate_handle(thing,
-			      &object,
+	if (!local_focus) {
+		rc = remote_expand(thing_handle, input_objects, options);
+		LOG_PROC_EXIT_INT(rc);
+		return rc;
+	}
+
+	rc = translate_handle(thing_handle,
+			      &thing,
 			      &type);
 
 	if (rc != HANDLE_MANAGER_NO_ERROR) {
@@ -767,18 +861,22 @@ int evms_expand(object_handle_t thing, handle_array_t * input_objects, option_ar
 
 	switch (type) {
 		case VOLUME:
-			rc = do_volume_expand((logical_volume_t *) object);
+			rc = do_volume_expand((logical_volume_t *) thing);
 			break;
 
 		case EVMS_OBJECT:
 		case REGION:
 		case SEGMENT:
 		case DISK:
-			rc = do_object_expand((storage_object_t *) object, input_objects, options);
+			rc = do_object_expand((storage_object_t *) thing, input_objects, options);
+			break;
+
+		case CONTAINER:
+			rc = do_container_expand((storage_container_t *) thing, input_objects, options);
 			break;
 
 		default:
-			LOG_ERROR("Object of type %d cannot be expanded.\n", type);
+			LOG_ERROR("A thing of type %d cannot be expanded.\n", type);
 			rc = EINVAL;
 	}
 
@@ -807,7 +905,8 @@ int engine_can_expand_by(storage_object_t * object, sector_count_t * delta_size)
 
 	LOG_PROC_ENTRY();
 
-	while ((rc == 0) && !list_empty(curr_object->parent_objects)) {
+	while ((rc == 0) && (curr_object->consuming_container == NULL) &&
+	       !list_empty(curr_object->parent_objects)) {
 
 		curr_object = first_thing(curr_object->parent_objects, NULL);
 
@@ -817,20 +916,26 @@ int engine_can_expand_by(storage_object_t * object, sector_count_t * delta_size)
 	}
 
 	if (rc == 0) {
-		/*
-		 * If this object is part of a volume and the volume has an FSIM that is
-		 * not scheduled to be removed, check to make sure the final size does
-		 * not go over the FSIM's maximum size it can handle for its backing
-		 * storage object.
-		 */
-		if (object->volume != NULL) {
+		if (curr_object->consuming_container != NULL) {
+			storage_container_t * con = curr_object->consuming_container;
+
+			rc = con->plugin->container_functions->can_expand_container_by(con, curr_object, delta_size);
+
+		} else if (object->volume != NULL) {
+			/*
+			 * If this object is part of a volume and the volume has
+			 * an FSIM that is not scheduled to be removed, check to
+			 * make sure the final size does not go over the FSIM's
+			 * maximum size it can handle for its backing storage
+			 * object.
+			 */
 
 			if (object->volume->flags & VOLFLAG_COMPATIBILITY) {
 				if (is_kernel_volume_mounted(object->volume, DETAILS)) {
 					rc = EBUSY;
 				}
 			}
-			
+
 			if (rc == 0) {
 				if ((object->volume->original_fsim != NULL) &&
 				    !(object->volume->flags & VOLFLAG_UNMKFS)) {
