@@ -1,6 +1,6 @@
 /*
  *
- *   (C) Copyright IBM Corp. 2001, 2003
+ *   (C) Copyright IBM Corp. 2001, 2004
  *
  *   This program is free software;  you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -40,6 +40,7 @@
 #include "fullengine.h"
 #include "internalAPI.h"
 #include "engine.h"
+#include "copy.h"
 #include "crc.h"
 #include "discover.h"
 #include "handlemgr.h"
@@ -250,7 +251,7 @@ int engine_get_volume_list(plugin_record_t     * fsim,
 		if ((fsim != NULL) || (disk_group != NULL) || (flags != 0)) {
 			list_element_t iter1;
 			list_element_t iter2;
-			logical_volume_t * vol;;
+			logical_volume_t * vol;
 
 			LIST_FOR_EACH_SAFE(new_volume_list, iter1, iter2, vol) {
 				boolean remove = FALSE;
@@ -546,7 +547,7 @@ int engine_get_container_list(plugin_record_t        * plugin,
  * allocate any kind of storage object.  It allocates zero filled memory,
  * creates the lists, and initializes the flags.
  */
-static int allocate_new_storage_object(storage_object_t * * new_object) {
+int allocate_new_storage_object(storage_object_t * * new_object) {
 	int rc = 0;
 
 	storage_object_t * object = NULL;
@@ -556,41 +557,65 @@ static int allocate_new_storage_object(storage_object_t * * new_object) {
 	object = (storage_object_t *) engine_alloc(sizeof(storage_object_t));
 	if (object != NULL) {
 
-		object->parent_objects = allocate_list();
+                object->parent_objects = allocate_list();
 		if (object->parent_objects != NULL) {
 
 			object->child_objects = allocate_list();
 			if (object->child_objects != NULL) {
 
-				/*
-				 * The default data type is DATA_TYPE.  The plug-in can change
-				 * it if necessary.
-				 */
-				object->data_type = DATA_TYPE;
+				object->associated_parents = allocate_list();
+				if (object->associated_parents != NULL) {
 
-				/*
-				 * If we are not in discovery, this must be a new object so mark
-				 * it new and dirty.
-				 */
-				if (!discover_in_progress) {
-					object->flags |= SOFLAG_NEW | SOFLAG_DIRTY;
+					object->associated_children = allocate_list();
+					if (object->associated_children != NULL) {
+
+						/*
+						 * The default data type is
+						 * DATA_TYPE.  The plug-in can
+						 * change it if necessary.
+						 */
+						object->data_type = DATA_TYPE;
+
+						/*
+						 * If we are not in discovery,
+						 * this must be a new object, so
+						 * mark it new and dirty.
+						 */
+						if (!discover_in_progress) {
+							object->flags |= SOFLAG_NEW | SOFLAG_DIRTY;
+						}
+
+					} else {
+						LOG_CRITICAL("Unable to get memory for the storage object's associated children list.\n");
+						rc = ENOMEM;
+					}
+
+				} else {
+					LOG_CRITICAL("Unable to get memory for the storage object's associated parents list.\n");
+					rc = ENOMEM;
 				}
 
 			} else {
-				destroy_list(object->parent_objects);
+				LOG_CRITICAL("Unable to get memory for the storage object's child list.\n");
 				rc = ENOMEM;
 			}
 
 		} else {
+			LOG_CRITICAL("Unable to get memory for the storage object's parent list.\n");
 			rc = ENOMEM;
 		}
 
 		if (rc != 0) {
+			/* destroy_list() handles NULL list pointers. */
+			destroy_list(object->associated_parents);
+			destroy_list(object->child_objects);
+			destroy_list(object->parent_objects);
 			engine_free(object);
 			object = NULL;
 		}
 
 	} else {
+		LOG_CRITICAL("Unable to get memory for a storage object.\n");
 		rc = ENOMEM;
 	}
 
@@ -606,7 +631,7 @@ static int allocate_new_storage_object(storage_object_t * * new_object) {
  * cleaning up any type of storage object.  It destroys the app-handle, if one
  * was created, destroys the lists in the object, and frees the object.
  */
-static void free_old_storage_object(storage_object_t * obj) {
+void free_old_storage_object(storage_object_t * obj) {
 
 	LOG_PROC_ENTRY();
 
@@ -615,14 +640,20 @@ static void free_old_storage_object(storage_object_t * obj) {
 	}
 
 	destroy_list(obj->parent_objects);
+	obj->parent_objects = NULL;
 	destroy_list(obj->child_objects);
+	obj->child_objects = NULL;
+	destroy_list(obj->associated_parents);
+	obj->associated_parents = NULL;
+	destroy_list(obj->associated_children);
+	obj->associated_children = NULL;
 
 	if ((!(engine_mode & ENGINE_DAEMON)) &&
 	    (obj->flags & SOFLAG_ACTIVE) &&
 	    (obj->data_type == DATA_TYPE)) {
 
 		/*
-		 * Order is important in the DeactivateList.  Objects must
+		 * Order is important in the deactivate_list.  Objects must
 		 * be appended to the list so that they are deactivated in
 		 * the order they were deleted.
 		 */
@@ -631,6 +662,7 @@ static void free_old_storage_object(storage_object_t * obj) {
 			     INSERT_AFTER,
 			     NULL);
 
+		obj->flags |= SOFLAG_NEEDS_DEACTIVATE;
 	} else {
 		engine_free(obj);
 	}
@@ -1133,7 +1165,7 @@ static int engine_open_object(storage_object_t * obj, int oflags, ...) {
 	LOG_DEBUG("Request to open object %s.  flags: %#x\n", obj->name, oflags);
 
 	if (obj->dev_major == 0) {
-		LOG_ERROR("Object %s does not have a non-zero major.\n", obj->name);
+		LOG_DEBUG("Object %s does not have a non-zero major.\n", obj->name);
 		LOG_PROC_EXIT_INT(EINVAL);
 		return -EINVAL;
 	}
@@ -2047,19 +2079,20 @@ static boolean engine_is_mounted(char * volume_name, char * * mount_name) {
 		*mount_name = NULL;
 	}
 
-	/* Search the VolumeList for a volume whose name matches volume_name. */
+	/* Search the volumes_list for a volume whose dev node matches volume_name. */
 	LIST_FOR_EACH(&volumes_list, iter, vol) {
-		if (strcmp(volume_name, vol->name) == 0) {
+		if (strcmp(volume_name, vol->dev_node) == 0) {
 			break;
 		}
 	}
 
 	if (vol == NULL) {
-		/* Didn't find it in the regular volume list. Search the
-		 * deleted-volumes list as well.
+		/*
+		 * Didn't find it in the regular volume list. Search the
+		 * volume_delete_list list as well.
 		 */
 		LIST_FOR_EACH(&volume_delete_list, iter, vol) {
-			if (strcmp(volume_name, vol->name) == 0) {
+			if (strcmp(volume_name, vol->dev_node) == 0) {
 				break;
 			}
 		}
@@ -2123,8 +2156,8 @@ static int engine_unassign_fsim_from_volume(logical_volume_t * volume) {
 	/* Reinitialize the file system and volume limits. */
 	volume->fs_size = volume->vol_size;
 	volume->min_fs_size = 0;
-	volume->max_fs_size = -1;
-	volume->max_vol_size = -1;
+	volume->max_fs_size = UINT64_MAX;
+	volume->max_vol_size = UINT64_MAX;
 
 	LOG_PROC_EXIT_INT(rc);
 	return rc;
@@ -2179,33 +2212,38 @@ static int engine_get_nodeid(ece_nodeid_t * nodeid) {
 
 
 static int engine_nodeid_to_string(ece_nodeid_t * nodeid,
-				   char         * string,
-				   uint         * len) {
+				   const char * * string) {
 
 	int rc = 0;
 
 	LOG_PROC_ENTRY();
 
-	if (num_config_nodes == 0) {
-		LOG_ERROR("The Engine is not running in a clustered environment.\n");
-		rc = ENOSYS;
+	if (nodeid == NULL) {
+		LOG_ERROR("No nodeid given.\n");
+		rc = EINVAL;
+	}
+
+	if (string == NULL) {
+		LOG_ERROR("No string destination given.\n");
+		rc = EINVAL;
 
 	} else {
-		const char * node_string = nodeid_to_string(nodeid);
+		*string = NULL;
+	}
 
-		if (node_string != NULL) {
-			if (strlen(node_string) < *len) {
-				strcpy(string, node_string);
-
-			} else {
-				LOG_ERROR("Buffer length %u is too small for the node name string.  Need %zu bytes.\n", *len, strlen(node_string) + 1);
-				*len = strlen(node_string) + 1;
-				rc = ENOSPC;
-			}
+	if (rc == 0) {
+		if (num_config_nodes == 0) {
+			LOG_ERROR("The Engine is not running in a clustered environment.\n");
+			*string = NULL;
+			rc = ENODEV;
 
 		} else {
-			LOG_ERROR("Node ID is not valid.\n");
-			rc = EINVAL;
+			*string = nodeid_to_string(nodeid);
+
+			if (*string == NULL) {
+				LOG_ERROR("Node ID is not valid.\n");
+				rc = EINVAL;
+			}
 		}
 	}
 
@@ -2221,27 +2259,42 @@ static int engine_string_to_nodeid(const char   * string,
 
 	LOG_PROC_ENTRY();
 
-	if (num_config_nodes == 0) {
-		LOG_ERROR("The Engine is not running in a clustered environment.\n");
-		rc = ENODEV;
+	if (string == NULL) {
+		LOG_ERROR("No string given.\n");
+		rc = EINVAL;
+	}
+
+	if (nodeid == NULL) {
+		LOG_ERROR("No nodeid destination given.\n");
+		rc = EINVAL;
 
 	} else {
-		int i = 0;
+		memcpy(nodeid, &no_nodeid, sizeof(ece_nodeid_t));
+	}
 
-		while (i < num_config_nodes) {
-			if (strcmp(config_node_names->node_info[i].node_name, string) == 0) {
-				break;
-			} else {
-				i++;
-			}
-		}
-
-		if (i < num_config_nodes) {
-			memcpy(nodeid, &config_nodes[i], sizeof(ece_nodeid_t));
+	if (rc == 0) {
+		if (num_config_nodes == 0) {
+			LOG_ERROR("The Engine is not running in a clustered environment.\n");
+			rc = ENODEV;
 
 		} else {
-			LOG_ERROR("There is no node ID to match node name \"%s\".\n", string);
-			rc = EINVAL;
+			int i = 0;
+
+			while (i < num_config_nodes) {
+				if (strcmp(config_node_names->node_info[i].node_name, string) == 0) {
+					break;
+				} else {
+					i++;
+				}
+			}
+
+			if (i < num_config_nodes) {
+				memcpy(nodeid, &config_nodes[i], sizeof(ece_nodeid_t));
+
+			} else {
+				LOG_ERROR("There is no node ID to match node name \"%s\".\n", string);
+				rc = EINVAL;
+			}
 		}
 	}
 
@@ -2303,739 +2356,6 @@ boolean engine_is_offline(storage_object_t   * obj,
 }
 
 
-#define NUM_TIMERS	1024
-typedef struct timer_data_s {
-	u_int64_t	timers[NUM_TIMERS];
-	u_int64_t	counts[NUM_TIMERS];
-	u_int64_t	rates[NUM_TIMERS];
-	int		oldest;
-	int		newest;
-} timer_data_t;
-
-
-void init_copy_progress(progress_t    * progress,
-			char          * title,
-			char          * description,
-			progress_type_t type,
-			u_int64_t       total_count) {
-
-	LOG_PROC_ENTRY();
-
-	memset(progress, 0, sizeof(*progress));
-	progress->title       = title;
-	progress->description = description;
-	progress->type        = type;
-	progress->total_count = total_count;
-	if (!(type == INDETERMINATE)) {
-		timer_data_t * timer_data = engine_alloc(sizeof(timer_data_t));
-
-		if (timer_data != NULL) {
-			struct timeval  time;
-			struct timezone tz;
-
-			/* Fill in the first entry */
-			gettimeofday(&time, &tz);
-			timer_data->timers[0] = (u_int64_t) time.tv_sec * (u_int64_t) 1000000 + (u_int64_t) time.tv_usec;
-			timer_data->oldest = 0;
-
-                        progress->plugin_private_data = timer_data;
-		}
-	}
-
-	plugin_progress(progress);
-
-	LOG_PROC_EXIT_VOID();
-}
-
-
-void report_copy_progress(progress_t * progress,
-			  u_int64_t    chunks_copied) {
-
-	LOG_PROC_ENTRY();
-
-	if (progress->type != INDETERMINATE) {
-
-		progress->count = chunks_copied;
-
-		if (progress->plugin_private_data != NULL) {
-			timer_data_t * timer_data = (timer_data_t *) progress->plugin_private_data;
-			int newest = timer_data->newest;
-			int oldest = timer_data->oldest;
-			u_int64_t block_time;
-			u_int64_t block_count;
-			u_int64_t ms_per_chunk;
-			struct timeval time;
-			struct timezone tz;
-
-			if (newest < NUM_TIMERS - 1) {
-				newest++;
-			} else {
-				newest = 0;
-			}
-			timer_data->newest = newest;
-
-			if (newest == oldest) {
-				if (oldest < NUM_TIMERS - 1) {
-					oldest++;
-				} else {
-					oldest = 0;
-				}
-			}
-
-			gettimeofday(&time, &tz);
-			timer_data->timers[newest] = (u_int64_t) time.tv_sec * (u_int64_t) 1000000 + (u_int64_t) time.tv_usec;
-			timer_data->counts[newest] = chunks_copied;
-
-			/*
-			 * Don't use more than the last ten seconds of data
-			 * for making an estimate.
-			 */
-			while ((timer_data->timers[newest] - timer_data->timers[oldest] > 10000000) &&
-			       (oldest != newest)) {
-				if (oldest < NUM_TIMERS - 1) {
-					oldest++;
-				} else {
-					oldest = 0;
-				}
-			}
-			timer_data->oldest = oldest;
-
-			block_time  = timer_data->timers[newest] - timer_data->timers[oldest];
-			block_count = timer_data->counts[newest] - timer_data->counts[oldest];
-			if (block_count != 0) {
-				ms_per_chunk = block_time / block_count;
-			} else {
-				ms_per_chunk = -1;
-			}
-			timer_data->rates[newest] = ms_per_chunk;
-
-			/*
-			 * Wait until we have at least five seconds of data
-			 * before making an estimate of the remaining time.
-			 */
-			if (timer_data->timers[newest] - timer_data->timers[oldest] > 5000000) {
-
-				/*
-				 * Can't give an estimate if the count hasn't
-				 * incremented at all across the array of
-				 * timers.
-				 */
-				if (block_count != 0) {
-					u_int64_t rem_chunks = progress->total_count - chunks_copied;
-					u_int64_t average_ms_per_chunk = 0;
-					u_int64_t rem_time_ms;
-					uint rem_secs;
-					int i = 0;
-					int j = oldest;
-
-					while (j != newest) {
-						if (timer_data->rates[j] != -1) {
-							average_ms_per_chunk += timer_data->rates[j];
-							i++;
-						}
-						if (j < NUM_TIMERS - 1) {
-							j++;
-						} else {
-							j = 0;
-						}
-					}
-
-					average_ms_per_chunk /= i;
-
-					/*
-					 * The calculation:
-					 *   rem_time_ms = rem_chunks * ms_per_chunk;
-					 * can cause rem_time_ms to drift wildly
-					 * between between progress reports.
-					 * ms_per_chunk fluctuates.  It also
-					 * loses precision from the division
-					 * since it is an integer.  For large
-					 * values of rem_chunks, the
-					 * fluctuations and loss of precision
-					 * get amplified, resulting in drifting
-					 * time estimates.
-					 * So, use the time and count for the
-					 * whole block when calculating
-					 * rem_time_ms.  It helps preserve some
-					 * precision which keeps the times more
-					 * consistent between the progress
-					 * reports.
-					 */
-					rem_time_ms = (rem_chunks / block_count) * block_time +
-						      (rem_chunks % block_count) * average_ms_per_chunk;
-
-					/*
-					 * Add a half second before rounding to
-					 * seconds.  Without it, the calculation
-					 * will round down to zero as soon as
-					 * rem_time_ms is under 1000000.  That
-					 * leaves the time estimate at zero for
-					 * the last second.  Depending on how
-					 * the UI handles it, the time estimate
-					 * either reads 00:00 or it disappears
-					 * for the last second before progress
-					 * is complete.  Either one looks funny.
-					 * The extra half second makes the time
-					 * estimate appear to hit zero when
-					 * progress is complete.
-					 */
-					rem_secs = (rem_time_ms + 500000) / 1000000;
-
-					/*
-					 * This check helps dampen the bouncing of the
-					 * estimate due to fractional noise from the
-					 * calculations above.
-					 */
-					if (abs(rem_secs - progress->remaining_seconds) > 3) {
-						progress->remaining_seconds += (int) (rem_secs - progress->remaining_seconds) / 2;
-					} else {
-						if (rem_secs < progress->remaining_seconds) {
-							progress->remaining_seconds = rem_secs;
-						}
-					}
-				}
-			}
-		}
-
-		plugin_progress(progress);
-	}
-
-	LOG_PROC_EXIT_VOID();
-}
-
-
-void final_copy_progress(progress_t * progress) {
-
-	LOG_PROC_ENTRY();
-
-	if (progress->count < progress->total_count) {
-		progress->count = progress->total_count;
-		progress->remaining_seconds = 0;
-
-		plugin_progress(progress);
-	}
-
-	engine_free(progress->plugin_private_data);
-	progress->plugin_private_data = NULL;
-
-	LOG_PROC_EXIT_VOID();
-}
-
-
-static int local_copy(copy_job_t * copy_job) {
-
-	int rc = 0;
-	void * copy_buffer;
-	lsn_t src_lsn = copy_job->src.start;
-	lsn_t trg_lsn = copy_job->trg.start;
-	sector_count_t len = min(copy_job->src.len, copy_job->trg.len);
-	u_int64_t chunk_size = min(len, 65536 >> EVMS_VSECTOR_SIZE_SHIFT);
-	sector_count_t sectors_copied = 0;
-	u_int64_t chunks_copied = 0;
-	progress_t progress;
-
-	LOG_PROC_ENTRY();
-	LOG_DEBUG("Source:	  %s\n", copy_job->src.obj->name);
-	LOG_DEBUG("Source offset: %"PRIu64"\n", src_lsn);
-	LOG_DEBUG("Target:	  %s\n", copy_job->trg.obj->name);
-	LOG_DEBUG("Target offset: %"PRIu64"\n", trg_lsn);
-	LOG_DEBUG("Length:        %"PRIu64"\n", len);
-
-	copy_buffer = engine_alloc(chunk_size << EVMS_VSECTOR_SIZE_SHIFT);
-
-	if (copy_job->title != NULL) {
-		init_copy_progress(&progress,
-				   copy_job->title,
-				   copy_job->description,
-				   DISPLAY_PERCENT,
-				   (len + chunk_size - 1) / chunk_size);
-	}
-
-	while ((rc == 0) && (sectors_copied < len)) {
-		sector_count_t sectors_to_copy = min(chunk_size, len - sectors_copied);
-
-		rc = copy_job->src.obj->plugin->functions.plugin->read(copy_job->src.obj, src_lsn, sectors_to_copy, copy_buffer);
-
-		if (rc == 0) {
-			rc = copy_job->trg.obj->plugin->functions.plugin->write(copy_job->trg.obj, trg_lsn, sectors_to_copy, copy_buffer);
-		}
-
-		src_lsn += sectors_to_copy;
-		trg_lsn += sectors_to_copy;
-		sectors_copied += sectors_to_copy;
-		chunks_copied ++;
-
-		if ((rc == 0) && (copy_job->title != NULL)) {
-			report_copy_progress(&progress, chunks_copied);
-		}
-	}
-
-	if (copy_job->title != NULL) {
-		final_copy_progress(&progress);
-	}
-
-	LOG_DEBUG("Total sectors copied: %"PRIu64"\n", sectors_copied);
-
-	engine_free(copy_buffer);
-
-	LOG_PROC_EXIT_INT(rc);
-	return rc;
-}
-
-
-static int engine_offline_copy(copy_job_t * copy_job) {
-
-	int rc = 0;
-
-	struct timezone tz;
-	struct timeval copy_start_time;
-	struct timeval copy_stop_time;
-	struct timeval copy_time;
-	int copy_hours;
-	int copy_minutes;
-
-	LOG_PROC_ENTRY();
-
-	gettimeofday(&copy_start_time, &tz);
-
-        rc = local_copy(copy_job);
-
-	gettimeofday(&copy_stop_time, &tz);
-
-	copy_time.tv_sec = copy_stop_time.tv_sec - copy_start_time.tv_sec;
-	copy_time.tv_usec = copy_stop_time.tv_usec - copy_start_time.tv_usec;
-	if (copy_time.tv_usec < 0) {
-		copy_time.tv_sec --;
-		copy_time.tv_usec += 1000000;
-	}
-
-	copy_hours = copy_time.tv_sec / 3600;
-	copy_time.tv_sec %= 3600;
-	copy_minutes = copy_time.tv_sec / 60;
-	copy_time.tv_sec %= 60;
-
-	LOG_DEFAULT("Copy time: %02d:%02d:%02ld.%06ld\n", copy_hours, copy_minutes, copy_time.tv_sec, copy_time.tv_usec);
-
-	LOG_PROC_EXIT_INT(rc);
-	return rc;
-}
-
-
-static int online_copy = -1;
-
-boolean engine_can_online_copy(void) {
-
-	LOG_PROC_ENTRY();
-
-	if (online_copy == -1) {
-		int rc;
-		storage_object_t * disk;
-		storage_object_t * mirror;
-
-		online_copy = 0;
-
-		if (dm_get_version() != 4) {
-			LOG_DETAILS("Online copy requires version 4 of the device-mapper ioctl.\n");
-			return online_copy;
-		}
-
-		disk = first_thing(&disks_list, NULL);
-		if (disk == NULL) {
-			LOG_DETAILS("Can't get a disk for testing if device-mapper raid 1 is available.\n");
-			return online_copy;
-		}
-
-		rc = allocate_new_storage_object(&mirror);
-		if (rc == 0) {
-			strcpy(mirror->name, "Test-mirror");
-			mirror->size = 512;
-			mirror->object_type = SEGMENT;
-
-			rc = dm_create(mirror);
-			if (rc == 0) {
-				dm_target_t * target;
-				
-
-				target = dm_allocate_target(DM_TARGET_MIRROR, 0, mirror->size, 2, 0);
-
-				if (target != NULL) {
-					target->data.mirror->num_mirrors = 2;
-					target->data.mirror->chunk_size = 32;
-					target->data.mirror->persistent = FALSE;
-					target->data.mirror->devices[0].major = disk->dev_major;
-					target->data.mirror->devices[0].minor = disk->dev_minor;
-					target->data.mirror->devices[0].start = 0;
-					target->data.mirror->devices[1].major = disk->dev_major;
-					target->data.mirror->devices[1].minor = disk->dev_minor;
-					target->data.mirror->devices[1].start = 0;
-
-					rc = dm_load_targets(mirror, target);
-					if (rc == 0) {
-						online_copy = 1;
-					}
-
-					dm_deallocate_targets(target);
-
-				} else {
-					LOG_SERIOUS("Error allocating a target list.\n");
-				}
-
-				dm_deactivate(mirror);
-
-			} else {
-				LOG_SERIOUS("Failed to create device-mapper mapping for mirror.  Error code is %d: %s.\n", rc, evms_strerror(rc));
-			}
-
-			free_old_storage_object(mirror);
-
-		} else {
-			LOG_SERIOUS("Error allocating a new storage object for the mirror: %d: %s\n", rc, evms_strerror(rc));
-		}
-
-		LOG_PROC_EXIT_BOOLEAN(online_copy);
-		return online_copy;
-	}
-
-	LOG_PROC_EXIT_BOOLEAN(online_copy);
-	return online_copy;
-}
-
-
-static int engine_copy_setup(copy_job_t * copy_job) {
-
-	int rc = 0;
-	storage_object_t * mirror;
-	dm_target_t * target;
-
-	LOG_PROC_ENTRY();
-
-	rc = allocate_new_storage_object(&mirror);
-	if (rc != 0) {
-		LOG_SERIOUS("Error allocating a new storage object for the mirror: %d: %s\n", rc, evms_strerror(rc));
-		LOG_PROC_EXIT_INT(rc);
-		return rc;
-	}
-
-	snprintf(mirror->name, EVMS_NAME_SIZE, "Copy_%s_to_%s", copy_job->src.obj->name, copy_job->trg.obj->name);
-	mirror->size = min(copy_job->src.len, copy_job->trg.len);
-	mirror->object_type = copy_job->src.obj->object_type;
-
-	insert_thing(mirror->child_objects,
-		     copy_job->src.obj,
-		     INSERT_BEFORE,
-		     NULL);
-
-	insert_thing(mirror->child_objects,
-		     copy_job->trg.obj,
-		     INSERT_AFTER,
-		     NULL);
-	/*
-	 * The device must have a size if it is to be used as the target of
-	 * another device (i.e., the device that will point to the mirror
-	 * instead of the source object).  The device must have an active map in
-	 * order to have a size.  But the mirror isn't ready to run until
-	 * copy_start() is called.  So, load the mirror device with an error map
-	 * and suspend the device.
-	 */
-	target = dm_allocate_target(DM_TARGET_ERROR, 0, mirror->size, 0, 0);
-
-	if (target != NULL) {
-		/*
-		 * 2.4 kernels keep device sizes in number of 1KB blocks, not in
-		 * sectors.  An odd size will be rounded down by the kernel.  If
-		 * the source has an odd size, then when the caller of the copy
-		 * service remaps to the mirror device the device can end up
-		 * being too small causing an error when loading the map.  Round
-		 * up the error target size so that it is big enough for the
-		 * device that will remap to the mirror.
-		 */
-		if (is_2_4_kernel) {
-			target->length += 1;
-			target->length &= ~(1);
-		}
-
-		rc = dm_activate(mirror, target);
-
-		if (rc == 0) {
-			rc = dm_suspend(mirror, TRUE);
-
-			if (rc != 0) {
-				LOG_SERIOUS("dm_suspend() to suspend the error map for the mirror returned %d: %s\n", rc, evms_strerror(rc));
-			}
-
-		} else {
-			LOG_SERIOUS("dm_activate() to activate the error map for the mirror returned %d: %s\n", rc, evms_strerror(rc));
-		}
-
-		dm_deallocate_targets(target);
-
-	} else {
-		LOG_CRITICAL("Error allocating target.\n");
-		rc = ENOMEM;
-	}
-
-	if (rc != 0) {
-		dm_deactivate(mirror);
-		free_old_storage_object(mirror);
-		LOG_PROC_EXIT_INT(rc);
-		return rc;
-	}
-
-	/*
-	 * Now load the mirror device's inactive table with the mapping for
-	 * the mirror.  When the device is resumed in copy_start(), the
-	 * error map will be replaced with the mirror map.
-	 */
-	target = dm_allocate_target(DM_TARGET_MIRROR, 0, mirror->size, 2, 0);
-
-	if (target != 0) {
-		/*
-		 * 2.4 kernels keep device sizes in number of 1KB blocks, not in
-		 * sectors.  An odd size will be rounded down by the kernel.  If
-		 * the source has an odd size it will be rounded down by the
-		 * kernel.  When the mirror map is created the same size as the
-		 * source, the source can end up being too small causing an
-		 * error when loading the map for the mirror.  Round down the
-		 * mirror target size so that it is not bigger than the source.
-		 */
-		if (is_2_4_kernel) {
-			target->length &= ~(1);
-		}
-
-		target->data.mirror->num_mirrors = 2;
-		target->data.mirror->chunk_size = 128;
-		target->data.mirror->persistent = FALSE;
-		target->data.mirror->devices[0].major = copy_job->src.obj->dev_major;
-		target->data.mirror->devices[0].minor = copy_job->src.obj->dev_minor;
-		target->data.mirror->devices[0].start = copy_job->src.start;
-		target->data.mirror->devices[1].major = copy_job->trg.obj->dev_major;
-		target->data.mirror->devices[1].minor = copy_job->trg.obj->dev_minor;
-		target->data.mirror->devices[1].start = copy_job->trg.start;
-
-		rc = dm_load_targets(mirror, target);
-
-		dm_deallocate_targets(target);
-
-	} else {
-		LOG_CRITICAL("Error allocating target.\n");
-		rc = ENOMEM;
-	}
-
-	if (rc == 0) {
-		copy_job->mirror = mirror;
-
-	} else {
-		LOG_SERIOUS("dm_load_targets() to load the mirror target returned %d: %s\n", rc, evms_strerror(rc));
-		dm_deactivate(mirror);
-		free_old_storage_object(mirror);
-	}
-
-	LOG_PROC_EXIT_INT(rc);
-	return rc;
-}
-
-
-static int engine_copy_start(copy_job_t * copy_job) {
-
-	int rc = 0;
-
-	LOG_PROC_ENTRY();
-
-	rc = dm_suspend(copy_job->mirror, FALSE);
-
-	LOG_PROC_EXIT_INT(rc);
-	return rc;
-}
-
-
-typedef struct ocp_parms_s {
-	char * dev_name;
-	progress_t progress;
-} ocp_parms_t;
-
-static void * online_copy_progress_thread(void * arg) {
-
-	ocp_parms_t * ocp_parms = (ocp_parms_t *) arg;
-	int old_cancel_state;
-	int rc;
-	char * info;
-	int count;
-	int nr_mirrors;
-	int src_major;
-	int src_minor;
-	int trg_major;
-	int trg_minor;
-	unsigned long sync_count;
-	unsigned long nr_regions;
-
-	/*
-	 * Don't allow the thread to be canceled unless it is sleeping.  There
-	 * is a very real risk that the thread can be canceled in the middle
-	 * of writing to the log file while the log mutex lock is held.  Bad
-	 * things happen if the thread leaves while holding the log lock.
-	 */
-	pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, &old_cancel_state);
-
-	LOG_PROC_ENTRY();
-
-	do {
-		rc = dm_get_info_v4(ocp_parms->dev_name, &info);
-
-		if (rc == 0) {
-			count = sscanf(info, "%d %d:%d %d:%d %lu/%lu",
-				       &nr_mirrors,
-				       &src_major,  &src_minor,
-				       &trg_major,  &trg_minor,
-				       &sync_count, &nr_regions);
-
-			if (count == 7) {
-				ocp_parms->progress.total_count = nr_regions;
-
-				report_copy_progress(&ocp_parms->progress, sync_count);
-
-				if (sync_count < nr_regions) {
-					/*
-					 * The thread can handle being canceled at this point.
-					 */
-					pthread_setcancelstate(old_cancel_state, NULL);
-					pthread_testcancel();
-
-					usleep(100000);
-
-					/* Here, too. */
-					pthread_testcancel();
-					pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, &old_cancel_state);
-
-				} else {
-					/*
-					 * Set an "error code" to exit the loop.
-					 */
-					rc = ENOSPC;
-				}
-
-			} else {
-				LOG_WARNING("Scanned %d variables from string \"%s\".  Need 7 variables.\n", count, info);
-				rc = ENODATA;
-			}
-
-		} else {
-			LOG_DETAILS("dm_get_info_v4() retruned error code %d: %s\n", rc, evms_strerror(rc));
-		}
-
-	} while (rc == 0);
-
-	LOG_PROC_EXIT_PTR(NULL);
-
-	pthread_setcancelstate(old_cancel_state, NULL);
-
-	return NULL;
-}
-
-
-static unsigned char sector_buf[EVMS_VSECTOR_SIZE];
-
-static int engine_copy_wait(copy_job_t * copy_job) {
-
-	int rc = 0;
-	unsigned int event_nr = 0;
-	char * info = NULL;
-	ocp_parms_t ocp_parms;
-	pthread_t progress_tid = 0;
-	boolean orig_log_pid = log_pid;
-
-	LOG_PROC_ENTRY();
-
-	if (copy_job->title != NULL) {
-		init_copy_progress(&ocp_parms.progress,
-				   copy_job->title,
-				   copy_job->description,
-				   DISPLAY_PERCENT,
-				   1);
-
-		/* Start a progress thread. */
-		ocp_parms.dev_name = copy_job->mirror->name;
-
-		/*
-		 * Since progress is reported on a separate thread,
-		 * turn on the logging of the PID in the log timestamp.
-		 */
-		log_pid = TRUE;
-
-		pthread_create(&progress_tid,
-			       &pthread_attr_detached,
-			       online_copy_progress_thread,
-			       &ocp_parms);
-
-		if (progress_tid == 0) {
-			log_pid = orig_log_pid;
-			ocp_parms.progress.type = INDETERMINATE;
-			report_copy_progress(&ocp_parms.progress, 0);
-		}
-	}
-
-	rc = dm_wait(copy_job->mirror, &event_nr, &info);
-
-	if (rc == 0) {
-		/*
-		 * 2.4 kernels use block sizes of 1KB.  They don't handle
-		 * any odd sector at the end of the device.  If we are
-		 * running on a 2.4 kernel and the mirror has an odd
-		 * number of sectors, copy the las sector by hand.
-		 */
-		if (is_2_4_kernel && (copy_job->mirror->size & 1)) {
-			rc = copy_job->src.obj->plugin->functions.plugin->read(copy_job->src.obj, copy_job->src.start + copy_job->mirror->size - 1, 1, sector_buf);
-			if (rc == 0) {
-				rc = copy_job->trg.obj->plugin->functions.plugin->write(copy_job->trg.obj, copy_job->trg.start + copy_job->mirror->size - 1, 1, sector_buf);
-
-				if (rc != 0) {
-					LOG_SERIOUS("Error code %d when writing the last sector to target object %s: %s\n", rc, copy_job->trg.obj->name, evms_strerror(rc));
-				}
-
-			} else {
-				LOG_SERIOUS("Error code %d when reading the last sector from source object %s: %s\n", rc, copy_job->src.obj->name, evms_strerror(rc));
-			}
-		}
-	}
-
-	if (progress_tid != 0) {
-		pthread_cancel(progress_tid);
-
-		/* Wait for the thread to exit. */
-		pthread_join(progress_tid, NULL);
-
-		log_pid = orig_log_pid;
-	}
-
-	if (copy_job->title != NULL) {
-		final_copy_progress(&ocp_parms.progress);
-	}
-
-	engine_free(info);
-
-	LOG_PROC_EXIT_INT(rc);
-	return rc;
-}
-
-
-static int engine_copy_cleanup(copy_job_t * copy_job) {
-
-	int rc = 0;
-
-	LOG_PROC_ENTRY();
-
-	if (copy_job->mirror != NULL) {
-		dm_deactivate(copy_job->mirror);
-
-		free_old_storage_object(copy_job->mirror);
-
-		copy_job->mirror = NULL;
-	}
-
-	LOG_PROC_EXIT_INT(rc);
-	return rc;
-}
-
-
 static pid_t do_fork_and_execvp(char * argv[],
 				int    stdin_pipe[2],
 				int    stdout_pipe[2],
@@ -3051,14 +2371,34 @@ static pid_t do_fork_and_execvp(char * argv[],
 	if (pid == 0) {
 		/* Child process */
 
+		int dev_null_fd = -1;
+
+		if ((stdout_pipe == NULL) || (stderr_pipe == NULL)) {
+			LOG_DEBUG("Open /dev/null\n");
+			dev_null_fd = open("/dev/null", O_WRONLY);
+			LOG_DEBUG("dev_null_fd is %d\n", dev_null_fd);
+		}
+
 		if (stdin_pipe != NULL) {
 			dup2(stdin_pipe[0], 0);	/* Route stdin to pipe's read fd */
 		}
+
 		if (stdout_pipe != NULL) {
 			dup2(stdout_pipe[1], 1);/* Route stdout to pipe's write fd */
+		} else {
+			if (dev_null_fd > 0) {
+				LOG_DEBUG("Send stdout to the bit bucket.\n");
+				dup2(dev_null_fd, 1);/* Route stdout to the bit bucket */
+			}
 		}
+
 		if (stderr_pipe != NULL) {
 			dup2(stderr_pipe[1], 2);/* Route stderr to pipe's write fd */
+		} else {
+			if (dev_null_fd > 0) {
+				LOG_DEBUG("Send stderr to the bit bucket.\n");
+				dup2(dev_null_fd, 2);/* Route stderr to the bit bucket */
+			}
 		}
 
 		if (debug_level >= DEBUG) {
@@ -3147,7 +2487,7 @@ static int can_discard_volume(logical_volume_t * vol) {
 
 	if (vol->flags & (VOLFLAG_DIRTY | VOLFLAG_NEW |
 			  VOLFLAG_NEEDS_ACTIVATE | VOLFLAG_NEEDS_DEACTIVATE |
-			  VOLFLAG_MKFS | VOLFLAG_UNMKFS | VOLFLAG_FSCK | VOLFLAG_DEFRAG |
+			  VOLFLAG_MKFS | VOLFLAG_UNMKFS | VOLFLAG_FSCK |
 			  VOLFLAG_SYNC_FS | VOLFLAG_PROBE_FS)) {
 		rc = EBUSY;
 	}
@@ -3218,12 +2558,34 @@ static int engine_can_discard(list_anchor_t objects) {
 }
 
 
+static void clear_volume_in_object(storage_object_t * obj) {
+
+	list_element_t iter;
+	storage_object_t * child;
+
+	LOG_PROC_ENTRY();
+
+	obj->volume = NULL;
+
+	if (obj->producing_container == NULL) {
+		LIST_FOR_EACH(obj->child_objects, iter, child) {
+			clear_volume_in_object(child);
+		}
+	}
+
+	LOG_PROC_EXIT_VOID();
+	return;
+}
+
+
 int discard_volume(logical_volume_t * vol) {
 
 	int rc = 0;
 
 	LOG_PROC_ENTRY();
 
+	LOG_DEBUG("Request to discard volume %s.\n", vol->name);
+
 	if (vol->file_system_manager != NULL) {
 		rc = vol->file_system_manager->functions.fsim->discard(vol);
 	}
@@ -3233,11 +2595,9 @@ int discard_volume(logical_volume_t * vol) {
 
 		/*
 		 * Clear out the volume pointer on this object and on all the child
-		 * objects in the tree.  Notify all the objects that the volume has been
-		 * deleted.
+		 * objects in the tree.
 		 */
-		set_volume_in_object(vol->object,
-				     NULL);    /* volume pointer */
+		clear_volume_in_object(vol->object);
 
 		remove_thing(&volumes_list, vol);
 
@@ -3287,26 +2647,45 @@ int engine_discard(list_anchor_t objects) {
 	}
 
 	LIST_FOR_EACH(objects, iter, obj) {
+		LOG_DEBUG("Request to discard object %s.\n", obj->name);
+
 		if (obj->data_type == DATA_TYPE) {
-			if (no_data_objects(obj->parent_objects)) {
-				if (obj->volume != NULL) {
-					rc = discard_volume(obj->volume);
+			if (obj->consuming_container != NULL) {
+				if (!no_data_objects(obj->consuming_container->objects_produced)) {
+					rc = engine_discard(obj->consuming_container->objects_produced);
+					if (rc != 0) {
+						break;
+					}
+				}
+				rc = obj->consuming_container->plugin->container_functions->discard_container(obj->consuming_container);
+				if (rc != 0) {
+					break;
+				}
+			} else {
+				if (no_data_objects(obj->parent_objects)) {
+					if (obj->volume != NULL) {
+						rc = discard_volume(obj->volume);
+						if (rc != 0) {
+							break;
+						}
+					}
+				} else {
+					rc = engine_discard(obj->parent_objects);
+					if (rc != 0) {
+						break;
+					}
 				}
 
-			} else {
-				rc = engine_discard(obj->parent_objects);
+				if (!no_data_objects(obj->associated_parents)) {
+					rc = engine_discard(obj->associated_parents);
+					if (rc != 0) {
+						break;
+					}
+				}
 			}
 		}
 	}
 
-	LIST_FOR_EACH(objects, iter, obj) {
-		if ((obj->data_type == DATA_TYPE) &&
-		    (obj->consuming_container != NULL) &&
-		    (no_data_objects(obj->consuming_container->objects_produced))) {
-			rc = obj->consuming_container->plugin->container_functions->discard_container(obj->consuming_container);
-		}
-	}
-
 	if (rc == 0) {
 		/*
 		 * Pass a copy of the objects list rather than the list
@@ -3395,6 +2774,60 @@ static int engine_rediscover_containers(list_anchor_t containers){
 }
 
 
+int engine_save_metadata(char         * parent,
+			 char         * child,
+			 sector_count_t offset,
+			 sector_count_t length,
+			 void         * metadata) {
+	int rc = 0;
+	int bytes_written;
+	metadata_db_entry_t metadata_entry;
+
+	LOG_PROC_ENTRY();
+
+	memset(&metadata_entry, 0, sizeof(metadata_entry));
+
+	strncpy(metadata_entry.signature, EVMS_METADATA_DB_SIGNATURE, sizeof(metadata_entry.signature));
+	metadata_entry.metadata_crc = CPU_TO_DISK32(evms_calculate_crc(0xffffffff,
+								       metadata,
+								       (u_int32_t) (length << EVMS_VSECTOR_SIZE_SHIFT)));
+	if (parent != NULL) {
+		strcpy(metadata_entry.parent, parent);
+	}
+	if (child != NULL) {
+		strcpy(metadata_entry.child, child);
+	}
+	metadata_entry.offset = CPU_TO_DISK64(offset);
+	metadata_entry.length = CPU_TO_DISK64(length);
+	metadata_entry.header_crc = CPU_TO_DISK32(evms_calculate_crc(0xffffffff, &metadata_entry, sizeof(metadata_entry)));
+
+	bytes_written = write(metadata_db_fd, &metadata_entry, sizeof(metadata_entry));
+	if (bytes_written < 0) {
+		rc = errno;
+		LOG_SERIOUS("Error writing the metadata DB entry header.  Return code is %d: %s\n",
+			    rc, strerror(rc));
+
+		LOG_PROC_EXIT_INT(rc);
+		return rc;
+	}
+
+	if (length != 0) {
+		bytes_written = write(metadata_db_fd, metadata, length << EVMS_VSECTOR_SIZE_SHIFT);
+		if (bytes_written < 0) {
+			rc = errno;
+			LOG_SERIOUS("Error writing the metadata.  Return code is %d: %s\n",
+				    rc, strerror(rc));
+
+			LOG_PROC_EXIT_INT(rc);
+			return rc;
+		}
+	}
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+
 engine_functions_t engine_functions = {
 	get_engine_plugin_api_version: engine_get_plugin_api_version,
 	get_plugin_list:               engine_get_plugin_list,
@@ -3426,11 +2859,13 @@ engine_functions_t engine_functions = {
 	close_volume:                  engine_close_volume,
 	engine_alloc:                  engine_alloc,
 	engine_realloc:                engine_realloc,
+	engine_strdup:                 engine_strdup,
 	engine_free:                   engine_free,
 	commit_in_progress:            engine_commit_in_progress,
 	write_log_entry:               plugin_write_log_entry,
-	calculate_CRC:                 calculate_CRC,
+	calculate_CRC:                 evms_calculate_crc,
 	calculate_checksum:            engine_calculate_checksum,
+	save_metadata:                 engine_save_metadata,
 	add_sectors_to_kill_list:      engine_add_sectors_to_kill_list,
 	validate_name:                 engine_validate_name,
 	register_name:                 engine_register_name,
