@@ -196,9 +196,10 @@ void free_option_descriptor_contents(void * object) {
 	LOG_PROC_EXIT_VOID();
 }
 
-/* Free an option array and all its associated memory. */
-void free_option_array(option_array_t * options)
+
+void free_option_array_contents(void * arg)
 {
+	option_array_t * options = (option_array_t *) arg;
 	int i;
 
 	for (i = 0; i < options->count; i++) {
@@ -225,8 +226,6 @@ void free_option_array(option_array_t * options)
 			}
 		}
 	}
-
-	engine_free(options);
 }
 
 
@@ -264,11 +263,7 @@ static int duplicate_value_list(value_list_t  * source,
 	if (source != NULL) {
 		uint size;
 
-		if (source->count == 0) {
-			size = sizeof(source->count);
-		} else {
-			size = sizeof(value_list_t) + (sizeof(value_t) * (source->count - 1));
-		}
+		size = sizeof(value_list_t) + (sizeof(value_t) * source->count);
 
 		*target = engine_alloc(size);
 		if (*target != NULL) {
@@ -285,7 +280,7 @@ static int duplicate_value_list(value_list_t  * source,
 
 			} else {
 				/* Copy the value list. */
-				memcpy((value_list_t *) *target, source, size);
+				memcpy(*target, source, size);
 			}
 
 		} else {
@@ -396,7 +391,7 @@ option_array_t * create_option_array_from_descriptors(option_desc_array_t *optio
 	option_array_t *option_array;
 	
 	option_array = engine_alloc(sizeof(option_array_t) +
-				(sizeof(key_value_pair_t) * (option_descriptors->count - 1)));
+				    (sizeof(key_value_pair_t) * option_descriptors->count));
 
 	if (option_array) {
 		int i, j, rc = 0;
@@ -442,7 +437,8 @@ option_array_t * create_option_array_from_descriptors(option_desc_array_t *optio
 		
 		if (rc) {
 			/* Had a problem somewhere. Free the option_array and return NULL */
-			free_option_array(option_array);
+			free_option_array_contents(option_array);
+			engine_free(option_array);
 			option_array = NULL;
 		}
 	}
@@ -515,7 +511,7 @@ int evms_get_option_descriptor(task_handle_t           handle,
 					rc = ENOMEM;
 				}
 			} else {
-				LOG_ERROR("Invalid option index %d.\n", index);
+				LOG_ERROR("Option index %d is not valid.\n", index);
 				rc = EINVAL;
 			}
 		} else {
@@ -628,7 +624,7 @@ static inline void copy_value (task_context_t *task, u_int32_t index, value_t *v
 		value_list_t *list;
 
 		list = task->option_descriptors->option[index].value.list;
-		size = sizeof (value_list_t) + ((list->count - 1) * sizeof (value_t));
+		size = sizeof (value_list_t) + (list->count * sizeof (value_t));
 
 		if (task->option_descriptors->option[index].type == EVMS_Type_String) {
 			int i;
