@@ -312,7 +312,7 @@ static int msg_user_msg(talk_t * talk) {
 	const char * node_name = nodeid_to_string(&talk->hear.node);
 	char * message_text;
 	char * new_message_text;
-	int * answer;
+	int  * answer;
 	char * * choices = NULL;
 	u_int32_t response[2];
 
@@ -361,7 +361,7 @@ static int msg_user_msg(talk_t * talk) {
 		}
 
 		if (rc == 0) {
-			if ((&answer != NULL) && (choices != NULL)) {
+			if ((answer != NULL) && (choices != NULL)) {
 				LOG_CRITICAL("Answer is: \"%s\"\n", choices[*answer]);
 			}
 		}
@@ -378,6 +378,9 @@ static int msg_user_msg(talk_t * talk) {
 
 		talk->say.size = sizeof(response);
 
+		evms_free(answer);
+		evms_free(choices);
+
 	} else {
 		evms_host_to_net(response, int_f,
 				 rc);
@@ -390,8 +393,6 @@ static int msg_user_msg(talk_t * talk) {
 
 	rc = say(talk);
 
-	engine_free(choices);
-
 	LOG_PROC_EXIT_INT(rc);
 	return rc;
 }
@@ -414,6 +415,13 @@ static int msg_progress(talk_t * talk) {
 			      &progress);
 
 	if (rc == 0) {
+		/*
+		 * The time estimate is calculated on the other node.
+		 * Don't calculate one here.
+		 */
+		progress->timer_data = NULL;
+		progress->flags |= PROGRESS_NO_TIME_ESTIMATE;
+
 		rc = plugin_progress(progress);
 
 		evms_host_to_net(response, user_progress_rets_f,
@@ -423,6 +431,8 @@ static int msg_progress(talk_t * talk) {
 
 		talk->say.size = sizeof(response);
 
+		evms_free(progress);
+
 	} else {
 		evms_host_to_net(response, int_f,
 				 rc);
@@ -435,8 +445,6 @@ static int msg_progress(talk_t * talk) {
 
 	rc = say(talk);
 
-	engine_free(progress);
-
 	LOG_PROC_EXIT_INT(rc);
 	return rc;
 }
@@ -562,38 +570,6 @@ static void * transact_message(ece_nodeid_t * nodeid,
 }
 
 
-static int make_user_ha(handle_array_t *   my_ha,
-			handle_array_t * * user_ha) {
-
-	int rc = 0;
-	handle_array_t * ha;
-	uint size;
-
-	LOG_PROC_ENTRY();
-
-	if (my_ha == NULL) {
-		*user_ha = NULL;
-		return rc;
-	}
-
-	size = sizeof(handle_array_t) + (my_ha->count - 1) * sizeof(object_handle_t);
-
-	ha = alloc_app_struct(size, NULL);
-
-	if (ha != NULL) {
-		memcpy(ha, my_ha, size);
-		*user_ha = ha;
-
-	} else {
-		LOG_CRITICAL("Error getting memory for a user handle array.\n");
-		rc = ENOMEM;
-	}
-
-	LOG_PROC_EXIT_INT(rc);
-	return rc;
-}
-
-
 static int one_handle_arg(msg_cmd_t cmd, object_handle_t thing) {
 
 	int rc;
@@ -747,7 +723,7 @@ int remote_verify_version(void) {
 		struct timezone tz;
 
 		gettimeofday(&now, &tz);
-		timeout.tv_sec = now.tv_sec + REQUEST_TIMEOUT;
+		timeout.tv_sec = now.tv_sec + REQUEST_TIMEOUT + 10;
 		timeout.tv_nsec = 0;
 		pthread_cond_timedwait(&cond, &mutex, &timeout);
 	}
@@ -773,13 +749,15 @@ int remote_verify_version(void) {
 
 			if (tmp_rc != 0) {
 				if (tmp_rc < 0) {
-					engine_user_message(NULL, NULL, "The Engine on node %s is running daemon protocol version (%d.%d.%d) which is greater than this Engine's daemon protocol version (%d.%d.%d).\n",
+					engine_user_message(NULL, NULL,
+							    _("The Engine on node %s is running daemon protocol version (%d.%d.%d) which is greater than this Engine's daemon protocol version (%d.%d.%d).\n"),
 							    nodeid_to_string(&talk->say.node),
 							    version->major, version->minor, version->patchlevel,
 							    engine_daemon_msg_version.major, engine_daemon_msg_version.minor, engine_daemon_msg_version.patchlevel);
 
 				} else {
-					engine_user_message(NULL, NULL, "The Engine on node %s is running daemon protocol version (%d.%d.%d) which is less than this Engine's daemon protocol version (%d.%d.%d).\n",
+					engine_user_message(NULL, NULL,
+							    _("The Engine on node %s is running daemon protocol version (%d.%d.%d) which is less than this Engine's daemon protocol version (%d.%d.%d).\n"),
 							    nodeid_to_string(&talk->say.node),
 							    version->major, version->minor, version->patchlevel,
 							    engine_daemon_msg_version.major, engine_daemon_msg_version.minor, engine_daemon_msg_version.patchlevel);
@@ -790,8 +768,8 @@ int remote_verify_version(void) {
 
 		} else {
 			engine_user_message(NULL, NULL,
-					    "There was an error when getting the version of the daemon protocol on node %s.  "
-					    "The error code was %d: %s.\n",
+					    _("There was an error when getting the version of the daemon protocol on node %s.  "
+					      "The error code was %d: %s.\n"),
 					    nodeid_to_string(&talk->say.node), tmp_rc, evms_strerror(tmp_rc));
 		}
 
@@ -823,7 +801,7 @@ int remote_verify_version(void) {
 			struct timezone tz;
 
 			gettimeofday(&now, &tz);
-			timeout.tv_sec = now.tv_sec + REQUEST_TIMEOUT;
+			timeout.tv_sec = now.tv_sec + REQUEST_TIMEOUT + 10;
 			timeout.tv_nsec = 0;
 			rc = pthread_cond_timedwait(&cond, &mutex, &timeout);
 		}
@@ -850,13 +828,15 @@ int remote_verify_version(void) {
 
 				if (tmp_rc != 0) {
 					if (tmp_rc < 0) {
-						engine_user_message(NULL, NULL, "The Engine on node %s is at version (%d.%d.%d) which is greater than this Engine's version (%d.%d.%d).\n",
+						engine_user_message(NULL, NULL,
+								    _("The Engine on node %s is at version (%d.%d.%d) which is greater than this Engine's version (%d.%d.%d).\n"),
 								    nodeid_to_string(&talk->say.node),
 								    version->major, version->minor, version->patchlevel,
 								    engine_version.major, engine_version.minor, engine_version.patchlevel);
 
 					} else {
-						engine_user_message(NULL, NULL, "The Engine on node %s is at version (%d.%d.%d) which is less than this Engine's version (%d.%d.%d).\n",
+						engine_user_message(NULL, NULL,
+								    _("The Engine on node %s is at version (%d.%d.%d) which is less than this Engine's version (%d.%d.%d).\n"),
 								    nodeid_to_string(&talk->say.node),
 								    version->major, version->minor, version->patchlevel,
 								    engine_version.major, engine_version.minor, engine_version.patchlevel);
@@ -867,8 +847,8 @@ int remote_verify_version(void) {
 
 			} else {
 				engine_user_message(NULL, NULL,
-						    "There was an error when getting the version of the daemon protocol on node %s.  "
-						    "The error code was %d: %s.\n",
+						    _("There was an error when getting the version of the daemon protocol on node %s.  "
+						      "The error code was %d: %s.\n"),
 						    nodeid_to_string(&talk->say.node), tmp_rc, evms_strerror(tmp_rc));
 			}
 
@@ -1000,6 +980,12 @@ int remote_open_engine(char           * node_name,
 
 	LOG_PROC_ENTRY();
 
+	if (membership == NULL) {
+		LOG_DETAILS("The is no membership available for opening the Engine on the other nodes.\n");
+		LOG_PROC_EXIT_INT(0);
+		return 0;
+	}
+
 	status_message("Starting EVMS on the other nodes in the cluster...\n");
 
 	rc = evms_sizeof_host_to_net(&arg_size, evms_open_engine_args_f,	
@@ -1089,7 +1075,7 @@ int remote_open_engine(char           * node_name,
 		struct timezone tz;
 
 		gettimeofday(&now, &tz);
-		timeout.tv_sec = now.tv_sec + REQUEST_TIMEOUT;
+		timeout.tv_sec = now.tv_sec + REQUEST_TIMEOUT + 10;
 		timeout.tv_nsec = 0;
 		rc = pthread_cond_timedwait(&cond, &mutex, &timeout);
 		if (rc != 0) {
@@ -1185,20 +1171,9 @@ int remote_get_plugin_list(evms_plugin_code_t    type,
 	response = transact_message(current_nodeid, EVMS_GET_PLUGIN_LIST, arg_size, net_args, &rc);
 
 	if (rc == 0) {
-		handle_array_t * ha;
-
 		evms_net_to_host(response, evms_get_plugin_list_rets_f,
 				 &rc,
-				 &ha);
-
-		if (rc == 0) {
-			rc = make_user_ha(ha, plugin_handle_list);
-
-		} else {
-			*plugin_handle_list = NULL;
-		}
-
-		engine_free(ha);
+				 plugin_handle_list);
 	}
 
 	engine_free(response);
@@ -1273,36 +1248,20 @@ int remote_changes_pending(boolean                 * result,
 	response = transact_message(current_nodeid, EVMS_CHANGES_PENDING, 0, NULL, &rc);
 
 	if (rc == 0) {
-		change_record_array_t * cra;
+		change_record_array_t * changes;
 
 		evms_net_to_host(response, evms_changes_pending_rets_f,
 				 &rc,
 				 result,
-				 &cra);
+				 &changes);
 
 		if (p_changes != NULL) {
-			if ((rc == 0) && (cra != NULL)) {
-				change_record_array_t * user_cra;
-				uint size = sizeof(change_record_array_t) + sizeof(change_record_t) * cra->count;
+			*p_changes = changes;
 
-				user_cra = alloc_app_struct(size, free_changes_pending_record_array_contents);
-
-				if (user_cra != NULL) {
-					memcpy(user_cra, cra, size);
-					*p_changes = user_cra;
-
-				} else {
-					LOG_CRITICAL("Error getting memory for a handle_object_info_t.\n");
-					rc = ENOMEM;
-					*p_changes = NULL;
-				}
-
-			} else {
-				*p_changes = NULL;
-			}
+		} else {
+			evms_free(changes);
 		}
 
-		engine_free(cra);
 	}
 
 	engine_free(response);
@@ -1372,6 +1331,12 @@ int remote_set_debug_level(debug_level_t level) {
 
 	LOG_PROC_ENTRY();
 
+	if (membership == NULL) {
+		LOG_DETAILS("The is no membership available for setting the debug level on the other nodes.\n");
+		LOG_PROC_EXIT_INT(0);
+		return 0;
+	}
+
 	evms_host_to_net(&net_debug_level, evms_set_debug_level_args_f,
 			 level);
 
@@ -1427,7 +1392,7 @@ int remote_set_debug_level(debug_level_t level) {
 		struct timezone tz;
 
 		gettimeofday(&now, &tz);
-		timeout.tv_sec = now.tv_sec + REQUEST_TIMEOUT;
+		timeout.tv_sec = now.tv_sec + REQUEST_TIMEOUT + 10;
 		timeout.tv_nsec = 0;
 		pthread_cond_timedwait(&cond, &mutex, &timeout);
 	}
@@ -1582,6 +1547,12 @@ int remote_close_engine() {
 
 	LOG_PROC_ENTRY();
 
+	if (membership == NULL) {
+		LOG_DETAILS("The is no membership available for closing the Engine on the other nodes.\n");
+		LOG_PROC_EXIT_INT(0);
+		return 0;
+	}
+
 	response_count = membership->num_entries - 1;
 
 	for (i = 0; (i < membership->num_entries) && (rc == 0); i++) {
@@ -1641,7 +1612,7 @@ int remote_close_engine() {
 		struct timezone tz;
 
 		gettimeofday(&now, &tz);
-		timeout.tv_sec = now.tv_sec + REQUEST_TIMEOUT;
+		timeout.tv_sec = now.tv_sec + REQUEST_TIMEOUT + 10;
 		timeout.tv_nsec = 0;
 		rc = pthread_cond_timedwait(&cond, &mutex, &timeout);
 		if (rc != 0) {
@@ -1826,31 +1797,9 @@ int remote_get_expand_points(object_handle_t           thing,
 	response = transact_message(current_nodeid, EVMS_GET_EXPAND_POINTS, sizeof(net_thing), &net_thing, &rc);
 
 	if (rc == 0) {
-		uint size;
-		expand_handle_array_t * eha;
-
                 evms_net_to_host(response, evms_get_expand_points_rets_f,
 				 &rc,
-				 &eha);
-
-		if (rc == 0) {
-			expand_handle_array_t * user_eha;
-
-			if (eha->count > 1) {
-				size = sizeof(expand_handle_array_t) + ((eha->count -1) * sizeof(expand_handle_t));
-			} else {
-				size = sizeof(expand_handle_array_t);
-			}
-
-			user_eha = alloc_app_struct(size, NULL);
-			if (user_eha != NULL) {
-				memcpy(user_eha, eha, size);
-				*expand_points = user_eha;
-
-			} else {
-				rc = ENOMEM;
-			}
-		}
+				 expand_points);
 	}
 
 	engine_free(response);
@@ -1923,31 +1872,9 @@ int remote_get_shrink_points(object_handle_t           thing,
 	response = transact_message(current_nodeid, EVMS_GET_SHRINK_POINTS, sizeof(net_thing), &net_thing, &rc);
 
 	if (rc == 0) {
-		uint size;
-		shrink_handle_array_t * sha;
-
 		evms_net_to_host(response, evms_get_shrink_points_rets_f,
 				 &rc,
-				 &sha);
-
-		if (rc == 0) {
-			shrink_handle_array_t * user_sha;
-
-			if (sha->count > 1) {
-				size = sizeof(shrink_handle_array_t) + ((sha->count -1) * sizeof(shrink_handle_t));
-			} else {
-				size = sizeof(shrink_handle_array_t);
-			}
-
-			user_sha = alloc_app_struct(size, NULL);
-			if (user_sha != NULL) {
-				memcpy(user_sha, sha, size);
-				*shrink_points = user_sha;
-
-			} else {
-				rc = ENOMEM;
-			}
-		}
+				 shrink_points);
 	}
 
 	engine_free(response);
@@ -2057,31 +1984,9 @@ int remote_get_info(object_handle_t          thing,
 	response = transact_message(current_nodeid, EVMS_GET_INFO, sizeof(net_thing), &net_thing, &rc);
 
 	if (rc == 0) {
-		handle_object_info_t * hoi;
-
 		evms_net_to_host(response, evms_get_info_rets_f,
 				 &rc,
-				 &hoi);
-
-		if (rc == 0) {
-			handle_object_info_t * user_hoi;
-
-			user_hoi = alloc_app_struct(sizeof(handle_object_info_t), free_info_object_contents);
-
-			if (user_hoi != NULL) {
-				memcpy(user_hoi, hoi, sizeof(handle_object_info_t));
-				*user_info = user_hoi;
-
-			} else {
-				LOG_CRITICAL("Error getting memory for a handle_object_info_t.\n");
-				rc = ENOMEM;
-			}
-
-		} else {
-			*user_info = NULL;
-		}
-
-		engine_free(hoi);
+				 user_info);
 	}
 
 	engine_free(response);
@@ -2126,32 +2031,9 @@ int remote_get_extended_info(object_handle_t           thing,
 	response = transact_message(current_nodeid, EVMS_GET_EXTENDED_INFO, arg_size, net_args, &rc);
 
 	if (rc == 0) {
-		extended_info_array_t * eia;
-
 		evms_net_to_host(response, evms_get_extended_info_rets_f,
 				 &rc,
-				 &eia);
-
-		if (rc == 0) {
-			extended_info_array_t * user_eia;
-			uint size = sizeof(extended_info_array_t) + (eia->count - 1) * sizeof(extended_info_t);
-
-			user_eia = alloc_app_struct(size, free_info_object_contents);
-
-			if (user_eia != NULL) {
-				memcpy(user_eia, eia, size);
-				*info = user_eia;
-
-			} else {
-				LOG_CRITICAL("Error getting memory for a handle_object_info_t.\n");
-				rc = ENOMEM;
-			}
-
-		} else {
-			*info = NULL;
-		}
-
-		engine_free(eia);
+				 info);
 	}
 
 	engine_free(response);
@@ -2271,20 +2153,9 @@ int remote_get_feature_list(object_handle_t    thing,
 	response = transact_message(current_nodeid, EVMS_GET_FEATURE_LIST, sizeof(net_thing), &net_thing, &rc);
 
 	if (rc == 0) {
-		handle_array_t * ha;
-
 		evms_net_to_host(response, evms_get_feature_list_rets_f,
 				 &rc,
-				 &ha);
-
-		if (rc == 0) {
-			rc = make_user_ha(ha, plugin_list);
-
-		} else {
-			*plugin_list = NULL;
-		}
-
-		engine_free(ha);
+				 plugin_list);
 	}
 
 	engine_free(response);
@@ -2337,20 +2208,9 @@ int remote_create(plugin_handle_t    plugin,
 	response = transact_message(current_nodeid, EVMS_CREATE, arg_size, net_args, &rc);
 
 	if (rc == 0) {
-		handle_array_t * ha;
-
 		evms_net_to_host(response, evms_create_rets_f,
 				 &rc,
-				 &ha);
-
-		if (rc == 0) {
-			rc = make_user_ha(ha, output_objects);
-
-		} else {
-			*output_objects = NULL;
-		}
-
-		engine_free(ha);
+				 output_objects);
 	}
 
 	engine_free(response);
@@ -2362,54 +2222,6 @@ int remote_create(plugin_handle_t    plugin,
 }
 
 
-int remote_transfer(object_handle_t  object,
-		    plugin_handle_t  plugin,
-		    object_handle_t  container,
-		    option_array_t * options) {
-	int rc = 0;
-	size_t arg_size;
-	void * net_args;
-	void * response;
-
-	LOG_PROC_ENTRY();
-
-	rc = evms_sizeof_host_to_net(&arg_size, evms_transfer_args_f,	
-				     object,
-				     plugin,
-				     container,
-				     options);
-
-	if (rc != 0) {
-		LOG_SERIOUS("evms_sizeof_host_to_net() returned error code %d: %s\n", rc, evms_strerror(rc));
-		LOG_PROC_EXIT_INT(rc);
-		return rc;
-	}
-
-	net_args = engine_alloc(arg_size);
-
-	if (net_args == NULL) {
-		LOG_CRITICAL("Error getting memory for net args.\n");
-		LOG_PROC_EXIT_INT(ENOMEM);
-		return ENOMEM;
-	}
-
-	evms_host_to_net(net_args, evms_transfer_args_f,
-			 object,
-			 plugin,
-			 container,
-			 options);
-
-	response = transact_message(current_nodeid, EVMS_TRANSFER, arg_size, net_args, &rc);
-
-	engine_free(response);
-
-	engine_free(net_args);
-
-	LOG_PROC_EXIT_INT(rc);
-	return rc;
-}
-
-
 int remote_assign(object_handle_t  object,
 		  plugin_handle_t  plugin,
 		  option_array_t * options) {
@@ -2527,20 +2339,9 @@ int remote_get_object_list(object_type_t         object_type,
 	response = transact_message(current_nodeid, EVMS_GET_OBJECT_LIST, arg_size, net_args, &rc);
 
 	if (rc == 0) {
-		handle_array_t * ha;
-
 		evms_net_to_host(response, evms_get_object_list_rets_f,
 				 &rc,
-				 &ha);
-
-		if (rc == 0) {
-			rc = make_user_ha(ha, object_handle_list);
-
-		} else {
-			*object_handle_list = NULL;
-		}
-
-		engine_free(ha);
+				 object_handle_list);
 	}
 
 	engine_free(response);
@@ -2566,37 +2367,9 @@ int remote_get_plugin_functions(engine_handle_t           thing,
 	response = transact_message(current_nodeid, EVMS_GET_PLUGIN_FUNCTIONS, sizeof(net_thing), &net_thing, &rc);
 
 	if (rc == 0) {
-		function_info_array_t * fia;
-
 		evms_net_to_host(response, evms_get_plugin_functions_rets_f,
 				 &rc,
-				 &fia);
-
-		if (rc == 0) {
-			uint size = sizeof(function_info_array_t);
-			function_info_array_t * user_fia;
-
-			if (fia->count > 1) {
-				size += (fia->count - 1) * sizeof(function_info_t);
-			}
-
-			user_fia = alloc_app_struct(size, free_function_info_array_contents);
-
-			if (user_fia != NULL) {
-				/* We do not copy the strings, just the pointers to the strings. */
-				memcpy(user_fia, fia, size);
-				*actions = user_fia;
-
-			} else {
-				rc = ENOMEM;
-				*actions = NULL;
-			}
-
-		} else {
-			*actions = NULL;
-		}
-
-		engine_free(fia);
+				 actions);
 	}
 
 	engine_free(response);
@@ -2976,19 +2749,6 @@ int remote_can_fsck(object_handle_t volume) {
 }
 
 
-int remote_can_defrag(object_handle_t volume) {
-
-	int rc;
-
-	LOG_PROC_ENTRY();
-
-	rc = one_handle_arg(EVMS_CAN_DEFRAG, volume);
-
-	LOG_PROC_EXIT_INT(rc);
-	return rc;			
-}
-
-
 int remote_mkfs(object_handle_t  volume,
 		plugin_handle_t  fsim,
 		option_array_t * options) {
@@ -3091,49 +2851,6 @@ int remote_fsck(object_handle_t  volume,
 }
 
 
-int remote_defrag(object_handle_t  volume,
-		  option_array_t * options) {
-
-	int rc = 0;
-	size_t arg_size;
-	void * net_args;
-	void * response;
-
-	LOG_PROC_ENTRY();
-
-	rc = evms_sizeof_host_to_net(&arg_size, evms_defrag_args_f,	
-				     volume,
-				     options);
-
-	if (rc != 0) {
-		LOG_SERIOUS("evms_sizeof_host_to_net() returned error code %d: %s\n", rc, evms_strerror(rc));
-		LOG_PROC_EXIT_INT(rc);
-		return rc;
-	}
-
-	net_args = engine_alloc(arg_size);
-
-	if (net_args == NULL) {
-		LOG_CRITICAL("Error getting memory for net args.\n");
-		LOG_PROC_EXIT_INT(ENOMEM);
-		return ENOMEM;
-	}
-
-	evms_host_to_net(net_args, evms_defrag_args_f,
-			 volume,
-			 options);
-
-	response = transact_message(current_nodeid, EVMS_DEFRAG, arg_size, net_args, &rc);
-
-	engine_free(response);
-
-	engine_free(net_args);
-
-	LOG_PROC_EXIT_INT(rc);
-	return rc;
-}
-
-
 int remote_get_volume_list(object_handle_t       fsim_handle,
 			   object_handle_t       disk_group_handle,
 			   volume_search_flags_t flags,
@@ -3173,20 +2890,9 @@ int remote_get_volume_list(object_handle_t       fsim_handle,
 	response = transact_message(current_nodeid, EVMS_GET_VOLUME_LIST, arg_size, net_args, &rc);
 
 	if (rc == 0) {
-		handle_array_t * ha;
-
 		evms_net_to_host(response, evms_get_volume_list_rets_f,
 				 &rc,
-				 &ha);
-
-		if (rc == 0) {
-			rc = make_user_ha(ha, volume_handle_list);
-
-		} else {
-			*volume_handle_list = NULL;
-		}
-
-		engine_free(ha);
+				 volume_handle_list);
 	}
 
 	engine_free(response);
@@ -3343,32 +3049,6 @@ int remote_remount(object_handle_t volume,
 * Storage Container functions *
 ******************************/
 
-int remote_can_add_to_container(object_handle_t object,
-				object_handle_t container) {
-	int rc;
-
-	LOG_PROC_ENTRY();
-
-	rc = two_handle_arg(EVMS_CAN_ADD_TO_CONTAINER, object, container);
-
-	LOG_PROC_EXIT_INT(rc);
-	return rc;
-}
-
-
-int remote_can_remove_from_container(object_handle_t object) {
-	
-	int rc;
-
-	LOG_PROC_ENTRY();
-
-	rc = one_handle_arg(EVMS_CAN_REMOVE_FROM_CONTAINER, object);
-
-	LOG_PROC_EXIT_INT(rc);
-	return rc;
-}
-
-
 int remote_create_container(plugin_handle_t   plugin,
 			    handle_array_t  * input_objects,
 			    option_array_t  * options,
@@ -3460,20 +3140,9 @@ int remote_get_container_list(object_handle_t          plugin_handle,
 	response = transact_message(current_nodeid, EVMS_GET_CONTAINER_LIST, arg_size, net_args, &rc);
 
 	if (rc == 0) {
-		handle_array_t * ha;
-
 		evms_net_to_host(response, evms_get_container_list_rets_f,
 				 &rc,
-				 &ha);
-
-		if (rc == 0) {
-			rc = make_user_ha(ha, container_handle_list);
-
-		} else {
-			*container_handle_list = NULL;
-		}
-
-		engine_free(ha);
+				 container_handle_list);
 	}
 
 	engine_free(response);
@@ -3553,20 +3222,9 @@ int remote_invoke_task(task_handle_t      task,
 	response = transact_message(current_nodeid, EVMS_INVOKE_TASK, sizeof(net_task), &net_task, &rc);
 
 	if (rc == 0) {
-		handle_array_t * ha;
-
 		evms_net_to_host(response, evms_invoke_task_rets_f,
 				 &rc,
-				 &ha);
-
-		if (rc == 0) {
-			rc = make_user_ha(ha, resulting_objects);
-
-		} else {
-			*resulting_objects = NULL;
-		}
-
-		engine_free(ha);
+				 resulting_objects);
 	}
 
 	engine_free(response);
@@ -3635,20 +3293,9 @@ int remote_get_acceptable_objects(task_handle_t      task,
 	response = transact_message(current_nodeid, EVMS_GET_ACCEPTABLE_OBJECTS, sizeof(net_task), &net_task, &rc);
 
 	if (rc == 0) {
-		handle_array_t * ha;
-
 		evms_net_to_host(response, evms_get_acceptable_objects_rets_f,
 				 &rc,
-				 &ha);
-
-		if (rc == 0) {
-			rc = make_user_ha(ha, acceptable_object_list);
-
-		} else {
-			*acceptable_object_list = NULL;
-		}
-
-		engine_free(ha);
+				 acceptable_object_list);
 	}
 
 	engine_free(response);
@@ -3673,20 +3320,9 @@ int remote_get_selected_objects(task_handle_t      task,
 	response = transact_message(current_nodeid, EVMS_GET_SELECTED_OBJECTS, sizeof(net_task), &net_task, &rc);
 
 	if (rc == 0) {
-		handle_array_t * ha;
-
 		evms_net_to_host(response, evms_get_selected_objects_rets_f,
 				 &rc,
-				 &ha);
-
-		if (rc == 0) {
-			rc = make_user_ha(ha, selected_object_list);
-
-		} else {
-			*selected_object_list = NULL;
-		}
-
-		engine_free(ha);
+				 selected_object_list);
 	}
 
 	engine_free(response);
@@ -3770,31 +3406,15 @@ int remote_set_selected_objects(task_handle_t               task,
 				 &eff);
 
 		if (declined_list != NULL) {
-			if ((rc == 0) && (dha != NULL)) {
-				uint size = sizeof(declined_handle_array_t) + (dha->count - 1) * sizeof(declined_handle_t);
-				declined_handle_array_t * user_dha;
+			*declined_list = dha;
 
-				user_dha = alloc_app_struct(size, NULL);
-
-				if (user_dha != NULL) {
-					memcpy(user_dha, dha, size);
-					*declined_list = user_dha;
-
-				} else {
-					LOG_CRITICAL("Error getting memory for a user handle array.\n");
-					rc = ENOMEM;
-				}
-
-			} else {
-				*declined_list = NULL;
-			}
+		} else {
+			evms_free(dha);
 		}
 
 		if (effect != NULL) {
 		    *effect = eff;
 		}
-
-		engine_free(dha);
 	}
 
 	engine_free(response);
@@ -3873,32 +3493,9 @@ int remote_get_option_descriptor(task_handle_t           task,
 	response = transact_message(current_nodeid, EVMS_GET_OPTION_DESCRIPTOR, arg_size, net_args, &rc);
 
 	if (rc == 0) {
-		option_descriptor_t * od;
-
 		evms_net_to_host(response, evms_get_option_descriptor_rets_f,
 				 &rc,
-				 &od);
-
-		if (rc == 0) {
-			option_descriptor_t * user_od;
-
-			user_od = alloc_app_struct(sizeof(option_descriptor_t), free_option_descriptor_contents);
-
-			if (user_od != NULL) {
-				memcpy(user_od, od, sizeof(option_descriptor_t));
-				*descriptor = user_od;
-
-			} else {
-				LOG_CRITICAL("Error getting memory for a user handle array.\n");
-				rc = ENOMEM;
-			}
-
-
-		} else {
-			*descriptor = NULL;
-		}
-
-		engine_free(od);
+				 descriptor);
 	}
 
 	engine_free(response);
@@ -4025,32 +3622,9 @@ int remote_get_option_descriptor_by_name(task_handle_t           task,
 	response = transact_message(current_nodeid, EVMS_GET_OPTION_DESCRIPTOR_BY_NAME, arg_size, net_args, &rc);
 
 	if (rc == 0) {
-		option_descriptor_t * od;
-
 		evms_net_to_host(response, evms_get_option_descriptor_by_name_rets_f,
 				 &rc,
-				 &od);
-
-		if (rc == 0) {
-			option_descriptor_t * user_od;
-
-			user_od = alloc_app_struct(sizeof(option_descriptor_t), free_option_descriptor_contents);
-
-			if (user_od != NULL) {
-				memcpy(user_od, od, sizeof(option_descriptor_t));
-				*descriptor = user_od;
-
-			} else {
-				LOG_CRITICAL("Error getting memory for a user handle array.\n");
-				rc = ENOMEM;
-			}
-
-
-		} else {
-			*descriptor = NULL;
-		}
-
-		engine_free(od);
+				 descriptor);
 	}
 
 	engine_free(response);
@@ -4204,6 +3778,12 @@ int remote_mark_for_rediscover(char * name) {
 
 	LOG_PROC_ENTRY();
 
+	if (membership == NULL) {
+		LOG_DETAILS("The is no membership available for marking for rediscover on the other nodes.\n");
+		LOG_PROC_EXIT_INT(0);
+		return 0;
+	}
+
 	rc = evms_sizeof_host_to_net(&arg_size, evms_mark_for_rediscover_args_f,	
 				     name);
 
@@ -4276,7 +3856,7 @@ int remote_mark_for_rediscover(char * name) {
 		struct timezone tz;
 
 		gettimeofday(&now, &tz);
-		timeout.tv_sec = now.tv_sec + REQUEST_TIMEOUT;
+		timeout.tv_sec = now.tv_sec + REQUEST_TIMEOUT + 10;
 		timeout.tv_nsec = 0;
 		rc = pthread_cond_timedwait(&cond, &mutex, &timeout);
 	}
@@ -4419,6 +3999,12 @@ int remote_rediscover(void) {
 
 	LOG_PROC_ENTRY();
 
+	if (membership == NULL) {
+		LOG_DETAILS("The is no membership available for rediscovering on the other nodes.\n");
+		LOG_PROC_EXIT_INT(0);
+		return 0;
+	}
+
 	response_count = membership->num_entries - 1;
 
 	for (i = 0; (i < membership->num_entries) && (rc == 0); i++) {
@@ -4477,7 +4063,7 @@ int remote_rediscover(void) {
 		struct timezone tz;
 
 		gettimeofday(&now, &tz);
-		timeout.tv_sec = now.tv_sec + REQUEST_TIMEOUT;
+		timeout.tv_sec = now.tv_sec + REQUEST_TIMEOUT + 10;
 		timeout.tv_nsec = 0;
 		rc = pthread_cond_timedwait(&cond, &mutex, &timeout);
 		if (rc != 0) {
@@ -4639,6 +4225,12 @@ int remote_shutdown() {
 
 	LOG_PROC_ENTRY();
 
+	if (membership == NULL) {
+		LOG_DETAILS("The is no membership available for shutting down on the other nodes.\n");
+		LOG_PROC_EXIT_INT(0);
+		return 0;
+	}
+
 	response_count = membership->num_entries - 1;
 
 	for (i = 0; (i < membership->num_entries) && (rc == 0); i++) {
@@ -4698,7 +4290,7 @@ int remote_shutdown() {
 		struct timezone tz;
 
 		gettimeofday(&now, &tz);
-		timeout.tv_sec = now.tv_sec + REQUEST_TIMEOUT;
+		timeout.tv_sec = now.tv_sec + REQUEST_TIMEOUT + 10;
 		timeout.tv_nsec = 0;
 		rc = pthread_cond_timedwait(&cond, &mutex, &timeout);
 		if (rc != 0) {
@@ -4753,6 +4345,231 @@ int remote_shutdown() {
 }
 
 
+int remote_can_activate(object_handle_t thing) {
+
+	int rc;
+
+	LOG_PROC_ENTRY();
+
+	rc = one_handle_arg(EVMS_CAN_ACTIVATE, thing);
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;			
+}
+
+
+int remote_activate(object_handle_t thing) {
+
+	int rc;
+
+	LOG_PROC_ENTRY();
+
+	rc = one_handle_arg(EVMS_ACTIVATE, thing);
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;			
+}
+
+
+int remote_can_deactivate(object_handle_t thing) {
+
+	int rc;
+
+	LOG_PROC_ENTRY();
+
+	rc = one_handle_arg(EVMS_CAN_DEACTIVATE, thing);
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;			
+}
+
+
+int remote_deactivate(object_handle_t thing) {
+
+	int rc;
+
+	LOG_PROC_ENTRY();
+
+	rc = one_handle_arg(EVMS_DEACTIVATE, thing);
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;			
+}
+
+
+int remote_metadata_backup(const char * dir) {
+
+	int rc = 0;
+	size_t arg_size;
+	void * net_args;
+	void * response;
+
+	LOG_PROC_ENTRY();
+
+	rc = evms_sizeof_host_to_net(&arg_size, evms_metadata_backup_args_f,
+				     dir);
+
+	if (rc != 0) {
+		LOG_SERIOUS("evms_sizeof_host_to_net() returned error code %d: %s\n", rc, evms_strerror(rc));
+		LOG_PROC_EXIT_INT(rc);
+		return rc;
+	}
+
+	net_args = engine_alloc(arg_size);
+
+	if (net_args == NULL) {
+		LOG_CRITICAL("Error getting memory for net args.\n");
+		LOG_PROC_EXIT_INT(ENOMEM);
+		return ENOMEM;
+	}
+
+	evms_host_to_net(net_args, evms_metadata_backup_args_f,
+			 dir);
+
+	response = transact_message(current_nodeid, EVMS_METADATA_BACKUP, arg_size, net_args, &rc);
+
+	if (rc == 0) {
+		evms_net_to_host(response, evms_metadata_backup_rets_f,
+				 &rc);
+	}
+
+	engine_free(response);
+
+	engine_free(net_args);
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+
+int remote_engine_read(object_handle_t handle,
+		       lsn_t           lsn,
+		       sector_count_t  length,
+		       void          * buffer) {
+
+	int rc = 0;
+	size_t arg_size;
+	void * net_args;
+	void * response;
+	char rets_format[32];
+	void * read_buff;
+
+	LOG_PROC_ENTRY();
+
+	rc = evms_sizeof_host_to_net(&arg_size, evms_engine_read_args_f,
+				     handle,
+				     lsn,
+				     length);
+
+	if (rc != 0) {
+		LOG_SERIOUS("evms_sizeof_host_to_net() returned error code %d: %s\n", rc, evms_strerror(rc));
+		LOG_PROC_EXIT_INT(rc);
+		return rc;
+	}
+
+	net_args = engine_alloc(arg_size);
+
+	if (net_args == NULL) {
+		LOG_CRITICAL("Error getting memory for net args.\n");
+		LOG_PROC_EXIT_INT(ENOMEM);
+		return ENOMEM;
+	}
+
+	evms_host_to_net(net_args, evms_engine_read_args_f,
+			 handle,
+			 lsn,
+			 length);
+
+	response = transact_message(current_nodeid, EVMS_ENGINE_READ, arg_size, net_args, &rc);
+
+	if (rc == 0) {
+		sprintf(rets_format, "%sp{x%"PRIu64"}",
+			evms_engine_read_rets_f,
+			length << EVMS_VSECTOR_SIZE_SHIFT);
+
+		evms_net_to_host(response, rets_format,
+				 &rc,
+				 &read_buff);
+
+		if (read_buff != NULL) {
+			memcpy(buffer, read_buff, length << EVMS_VSECTOR_SIZE_SHIFT);
+
+			evms_free(read_buff);
+		}
+
+	} else {
+		evms_net_to_host(response, evms_engine_read_rets_f,
+				 &rc);
+	}
+
+	engine_free(response);
+
+	engine_free(net_args);
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+
+int remote_engine_write(object_handle_t handle,
+		       lsn_t           lsn,
+		       sector_count_t  length,
+		       void          * buffer) {
+
+	int rc = 0;
+	size_t arg_size;
+	void * net_args;
+	void * response;
+	char args_format[32];
+
+	LOG_PROC_ENTRY();
+
+	sprintf(args_format, "%sp{x%"PRIu64"}",
+		evms_engine_write_args_f,
+		length << EVMS_VSECTOR_SIZE_SHIFT);
+
+	rc = evms_sizeof_host_to_net(&arg_size, args_format,
+				     handle,
+				     lsn,
+				     length,
+				     buffer);
+
+	if (rc != 0) {
+		LOG_SERIOUS("evms_sizeof_host_to_net() returned error code %d: %s\n", rc, evms_strerror(rc));
+		LOG_PROC_EXIT_INT(rc);
+		return rc;
+	}
+
+	net_args = engine_alloc(arg_size);
+
+	if (net_args == NULL) {
+		LOG_CRITICAL("Error getting memory for net args.\n");
+		LOG_PROC_EXIT_INT(ENOMEM);
+		return ENOMEM;
+	}
+
+	evms_host_to_net(net_args, args_format,
+			 handle,
+			 lsn,
+			 length,
+			 buffer);
+
+	response = transact_message(current_nodeid, EVMS_ENGINE_WRITE, arg_size, net_args, &rc);
+
+	if (rc == 0) {
+		evms_net_to_host(response, evms_engine_write_rets_f,
+				 &rc);
+	}
+
+	engine_free(response);
+
+	engine_free(net_args);
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+
 void engine_router(ece_msg_t * msg) {
 
 	u_int32_t cmd = msg->cmd;
