@@ -25,6 +25,7 @@
 #include <string.h>
 #include <errno.h>
 #include <unistd.h>
+#include <csm_funcs.h>
 
 #include "fullengine.h"
 #include "shrink.h"
@@ -42,7 +43,9 @@
 #include "volume.h"
 
 
-int get_object_shrink_points(storage_object_t * obj, u_int64_t * max_delta_size, list_anchor_t shrink_points) {
+int get_object_shrink_points(storage_object_t * obj,
+			     u_int64_t          max_delta_size,
+			     list_anchor_t      shrink_points) {
 
 	int rc = 0;
 
@@ -58,7 +61,7 @@ int get_object_shrink_points(storage_object_t * obj, u_int64_t * max_delta_size,
 int get_volume_shrink_points(logical_volume_t * volume, list_anchor_t shrink_points) {
 
 	int rc = 0;
-	u_int64_t max_delta_size = -1;
+	u_int64_t max_delta_size = volume->vol_size;
 
 	LOG_PROC_ENTRY();
 
@@ -80,56 +83,12 @@ int get_volume_shrink_points(logical_volume_t * volume, list_anchor_t shrink_poi
 	if ((volume->original_fsim != NULL) &&
 	    !(volume->flags & VOLFLAG_UNMKFS)) {
 
-		/*
-		 * Reget the current size and limits because the file system could be
-		 * mounted and be active.
-		 */
-		rc = get_volume_sizes_and_limits(volume);
-
-		if (rc == 0) {
-			max_delta_size = volume->fs_size - volume->min_fs_size;
-		}
+		max_delta_size = volume->fs_size;
+                rc = volume->original_fsim->functions.fsim->can_shrink_by(volume, &max_delta_size);
 	}
 
 	if (rc == 0) {
-		rc = get_object_shrink_points(volume->object, &max_delta_size, shrink_points);
-	}
-
-	if (rc == 0) {
-		/*
-		 * If the volume has an FSIM and the FSIM is not scheduled to be removed
-		 * and there are shrink points, check if the FSIM can handle the shrink.
-		 */
-		if ((volume->original_fsim != NULL) &&
-		    !(volume->flags & VOLFLAG_UNMKFS) &&
-		    !list_empty(shrink_points)) {
-
-			max_delta_size = round_down_to_hard_sector(max_delta_size, volume->object);
-
-			rc = volume->original_fsim->functions.fsim->can_shrink_by(volume, &max_delta_size);
-
-			/*
-			 * The FSIM cannot handle the shrink.  Delete all the shrink
-			 * points from the list.
-			 */
-			if (rc != 0) {
-				/*
-				 * Can't use DeleteAllItems(shrink_ponts, TRUE)
-				 * because it uses free().  Shrink points are
-				 * allocted with engine_alloc() which may add a
-				 * debug header.  Extract each item and use
-				 * engine_free().
-				 */
-				list_element_t iter1;
-				list_element_t iter2;
-				void * thing;
-
-				LIST_FOR_EACH_SAFE(shrink_points, iter1, iter2, thing) {
-					delete_element(iter1);
-					engine_free(thing);
-				}
-			}
-		}
+		rc = get_object_shrink_points(volume->object, max_delta_size, shrink_points);
 	}
 
 	LOG_PROC_EXIT_INT(rc);
@@ -137,10 +96,24 @@ int get_volume_shrink_points(logical_volume_t * volume, list_anchor_t shrink_poi
 }
 
 
+int get_container_shrink_points(storage_container_t * con,
+				list_anchor_t         shrink_points) {
+
+	int rc = 0;
+
+	LOG_PROC_ENTRY();
+
+	rc = con->plugin->container_functions->can_shrink_container(con, shrink_points);
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+
 /*
- * Find out if an object can be shrunk.  The "thing" must be either a volume
- * or a top level object.  What we'll do is call the internal
- * get_shrink_points() and see if any shrink candidates show up.  If we get
+ * Find out if an object can be shrunk.  The "thing" must be either a volume,
+ * top level object, or container.  What we'll do is call the internal
+ * get_*_shrink_points() and see if any shrink candidates show up.  If we get
  * shrink points then return 0 to say that the object can be shrunk.  Else,
  * return an error.
  */
@@ -183,9 +156,7 @@ int evms_can_shrink(object_handle_t thing) {
 					 */
 					if (is_top_object(obj)) {
 						STATIC_LIST_DECL(shrink_points);
-						u_int64_t max_delta_size = obj->size;
-
-						rc = get_object_shrink_points(obj, &max_delta_size, &shrink_points);
+						rc = get_object_shrink_points(obj, obj->size, &shrink_points);
 
 						if (rc == 0) {
 							if (list_empty(&shrink_points)) {
@@ -230,6 +201,22 @@ int evms_can_shrink(object_handle_t thing) {
 				}
 				break;
 
+			case CONTAINER:
+				{
+					storage_container_t * con = object;
+
+					STATIC_LIST_DECL(shrink_points);
+
+					rc = get_container_shrink_points(con, &shrink_points);
+
+					if (rc == 0) {
+						if (list_empty(&shrink_points)) {
+							rc = ENOENT;
+						}
+					}
+				}
+				break;
+
 			default:
 				rc = EINVAL;
 				break;
@@ -258,11 +245,7 @@ static int make_shrink_handle_array(list_anchor_t list, shrink_handle_array_t *
 
 	if (rc == 0) {
 		LOG_DEBUG("Number of objects in the list:  %d\n", count);
-		if (count > 1) {
-			size = sizeof(shrink_handle_array_t) + ((count -1) * sizeof(shrink_handle_t));
-		} else {
-			size = sizeof(shrink_handle_array_t);
-		}
+		size = sizeof(shrink_handle_array_t) + (count * sizeof(shrink_handle_t));
 
 		sha = alloc_app_struct(size, NULL);
 		if (sha != NULL) {
@@ -271,11 +254,20 @@ static int make_shrink_handle_array(list_anchor_t list, shrink_handle_array_t *
 
 			LIST_FOR_EACH(list, iter, object_info) {
 
-				LOG_DEBUG("Add entry for storage object %s.\n", object_info->object->name);
+				if (object_info->object != NULL) {
+					LOG_DEBUG("Add entry for storage object %s.\n", object_info->object->name);
+					rc = ensure_app_handle(object_info->object);
+				} else if (object_info->container != NULL) {
+					LOG_DEBUG("Add entry for storage object %s.\n", object_info->container->name);
+					rc = ensure_app_handle(object_info->container);
+				}
 
-				rc = ensure_app_handle(object_info->object);
 				if (rc == 0) {
-					sha->shrink_point[sha->count].object = object_info->object->app_handle;
+					if (object_info->object != NULL) {
+						sha->shrink_point[sha->count].object = object_info->object->app_handle;
+					} else if (object_info->container != NULL) {
+						sha->shrink_point[sha->count].object = object_info->container->app_handle;
+					}
 					sha->shrink_point[sha->count].max_shrink_size = object_info->max_shrink_size;
 					sha->count++;
 				}
@@ -299,10 +291,10 @@ static int make_shrink_handle_array(list_anchor_t list, shrink_handle_array_t *
  * list.  The list is then converted into a shrink_handle_array_t for return
  * to the user.
  */
-int evms_get_shrink_points(object_handle_t thing, shrink_handle_array_t * * shrink_points) {
+int evms_get_shrink_points(object_handle_t thing_handle, shrink_handle_array_t * * shrink_points) {
 
 	int rc = 0;
-	void * object;
+	void * thing;
 	object_type_t type;
 	STATIC_LIST_DECL(shrink_point_list);
 
@@ -315,13 +307,13 @@ int evms_get_shrink_points(object_handle_t thing, shrink_handle_array_t * * shri
 	}
 
 	if (!local_focus) {
-		rc = remote_get_shrink_points(thing, shrink_points);
+		rc = remote_get_shrink_points(thing_handle, shrink_points);
 		LOG_PROC_EXIT_INT(rc);
 		return rc;
 	}
 
-	rc = translate_handle(thing,
-			      &object,
+	rc = translate_handle(thing_handle,
+			      &thing,
 			      &type);
 
 	if (rc != HANDLE_MANAGER_NO_ERROR) {
@@ -338,7 +330,7 @@ int evms_get_shrink_points(object_handle_t thing, shrink_handle_array_t * * shri
 	switch (type) {
 		case VOLUME:
 			{
-				logical_volume_t * volume = (logical_volume_t *) object;
+				logical_volume_t * volume = (logical_volume_t *) thing;
 
 
 				if (volume->flags & VOLFLAG_COMPATIBILITY) {
@@ -364,14 +356,12 @@ int evms_get_shrink_points(object_handle_t thing, shrink_handle_array_t * * shri
 		case SEGMENT:
 		case DISK:
 			{
-				storage_object_t * obj = (storage_object_t *) object;
+				storage_object_t * obj = (storage_object_t *) thing;
 				/*
 				 * Only top level objects can be shrunk.
 				 */
 				if (is_top_object(obj)) {
-					u_int64_t max_delta_size = obj->size;
-
-					rc = get_object_shrink_points(obj, &max_delta_size, &shrink_point_list);
+					rc = get_object_shrink_points(obj, obj->size, &shrink_point_list);
 
 				} else {
 					LOG_ERROR("Object %s is not a top level object.  Only volumes and top level objects can be shrunk.\n", obj->name);
@@ -380,6 +370,14 @@ int evms_get_shrink_points(object_handle_t thing, shrink_handle_array_t * * shri
 			}
 			break;
 
+		case CONTAINER:
+			{
+				storage_container_t * con = (storage_container_t *) thing;
+				
+				rc = get_container_shrink_points(con, &shrink_point_list);
+			}
+			break;
+
 		default:
 			LOG_ERROR("An object of type %d cannot be shrunk.\n", type);
 			rc = EINVAL;
@@ -413,10 +411,10 @@ static int warn_if_no_fsim(storage_object_t * obj) {
 			int answer = 0;	    /* Initialize to "Continue" */
 
 			engine_user_message(&answer, choices,
-					    "WARNING: Volume \"%s\" does not have an associated File System Interface Module which could coordinate the shrinking of the file system on the volume.  "
-					    "The file system will not be shrunk.  "
-					    "You may need to run a separate utility to shrink the file system before this operation starts.  "
-					    "Shrinking the volume may make the file system unusable.\n",
+					    _("WARNING: Volume \"%s\" does not have an associated File System Interface Module which could coordinate the shrinking of the file system on the volume.  "
+					      "The file system will not be shrunk.  "
+					      "You may need to run a separate utility to shrink the file system before this operation starts.  "
+					      "Shrinking the volume may make the file system unusable.\n"),
 					    obj->volume->name);
 
 			if (answer != 0) {
@@ -503,27 +501,26 @@ static void post_shrink_cleanup(list_anchor_t      input_object_list,
 		}
 	}
 
-	if (obj->object_type == EVMS_OBJECT) {
-		/*
-		 * Mark all the child feature headers dirty to ensure that they
-		 * get the correct depth and get written in the right place.
-		 */
-		LIST_FOR_EACH(obj->child_objects, iter, child_obj) {
-			mark_feature_headers_dirty(child_obj);
-		}
-	}
-
 	/* If the object is part of a volume... */
 	if (obj->volume != NULL) {
 
-		/*
-		 * Mark it dirty so that the feature headers get written to
-		 * their new locations.
-		 */
-		obj->volume->flags |= VOLFLAG_DIRTY;
-
+		/* An EVMS volume needs some adjusting after shrinking. */
 		if (!(obj->volume->flags & VOLFLAG_COMPATIBILITY)) {
-			obj->volume->flags |= VOLFLAG_NEEDS_ACTIVATE;
+
+			/*
+			 * Mark the feature headers dirty so that they get
+			 * written to their new locations.
+			 */
+			obj->volume->flags |= VOLFLAG_FEATURE_HEADER_DIRTY;
+
+			if (obj->volume->flags & VOLFLAG_HAS_OWN_DEVICE) {
+
+				/*
+				 * The volume device needs to be reactivated
+				 * with the new size.
+				 */
+				obj->volume->flags |= VOLFLAG_NEEDS_ACTIVATE;
+			}
 		}
 
 		/* Update the volume size. */
@@ -547,53 +544,15 @@ static void post_shrink_cleanup(list_anchor_t      input_object_list,
 }
 
 
-/*
- * Shrink the size of an object.  The shrink gets targeted at the top object of
- * the feature stack.  The plug-ins pass it on down until it hits the object
- * that is to do the real shrink.
- */
-int evms_shrink(object_handle_t thing, handle_array_t * input_objects, option_array_t * options) {
+int do_object_shrink(storage_object_t * obj,
+		     handle_array_t   * input_objects,
+		     option_array_t   * options) {
 
 	int rc = 0;
-	void * object;
-	object_type_t type;
-	storage_object_t * obj;
-	list_anchor_t input_object_list;
+	STATIC_LIST_DECL(input_object_list);
 
 	LOG_PROC_ENTRY();
 
-	rc = check_engine_write_access();
-        if (rc != 0) {
-		LOG_PROC_EXIT_INT(rc);
-		return rc;
-	}
-
-	if (!local_focus) {
-		rc = remote_shrink(thing, input_objects, options);
-		LOG_PROC_EXIT_INT(rc);
-		return rc;
-	}
-
-	rc = translate_handle(thing,
-			      &object,
-			      &type);
-
-	if (rc != HANDLE_MANAGER_NO_ERROR) {
-		LOG_PROC_EXIT_INT(rc);
-		return rc;
-	}
-
-	if ((type != EVMS_OBJECT) &&
-	    (type != REGION) &&
-	    (type != SEGMENT) &&
-	    (type != DISK)) {
-		LOG_ERROR("Object of type %d cannot be shrunk.\n", type);
-		LOG_PROC_EXIT_INT(EINVAL);
-		return EINVAL;
-	}
-
-	obj = (storage_object_t *) object;
-
 	if (obj->volume != NULL) {
 
 		if (obj->volume->flags & VOLFLAG_COMPATIBILITY) {
@@ -617,15 +576,7 @@ int evms_shrink(object_handle_t thing, handle_array_t * input_objects, option_ar
 		}
 	}
 
-	input_object_list = allocate_list();
-
-	if (input_object_list == NULL) {
-		LOG_CRITICAL("Error allocating memory to create the input object list.\n");
-		LOG_PROC_EXIT_INT(ENOMEM);
-		return ENOMEM;
-	}
-
-	rc = make_list(input_objects, input_object_list);
+	rc = make_list(input_objects, &input_object_list);
 
 	if (rc == 0) {
 		list_element_t iter;
@@ -636,7 +587,7 @@ int evms_shrink(object_handle_t thing, handle_array_t * input_objects, option_ar
 		 * (The caller could have supplied handles for bogus items such
 		 * as plug-ins or volumes.)
 		 */
-		LIST_FOR_EACH(input_object_list, iter, input_obj) {
+		LIST_FOR_EACH(&input_object_list, iter, input_obj) {
 			rc = isa_valid_shrink_input_object(input_obj, obj->disk_group);
 			if (rc != 0) {
 				break;
@@ -651,44 +602,61 @@ int evms_shrink(object_handle_t thing, handle_array_t * input_objects, option_ar
 
 			parent_count = list_count(top_object->parent_objects);
 
-			while (parent_count == 1) {
+			while ((parent_count == 1) && (top_object->consuming_container == NULL)) {
 				top_object = first_thing(top_object->parent_objects, NULL);
 				parent_count = list_count(top_object->parent_objects);
 			}
 
-			if (rc == 0) {
-				/*
-				 * Warn the user if there is no FSIM to handle
-				 * the shrink.
-				 */
-				rc = warn_if_no_fsim(obj);
+			if (top_object->consuming_container != NULL) {
+				storage_container_t * con = top_object->consuming_container;
 
-				if (rc == 0) {
-					/*
-					 * If the object is not part of a volume
-					 * and it has stop data, remove the stop
-					 * data from its current location.  It
-					 * will be reapplied during commit.
-					 */
-					if ((top_object->volume == NULL) &&
-					    (top_object->flags & SOFLAG_HAS_STOP_DATA)) {
-						top_object->plugin->functions.plugin->add_sectors_to_kill_list(top_object, top_object->size - (EVMS_FEATURE_HEADER_SECTORS * 2), EVMS_FEATURE_HEADER_SECTORS * 2);
-						top_object->flags &= ~SOFLAG_HAS_STOP_DATA;
-					}
-
-					/*
-					 * Start the shrink at the top level
-					 * object.
-					 */
-					rc = top_object->plugin->functions.plugin->shrink(top_object, obj, input_object_list, options);
-
-					if (rc == 0) {
-						post_shrink_cleanup(input_object_list, top_object, obj);
-					}
-				}
+				rc = con->plugin->container_functions->shrink_container(con,
+											top_object,
+											obj,
+											&input_object_list,
+											options);
 
 			} else {
-				LOG_WARNING("Error code %d encountered when trying to find the top level object.\n", rc);
+				if (parent_count <= 1) {
+
+					/*
+					 * Warn the user if there is no FSIM to handle
+					 * the shrink.
+					 */
+					rc = warn_if_no_fsim(obj);
+
+					if (rc == 0) {
+						/*
+						 * If the object is not part of a volume
+						 * and it has stop data, remove the stop
+						 * data from its current location.  It
+						 * will be reapplied during commit.
+						 */
+						if ((top_object->volume == NULL) &&
+						    (top_object->flags & SOFLAG_HAS_STOP_DATA)) {
+							top_object->plugin->functions.plugin->add_sectors_to_kill_list(top_object, top_object->size - (EVMS_FEATURE_HEADER_SECTORS * 2), EVMS_FEATURE_HEADER_SECTORS * 2);
+							top_object->flags &= ~SOFLAG_HAS_STOP_DATA;
+						}
+
+						/*
+						 * Start the shrink at the top level
+						 * object.
+						 */
+						rc = top_object->plugin->functions.plugin->shrink(top_object, obj, &input_object_list, options);
+
+						if (rc == 0) {
+							post_shrink_cleanup(&input_object_list, top_object, obj);
+						}
+					}
+
+				} else {
+					if (parent_count > 1) {
+						LOG_ERROR("Object %s cannot be shrunk because it has multiple parents.\n", obj->name);
+						rc = EINVAL;
+					} else {
+						LOG_WARNING("Error code %d encountered when trying to find the top level object.\n", rc);
+					}
+				}
 			}
 
 		} else {
@@ -700,7 +668,124 @@ int evms_shrink(object_handle_t thing, handle_array_t * input_objects, option_ar
 		LOG_ERROR("Error code %d when making a list from the input_objects handle array.\n", rc);
 	}
 
-	destroy_list(input_object_list);
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+
+static int do_container_shrink(storage_container_t * con,
+			       handle_array_t      * input_objects,
+			       option_array_t      * options) {
+
+	int rc = 0;
+	STATIC_LIST_DECL(input_object_list);
+
+	LOG_PROC_ENTRY();
+
+	rc = make_list(input_objects, &input_object_list);
+
+	if (rc == 0) {
+		list_element_t iter;
+		storage_object_t * input_obj;
+		storage_container_t * disk_group;
+
+		/*
+		 * The objects (disks) in a CSM container don't beling to a
+		 * disk group.  It is the CSM conatainer and the objects it
+		 * produces that are in a disk group.  Other container plug-ins
+		 * must obey the rule of having everything in the same
+		 * disk group.
+		 */
+		if (con->plugin->id == CSM_PLUGIN_ID) {
+			disk_group = NULL;
+		} else {
+			disk_group = con->disk_group;
+		}
+
+		/*
+		 * Make sure all of the input objects are storage
+		 * objects.  (The caller could have supplied handles
+		 * for bogus items such as plug-ins or volumes.)
+		 */
+		LIST_FOR_EACH(&input_object_list, iter, input_obj) {
+			rc = isa_valid_shrink_input_object(input_obj, disk_group);
+			if (rc != 0) {
+				break;
+			}
+		}
+
+		if (rc == 0) {
+			rc = con->plugin->container_functions->shrink_container(con,
+										NULL,
+										NULL,
+										&input_object_list,
+										options);
+
+		} else {
+			LOG_ERROR("One or more items in the input object list is not a valid input object.\n");
+			rc = EINVAL;
+		}
+
+	} else {
+		LOG_ERROR("Error code %d when making a list from the input_objects handle array.\n", rc);
+		rc = EINVAL;
+	}
+
+	LOG_PROC_EXIT_INT(rc);
+	return rc;
+}
+
+
+/*
+ * Shrink the size of an object.  The shrink gets targeted at the top object of
+ * the feature stack.  The plug-ins pass it on down until it hits the object
+ * that is to do the real shrink.
+ */
+int evms_shrink(object_handle_t thing_handle, handle_array_t * input_objects, option_array_t * options) {
+
+	int rc = 0;
+	void * thing;
+	object_type_t type;
+
+	LOG_PROC_ENTRY();
+
+	rc = check_engine_write_access();
+        if (rc != 0) {
+		LOG_PROC_EXIT_INT(rc);
+		return rc;
+	}
+
+	if (!local_focus) {
+		rc = remote_shrink(thing_handle, input_objects, options);
+		LOG_PROC_EXIT_INT(rc);
+		return rc;
+	}
+
+	rc = translate_handle(thing_handle,
+			      &thing,
+			      &type);
+
+	if (rc != HANDLE_MANAGER_NO_ERROR) {
+		LOG_PROC_EXIT_INT(rc);
+		return rc;
+	}
+
+	switch (type) {
+		case EVMS_OBJECT:
+		case REGION:
+		case SEGMENT:
+		case DISK:
+			rc = do_object_shrink((storage_object_t *) thing, input_objects, options);
+			break;
+
+		case CONTAINER:
+			rc = do_container_shrink((storage_container_t *) thing, input_objects, options);
+			break;
+
+		default:
+			LOG_ERROR("A thing of type %d cannot be shrunk.\n", type);
+			rc = EINVAL;
+	}
 
 	LOG_PROC_EXIT_INT(rc);
 	return rc;
@@ -727,7 +812,8 @@ int engine_can_shrink_by(storage_object_t * object, sector_count_t * delta_size)
 
 	LOG_PROC_ENTRY();
 
-	while ((rc == 0) && !list_empty(curr_object->parent_objects)) {
+	while ((rc == 0) && (curr_object->consuming_container == NULL) &&
+	       !list_empty(curr_object->parent_objects)) {
 
 		curr_object = first_thing(curr_object->parent_objects, NULL);
 
@@ -737,12 +823,19 @@ int engine_can_shrink_by(storage_object_t * object, sector_count_t * delta_size)
 	}
 
 	if (rc == 0) {
-		/*
-		 * If this object is part of a volume and the volume has an FSIM that is
-		 * not scheduled to be removed, check to make sure the final size does
-		 * not go below the FSIM's minimum size for the file system.
-		 */
-		if (object->volume != NULL) {
+		if (curr_object->consuming_container != NULL) {
+			storage_container_t * con = curr_object->consuming_container;
+
+			rc = con->plugin->container_functions->can_shrink_container_by(con, curr_object, delta_size);
+
+		} else if (object->volume != NULL) {
+			/*
+			 * If this object is part of a volume and the volume has
+			 * an FSIM that is not scheduled to be removed, check to
+			 * make sure the final size does not go over the FSIM's
+			 * maximum size it can handle for its backing storage
+			 * object.
+			 */
 
 			if (object->volume->flags & VOLFLAG_COMPATIBILITY) {
 				if (is_kernel_volume_mounted(object->volume, ERROR)) {
@@ -750,7 +843,7 @@ int engine_can_shrink_by(storage_object_t * object, sector_count_t * delta_size)
 				}
 
 			}
-			
+
 			if (rc == 0) {
 				if ((object->volume->original_fsim != NULL) &&
 				    !(object->volume->flags & VOLFLAG_UNMKFS)) {
