@@ -44,6 +44,7 @@
 #include "memman.h"
 #include "remote.h"
 #include "dm.h"
+#include "option.h"
 
 
 boolean is_kernel_volume_mounted(logical_volume_t * vol,
@@ -177,7 +178,10 @@ boolean is_mounted(char * volume_name, int dev_major, int dev_minor, char * * mo
 
 					status = stat(mount_entry->mnt_fsname, &statbuf);
 					if (status == 0) {
-						if (statbuf.st_rdev == vol_devt) {
+						if ((vol_devt != 0) &&
+						    (statbuf.st_rdev == vol_devt)) {
+							LOG_DEBUG("%s has same device number as volume %s (%u:%u)",
+								  mount_entry->mnt_fsname, volume_name, dev_major, dev_minor);
 							result = TRUE;
 						}
 					}
@@ -231,7 +235,8 @@ boolean is_volume_mounted(logical_volume_t * vol) {
 
 	LOG_PROC_ENTRY();
 
-	LOG_DEBUG("Checking if %s is mounted.\n", vol->name);
+	LOG_DEBUG("Checking if volume %s (dev node %s) is mounted.\n",
+		  vol->name, vol->dev_node);
 
 	if (vol->mount_point != NULL) {
 		engine_free(vol->mount_point);
@@ -632,7 +637,7 @@ int make_volume(storage_object_t * obj,
 				  NULL);
 
 		if (el != NULL) {
-			sort_list(&volumes_list, compare_volumes);
+			sort_list(&volumes_list, compare_volumes, NULL);
 
 			set_volume_in_object(obj, vol);
 		}
@@ -922,10 +927,6 @@ int make_evms_volume_for_object(storage_object_t * obj,
 				}
 			}
 
-			if (!(vol->flags & VOLFLAG_ACTIVE)) {
-				vol->flags |= VOLFLAG_NEEDS_ACTIVATE;
-			}
-
 		} else {
 			/*
 			 * The volume is made from an EVMS object.  The volume
@@ -1012,7 +1013,12 @@ int evms_create_volume(object_handle_t object_handle,
 		rc = make_evms_volume_for_object(obj, name,new_serial);
 
 		if (rc == 0) {
-			obj->volume->flags |= (VOLFLAG_NEW | VOLFLAG_DIRTY | VOLFLAG_NEEDS_ACTIVATE);
+			obj->volume->flags |= (VOLFLAG_NEW | VOLFLAG_FEATURE_HEADER_DIRTY);
+			if ((obj->flags & SOFLAG_NEEDS_ACTIVATE) ||
+			    ((obj->flags & SOFLAG_ACTIVE) &&
+			     !(obj->flags & SOFLAG_NEEDS_DEACTIVATE))) {
+				obj->volume->flags |= VOLFLAG_NEEDS_ACTIVATE;
+			}
 		}
 	}
 
@@ -1074,7 +1080,6 @@ int evms_set_volume_name(engine_handle_t volume_handle,
 	object_type_t type;
 	logical_volume_t * vol;
 	char new_name[EVMS_VOLUME_NAME_SIZE+1];
-	char * base_name = new_name + EVMS_DEV_NODE_PATH_LEN;
 
 	LOG_PROC_ENTRY();
 
@@ -1113,20 +1118,30 @@ int evms_set_volume_name(engine_handle_t volume_handle,
 		/* Make sure this volume name does not already exist. */
 		rc = engine_validate_name(new_name);
 		if ((rc == 0) && (vol->flags & VOLFLAG_HAS_OWN_DEVICE)) {
-			rc = engine_validate_name(base_name);
+			rc = engine_validate_name(new_name + EVMS_DEV_NODE_PATH_LEN);
 		}
 
 		if (rc == 0) {
-			rc = add_volume_to_rename_list(vol, new_name);
+			/*
+			 * If the volume is active, we must schedule
+			 * a rename with device-mapper.
+			 */
+			if (vol->flags & VOLFLAG_ACTIVE) {
+				rc = add_volume_to_rename_list(vol, new_name);
+			}
+
 			if (rc == 0) {
 				engine_unregister_name(vol->name);
 				engine_unregister_name(vol->name + EVMS_DEV_NODE_PATH_LEN);
 				memset(&vol->name, 0, EVMS_VOLUME_NAME_SIZE + 1);
 				strcpy(vol->name, new_name);
 				engine_register_name(new_name);
-				engine_register_name(base_name);
-				vol->flags |= VOLFLAG_DIRTY | VOLFLAG_NEEDS_ACTIVATE;
-				sort_list(&volumes_list, compare_volumes);
+				engine_register_name(new_name + EVMS_DEV_NODE_PATH_LEN);
+				vol->flags |= VOLFLAG_FEATURE_HEADER_DIRTY;
+				if (vol->flags & VOLFLAG_ACTIVE) {
+					vol->flags |= VOLFLAG_NEEDS_ACTIVATE;
+				}
+				sort_list(&volumes_list, compare_volumes, NULL);
 			}
 		}
 	}
@@ -1208,7 +1223,12 @@ int evms_create_compatibility_volume(object_handle_t object_handle) {
 	rc = make_compatibility_volume_for_object(obj);
 
 	if (rc == 0) {
-		obj->volume->flags |= (VOLFLAG_NEW | VOLFLAG_DIRTY | VOLFLAG_NEEDS_ACTIVATE);
+		obj->volume->flags |= VOLFLAG_NEW;
+		if ((obj->flags & SOFLAG_NEEDS_ACTIVATE) ||
+		    ((obj->flags & SOFLAG_ACTIVE) &&
+		     !(obj->flags & SOFLAG_NEEDS_DEACTIVATE))) {
+			obj->volume->flags |= VOLFLAG_NEEDS_ACTIVATE;
+		}
 		/*
 		 * Kill any stop data sectors
 		 * that might be on the object.
@@ -1222,7 +1242,6 @@ int evms_create_compatibility_volume(object_handle_t object_handle) {
 		}
 
 		obj->flags &= ~SOFLAG_HAS_STOP_DATA;
-
 	}
 
 	LOG_PROC_EXIT_INT(rc);
@@ -1287,7 +1306,7 @@ static int can_mkfs(object_handle_t volume_handle,
 		rc = EBUSY;
 	}
 
-	if (!(vol->flags & (VOLFLAG_ACTIVE | VOLFLAG_NEW))) {
+	if (!(vol->flags & (VOLFLAG_ACTIVE | VOLFLAG_NEW | VOLFLAG_NEEDS_ACTIVATE))) {
 		LOG(log_level, "Volume \"%s\" is not active.\n", vol->name);
 		rc = EINVAL;
 	}
@@ -1374,6 +1393,11 @@ static int can_unmkfs(object_handle_t volume_handle,
 		return EBUSY;
 	}
 
+	if (!(vol->flags & (VOLFLAG_ACTIVE | VOLFLAG_NEW | VOLFLAG_NEEDS_ACTIVATE))) {
+		LOG(log_level, "Volume \"%s\" is not active.\n", vol->name);
+		rc = EINVAL;
+	}
+
 	if (vol->flags & VOLFLAG_READ_ONLY) {
 		LOG(log_level, "Volume \"%s\" is read only.\n", vol->name);
 		LOG_PROC_EXIT_INT(EINVAL);
@@ -1461,6 +1485,11 @@ static int can_fsck(object_handle_t volume_handle,
 		return EBUSY;
 	}
 
+	if (!(vol->flags & (VOLFLAG_ACTIVE | VOLFLAG_NEW | VOLFLAG_NEEDS_ACTIVATE))) {
+		LOG(log_level, "Volume \"%s\" is not active.\n", vol->name);
+		rc = EINVAL;
+	}
+
 	fsim = vol->file_system_manager;
 
 	if (fsim == NULL) {
@@ -1503,93 +1532,6 @@ int evms_can_fsck(object_handle_t volume_handle) {
 }
 
 
-static int can_defrag(object_handle_t volume_handle,
-		      debug_level_t   log_level) {
-
-	int rc = 0;
-	void * thing;
-	object_type_t type;
-	logical_volume_t * vol;
-	plugin_record_t * fsim;
-
-	LOG_PROC_ENTRY();
-
-	rc = check_engine_write_access();
-	if (rc != 0) {
-		LOG_PROC_EXIT_INT(rc);
-		return rc;
-	}
-
-	rc = translate_handle(volume_handle,
-			      &thing,
-			      &type);
-
-	if (rc != HANDLE_MANAGER_NO_ERROR) {
-		LOG_PROC_EXIT_INT(rc);
-		return rc;
-	}
-	if (type != VOLUME) {
-		LOG(log_level, "Handle %d is not for a volume.\n", volume_handle);
-		LOG_PROC_EXIT_INT(EINVAL);
-		return EINVAL;
-	}
-
-	vol = (logical_volume_t *) thing;
-
-	if (is_kernel_volume_mounted(vol, DETAILS)) {
-		LOG_PROC_EXIT_INT(EBUSY);
-		return EBUSY;
-	}
-
-	if (vol->flags & VOLFLAG_READ_ONLY) {
-		LOG(log_level, "Volume \"%s\" is read only.\n", vol->name);
-		LOG_PROC_EXIT_INT(EINVAL);
-		return EINVAL;
-	}
-
-	fsim = vol->file_system_manager;
-
-	if (fsim == NULL) {
-		LOG(log_level, "Volume \"%s\" does not have a File System Interface Module associated with it.\n", vol->name);
-		LOG_PROC_EXIT_INT(ENOSYS);
-		return ENOSYS;
-	}
-						
-	rc = fsim->functions.fsim->can_defrag(vol);
-
-	LOG_PROC_EXIT_INT(rc);
-	return rc;
-}
-
-
-/*
- * Can the volume be defragmented?
- */
-int evms_can_defrag(object_handle_t volume_handle) {
-
-	int rc = 0;
-
-	LOG_PROC_ENTRY();
-
-	rc = check_engine_write_access();
-	if (rc != 0) {
-		LOG_PROC_EXIT_INT(rc);
-		return rc;
-	}
-
-	if (!local_focus) {
-		rc = remote_can_defrag(volume_handle);
-		LOG_PROC_EXIT_INT(rc);
-		return rc;
-	}
-
-	rc = can_defrag(volume_handle, DETAILS);
-
-	LOG_PROC_EXIT_INT(rc);
-	return rc;
-}
-
-
 /* Make a deep copy of a value list. */
 static value_list_t * dup_value_list(value_list_t * vl, value_type_t type) {
 
@@ -1598,7 +1540,7 @@ static value_list_t * dup_value_list(value_list_t * vl, value_type_t type) {
 
 	LOG_PROC_ENTRY()
 
-	size = sizeof(value_list_t) + (vl->count - 1) * sizeof(value_t);
+	size = sizeof(value_list_t) + vl->count * sizeof(value_t);
 
 	dup_vl = engine_alloc(size);
 
@@ -1625,7 +1567,7 @@ static value_list_t * dup_value_list(value_list_t * vl, value_type_t type) {
 }
 
 
-/* Make a deep copy of an optin array. */
+/* Make a deep copy of an option array. */
 static option_array_t * dup_option_array(option_array_t * oa) {
 
 	uint size;
@@ -1633,7 +1575,7 @@ static option_array_t * dup_option_array(option_array_t * oa) {
 
 	LOG_PROC_ENTRY()
 
-	size = sizeof(option_array_t) + (oa->count - 1) * sizeof(key_value_pair_t);
+	size = sizeof(option_array_t) + oa->count * sizeof(key_value_pair_t);
 
 	dup_oa = engine_alloc(size);
 
@@ -1776,22 +1718,20 @@ int evms_unmkfs(object_handle_t volume_handle) {
 		vol->file_system_manager = NULL;
 
 		/*
-		 * Since it has no FSIM any pending mkfs, fsck, or defrag can't
+		 * Since it has no FSIM any pending mkfs, or fsck can't
 		 * be run.
 		 */
-		vol->flags &= ~(VOLFLAG_MKFS | VOLFLAG_FSCK | VOLFLAG_DEFRAG);
+		vol->flags &= ~(VOLFLAG_MKFS | VOLFLAG_FSCK);
 		if (vol->mkfs_options != NULL) {
-			free_option_array(vol->mkfs_options);
+			free_option_array_contents(vol->mkfs_options);
+			engine_free(vol->mkfs_options);
 			vol->mkfs_options = NULL;
 		}
 		if (vol->fsck_options != NULL) {
-			free_option_array(vol->fsck_options);
+			free_option_array_contents(vol->fsck_options);
+			engine_free(vol->fsck_options);
 			vol->fsck_options = NULL;
 		}
-		if (vol->defrag_options != NULL) {
-			free_option_array(vol->defrag_options);
-			vol->defrag_options = NULL;
-		}
 
 		/* If this FSIM claimed the volume during discovery ... */
 		if (fsim == vol->original_fsim) {
@@ -1869,60 +1809,6 @@ int evms_fsck(object_handle_t  volume_handle,
 }
 
 
-/*
- * Defragment a volume.
- */
-int evms_defrag(object_handle_t  volume_handle,
-		option_array_t * options) {
-
-	int rc = 0;
-	void * thing;
-	object_type_t type;
-	logical_volume_t * vol;
-	plugin_record_t * fsim;
-
-	LOG_PROC_ENTRY();
-
-	rc = check_engine_write_access();
-	if (rc != 0) {
-		LOG_PROC_EXIT_INT(rc);
-		return rc;
-	}
-
-	if (!local_focus) {
-		rc = remote_defrag(volume_handle,options);
-		LOG_PROC_EXIT_INT(rc);
-		return rc;
-	}
-
-	rc = can_defrag(volume_handle, ERROR);
-	if (rc != 0) {
-		LOG_PROC_EXIT_INT(rc);
-		return rc;
-	}
-
-	translate_handle(volume_handle,
-			 &thing,
-			 &type);
-
-	vol = (logical_volume_t *) thing;
-
-	fsim = vol->file_system_manager;
-
-	vol->defrag_options = dup_option_array(options);
-	if (vol->fsck_options != NULL) {
-		vol->flags |= VOLFLAG_DEFRAG;
-
-	} else {
-		LOG_CRITICAL("Error allocating memory for a copy of the options.\n");
-		rc = ENOMEM;
-	}
-
-	LOG_PROC_EXIT_INT(rc);
-	return rc;
-}
-
-
 /*
  * evms_get_volume_list returns a pointer to a handle_array_t with handles for
  * logical volumes, optionally filtering on the FSIM that manages the volume.
@@ -2099,6 +1985,15 @@ static int can_convert_to_evms_volume(object_handle_t volume_handle,
 		rc = EBUSY;
 	}
 
+	/*
+	 * The volume must be active or new since we may have to ask the
+	 * file system if it can shrink.
+	 */
+	if (!(vol->flags & (VOLFLAG_ACTIVE | VOLFLAG_NEW))) {
+		LOG(log_level, "Volume %s is not activee.\n", vol->name);
+		rc = EINVAL;
+	}
+
 	if (rc == 0) {
 		/*
 		 * The volume's object must allow the volume name to be
@@ -2212,8 +2107,8 @@ int evms_convert_to_evms_volume(object_handle_t volume_handle, char * name) {
 		int answer = 0;	    /* Default is "Continue" */
 
 		rc = engine_user_message(&answer, choices,
-					 "Volume %s does not have a File System Interface Module (FSIM) associated with it.  "
-					 "The file system (if any) on the volume cannot be shrunk to make space at the end of the volume for the metadata necessary to make an EVMS volume.\n",
+					 _("Volume %s does not have a File System Interface Module (FSIM) associated with it.  "
+					   "The file system (if any) on the volume cannot be shrunk to make space at the end of the volume for the metadata necessary to make an EVMS volume.\n"),
 					 vol->name);
 
 		if (answer == 1) {
@@ -2259,13 +2154,10 @@ int evms_convert_to_evms_volume(object_handle_t volume_handle, char * name) {
 			new_vol->original_vol_size          = vol->original_vol_size;
 			new_vol->max_vol_size               = vol->max_vol_size;
 			new_vol->shrink_vol_size            = min(vol->shrink_vol_size, new_vol->vol_size);
-			new_vol->associated_volume          = vol->associated_volume;
 			new_vol->mkfs_options               = vol->mkfs_options;
 			new_vol->fsck_options               = vol->fsck_options;
-			new_vol->defrag_options             = vol->defrag_options;
-			new_vol->flags                      = vol->flags;
-			new_vol->flags                     &= ~(VOLFLAG_ACTIVE | VOLFLAG_COMPATIBILITY);
-			new_vol->flags                     |= (VOLFLAG_DIRTY | VOLFLAG_NEEDS_ACTIVATE);
+			new_vol->flags                     |= vol->flags &
+							      ~(VOLFLAG_ACTIVE | VOLFLAG_COMPATIBILITY);
 			new_vol->private_data               = vol->private_data;
 			new_vol->original_fsim_private_data = vol->original_fsim_private_data;
 			memcpy(new_vol->dev_node, vol->dev_node, sizeof(vol->dev_node));
@@ -2503,13 +2395,10 @@ int evms_convert_to_compatibility_volume(object_handle_t volume_handle) {
 		new_vol->original_vol_size          = vol->original_vol_size;
 		new_vol->max_vol_size               = vol->max_vol_size;
 		new_vol->shrink_vol_size            = min(vol->shrink_vol_size, new_vol->vol_size);
-		new_vol->associated_volume          = vol->associated_volume;
 		new_vol->mkfs_options               = vol->mkfs_options;
 		new_vol->fsck_options               = vol->fsck_options;
-		new_vol->defrag_options             = vol->defrag_options;
-		new_vol->flags                      = vol->flags;
-		new_vol->flags                     &= ~VOLFLAG_ACTIVE;
-		new_vol->flags                     |= (VOLFLAG_DIRTY | VOLFLAG_COMPATIBILITY | VOLFLAG_NEEDS_ACTIVATE);
+		new_vol->flags                      |= vol->flags &
+						       ~(VOLFLAG_ACTIVE | VOLFLAG_HAS_OWN_DEVICE);
 		new_vol->private_data               = vol->private_data;
 		new_vol->original_fsim_private_data = vol->original_fsim_private_data;
 		memcpy(new_vol->dev_node, vol->dev_node, sizeof(vol->dev_node));
@@ -2782,8 +2671,8 @@ int evms_add_feature_to_volume(object_handle_t  volume_handle,
 		int answer = 0;	    /* Default is "Continue" */
 
 		rc = engine_user_message(&answer, choices,
-					 "Volume %s does not have a File System Interface Module (FSIM) associated with it.  "
-					 "The file system (if any) on the volume cannot be shrunk to make space at the end of the volume for the metadata necessary to make an EVMS volume.\n",
+					 _("Volume %s does not have a File System Interface Module (FSIM) associated with it.  "
+					   "The file system (if any) on the volume cannot be shrunk to make space at the end of the volume for the metadata necessary to make an EVMS volume.\n"),
 					 vol->name);
 
 		if (answer == 1) {
@@ -2833,15 +2722,16 @@ int evms_add_feature_to_volume(object_handle_t  volume_handle,
 
 	if (rc == 0) {
 		storage_object_t * prev_vol_obj = vol->object;
-		handle_array_t input_ha;
+		u_int8_t ha[sizeof(handle_array_t) + sizeof(object_handle_t)];
+		handle_array_t * input_ha = (handle_array_t *) ha;
 		handle_array_t * output_ha;
 
 		/* Build a handle_array for the call to evms_create(). */
 		rc = ensure_app_handle(vol->object);
 
 		if (rc == 0) {
-			input_ha.count = 1;
-			input_ha.handle[0] = vol->object->app_handle;
+			input_ha->count = 1;
+			input_ha->handle[0] = vol->object->app_handle;
 
 			/*
 			 * If this is a featureless EVMS volume, remove the
@@ -2860,7 +2750,7 @@ int evms_add_feature_to_volume(object_handle_t  volume_handle,
 			 */
 			prev_vol_obj->volume = NULL;
 
-			rc = evms_create(feature_handle, &input_ha, options, &output_ha);
+			rc = evms_create(feature_handle, input_ha, options, &output_ha);
 
 			if (rc == 0) {
 
@@ -2894,7 +2784,7 @@ int evms_add_feature_to_volume(object_handle_t  volume_handle,
 					 * the feature headers is
 					 * updated.
 					 */
-					vol->flags |= VOLFLAG_DIRTY;
+					vol->flags |= VOLFLAG_FEATURE_HEADER_DIRTY;
 				}
 
 			} else {
@@ -3182,7 +3072,7 @@ int evms_mount(object_handle_t volume_handle, char * mount_point, char * mount_o
 					if (bytes_read > 0) {
 						mount_output[bytes_read] = '\0';
 						engine_user_message(NULL, NULL,
-								    "mount: %s", mount_output);
+								    _("mount: %s"), mount_output);
 					}
 
 					rc = WEXITSTATUS(status);
@@ -3348,7 +3238,7 @@ int evms_unmount(object_handle_t volume_handle) {
 					if (bytes_read > 0) {
 						mount_output[bytes_read] = '\0';
 						engine_user_message(NULL, NULL,
-								    "umount: %s", mount_output);
+								    _("umount: %s"), mount_output);
 					}
 
 					rc = WEXITSTATUS(status);
@@ -3550,7 +3440,7 @@ int evms_remount(object_handle_t volume_handle, char * mount_options) {
 						if (bytes_read > 0) {
 							mount_output[bytes_read] = '\0';
 							engine_user_message(NULL, NULL,
-									    "mount: %s", mount_output);
+									    _("mount: %s"), mount_output);
 						}
 
 						rc = WEXITSTATUS(status);
