@@ -25,6 +25,7 @@
 #include <stdlib.h>
 #include <stdint.h>
 #include <sys/types.h>
+#include <gettext.h>
 
 #define __STDC_FORMAT_MACROS 1
 #include <inttypes.h>
@@ -68,36 +69,71 @@
  *
  * b		boolean
  * c[...]	integer count of array entries
- * C[...]	32-bit integer count of array entries of the given format
- * d		option_descriptor_t (see options.h)
- * e		extended_info_t (see options.h)
- * g		8-bit integer
- * h		16-bit integer
+ * d[...]	32-bit integer count of array entries
  * i		integer - size dependant on the archtecture
- * I		32-bit integer
- * k		key_value_pair_t (see options.h)
- * l		64-bit integer
- * o		object_info_t structure
+ * j		8-bit integer
+ * k		16-bit integer
+ * l		32-bit integer
+ * m		64-bit integer
  * p		pointer
  * p{...}	pointer to a structure of the given format
  * s		pointer to a string
  * t		pointer to an array of strings
- * v		a value_type_t followed by a value_t
  * xnn		byte string of nn bytes
  *
+ * D	        option_descriptor_t *		(see options.h)
+ * E		expand_handle_array_t *		(see appstruct.h)
+ * F		function_info_array_t *		(see options.h)
+ * H		handle_array_t *		(see appstruct.h)
+ * I		handle_object_info_t *		(see appstruct.h)
+ * N		declined_handle_array_t *	(see options.h)
+ * O		option_array_t *		(see options.h)
+ * R		change_record_array_t *		(see appstruct.h)
+ * S		shrink_handle_array_t *		(see appstruct.h)
+ * V		a value_type_t			(see optins.h)
+ *		followed by a boolean that says if it is a list
+ *		followed by a value_t		(see options.h)
+ * X	        extended_info_array_t *		(see options.h)
+ *
  */
 
 /* Formats for basic data types */
+#define boolean_c 'b'
 #define boolean_f "b"
+
+#define count_c   'c'
+#define count_f   "c"
+
+#define count32_c 'd'
+#define count32_f "d"
+
+#define int_c     'i'
 #define int_f     "i"
+
+#define u_int8_c  'j'
 #define u_int8_f  "j"
+
+#define u_int16_c 'k'
 #define u_int16_f "k"
-#define u_int32_f "I"
-#define u_int64_f "l"
+
+#define u_int32_c 'l'
+#define u_int32_f "l"
+
+#define u_int64_c 'm'
+#define u_int64_f "m"
+
+#define ptr_c     'p'
 #define ptr_f     "p"
+
+#define string_c  's'
 #define string_f  "s"
+
+#define strings_c 't'
 #define strings_f "t"
 
+#define bytes_c   'x'
+#define bytes_f   "x"
+
 
 /* EVMS will always use 64-bit fields for sectors. */
 typedef u_int64_t evms_sector_t;
@@ -280,15 +316,15 @@ typedef struct evms_feature_header {
 #define VOLFLAG_DIRTY                   (1<<0)
 #define VOLFLAG_NEW                     (1<<1)
 #define VOLFLAG_READ_ONLY               (1<<2)
-#define VOLFLAG_COMPATIBILITY           (1<<3)
-#define VOLFLAG_HAS_OWN_DEVICE          (1<<4)
-#define VOLFLAG_ACTIVE                  (1<<5)
-#define VOLFLAG_NEEDS_ACTIVATE          (1<<6)
-#define VOLFLAG_NEEDS_DEACTIVATE        (1<<7)
-#define VOLFLAG_MKFS                    (1<<8)
-#define VOLFLAG_UNMKFS                  (1<<9)
-#define VOLFLAG_FSCK                    (1<<10)
-#define VOLFLAG_DEFRAG                  (1<<11)
+#define VOLFLAG_FEATURE_HEADER_DIRTY    (1<<3)
+#define VOLFLAG_COMPATIBILITY           (1<<4)
+#define VOLFLAG_HAS_OWN_DEVICE          (1<<5)
+#define VOLFLAG_ACTIVE                  (1<<6)
+#define VOLFLAG_NEEDS_ACTIVATE          (1<<7)
+#define VOLFLAG_NEEDS_DEACTIVATE        (1<<8)
+#define VOLFLAG_MKFS                    (1<<9)
+#define VOLFLAG_UNMKFS                  (1<<10)
+#define VOLFLAG_FSCK                    (1<<11)
 #define VOLFLAG_SYNC_FS                 (1<<12)
 #define VOLFLAG_PROBE_FS                (1<<13)
 #define VOLFLAG_NOT_MOUNTABLE           (1<<14)
@@ -484,6 +520,16 @@ typedef struct progress_s {
 	 * indicator.
 	 */
 	void          * ui_private_data;
+
+	/*
+	 * A pointer to the Engine's data structure for calculating remaining
+	 * time estimates.  Neither the plug-in nor the user interface should
+	 * modify this field.
+	 */
+	struct timer_data_s * timer_data;
+
+	u_int32_t       flags;
+
 } progress_t;
 #define progress_f      \
 	int_f           \
@@ -494,7 +540,13 @@ typedef struct progress_s {
 	u_int64_f       \
 	int_f           \
 	ptr_f           \
-	ptr_f
+	ptr_f           \
+	ptr_f           \
+	u_int32_f
+
+#define PROGRESS_NO_TIME_ESTIMATE (1<<0)	/* The Engine should not */
+						/* calcualte a reamining */
+						/* time estimate.        */
 
 /*
  * The data types which a storage object can be.
@@ -690,7 +742,7 @@ typedef struct node_list_s {
 /* Macro to set attributes on log functions to make the compiler treat them
  * as printfs. This only seems to work on recent compilers.
  */
-#if __GNUC__ >= 3
+#if __GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 2)
 #define __PRINTF_ATTRIBUTE__(format_idx, arg_idx) __attribute__((__format__ (__printf__, (format_idx), (arg_idx))))
 #else
 #define __PRINTF_ATTRIBUTE__(format_idx, arg_idx)
@@ -718,10 +770,10 @@ static inline u_int64_t sectors_to_bytes(u_int64_t sectors)
  * calc_log2
  *	Calculate the log2 of a number.
  **/
-static inline long calc_log2(long arg)
+static inline long calc_log2(u_int64_t arg)
 {
 	long result = -1;
-	long tmp;
+	u_int64_t tmp;
 
 	if (arg) {
 		tmp = arg;
@@ -738,5 +790,62 @@ static inline long calc_log2(long arg)
 	return result;
 }
 
+/*
+ * Standard gettext macros.
+ */
+
+#ifdef ENABLE_NLS
+#  undef _
+#  define _(String) gettext(String)
+#  ifdef gettext_noop
+#    define N_(String) gettext_noop (String)
+#  else
+#    define N_(String) (String)
+#  endif
+#else
+#  define _(String) (String)
+#  define N_(String) (String)
+#endif
+
+#ifndef INT8_MIN
+#define INT8_MIN	0x80
+#endif
+#ifndef INT8_MAX
+#define INT8_MAX	0x7f
+#endif
+#ifndef UINT8_MAX
+#define UINT8_MAX	0xff
+#endif
+
+#ifndef INT16_MIN
+#define INT16_MIN	0x8000
+#endif
+#ifndef INT16_MAX
+#define INT16_MAX	0x7fff
+#endif
+#ifndef UINT16_MAX
+#define UINT16_MAX	0xffff
+#endif
+
+#ifndef INT32_MIN
+#define INT32_MIN	0x80000000
+#endif
+#ifndef INT32_MAX
+#define INT32_MAX	0x7fffffff
+#endif
+#ifndef UINT32_MAX
+#define UINT32_MAX	0xffffffff
+#endif
+
+#ifndef INT64_MIN
+#define INT64_MIN	0x8000000000000000
+#endif
+#ifndef INT64_MAX
+#define INT64_MAX	0x7fffffffffffffff
+#endif
+#ifndef UINT64_MAX
+#define UINT64_MAX	0xffffffffffffffff
+#endif
+
 #endif
 
