@@ -24,7 +24,7 @@
 #ifndef _BSD_COMMIT_HEADER
 #define _BSD_COMMIT_HEADER 1
 
-int  commit_bsd_segments( DISKSEG  *seg, LOGICALDISK *ld, uint engine_commit_phase);
+int  commit_bsd_segments( DISKSEG  *seg, LOGICALDISK *ld, uint engine_commit_phase, boolean backup);
 
 #endif
 
