@@ -120,36 +120,6 @@ int add_disk_to_container( storage_object_t    * ld,
 }
 
 
-/*
- *  Function: csm_can_create_container
- *
- *  Our containers hold logical disks that produce CSM
- *  segments ... meaning they have CSM metadata.  So, we
- *  can only create a container if all the objects are
- *  known to us as csm logical disks.
- */
-int csm_can_create_container( list_anchor_t objects )
-{
-        LOGICALDISK *ld;
-        list_element_t iter;
-
-        LOG_ENTRY();
-
-        REQUIRE( objects != NULL );
-        REQUIRE( EngFncs->list_count(objects) > 0);
-        REQUIRE( csm_has_quorum == TRUE || csm_admin_mode == TRUE);
-
-        LIST_FOR_EACH( objects, iter, ld ) {
-                REQUIRE(isa_cluster_logical_disk(ld) == FALSE);
-		REQUIRE(EngFncs->list_count(ld->parent_objects) == 0);
-                REQUIRE(ld->consuming_container == NULL);
-        }
-
-        LOG_EXIT_INT(0);
-        return 0;
-}
-
-
 /*
  *  Function: csm_can_delete_container
  *
@@ -211,23 +181,136 @@ int csm_can_add_object( LOGICALDISK           * ld,
 }
 
 
-/*
- *  Function: csm_can_remove_object
+/**
+ * csm_can_expand_container
  *
- *  We can remove the object if it is consumed by one of
- *  our containers.
- */
-int csm_can_remove_object( LOGICALDISK * ld )
+ * Get the list of available disks and see which ones can be added to this
+ * container. If any object can be added, create an expand-point to add to
+ * this list.
+ **/
+int csm_can_expand_container(storage_container_t *container,
+			     list_anchor_t expand_points)	/* of type expand_object_info_t */
 {
-        LOG_ENTRY();
+	expand_object_info_t *expand_point;
+	list_anchor_t objects = NULL;
+	storage_object_t *object;
+	sector_count_t limit = 0;
+	list_element_t iter;
+	int rc;
 
-        REQUIRE( csm_has_quorum == TRUE || csm_admin_mode == TRUE);
-        REQUIRE(isa_cluster_container(ld->consuming_container)==TRUE);
+	LOG_ENTRY();
+	LOG_DEBUG("Checking if container %s can expand.\n", container->name);
 
-        LOG_EXIT_INT(0);
-        return 0;
+        rc = EngFncs->get_object_list(DISK, DATA_TYPE, NULL,
+				      NULL, TOPMOST, &objects);
+	if (rc) {
+		goto out;
+	}
+
+	prune_acceptable_disks(objects);
+	LIST_FOR_EACH(objects, iter, object) {
+		limit += object->size;
+	}
+
+	if (!limit) {
+		LOG_DEBUG("No objects available to add to container %s.\n",
+			  container->name);
+		rc = ENOSPC;
+		goto out;
+	}
+
+	expand_point = EngFncs->engine_alloc(sizeof(*expand_point));
+	if (!expand_point) {
+		rc = ENOMEM;
+		goto out;
+	}
+
+	expand_point->container = container;
+	expand_point->max_expand_size = limit;
+	EngFncs->insert_thing(expand_points, expand_point, INSERT_AFTER, NULL);
+	rc = 0;
+
+out:
+	EngFncs->destroy_list(objects);
+	LOG_EXIT_INT(rc);
+	return rc;
 }
 
+/**
+ * csm_can_shrink_container
+ *
+ * If any one of the consumed objects can be removed, then the container can
+ * shrink. Create a shrink_object_info_t and add it to the shrink_points list.
+ **/
+int csm_can_shrink_container(storage_container_t *container,
+			     list_anchor_t shrink_points)	/* of type shrink_object_info_t */
+{
+	shrink_object_info_t *shrink_point;
+	storage_object_t *object;
+	sector_count_t limit = 0;
+	list_element_t iter;
+	int rc;
+
+	LOG_ENTRY();
+	LOG_DEBUG("Checking if container %s can shrink.\n", container->name);
+
+	LIST_FOR_EACH(container->objects_consumed, iter, object) {
+		rc = csm_can_unassign(object);
+		if (!rc) {
+			limit += object->size;
+		}
+	}
+
+	if (!limit) {
+		LOG_DEBUG("No objects can be removed from container %s.\n",
+			  container->name);
+		rc = EBUSY;
+		goto out;
+	}
+
+	shrink_point = EngFncs->engine_alloc(sizeof(*shrink_point));
+	if (!shrink_point) {
+		rc = ENOMEM;
+		goto out;
+	}
+
+	shrink_point->container = container;
+	shrink_point->max_shrink_size = limit;
+	EngFncs->insert_thing(shrink_points, shrink_point, INSERT_AFTER, NULL);
+	rc = 0;
+
+out:
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
+/**
+ * csm_can_expand_container_by
+ *
+ * Don't currently support expanding consumed disks.
+ **/
+int csm_can_expand_container_by(storage_container_t *container,
+				storage_object_t *consumed_object,
+				sector_count_t *delta_size)
+{
+	LOG_ENTRY();
+	LOG_EXIT_INT(ENOSYS);
+	return ENOSYS;
+}
+
+/**
+ * csm_can_shrink_container_by
+ *
+ * Don't currently support shrinking consumed disks.
+ **/
+int csm_can_shrink_container_by(storage_container_t *container,
+				storage_object_t *consumed_object,
+				sector_count_t *delta_size)
+{
+	LOG_ENTRY();
+	LOG_EXIT_INT(ENOSYS);
+	return ENOSYS;
+}
 
 /*
  *  Function: csm_create_container_storage_object
@@ -373,15 +456,90 @@ int csm_add_object( LOGICALDISK         *ld,
 }
 
 
-int csm_transfer_object( DISKSEG                * seg,
-                         storage_container_t    * target_container,
-                         option_array_t         * options )
+/**
+ * csm_expand_container
+ *
+ * Expand the specified container by adding one or more objects, which are
+ * specified in the input_objects list. Add each object using the specified
+ * options.
+ **/
+int csm_expand_container(storage_container_t *container,
+			 storage_object_t *consumed_object,
+			 storage_object_t *expand_object,
+			 list_anchor_t input_objects,
+			 option_array_t *options)
 {
-        LOG_ENTRY();
-        LOG_EXIT_INT(ENOSYS);
-        return ENOSYS;
+	storage_object_t *object;
+	list_element_t iter;
+	int rc = 0;
+
+	LOG_ENTRY();
+	LOG_DEBUG("Expanding container %s.\n", container->name);
+
+	if (expand_object) {
+		/* One of the consumed objects will be expanding.
+		 * Not currently supported.
+		 */
+		rc = ENOSYS;
+	} else {
+		/* Add this list of new objects to the container. */
+		LIST_FOR_EACH(input_objects, iter, object) {
+			rc = csm_add_object(object, container, options);
+			if (rc) {
+				LOG_ERROR("Error adding object %s to container "
+					  "%s. Aborting remaining expands.\n",
+					  object->name, container->name);
+				break;
+			}
+		}
+	}
+
+	LOG_EXIT_INT(rc);
+	return rc;
 }
 
+/**
+ * csm_shrink_container
+ *
+ * Shrink the specified container by removing one or more objects, which are
+ * specified in the input_objects list. Remove each object using the specified
+ * options.
+ **/
+int csm_shrink_container(storage_container_t *container,
+			 storage_object_t *consumed_object,
+			 storage_object_t *shrink_object,
+			 list_anchor_t input_objects,
+			 option_array_t *options)
+{
+	storage_object_t *object;
+	list_element_t iter;
+	int rc = 0;
+
+	LOG_ENTRY();
+	LOG_DEBUG("Shrinking container %s.\n", container->name);
+
+	if (shrink_object) {
+		/* One of the consumed objects will be shrinking.
+		 * Not currently supported.
+		 */
+		rc = ENOSYS;
+	} else {
+		/* Remove this list of objects from the container. */
+		LIST_FOR_EACH(input_objects, iter, object) {
+			rc = csm_remove_object(object);
+			if (rc) {
+				LOG_ERROR("Error removing object %s from "
+					  "container %s. Aborting remaining "
+					  "shrinks.\n", object->name,
+					  container->name);
+				break;
+			}
+		}
+	}
+
+	LOG_EXIT_INT(rc);
+	return rc;
+}
 
 int csm_discard_container( storage_container_t * container )
 {
@@ -510,6 +668,18 @@ int csm_commit_container_changes( storage_container_t   * container,
         return 0;
 }
 
+/*
+ *  Function:  csm_backup_container_metadata
+ *
+ *  CSM containers don't have metadata to backup. All metadata is backed-up
+ *  on a per-segment basis. See csm_backup_metadata().
+ */
+int csm_backup_container_metadata( storage_container_t * container )
+{
+	LOG_ENTRY();
+	LOG_EXIT_INT(0);
+	return 0;
+}
 
 /*
  *  Function:  csm_get_container_info
@@ -523,7 +693,7 @@ int csm_get_container_info( storage_container_t     *container,
         int rc=ENOSYS;
         extended_info_array_t   *Info;
         container_private_data_t *c_pdata;
-        char *str;
+        const char *nodeid_string;
 
         LOG_ENTRY();
 
@@ -531,84 +701,64 @@ int csm_get_container_info( storage_container_t     *container,
 
         c_pdata = (container_private_data_t *)container->private_data;
 
-        str  = (char *) calloc(1,(ECE_NODEID_SIZE*2)+1);
         Info = EngFncs->engine_alloc( sizeof(extended_info_array_t) + ( CONTAINER_GETINFO_COUNT * sizeof(extended_info_t) ) );
 
-        if (Info && str) {
+        if (Info) {
 
                 Info->count = CONTAINER_GETINFO_COUNT;
 
-                SET_STRING( Info->info[CONTAINER_GETINFO_NAME_INDEX].name,
-                            CONTAINER_GETINFO_NAME_STRING );
-                SET_STRING( Info->info[CONTAINER_GETINFO_NAME_INDEX].title,
-                            CONTAINER_GETINFO_NAME_STRING );
-                SET_STRING( Info->info[CONTAINER_GETINFO_NAME_INDEX].desc,
-                            "Enter the container name(e.g. node1-disk-group). Do not use any slashes in the name. Evms will prepend the csm namespace to the name(e.g. csm/node1-disk-group).");
+                Info->info[CONTAINER_GETINFO_NAME_INDEX].name =
+			EngFncs->engine_strdup( CONTAINER_GETINFO_NAME_STRING );
+                Info->info[CONTAINER_GETINFO_NAME_INDEX].title = EngFncs->engine_strdup( _("Name") );
+                Info->info[CONTAINER_GETINFO_NAME_INDEX].desc =
+			EngFncs->engine_strdup( _("Enter the container name(e.g. node1-disk-group). Do not use any slashes in the name. Evms will prepend the csm namespace to the name(e.g. csm/node1-disk-group)."));
                 Info->info[CONTAINER_GETINFO_NAME_INDEX].type               = EVMS_Type_String;
                 Info->info[CONTAINER_GETINFO_NAME_INDEX].unit               = EVMS_Unit_None;
-                SET_STRING( Info->info[CONTAINER_GETINFO_NAME_INDEX].value.s, container->name );
+                Info->info[CONTAINER_GETINFO_NAME_INDEX].value.s = EngFncs->engine_strdup( container->name );
                 Info->info[CONTAINER_GETINFO_NAME_INDEX].collection_type    = EVMS_Collection_None;
                 memset(&Info->info[CONTAINER_GETINFO_NAME_INDEX].group, 0, sizeof(group_info_t));
 
-                str = csm_nodeid_to_string( &c_pdata->nodeid );
-                if (!str) {
-                        rc = ENOMEM;
-                }
-                else {
-                        rc = 0;
-                }
+		rc = EngFncs->nodeid_to_string( &c_pdata->nodeid, &nodeid_string );
 
                 if (!rc) {
-                        SET_STRING( Info->info[CONTAINER_GETINFO_TYPE_INDEX].name,
-                                    CONTAINER_GETINFO_TYPE_STRING );
-                        SET_STRING( Info->info[CONTAINER_GETINFO_TYPE_INDEX].title,
-                                    CONTAINER_GETINFO_TYPE_STRING );
-                        SET_STRING( Info->info[CONTAINER_GETINFO_TYPE_INDEX].desc,
-                                    "Enter the cluster storage type. The allowed values are: private, shared or deported.");
+                        Info->info[CONTAINER_GETINFO_TYPE_INDEX].name =
+				EngFncs->engine_strdup( CONTAINER_GETINFO_TYPE_STRING );
+                        Info->info[CONTAINER_GETINFO_TYPE_INDEX].title = EngFncs->engine_strdup( _("Type") );
+                        Info->info[CONTAINER_GETINFO_TYPE_INDEX].desc =
+				EngFncs->engine_strdup( _("Enter the cluster storage type. The allowed values are: private, shared or deported."));
                         Info->info[CONTAINER_GETINFO_TYPE_INDEX].type = EVMS_Type_String;
                         Info->info[CONTAINER_GETINFO_TYPE_INDEX].unit = EVMS_Unit_None;
 
                         if (container->flags & SCFLAG_CLUSTER_PRIVATE) {
-                                SET_STRING( Info->info[CONTAINER_GETINFO_TYPE_INDEX].value.s, "private");
+                                Info->info[CONTAINER_GETINFO_TYPE_INDEX].value.s = EngFncs->engine_strdup( _("private") );
                         }
                         else if (container->flags & SCFLAG_CLUSTER_SHARED) {
-                                SET_STRING( Info->info[CONTAINER_GETINFO_TYPE_INDEX].value.s, "shared");
+                                Info->info[CONTAINER_GETINFO_TYPE_INDEX].value.s = EngFncs->engine_strdup( _("shared") );
                         }
                         else if (container->flags & SCFLAG_CLUSTER_DEPORTED) {
-                                SET_STRING( Info->info[CONTAINER_GETINFO_TYPE_INDEX].value.s, "deported");
+                                Info->info[CONTAINER_GETINFO_TYPE_INDEX].value.s = EngFncs->engine_strdup( _("deported") );
                         }
                         else {
-                                SET_STRING( Info->info[CONTAINER_GETINFO_TYPE_INDEX].value.s, "unknown");
+                                Info->info[CONTAINER_GETINFO_TYPE_INDEX].value.s = EngFncs->engine_strdup( _("unknown") );
                         }
 
                         Info->info[CONTAINER_GETINFO_TYPE_INDEX].collection_type    = EVMS_Collection_None;
                         memset( &Info->info[CONTAINER_GETINFO_TYPE_INDEX].group, 0, sizeof(group_info_t));
-                }
-                else {
-                        EngFncs->engine_free( Info );
-                        Info = NULL;
-                        rc = ENOMEM;
-                }
 
-
-                if (!rc) {
-
-                        SET_STRING( Info->info[CONTAINER_GETINFO_NODE_ID_INDEX].name,
-                                    CONTAINER_GETINFO_NODE_ID_STRING );
-                        SET_STRING( Info->info[CONTAINER_GETINFO_NODE_ID_INDEX].title,
-                                    CONTAINER_GETINFO_NODE_ID_STRING );
-                        SET_STRING( Info->info[CONTAINER_GETINFO_NODE_ID_INDEX].desc,
-                                    "Enter the owner of this container (nodeid). Every container must have an owner and it must be one of the configured cluster nodes.");
+                        Info->info[CONTAINER_GETINFO_NODE_ID_INDEX].name =
+				EngFncs->engine_strdup( CONTAINER_GETINFO_NODE_ID_STRING );
+                        Info->info[CONTAINER_GETINFO_NODE_ID_INDEX].title = EngFncs->engine_strdup( _("Node") );
+                        Info->info[CONTAINER_GETINFO_NODE_ID_INDEX].desc =
+				EngFncs->engine_strdup( _("Enter the owner of this container (nodeid). Every container must have an owner and it must be one of the configured cluster nodes.") );
                         Info->info[CONTAINER_GETINFO_NODE_ID_INDEX].type               = EVMS_Type_String;
                         Info->info[CONTAINER_GETINFO_NODE_ID_INDEX].unit               = EVMS_Unit_None;
-                        SET_STRING( Info->info[CONTAINER_GETINFO_NODE_ID_INDEX].value.s, str );
+                        Info->info[CONTAINER_GETINFO_NODE_ID_INDEX].value.s = EngFncs->engine_strdup( nodeid_string );
                         Info->info[CONTAINER_GETINFO_NODE_ID_INDEX].collection_type    = EVMS_Collection_None;
                         memset( &Info->info[CONTAINER_GETINFO_NODE_ID_INDEX].group, 0, sizeof(group_info_t));
                 }
                 else {
                         EngFncs->engine_free( Info );
                         Info = NULL;
-                        rc = ENOMEM;
                 }
 
 
@@ -617,8 +767,6 @@ int csm_get_container_info( storage_container_t     *container,
                 rc = ENOMEM;
         }
 
-        if (str) free(str);
-
         *info = Info;
         LOG_EXIT_INT(rc);
         return rc;
@@ -707,8 +855,8 @@ int csm_set_container_info( storage_container_t * container,
 
                         case CONTAINER_SETINFO_NODE_ID_INDEX:
 
-                                rc = csm_string_to_nodeid( options->option[i].value.s,
-                                                           &c_pdata->nodeid );
+                                rc = EngFncs->string_to_nodeid( options->option[i].value.s,
+								&c_pdata->nodeid );
 				if (rc) {
 					LOG_ERROR("Lookup of node name \"%s\" failed with error code %d: %s\n",
 						  options->option[i].value.s, rc, EngFncs->strerror(rc));
@@ -748,8 +896,8 @@ int csm_set_container_info( storage_container_t * container,
                         }
                         else if (strcmp(options->option[i].name,CONTAINER_SETINFO_NODE_ID_STRING) == 0) {
 
-                                rc = csm_string_to_nodeid( options->option[i].value.s,
-                                                           &c_pdata->nodeid);
+                                rc = EngFncs->string_to_nodeid( options->option[i].value.s,
+								&c_pdata->nodeid);
 				if (rc) {
 					LOG_ERROR("Lookup of node name \"%s\" failed with error code %d: %s\n",
 						  options->option[i].value.s, rc, EngFncs->strerror(rc));
@@ -781,17 +929,17 @@ int csm_set_container_info( storage_container_t * container,
 	}
 
         // update container flags
-        if (strcmp(storage_type,"private")==0) {
+        if (strcmp(storage_type,_("private"))==0) {
                 container->flags &= ~SCFLAG_CLUSTER_SHARED;
                 container->flags &= ~SCFLAG_CLUSTER_DEPORTED;
                 container->flags |=  SCFLAG_CLUSTER_PRIVATE;
         }
-        else if (strcmp(storage_type,"shared")==0) {
+        else if (strcmp(storage_type,_("shared"))==0) {
                 container->flags &= ~SCFLAG_CLUSTER_PRIVATE;
                 container->flags &= ~SCFLAG_CLUSTER_DEPORTED;
                 container->flags |=  SCFLAG_CLUSTER_SHARED;
         }
-        else if (strcmp(storage_type,"deported")==0) {
+        else if (strcmp(storage_type,_("deported"))==0) {
                 container->flags &= ~SCFLAG_CLUSTER_PRIVATE;
                 container->flags &= ~SCFLAG_CLUSTER_SHARED;
                 container->flags |=  SCFLAG_CLUSTER_DEPORTED;
@@ -812,7 +960,7 @@ int csm_set_container_info( storage_container_t * container,
 			memset(container->name, '\0', EVMS_NAME_SIZE+1);
 			strcpy(container->name, new_container_name);
 		} else {
-			MESSAGE("Register new container name \"%s\" failed with error code %d: %s\n",
+			MESSAGE(_("Register new container name \"%s\" failed with error code %d: %s\n"),
 				new_container_name, rc, EngFncs->strerror(rc));
 			rc = 0;
 		}
@@ -887,8 +1035,7 @@ int csm_set_container_info( storage_container_t * container,
 					seg = create_csm_data_segment( ld,
 								       container,
 								       hdr1->start_useable,
-								       hdr1->end_useable - hdr1->start_useable + 1,
-								       0 );
+								       hdr1->end_useable - hdr1->start_useable + 1 );
 
 					if (seg != NULL) {
 						insert_csm_segment_into_list( ld->parent_objects, seg );
