@@ -23,8 +23,12 @@
 #ifndef _CSM_DEV_MAPPER_HEADER_
 #define _CSM_DEV_MAPPER_HEADER_ 1
 
+int  csm_can_activate( storage_object_t *object );
+
 int  csm_activate( storage_object_t *object );
 
+int  csm_can_deactivate( storage_object_t *object );
+
 int  csm_deactivate( storage_object_t *object );
 
 #endif
