@@ -161,8 +161,7 @@ DISKSEG * create_csm_metadata_segment( LOGICALDISK         *ld,
 DISKSEG * create_csm_data_segment( LOGICALDISK          *ld,
                                    storage_container_t  *container,
                                    lba_t                 start,
-                                   sector_count_t        size,
-                                   u_int32_t             object_flags)
+                                   sector_count_t        size)
 {
         DISKSEG *seg;
 //      char devname[EVMS_VOLUME_NAME_SIZE+1];
@@ -176,7 +175,6 @@ DISKSEG * create_csm_data_segment( LOGICALDISK          *ld,
                 seg->size        = size;
                 seg->start       = start;
                 seg->data_type   = DATA_TYPE;
-                seg->flags       = object_flags;
 
                 set_segment_storage_type(seg,container);
 
@@ -294,7 +292,7 @@ int insert_csm_segment_into_ordered_list( list_anchor_t seglist, DISKSEG *seg )
                 e = EngFncs->insert_thing( seglist,
                                            seg,
                                            INSERT_BEFORE|EXCLUSIVE_INSERT,
-                                           EngFncs->find_in_list(seglist, seg2, NULL) );
+                                           EngFncs->find_in_list(seglist, seg2, NULL, NULL) );
                 if (e != NULL) {
                         rc = 0;
                 }
@@ -478,89 +476,17 @@ void prune_csm_seg_objects_from_list( list_anchor_t list )
 }
 
 
-/*
- *  Function:  csm_nodeid_to_string
- *
- *  Called to convert a Cluster Unique Identifier (CUID) to an
- *  asciiz string. CUID == ece_nodeid_t
- */
-char * csm_nodeid_to_string( ece_nodeid_t *id )
-{
-        char *nodeid_string=NULL;
-        int rc=0;
-        uint buffer_size = NODEID_STRING_SIZE+1;
-
-        LOG_ENTRY();
-
-        if (id) {
-
-                nodeid_string = (char *) calloc(1,buffer_size);
-                if (nodeid_string) {
-
-                        rc = EngFncs->nodeid_to_string( id,
-                                                        nodeid_string,
-                                                        &buffer_size);
-
-                        if (rc == ENOSPC) {
-                                free(nodeid_string);
-                                nodeid_string = (char *) calloc(1,buffer_size);
-
-                                rc = EngFncs->nodeid_to_string( id,
-                                                                nodeid_string,
-                                                                &buffer_size);
-                        }
-
-                        if (rc) {
-                                free(nodeid_string);
-                                nodeid_string=NULL;
-                        }
-
-                }
-
-        }
-
-        LOG_EXIT_PTR(nodeid_string);
-        return nodeid_string;
-}
-
-
-/*
- *  Function:  csm_string_to_nodeid
- *
- *  Called to convert an asciiz string representing a nodeid to a
- *  Cluster Unique Identifier (CUID) a.k.a. ece_nodeid_t
- */
-int csm_string_to_nodeid( char *str, ece_nodeid_t *nodeid )
-{
-        int rc=0;
-
-        LOG_ENTRY();
-
-        REQUIRE(str != NULL);
-        REQUIRE(nodeid != NULL);
-
-        rc = EngFncs->string_to_nodeid(str,nodeid);
-
-        LOG_EXIT_INT(rc);
-        return rc;
-}
-
-
 static boolean valid_cluster_node( ece_nodeid_t  *nodeid  )
 {
+	int rc;
         boolean  result=FALSE;
-        char    *nodeid_string=NULL;
+        const char *nodeid_string=NULL;
 
         LOG_ENTRY();
 
-        if (nodeid) {
-
-                nodeid_string = csm_nodeid_to_string(nodeid);
-                if (nodeid_string) {
-                        free(nodeid_string);
-                        result=TRUE;
-                }
-
+	rc = EngFncs->nodeid_to_string(nodeid, &nodeid_string);
+        if (nodeid_string) {
+                result=TRUE;
         }
 
         LOG_EXIT_BOOL(result);
