@@ -51,7 +51,7 @@ int csm_can_delete( DISKSEG * seg )
 
 
 int csm_can_expand( DISKSEG         *seg,
-                    sector_count_t  *expand_limit,
+                    sector_count_t   expand_limit,
                     list_anchor_t    expansion_points )
 {
         LOG_ENTRY();
@@ -69,7 +69,7 @@ int csm_can_expand_by( DISKSEG * seg, sector_count_t *size)
 
 
 int csm_can_shrink( DISKSEG          * seg,
-                    sector_count_t   * shrink_limit,
+                    sector_count_t     shrink_limit,
                     list_anchor_t      shrink_points )
 {
         LOG_ENTRY();
@@ -334,8 +334,7 @@ int assign_cluster_segment_manager_to_disk( LOGICALDISK         *ld,
                         dataseg = create_csm_data_segment( ld,
                                                            container,
                                                            start_useable,
-                                                           end_useable - start_useable + 1,
-                                                           SOFLAG_DIRTY | SOFLAG_NEEDS_ACTIVATE );
+                                                           end_useable - start_useable + 1 );
 
                         if (dataseg==NULL) {
                                 rc = ENOMEM;
@@ -426,7 +425,7 @@ static int get_assign_options( option_array_t  *options,
 
                         case CSM_ASSIGN_OPTION_NODE_ID_INDEX:
 
-                                rc = csm_string_to_nodeid( options->option[i].value.s, nodeid);
+                                rc = EngFncs->string_to_nodeid( options->option[i].value.s, nodeid);
 
                                 if (!rc) {
                                         got_nodeid = TRUE;
@@ -456,7 +455,7 @@ static int get_assign_options( option_array_t  *options,
 
                         if (strcmp(options->option[i].name,CSM_ASSIGN_OPTION_NODE_ID_STRING) == 0) {
 
-                                rc = csm_string_to_nodeid( options->option[i].value.s,nodeid);
+                                rc = EngFncs->string_to_nodeid( options->option[i].value.s,nodeid);
 
                                 if (!rc) {
                                         got_nodeid = TRUE;
@@ -529,13 +528,13 @@ int csm_assign( storage_object_t * input_object, option_array_t * options )
                 rc = csm_find_container( container_name, &container );
                 if (rc) {
 
-                        if (strcmp(storage_type,"private")==0) {
+                        if (strcmp(storage_type,_("private"))==0) {
                                 flags |= SCFLAG_CLUSTER_PRIVATE;
                         }
-                        else if (strcmp(storage_type,"shared")==0) {
+                        else if (strcmp(storage_type,_("shared"))==0) {
                                 flags |= SCFLAG_CLUSTER_SHARED;
                         }
-                        else if (strcmp(storage_type,"deported")==0) {
+                        else if (strcmp(storage_type,_("deported"))==0) {
                                 flags |= SCFLAG_CLUSTER_DEPORTED;
                         }
                         else {
@@ -596,8 +595,28 @@ int csm_can_unassign( LOGICALDISK *ld )
         REQUIRE( isa_cluster_logical_disk(ld) == TRUE );
         REQUIRE( csm_has_quorum == TRUE || csm_admin_mode == TRUE);
 
-        LIST_FOR_EACH(ld->parent_objects, iter, seg ) {
+	if ( !csm_admin_mode ) {
+
+		// We can't unassign from a disk if the container is deported or
+		// if the container is private and ownded by another node.  If
+		// either of those cases is true, we cannot check if the data
+		// segment is in use, because it is not produced, and therefore
+		// cannot safely allow the CSM to be unassigned from the disk.
+		REQUIRE( (ld->consuming_container->flags & SCFLAG_CLUSTER_DEPORTED) == 0 );
+
+		if ( ld->consuming_container->flags & SCFLAG_CLUSTER_PRIVATE ) {
+			container_private_data_t *c_pdata;
+			ece_nodeid_t nodeid;
+
+			c_pdata = (container_private_data_t *) ld->consuming_container->private_data;
+			EngFncs->get_nodeid(&nodeid);
+			REQUIRE( memcmp(&c_pdata->nodeid, &nodeid, sizeof(ece_nodeid_t)) == 0 );
+		}
+	}
+
+        LIST_FOR_EACH( ld->parent_objects, iter, seg ) {
                 REQUIRE( EngFncs->list_count(seg->parent_objects) == 0 );
+		REQUIRE( seg->consuming_container == NULL );
         }
 
         LOG_EXIT_INT(0);
@@ -833,6 +852,28 @@ int csm_commit_changes( DISKSEG *seg, uint engine_commit_phase )
 }
 
 
+int csm_backup_metadata( DISKSEG *seg )
+{
+	LOGICALDISK *ld = get_logical_disk(seg);
+	list_element_t iter;
+	DISKSEG *md_seg;
+	int rc = 0;
+
+	LOG_ENTRY();
+
+	LIST_FOR_EACH(ld->parent_objects, iter, md_seg) {
+		if (md_seg->data_type == META_DATA_TYPE) {
+			rc = write_metadata(md_seg, ld, seg, TRUE);
+			if (rc) {
+				break;
+			}
+		}
+	}
+
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
 int csm_read( DISKSEG           *seg,
               lsn_t              lsn,
               sector_count_t     count,
