@@ -29,13 +29,17 @@
 #include <unistd.h>
 #include <sys/ioctl.h>
 #include <sys/stat.h>
+#include <sys/types.h>
 #include <errno.h>
 #include <glob.h>
+#include <malloc.h>
+#include <wait.h>
 
 #include <plugin.h>
 #include <ldm_funcs.h>
 #include "localdskmgr.h"
 #include "cache.h"
+#include "info.h"
 
 engine_functions_t * EngFncs = NULL;
 
@@ -61,6 +65,9 @@ glob_t dev_names_glob;
 int glob_flags;
 static char pattern[PATH_MAX];
 
+static dm_device_list_t * dm_devices = NULL;
+static list_anchor_t multipath_children = NULL;
+
 /**
  * round_down
  * @value:	Value (in sectors) to be rounded down.
@@ -896,6 +903,285 @@ out:
 	return rc;
 }
 
+/**
+ * get_dm_device_list
+ *
+ * Get the list of current DM devices (if we haven't gotten it previously).
+ **/
+static dm_device_list_t * get_dm_device_list(void)
+{
+	int rc;
+
+	LOG_ENTRY();
+
+	if (!dm_devices) {
+		rc = EngFncs->dm_get_devices(&dm_devices);
+		if (rc) {
+			LOG_ERROR("Error calling dm_get_devices.\n");
+		}
+	}
+
+	LOG_EXIT_PTR(dm_devices);
+	return dm_devices;
+}
+
+/**
+ * find_disk_in_dm_devices
+ *
+ * Search the DM devices list for an entry with the same major:minor
+ * as this disk.
+ **/
+static dm_device_list_t * find_disk_in_dm_devices(storage_object_t * disk,
+						  dm_device_list_t * dm_list)
+{
+	dm_device_list_t * dm_entry;
+
+	LOG_ENTRY();
+
+	for (dm_entry = dm_list; dm_entry; dm_entry = dm_entry->next) {
+		if (dm_entry->dev_major == disk->dev_major &&
+		    dm_entry->dev_minor == disk->dev_minor) {
+			goto out;
+		}
+	}
+
+out:
+	LOG_EXIT_PTR(dm_entry);
+	return dm_entry;
+}
+
+/**
+ * check_multipath_name
+ *
+ * Other EVMS plugins can create multipath devices. We *don't* want to
+ * recognize those devices as disks. So check the name that we got from
+ * DM to see if it uses the naming format of the EVMS multipath plugins.
+ **/
+static int check_multipath_name(storage_object_t *disk)
+{
+	int rc;
+
+	LOG_ENTRY();
+
+	/* Multipath-segment-manager devices start with "mp/". */
+	rc = strncmp(disk->name, "mp/", 3);
+	if (rc) {
+		/* MD-multipath devices start with "md/". */
+		rc = strncmp(disk->name, "md/", 3);
+	}
+
+	rc = rc ? 0 : EINVAL;
+
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
+/**
+ * update_multipath_child_list
+ *
+ * Search the multipath target info for all the child devices. Add these
+ * devices to the global list so we can filter these out at the end of
+ * discovery. That way we won't discover both the multipath devices and
+ * their component disks, which would lead to duplicate discoveries in the
+ * higher levels.
+ *
+ * Use a temporary list to build up the list of children for this multipath.
+ * Then append this list to the global one once we've collected all the
+ * children. This will prevent hitting an error part way through processing
+ * the list of child devices, which could leave the global list in an
+ * inconsistent state.
+ **/
+static int update_multipath_child_list(dm_target_t * targets)
+{
+	dm_target_multipath_t * mp = targets->data.multipath;
+	dm_priority_group_t * pg;
+	dm_path_t * path;
+	dm_device_t * device;
+	list_anchor_t children = NULL;
+	list_element_t itr1, itr2;
+	int i, j, rc = 0;
+
+	LOG_ENTRY();
+
+	/* Allocate the global child list if it doesn't exist yet. */
+	if (!multipath_children) {
+		multipath_children = EngFncs->allocate_list();
+		if (!multipath_children) {
+			LOG_ERROR("Error allocating multipath_children list.\n");
+			rc = ENOMEM;
+			goto out;
+		}
+	}
+
+	/* Allocate a temporary list. */
+	children = EngFncs->allocate_list();
+	if (!children) {
+		LOG_ERROR("Error allocating temporary child list.\n");
+		rc = ENOMEM;
+		goto out;
+	}
+
+	/* For each priority group in the multipath. */
+	for (i = 0; i < mp->num_groups; i++) {
+		pg = mp->group + i;
+		/* For each path in the priority group. */
+		for (j = 0; j < pg->num_paths; j++) {
+			path = pg->path + j;
+			device = EngFncs->engine_alloc(sizeof(*device));
+			if (!device) {
+				LOG_ERROR("Error allocating device structure "
+					  "for path %d:%d.\n",
+					  path->device.major, path->device.minor);
+				rc = ENOMEM;
+				goto out;
+			}
+			device->major = path->device.major;
+			device->minor = path->device.minor;
+
+			/* Add this path's device to the temporary list. */
+			itr1 = EngFncs->insert_thing(children, device,
+						     INSERT_AFTER, NULL);
+			if (!itr1) {
+				LOG_ERROR("Error adding device %d:%d to the "
+					  "temporary child list.\n",
+					  device->major, device->minor);
+				rc = ENOMEM;
+				goto out;
+			}
+		}
+	}
+
+out:
+	if (!rc) {
+		/* Success. Append the temporary list to the global list. */
+		rc = EngFncs->merge_lists(multipath_children, children, NULL, NULL);
+		if (rc) {
+			LOG_ERROR("Error merging temporary list with "
+				  "multipath_children list.\n");
+		}
+	}
+	if (rc) {
+		/* Some error occurred. Delete the temporary
+		 * list and all of it's devices.
+		 */
+		if (children) {
+			LIST_FOR_EACH_SAFE(children, itr1, itr2, device) {
+				EngFncs->delete_element(itr1);
+				EngFncs->engine_free(device);
+			}
+		}
+	}
+	if (children) {
+		EngFncs->destroy_list(children);
+	}
+
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
+/**
+ * check_multipath
+ *
+ * Check if this disk is a DM multipath device.
+ **/
+static int check_multipath(storage_object_t * disk)
+{
+	dm_device_list_t * dm_list, * dm_entry;
+	dm_target_t * targets = NULL;
+	local_disk_t * ld = disk->private_data;
+	int rc = 0;
+
+	LOG_ENTRY();
+
+	/* Get the list of active DM devices. */
+	dm_list = get_dm_device_list();
+	if (!dm_list) {
+		LOG_WARNING("Cannot get list of DM devices.\n");
+		goto out;
+	}
+
+	/* Search the DM list for an entry that matches this disk. */
+	dm_entry = find_disk_in_dm_devices(disk, dm_list);
+	if (!dm_entry) {
+		LOG_DEBUG("Disk %s is not a DM device.\n", disk->name);
+		goto out;
+	}
+
+	/* Copy the DM name to this disk. */
+	LOG_DEBUG("Changing disk name from %s to %s.\n",
+		  disk->name, dm_entry->name);
+	strncpy(disk->name, dm_entry->name, EVMS_NAME_SIZE);
+
+	/* Get the DM mapping for this disk. */
+	rc = EngFncs->dm_get_targets(disk, &targets);
+	if (rc) {
+		LOG_ERROR("Error getting DM mapping for disk %s.\n", disk->name);
+		goto out;
+	}
+
+	/* Reject all non-multipath devices. */
+	if (targets->type != DM_TARGET_MULTIPATH) {
+		LOG_DEBUG("Disk %s is not a multipath device.\n", disk->name);
+		rc = EINVAL;
+		goto out;
+	}
+
+	/* Reject all multipath devices that
+	 * were created by other EVMS plugins.
+	 */
+	rc = check_multipath_name(disk);
+	if (rc) {
+		LOG_DEBUG("Multipath disk %s belongs to another EVMS plugin.\n",
+			  disk->name);
+		goto out;
+	}
+
+	rc = update_multipath_child_list(targets);
+	if (rc) {
+		LOG_DEBUG("Error building list of children of "
+			  "multipath disk %s.\n", disk->name);
+		goto out;
+	}
+
+	ld->flags |= LD_FLAG_MULTIPATH;
+
+out:
+	EngFncs->dm_deallocate_targets(targets);
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
+/**
+ * remove_multipath_children
+ *
+ * Compare the multipath_children list with the discovery output list. Any
+ * disks on the multipath_children list must be removed from the output list.
+ **/
+static void remove_multipath_children(list_anchor_t multipath_children,
+				      list_anchor_t output_list)
+{
+	list_element_t itr1, itr2, itr3;
+	storage_object_t * disk;
+	dm_device_t * child;
+
+	LOG_ENTRY();
+
+	LIST_FOR_EACH(multipath_children, itr3, child) {
+		LIST_FOR_EACH_SAFE(output_list, itr1, itr2, disk) {
+			if (child->major == disk->dev_major &&
+			    child->minor == disk->dev_minor) {
+				EngFncs->delete_element(itr1);
+				close_dev(disk);
+				EngFncs->engine_free(disk->private_data);
+				disk->flags &= ~SOFLAG_ACTIVE;
+				EngFncs->free_logical_disk(disk);
+			}
+		}
+	}
+
+	LOG_EXIT_VOID();
+}
+
 /**
  * get_geometry
  *
@@ -952,12 +1238,15 @@ out:
  **/
 static int get_fake_geometry(storage_object_t * disk)
 {
+	local_disk_t * ld = disk->private_data;
 	int rc = EINVAL;
 
 	LOG_ENTRY();
 
 	if (disk->dev_major == LOOP_MAJOR ||
-	    disk->dev_major == NBD_MAJOR) {
+	    disk->dev_major == NBD_MAJOR ||
+	    disk->dev_major == DRBD_MAJOR ||
+	    ld->flags & LD_FLAG_MULTIPATH) {
 		LOG_DEBUG("Creating fake geometry for disk %s.\n", disk->name);
 		disk->geometry.heads = 255;
 		disk->geometry.sectors_per_track = 63;
@@ -1051,6 +1340,35 @@ static void get_hardsector_size(storage_object_t * disk)
 	LOG_EXIT_VOID();
 }
 
+/**
+ * find_disk_type
+ *
+ * Determine if this disk is IDE, SCSI, or something else. This is done by
+ * examining the name of the disk. Names starting with "hd" or that contain
+ * "ide" will be marked as IDE. Names starting with "sd" or that contain
+ * "scsi" will be marked as SCSI.
+ **/
+static void find_disk_type(storage_object_t * disk)
+{
+	local_disk_t * ld = disk->private_data;
+
+	LOG_ENTRY();
+
+	if (!strncmp(disk->name, "hd", 2) ||
+	    strstr(disk->name, "ide")) {
+		ld->flags |= LD_FLAG_IDE;
+	} else if (!strncmp(disk->name, "sd", 2) ||
+		   strstr(disk->name, "scsi")) {
+		ld->flags |= LD_FLAG_SCSI;
+	}
+
+	LOG_DEBUG("Type of disk %s is %s\n", disk->name,
+		  (ld->flags & LD_FLAG_IDE) ? "IDE" :
+		  (ld->flags & LD_FLAG_SCSI) ? "SCSI" : "Unknown");
+
+	LOG_EXIT_VOID();
+}
+
 /**
  * create_logical_disk
  *
@@ -1072,7 +1390,7 @@ static storage_object_t * create_logical_disk(storage_object_t * working_disk)
 		goto out;
 	}
 
-	disk->private_data = EngFncs->engine_alloc(sizeof(int));
+	disk->private_data = EngFncs->engine_alloc(sizeof(local_disk_t));
 	if (!disk->private_data) {
 		LOG_SERIOUS("Error allocating private data for disk %s.\n",
 			    disk->name);
@@ -1105,6 +1423,8 @@ static storage_object_t * create_logical_disk(storage_object_t * working_disk)
 
 	ld->fd = working_ld->fd;
 
+	find_disk_type(disk);
+
 	LOG_DETAILS("New Logical Disk:\n");
 	LOG_DETAILS("  name:            %s\n", disk->name);
 	LOG_DETAILS("  size:            %"PRIu64"\n", disk->size);
@@ -1154,6 +1474,7 @@ static int LD_discover(list_anchor_t input_list,
 		memset(&working_disk, 0, sizeof(working_disk));
 		working_disk.private_data = &working_ld;
 		working_ld.fd = -1;
+		working_ld.flags = 0;
 		strncpy(working_disk.name, full_node_path + base_directory_len,
 			EVMS_NAME_SIZE);
 
@@ -1175,6 +1496,13 @@ static int LD_discover(list_anchor_t input_list,
 			continue;
 		}
 
+		/* Check for DM-multipath devices. */
+		rc = check_multipath(&working_disk);
+		if (rc) {
+			close_dev(&working_disk);
+			continue;
+		}
+
 		/* Get the disk's geometry. */
 		rc = get_geometry(&working_disk);
 		if (rc) {
@@ -1215,6 +1543,11 @@ static int LD_discover(list_anchor_t input_list,
 		}
 	}
 
+	remove_multipath_children(multipath_children, output_list);
+
+	EngFncs->dm_deallocate_device_list(dm_devices);
+	EngFncs->destroy_list(multipath_children);
+
 	count = EngFncs->list_count(output_list);
 	LOG_DEBUG("Discovered %d disks.\n", count);
 	LOG_EXIT_INT(0);
@@ -1253,6 +1586,44 @@ static int get_alignment_size(storage_object_t * disk)
 	return size;
 }
 
+/**
+ * check_alignment
+ * @align_size:
+ * @offset:
+ * @count:
+ * @buffer:
+ *
+ * Check whether the specified offset, count, and buffer are valid for the
+ * specified alignment restriction.
+ **/
+static int check_alignment(int align_size,
+			   lsn_t offset,
+			   sector_count_t count,
+			   void * buffer)
+{
+	int align_sectors = align_size >> EVMS_VSECTOR_SIZE_SHIFT;
+	int align_mask = align_size - 1;
+	int rc = 0;
+
+	LOG_ENTRY();
+	LOG_EXTRA("Checking alignment.\n");
+	LOG_EXTRA("\tAlignment Size: %d bytes\n", align_size);
+	LOG_EXTRA("\tBuffer:         0x%p\n", buffer);
+	LOG_EXTRA("\tSector Offset:  %"PRIu64"\n", offset);
+	LOG_EXTRA("\tSector Count:   %"PRIu64"\n", count);
+
+	if ((unsigned long)buffer & align_mask) {
+		rc = EINVAL;
+	} else if (offset % align_sectors) {
+		rc = EINVAL;
+	} else if (count % align_sectors) {
+		rc = EINVAL;
+	}
+
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
 /**
  * get_aligned_buffer
  * @offset:	Starting offset (in sectors) of engine I/O request.
@@ -1260,10 +1631,7 @@ static int get_alignment_size(storage_object_t * disk)
  * @align_size:	Size (in bytes) that the I/O must be aligned on.
  * @local_offset:	Aligned starting offset (in sectors).
  * @local_count:	Aligned I/O size (in sectors).
- * @local_buffer:	New data buffer. Not aligned, but need to return this
- *			pointer so it can be freed after using.
- * @local_buffer_aligned:	Aligned data buffer. This points somewhere
- *				within local_buffer.
+ * @buffer:	Aligned data buffer.
  *
  * To use O_DIRECT, the buffer passed to read() or write() must be aligned on
  * the device's block/sector size. The size and starting offset of the I/O must
@@ -1274,12 +1642,9 @@ static int get_aligned_buffer(lsn_t offset,
 			      int align_size,
 			      lsn_t * local_offset,
 			      sector_count_t * local_count,
-			      char ** local_buffer,
-			      char ** local_buffer_aligned)
+			      void ** buffer)
 {
-	u_int32_t offset_diff, offset_diff_bytes;
-	u_int32_t local_buffer_bytes;
-	u_int32_t alignment_base, alignment_diff;
+	u_int32_t offset_diff;
 	int rc = 0;
 
 	LOG_ENTRY();
@@ -1289,36 +1654,19 @@ static int get_aligned_buffer(lsn_t offset,
 
 	/* Difference between real offset and local offset. */
 	offset_diff = offset - *local_offset;
-	offset_diff_bytes = offset_diff << EVMS_VSECTOR_SIZE_SHIFT;
 
 	/* Round up total count of sectors to alignment size. */
 	*local_count = round_up(count + offset_diff, align_size);
 
-	/* Allocate the buffer that will actually perform the I/O. This needs
-	 * to allocate enough extra to compensate for the alignment that will
-	 * occur next.
+	/* Allocate the buffer that will actually perform the I/O. The
+	 * memalign call guarantees that the allocated buffer is
+	 * aligned on the desired alignment-size.
 	 */
-	local_buffer_bytes = *local_count << EVMS_VSECTOR_SIZE_SHIFT;
-	*local_buffer = EngFncs->engine_alloc(local_buffer_bytes +
-					      align_size - 1);
-	if (!*local_buffer) {
+	*buffer = memalign(align_size, *local_count << EVMS_VSECTOR_SIZE_SHIFT);
+	if (!*buffer) {
 		rc = ENOMEM;
-		goto out;
 	}
 
-	/* Align the buffer on hard-sector-size. */
-	alignment_base = (unsigned long)(*local_buffer) & (align_size - 1);
-	alignment_diff = (alignment_base + align_size - 1) & ~(align_size - 1);
-	alignment_diff -= alignment_base;
-	*local_buffer_aligned = *local_buffer + alignment_diff;
-
-	/* Sanity check */
-	if (((unsigned long)(*local_buffer_aligned) % align_size) != 0) {
-		LOG_ERROR("BUFFER NOT ALIGNED!!: buf: %p, size: %u\n",
-			  *local_buffer_aligned, align_size);
-	}
-
-out:
 	LOG_EXIT_INT(rc);
 	return rc;
 }
@@ -1331,11 +1679,11 @@ static int LD_read(storage_object_t * disk,
 		   sector_count_t count,
 		   void * buffer)
 {
-	char * local_buffer = NULL, * local_buffer_aligned;
+	void * local_buffer = NULL;
 	lsn_t local_offset;
 	sector_count_t local_count;
 	local_disk_t * ld = disk->private_data;
-	int rc, align_size;
+	int rc, align_size, aligned = FALSE;
 
 	LOG_ENTRY();
 	LOG_DEBUG("Read disk:%s offset:%"PRIu64" count:%"PRIu64"\n",
@@ -1362,16 +1710,26 @@ static int LD_read(storage_object_t * disk,
 	/* Get the alignment restriction for O_DIRECT. */
 	align_size = get_alignment_size(disk);
 
-	/* Get a data buffer aligned with this restriction. */
-	rc = get_aligned_buffer(offset, count, align_size,
-				&local_offset, &local_count,
-				&local_buffer, &local_buffer_aligned);
+	/* Check if the supplied buffer, offset, and count
+	 * are valid for the alignment restrictions.
+	 */
+	rc = check_alignment(align_size, offset, count, buffer);
 	if (rc) {
-		goto out;
+		/* Get a data buffer aligned with this restriction. */
+		rc = get_aligned_buffer(offset, count, align_size, &local_offset,
+					&local_count, &local_buffer);
+		if (rc) {
+			goto out;
+		}
+	} else {
+		aligned = TRUE;
+		local_offset = offset;
+		local_count = count;
+		local_buffer = buffer;
 	}
 
 	/* Send the read to the engine. */
-	rc = EngFncs->read_object(disk, ld->fd, local_buffer_aligned,
+	rc = EngFncs->read_object(disk, ld->fd, local_buffer,
 				  local_count << EVMS_VSECTOR_SIZE_SHIFT,
 				  local_offset << EVMS_VSECTOR_SIZE_SHIFT);
 	if (rc < 0) {
@@ -1380,16 +1738,20 @@ static int LD_read(storage_object_t * disk,
 	}
 
 	/* Copy the data back to the caller's buffer. */
-	memcpy(buffer, local_buffer_aligned +
-		       ((offset - local_offset) << EVMS_VSECTOR_SIZE_SHIFT),
-	       count << EVMS_VSECTOR_SIZE_SHIFT);
+	if (!aligned) {
+		memcpy(buffer, local_buffer +
+			       ((offset - local_offset) << EVMS_VSECTOR_SIZE_SHIFT),
+		       count << EVMS_VSECTOR_SIZE_SHIFT);
+	}
 
 	/* Record this I/O in the cache. */
 	write_to_cache(disk, offset, count, buffer);
 	rc = 0;
 
 out:
-	EngFncs->engine_free(local_buffer);
+	if (!aligned) {
+		free(local_buffer);
+	}
 	LOG_EXIT_INT(rc);
 	return rc;
 }
@@ -1402,11 +1764,11 @@ static int LD_write(storage_object_t * disk,
 		    sector_count_t count,
 		    void * buffer)
 {
-	char * local_buffer = NULL, * local_buffer_aligned;
+	void * local_buffer = NULL;
 	lsn_t local_offset;
 	sector_count_t local_count;
 	local_disk_t * ld = disk->private_data;
-	int rc, align_size;
+	int rc, align_size, aligned = FALSE;
 
 	LOG_ENTRY();
 	LOG_DEBUG("Write disk:%s offset:%"PRIu64" count:%"PRIu64"\n",
@@ -1427,16 +1789,26 @@ static int LD_write(storage_object_t * disk,
 	/* Get the alignment restriction for O_DIRECT. */
 	align_size = get_alignment_size(disk);
 
-	/* Get a data buffer aligned with this restriction. */
-	rc = get_aligned_buffer(offset, count, align_size,
-				&local_offset, &local_count,
-				&local_buffer, &local_buffer_aligned);
+	/* Check if the supplied buffer, offset, and count
+	 * are valid for the alignment restrictions.
+	 */
+	rc = check_alignment(align_size, offset, count, buffer);
 	if (rc) {
-		goto out;
+		/* Get a data buffer aligned with this restriction. */
+		rc = get_aligned_buffer(offset, count, align_size, &local_offset,
+					&local_count, &local_buffer);
+		if (rc) {
+			goto out;
+		}
+	} else {
+		aligned = TRUE;
+		local_offset = offset;
+		local_count = count;
+		local_buffer = buffer;
 	}
 
 	if (local_count != count) {
-		rc = EngFncs->read_object(disk, ld->fd, local_buffer_aligned,
+		rc = EngFncs->read_object(disk, ld->fd, local_buffer,
 					  local_count << EVMS_VSECTOR_SIZE_SHIFT,
 					  local_offset << EVMS_VSECTOR_SIZE_SHIFT);
 		if (rc < 0) {
@@ -1446,12 +1818,14 @@ static int LD_write(storage_object_t * disk,
 	}
 	
 	/* Put user data at the right place in the buffer */
-	memcpy(local_buffer_aligned +
-	       ((offset - local_offset) << EVMS_VSECTOR_SIZE_SHIFT),
-	       buffer, count << EVMS_VSECTOR_SIZE_SHIFT);
+	if (!aligned) {
+		memcpy(local_buffer +
+		       ((offset - local_offset) << EVMS_VSECTOR_SIZE_SHIFT),
+		       buffer, count << EVMS_VSECTOR_SIZE_SHIFT);
+	}
 	
 	/* Send the write to the engine. */
-	rc = EngFncs->write_object(disk, ld->fd, local_buffer_aligned,
+	rc = EngFncs->write_object(disk, ld->fd, local_buffer,
 				   local_count << EVMS_VSECTOR_SIZE_SHIFT,
 				   local_offset << EVMS_VSECTOR_SIZE_SHIFT);
 	if (rc < 0) {
@@ -1468,7 +1842,9 @@ static int LD_write(storage_object_t * disk,
 	rc = 0;
 
 out:
-	EngFncs->engine_free(local_buffer);
+	if (!aligned) {
+		free(local_buffer);
+	}
 	LOG_EXIT_INT(rc);
 	return rc;
 }
@@ -1522,107 +1898,29 @@ static int LD_commit_changes(storage_object_t * disk, commit_phase_t phase)
 /**
  * LD_get_info
  *
- * Returns DISK specific information
+ * Return information about this disk to display to the user.
  **/
 static int LD_get_info(storage_object_t * disk,
-		       char * descriptor_name,
+		       char * name,
 		       extended_info_array_t ** info)
 {
-	extended_info_array_t * Info;
-	uint count = 0;
-	int index = 0, rc = EINVAL;
+	local_disk_t * ld = disk->private_data;
+	int rc = EINVAL;
 
 	LOG_ENTRY();
 
-	if (!info) {
-		goto out;
-	}
 	*info = NULL;
 	
-        if (descriptor_name) {
-		goto out;
+	if (!name) {
+		rc = get_basic_info(disk, info);
+	} else if (!strncasecmp(name, "Type", 4)) {
+		if (ld->flags & LD_FLAG_IDE) {
+			rc = get_ide_info(disk, info);
+		} else if (ld->flags & LD_FLAG_SCSI) {
+			rc = get_scsi_info(disk, info);
+		}
 	}
-	
-	Info = EngFncs->engine_alloc(sizeof(extended_info_array_t) +
-				     8 * sizeof(extended_info_t));
-	if (!Info) {
-		rc = ENOMEM;
-		goto out;
-	}
-		
-	SET_STRING(Info->info[index].name, "Name");
-	SET_STRING(Info->info[index].title, "Name");
-	SET_STRING(Info->info[index].desc, "EVMS name for the DISK storage object");
-	Info->info[index].type = EVMS_Type_String;
-	SET_STRING(Info->info[index].value.s, disk->name);
-	index++;
 
-	SET_STRING(Info->info[index].name, "Size");
-	SET_STRING(Info->info[index].title, "Size");
-	SET_STRING(Info->info[index].desc, "Size of the disk in sectors");
-	Info->info[index].type = EVMS_Type_Unsigned_Int64;
-	Info->info[index].unit = EVMS_Unit_Sectors;
-	Info->info[index].flags |= EVMS_EINFO_FLAGS_NO_UNIT_CONVERSION;
-	Info->info[index].value.ui64 = disk->size;
-	index++;
-
-	SET_STRING(Info->info[index].name, "Cyl");
-	SET_STRING(Info->info[index].title, "Cylinders");
-	SET_STRING(Info->info[index].desc, "Drive geometry -- number of cylinders");
-	Info->info[index].type = EVMS_Type_Unsigned_Int64;
-	Info->info[index].value.ui64 = disk->geometry.cylinders;
-	index++;
-
-	SET_STRING(Info->info[index].name, "Heads");
-	SET_STRING(Info->info[index].title, "Heads");
-	SET_STRING(Info->info[index].desc, "Drive geometry -- number of heads");
-	Info->info[index].type = EVMS_Type_Unsigned_Int32;
-	Info->info[index].value.ui32 = disk->geometry.heads;
-	index++;
-
-	SET_STRING(Info->info[index].name, "Sectors");
-	SET_STRING(Info->info[index].title, "Sectors");
-	SET_STRING(Info->info[index].desc, "Drive geometry -- sectors per track");
-	Info->info[index].type = EVMS_Type_Unsigned_Int32;
-	Info->info[index].value.ui32 = disk->geometry.sectors_per_track;
-	index++;
-
-	SET_STRING(Info->info[index].name, "SectorSize");
-	SET_STRING(Info->info[index].title, "Sector Size");
-	SET_STRING(Info->info[index].desc, "Number of bytes per sector");
-	Info->info[index].type = EVMS_Type_Unsigned_Int32;
-	Info->info[index].unit = EVMS_Unit_Bytes;
-	Info->info[index].value.ui32 = disk->geometry.bytes_per_sector;
-	index++;
-
-	SET_STRING(Info->info[index].name, "BlockSize");
-	SET_STRING(Info->info[index].title, "Block Size");
-	SET_STRING(Info->info[index].desc, "Number of bytes per block");
-	Info->info[index].type = EVMS_Type_Unsigned_Int64;
-	Info->info[index].unit = EVMS_Unit_Bytes;
-	Info->info[index].value.ui64 = disk->geometry.block_size;
-	index++;
-
-	SET_STRING(Info->info[index].name, "BootLimit");
-	SET_STRING(Info->info[index].title, "Boot Cylinder Limit");
-	SET_STRING(Info->info[index].desc, "LBA of the first sector above the boot cylinder limit for this drive");
-	Info->info[index].type = EVMS_Type_Unsigned_Int64;
-	Info->info[index].value.ui64 = disk->geometry.boot_cylinder_limit;
-	index++;
-
-	count = EngFncs->list_count(disk->parent_objects);
-	SET_STRING(Info->info[index].name, "Segments");
-	SET_STRING(Info->info[index].title, "Segments");
-	SET_STRING(Info->info[index].desc, "Number of segments discovered on the drive (metadata, data, freespace)");
-	Info->info[index].type = EVMS_Type_Unsigned_Int32;
-	Info->info[index].value.ui32 = count;
-	index++;
-
-	Info->count = index;
-	*info = Info;
-	rc = 0;
-
-out:
 	LOG_EXIT_INT(rc);
 	return rc;
 }
@@ -1674,41 +1972,43 @@ static int LD_get_plugin_info(char * descriptor_name,
 		my_plugin_record->required_plugin_api_version.plugin.minor,
 		my_plugin_record->required_plugin_api_version.plugin.patchlevel);
 
-	SET_STRING(Info->info[0].name, "Short Name");
-	SET_STRING(Info->info[0].title, "Short Name");
-	SET_STRING(Info->info[0].desc, "A short name given to this plugin");
+	Info->info[0].name = EngFncs->engine_strdup("Short Name");
+	Info->info[0].title = EngFncs->engine_strdup(_("Short Name"));
+	Info->info[0].desc = EngFncs->engine_strdup(_("A short name given to this plug-in"));
 	Info->info[0].type = EVMS_Type_String;
-	SET_STRING(Info->info[0].value.s, my_plugin_record->short_name);
+	Info->info[0].value.s = EngFncs->engine_strdup(my_plugin_record->short_name);
 
-	SET_STRING(Info->info[1].name, "Long Name");
-	SET_STRING(Info->info[1].title, "Long Name");
-	SET_STRING(Info->info[1].desc, "A long name given to this plugin");
+	Info->info[1].name = EngFncs->engine_strdup("Long Name");
+	Info->info[1].title = EngFncs->engine_strdup(_("Long Name"));
+	Info->info[1].desc = EngFncs->engine_strdup(_("A longer, more descriptive name for this plug-in"));
 	Info->info[1].type = EVMS_Type_String;
-	SET_STRING(Info->info[1].value.s, my_plugin_record->long_name);
+	Info->info[1].value.s = EngFncs->engine_strdup(my_plugin_record->long_name);
 
-	SET_STRING(Info->info[2].name, "Type");
-	SET_STRING(Info->info[2].title, "Plug-in Type");
-	SET_STRING(Info->info[2].desc, "There are various types of plugins, each responsible for some kind of storage object.");
+	Info->info[2].name = EngFncs->engine_strdup("Type");
+	Info->info[2].title = EngFncs->engine_strdup(_("Plug-in Type"));
+	Info->info[2].desc = EngFncs->engine_strdup(_("There are various types of plug-ins, each responsible for some kind of storage object or logical volume."));
 	Info->info[2].type = EVMS_Type_String;
-	SET_STRING(Info->info[2].value.s, "Device Manager");
+	Info->info[2].value.s = EngFncs->engine_strdup(_("Device Manager"));
 
-	SET_STRING(Info->info[3].name, "Version");
-	SET_STRING(Info->info[3].title, "Plugin Version");
-	SET_STRING(Info->info[3].desc, "Version number of this plug-in");
+	Info->info[3].name = EngFncs->engine_strdup("Version");
+	Info->info[3].title = EngFncs->engine_strdup(_("Plug-in Version"));
+	Info->info[3].desc = EngFncs->engine_strdup(_("Version number of this plug-in"));
 	Info->info[3].type = EVMS_Type_String;
-	SET_STRING(Info->info[3].value.s, version_string);
+	Info->info[3].value.s = EngFncs->engine_strdup(version_string);
 
-	SET_STRING(Info->info[4].name, "Required Engine Services Version");
-	SET_STRING(Info->info[4].title, "Required Engine Services Version");
-	SET_STRING(Info->info[4].desc, "Version of the Engine services that this plug-in requires. It will not run on older versions of the Engine services.");
+	Info->info[4].name = EngFncs->engine_strdup("Required Engine Services Version");
+	Info->info[4].title = EngFncs->engine_strdup(_("Required Engine Services Version"));
+	Info->info[4].desc = EngFncs->engine_strdup(_("Version of the Engine services that this plug-in requires.  "
+						      "It will not run on older versions of the Engine services."));
 	Info->info[4].type = EVMS_Type_String;
-	SET_STRING(Info->info[4].value.s, required_engine_api_version_string);
+	Info->info[4].value.s = EngFncs->engine_strdup(required_engine_api_version_string);
 
-	SET_STRING(Info->info[5].name, "Required Plug-in API Version");
-	SET_STRING(Info->info[5].title, "Required Plug-in API Version");
-	SET_STRING(Info->info[5].desc, "Version of the Engine plug-in API that this plug-in requires. It will not run on older versions of the Engine plug-in API.");
+	Info->info[5].name = EngFncs->engine_strdup("Required Plug-in API Version");
+	Info->info[5].title = EngFncs->engine_strdup(_("Required Plug-in API Version"));
+	Info->info[5].desc = EngFncs->engine_strdup(_("Version of the Engine plug-in API that this plug-in requires.  "
+						      "It will not run on older versions of the Engine plug-in API."));
 	Info->info[5].type = EVMS_Type_String;
-	SET_STRING(Info->info[5].value.s, required_plugin_api_version_string);
+	Info->info[5].value.s = EngFncs->engine_strdup(required_plugin_api_version_string);
 
 	*info = Info;
 	rc = 0;
@@ -1742,6 +2042,8 @@ static int LD_plugin_function(storage_object_t * object,
 {
 	int rc = 0;
 
+	LOG_ENTRY();
+
 	switch (action) {
 		case LDM_Start_Caching:
 			LOG_DEBUG("Start caching\n");
@@ -1787,6 +2089,25 @@ static int LD_plugin_function(storage_object_t * object,
 }
 
 
+static int LD_backup_metadata(storage_object_t * disk) {
+
+	int rc;
+
+	LOG_ENTRY();
+
+	if (disk->plugin->id != LDM_PLUGIN_ID_2) {
+		LOG_ERROR("I don't own object %s.\n", disk->name);
+		LOG_EXIT_INT(EINVAL);
+		return EINVAL;
+        }
+
+	rc = EngFncs->save_metadata(disk->name, NULL, 0, 0, NULL);
+
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
+
 static plugin_functions_t ft_sysfs = {
 	.setup_evms_plugin		= LD_setup,
 	.cleanup_evms_plugin		= LD_cleanup,
@@ -1799,7 +2120,8 @@ static plugin_functions_t ft_sysfs = {
 	.read				= LD_read,
 	.write				= LD_write,
 	.get_plugin_functions		= LD_get_plugin_functions,
-	.plugin_function		= LD_plugin_function
+	.plugin_function		= LD_plugin_function,
+	.backup_metadata		= LD_backup_metadata
 };
 
 plugin_record_t LD_Plugin = {
@@ -1810,14 +2132,14 @@ plugin_record_t LD_Plugin = {
 		.patchlevel	= PATCH_LEVEL
 	},
 	.required_engine_api_version = {
-		.major		= 13,
+		.major		= 15,
 		.minor		= 0,
 		.patchlevel	= 0
 	},
 	.required_plugin_api_version = {
 		.plugin = {
-			.major		= 12,
-			.minor		= 0,
+			.major		= 13,
+			.minor		= 1,
 			.patchlevel	= 0
 		}
 	},
