@@ -504,11 +504,15 @@ int      isa_valid_partition_table_chain( LOGICALDISK        *ld,		  // disk
 
 				if (isa_ebr_partition_record(part)==TRUE) {
 					if (ebr_chaining == TRUE) {
-						MESSAGE("A partition record, chaining logical drives in the extended partition on drive %s, is invalid.\nSkipping segment discovery on this drive.\nSee /var/log/evmsEngine.log for more info.\n",
+						MESSAGE(_("A partition record chaining logical drives in the extended partition on drive %s is not valid.\n"
+							  "Skipping segment discovery on this drive.\n"
+							  "See the EVMS log for more information.\n"),
 							ld->name );
 					}
 					else {
-						MESSAGE("The partition record, describing the extended partition on drive %s, is invalid.\nSkipping segment discovery on this drive.\nSee /var/log/evmsEngine.log for more info.\n",
+						MESSAGE(_("The partition record describing the extended partition on drive %s is not valid.\n"
+							  "Skipping segment discovery on this drive.\n"
+							  "See the EVMS log for more information.\n"),
 							ld->name );
 					}
 				}
@@ -519,7 +523,8 @@ int      isa_valid_partition_table_chain( LOGICALDISK        *ld,		  // disk
 					else {
 						sprintf(pname, "%s%d", ld->name, partition_count+1 );
 					}
-					MESSAGE("A partition record, describing partition %s, is invalid.\nSkipping segment discovery on this drive %s.\n",
+					MESSAGE(_("A partition record describing partition %s is not valid.\n"
+						  "Skipping segment discovery on this drive %s.\n"),
 						pname, ld->name );
 				}
 
@@ -548,7 +553,9 @@ int      isa_valid_partition_table_chain( LOGICALDISK        *ld,		  // disk
 	if ( (extended_partition_count > 1) &&
 	     (ebr_chaining == FALSE)) {
 		LOG_ERROR("error, found more than 1 extended partition on disk %s\n", ld->name);
-		MESSAGE( "Found more than 1 extended partition on disk %s.\nSkipping segment discovery on this disk.\n", ld->name);
+		MESSAGE(_("Found more than 1 extended partition on disk %s.\n"
+			  "Skipping segment discovery on this disk.\n"),
+			 ld->name);
 		LOG_EXIT_INT(ENOSYS);
 		return ENOSYS;	// will not support this disk
 	}
@@ -559,7 +566,9 @@ int      isa_valid_partition_table_chain( LOGICALDISK        *ld,		  // disk
 	if ( (extended_partition_count > 1) &&
 	     (ebr_chaining == TRUE)) {
 		LOG_ERROR("error, found more than 1 extended partition record in an EBR partition table on disk %s.\n", ld->name);
-		MESSAGE( "Found more than 1 extended partition record in an EBR partition table on disk %s.\nSkipping segment discovery on this disk.\n", ld->name);
+		MESSAGE(_("Found more than 1 extended partition record in an EBR partition table on disk %s.\n"
+			  "Skipping segment discovery on this disk.\n"),
+			 ld->name);
 		LOG_EXIT_INT(ENOSYS);
 		return ENOSYS;	// will not support this disk
 	}
@@ -570,7 +579,9 @@ int      isa_valid_partition_table_chain( LOGICALDISK        *ld,		  // disk
 	if ( ( partition_count > 1) &&
 	     ( ebr_chaining == TRUE ) ) {
 		LOG_ERROR("error, found more than 1 logical partition in an EBR partition table on disk %s\n", ld->name);
-		MESSAGE( "Found more than 1 logical partition in an EBR partition table on disk %s.\nSkipping segment discovery on this disk.\n", ld->name);
+		MESSAGE(_("Found more than 1 logical partition in an EBR partition table on disk %s.\n"
+			  "Skipping segment discovery on this disk.\n"),
+			ld->name);
 		LOG_EXIT_INT(ENOSYS);
 		return ENOSYS;	// will not support this disk
 	}
