@@ -24,5 +24,8 @@
 #ifndef DEFSEGMGR_HEADER
 #define DEFSEGMGR_HEADER
 
+extern plugin_record_t                 *Seg_My_PluginRecord_Ptr;
+#define my_plugin_record                Seg_My_PluginRecord_Ptr
+extern struct engine_functions_s       *EngFncs;
 
 #endif
