@@ -342,7 +342,7 @@ DLA_Table_Sector * Read_Dlat_Sector( LOGICALDISK *ld,  lba_t ebr_lba  )
  *  for converting the DLAT to DISK format, updating the CRC and doing
  *  the I/O.
  */
-int Write_Dlat_Sector( LOGICALDISK *ld, DISKSEG *boot_seg )
+int Write_Dlat_Sector( LOGICALDISK *ld, DISKSEG *boot_seg, DISKSEG *parent, boolean backup )
 {
 	DLA_Table_Sector           *dlat_buffer=NULL;
 	u_int32_t                   Calculated_CRC;
@@ -371,7 +371,11 @@ int Write_Dlat_Sector( LOGICALDISK *ld, DISKSEG *boot_seg )
 				dlat_lba = (lba_t) (disk_pdata->geometry.sectors_per_track - 1) + boot_seg->start;
 
 				Fncs = (struct plugin_functions_s *)ld->plugin->functions.plugin;
-				rc = Fncs->write( ld, dlat_lba, 1,(void *) dlat_buffer);
+				if (backup) {
+					rc = EngFncs->save_metadata(parent->name, ld->name, dlat_lba, 1, dlat_buffer);
+				} else {
+					rc = Fncs->write( ld, dlat_lba, 1,(void *) dlat_buffer);
+				}
 
 				rc = 0;
 			}
