@@ -25,8 +25,12 @@
 
 int dl_get_devmap_info( storage_object_t *drivelink );
 	
+int dl_can_activate( storage_object_t *drivelink );
+
 int dl_activate( storage_object_t *drivelink );
 
+int dl_can_deactivate( storage_object_t *drivelink );
+
 int dl_deactivate( storage_object_t *drivelink );
 
 #endif
