@@ -30,7 +30,7 @@
 
 #include "dl_common.h"
 
- 
+
 /*
  *  Function: dl_shrink_drivelink
  *
@@ -81,19 +81,19 @@ static int dl_shrink_drivelink( storage_object_t  *drivelink,
 
                 rc = fncs->add_sectors_to_kill_list( child,
                                                      feature_header->feature_data1_start_lsn,
-                                                     feature_header->feature_data1_size );              
+                                                     feature_header->feature_data1_size );
                 if (rc) {
                         LOG_ERROR("error, add kill sectors call failed\n");
                 }
 
                 rc = fncs->add_sectors_to_kill_list( child,
                                                      feature_header->feature_data2_start_lsn,
-                                                     feature_header->feature_data2_size );                   
+                                                     feature_header->feature_data2_size );
                 if (rc) {
                         LOG_ERROR("error, add kill sectors call failed\n");
                 }
         }
-                
+
         (pdata->ordering_table+index)->child_serial_number = 0;
         (pdata->ordering_table+index)->child_vsize         = 0;
 
@@ -127,6 +127,7 @@ static int dl_expand_drivelink( storage_object_t *drivelink,
 {
         storage_object_t  *child;
         u_int32_t          child_serial_number;
+        u_int64_t          expand_size = 0;
         drivelink_private_data_t *pdata;
         list_element_t iter;
         int  rc=0,i;
@@ -138,6 +139,18 @@ static int dl_expand_drivelink( storage_object_t *drivelink,
 
         pdata = (drivelink_private_data_t *)drivelink->private_data;
 
+	// Check with the engine before starting the expand.
+	LIST_FOR_EACH(expand_objects, iter, child) {
+		expand_size += (child->size - 2*DRIVELINK_METADATA_SECTOR_COUNT -
+				2*FEATURE_HEADER_SECTOR_COUNT);
+	}
+	rc = EngFncs->can_expand_by(drivelink, &expand_size);
+	if (rc) {
+		LOG_ERROR("Expand of object %s rejected by the engine.\n", drivelink->name);
+		LOG_EXIT_INT(rc);
+		return rc;
+	}
+
         // get setup for any possible unwinding of the expand
         original_link_count = pdata->drive_link_count;
 
@@ -149,18 +162,18 @@ static int dl_expand_drivelink( storage_object_t *drivelink,
 
                 // create a feature header for the child object
                 child->feature_header = (evms_feature_header_t *) EngFncs->engine_alloc( sizeof(evms_feature_header_t) );
-                
+
                 if (child->feature_header==NULL){
                         rc=ENOMEM;
                         break;
                 }
-                
+
                 // create a drive link serial number for the child object we are consuming .
                 // The routine will check that it is unique for the ordering table.
-                child_serial_number = dl_gen_child_serial_number( drivelink );                                                            
+                child_serial_number = dl_gen_child_serial_number( drivelink );
 
                 if (child_serial_number == BAD_SERIAL_NUMBER) {
-                        rc=EINVAL; 
+                        rc=EINVAL;
                         break;
                 }
 
@@ -169,9 +182,9 @@ static int dl_expand_drivelink( storage_object_t *drivelink,
                 (pdata->ordering_table+pdata->drive_link_count)->child_serial_number = child_serial_number;
                 (pdata->ordering_table+pdata->drive_link_count)->child_vsize = 0;
                 ++pdata->drive_link_count;
- 
+
                 // add child to the drive link storage object
-                rc = dl_add_child_object_to_drivelink( drivelink, child, child_serial_number, NULL); 
+                rc = dl_add_child_object_to_drivelink( drivelink, child, child_serial_number, NULL);
                 if (rc==0 ) {
                         drive_link_t  *dlink = &pdata->drive_link[pdata->drive_link_count-1];
                         rc = dl_build_feature_header( drivelink, dlink, dlink->object->feature_header );
@@ -188,7 +201,7 @@ static int dl_expand_drivelink( storage_object_t *drivelink,
         // then we need to unwind the expand
         if (rc) {
                 for (i=original_link_count-1; i<pdata->drive_link_count; i++) {
-                        if (pdata->drive_link[i].object != NULL) {                      
+                        if (pdata->drive_link[i].object != NULL) {
                                 if (pdata->drive_link[i].object->feature_header) {
                                         EngFncs->engine_free(pdata->drive_link[i].object->feature_header);
                                 }
@@ -199,7 +212,7 @@ static int dl_expand_drivelink( storage_object_t *drivelink,
                                 pdata->ordering_table[i].child_vsize         = 0;
                         }
                 }
- 
+
                 // restore original drive_link_count
                 pdata->drive_link_count = original_link_count;
         }
@@ -238,7 +251,7 @@ int dl_can_delete( storage_object_t * object)
 }
 
 
-int dl_can_expand(storage_object_t *drivelink, sector_count_t *expand_limit, list_anchor_t expansion_points )  
+int dl_can_expand(storage_object_t *drivelink, sector_count_t expand_limit, list_anchor_t expansion_points )
 {
         int rc = EINVAL;
         storage_object_t        *x_obj;
@@ -255,7 +268,7 @@ int dl_can_expand(storage_object_t *drivelink, sector_count_t *expand_limit, lis
         REQUIRE(dl_isa_drivelink(drivelink)==TRUE);
         REQUIRE(dl_isa_complete_aggregate(drivelink)==TRUE);
         REQUIRE(expansion_points != NULL);
-        
+
         acceptable_objects = EngFncs->allocate_list();
 
         REQUIRE(acceptable_objects != NULL);
@@ -263,9 +276,9 @@ int dl_can_expand(storage_object_t *drivelink, sector_count_t *expand_limit, lis
         LOG_DEBUG("considering drivelink %s\n", drivelink->name );
 
         pdata = (drivelink_private_data_t *)drivelink->private_data;
-        
+
         if (pdata->drive_link_count < EVMS_DRIVELINK_MAX_ENTRIES) {
-        
+
                 rc = EngFncs->get_object_list( 0,
                                                       DATA_TYPE,
                                                       NULL,
@@ -273,11 +286,11 @@ int dl_can_expand(storage_object_t *drivelink, sector_count_t *expand_limit, lis
                                                       VALID_INPUT_OBJECT,
                                                       &acceptable_objects );
 
-                if (rc==0) {    
+                if (rc==0) {
                         LIST_FOR_EACH(acceptable_objects, iter, x_obj) {
                                 if ( x_obj != drivelink &&
                                      x_obj->size > metadata_sectors &&
-                                     (x_obj->size - metadata_sectors) <= *expand_limit ) {
+                                     (x_obj->size - metadata_sectors) <= expand_limit ) {
                                         dl_expand_size += x_obj->size - metadata_sectors;
                                 }
                         }
@@ -290,7 +303,7 @@ int dl_can_expand(storage_object_t *drivelink, sector_count_t *expand_limit, lis
                 if (expand_object) {
 
                         expand_object->object          = drivelink;
-                        expand_object->max_expand_size = dl_expand_size;
+                        expand_object->max_expand_size = min(dl_expand_size, expand_limit);
 
                         iter = EngFncs->insert_thing(expansion_points,
                                                      expand_object,
@@ -301,7 +314,7 @@ int dl_can_expand(storage_object_t *drivelink, sector_count_t *expand_limit, lis
                         }
                 }
         }
-      
+
         child = dl_get_last_child(drivelink);
         if (child) {
                 struct plugin_functions_s * fncs = (struct plugin_functions_s *)child->plugin->functions.plugin;
@@ -324,10 +337,10 @@ int dl_can_expand(storage_object_t *drivelink, sector_count_t *expand_limit, lis
 int dl_can_expand_by( storage_object_t *object, sector_count_t *size)
 {
         int rc=EINVAL;
-        
+
         LOG_ENTRY();
 
-        if ( dl_isa_drivelink(object) && 
+        if ( dl_isa_drivelink(object) &&
              dl_isa_complete_aggregate(object)==TRUE ) {
                 rc = 0;
         }
@@ -336,7 +349,7 @@ int dl_can_expand_by( storage_object_t *object, sector_count_t *size)
         return rc;
 }
 
-int dl_can_shrink( storage_object_t  *drivelink, sector_count_t *shrink_limit,  list_anchor_t shrink_points )
+int dl_can_shrink( storage_object_t  *drivelink, sector_count_t shrink_limit,  list_anchor_t shrink_points )
 {
         int i,rc = EINVAL;
         drivelink_private_data_t  *pdata;
@@ -353,7 +366,7 @@ int dl_can_shrink( storage_object_t  *drivelink, sector_count_t *shrink_limit,
 
         if ( pdata->drive_link_count > 1 ) {
 
-                if (pdata->drive_link[pdata->drive_link_count - 1].sector_count <= *shrink_limit) {
+                if (pdata->drive_link[pdata->drive_link_count - 1].sector_count <= shrink_limit) {
 
                         shrink_object = (shrink_object_info_t *) EngFncs->engine_alloc( sizeof(shrink_object_info_t) );
                         if (shrink_object) {
@@ -367,7 +380,7 @@ int dl_can_shrink( storage_object_t  *drivelink, sector_count_t *shrink_limit,
 
                                 for ( i = pdata->drive_link_count - 1;
                                      (i > 0) &&
-                                     (shrink_object->max_shrink_size + pdata->drive_link[i].sector_count <= *shrink_limit);
+                                     (shrink_object->max_shrink_size + pdata->drive_link[i].sector_count <= shrink_limit);
                                       i--) {
 
                                         shrink_object->max_shrink_size += pdata->drive_link[i].sector_count;
@@ -388,13 +401,13 @@ int dl_can_shrink( storage_object_t  *drivelink, sector_count_t *shrink_limit,
                 }
 
         }
-        
+
         child = dl_get_last_child(drivelink);
         if (child) {
                 struct plugin_functions_s * fncs = (struct plugin_functions_s *)child->plugin->functions.plugin;
-                rc = fncs->can_shrink( child, shrink_limit, shrink_points );            
+                rc = fncs->can_shrink( child, shrink_limit, shrink_points );
         }
-        
+
         if (rc && shrink_object==NULL) {
                 LOG_DEBUG("did not find any shrink points for %s\n", drivelink->name);
                 rc = ENOMSG;
@@ -462,7 +475,7 @@ int dl_can_shrink_by( storage_object_t  *drivelink,  sector_count_t *size)
 }
 
 
-int dl_can_replace_child( storage_object_t  * object, 
+int dl_can_replace_child( storage_object_t  * object,
                           storage_object_t  * child,
                           storage_object_t  * new_child)
 {
@@ -475,15 +488,15 @@ int dl_can_replace_child( storage_object_t  * object,
 
         REQUIRE(dl_isa_drivelink(object)==TRUE);
         REQUIRE(child != NULL);
-        REQUIRE(dl_isa_missing_child(child)==FALSE); // till we fix replace of missing child objects that 
+        REQUIRE(dl_isa_missing_child(child)==FALSE); // till we fix replace of missing child objects that
                                                      // have error mappings that kill the copy job
-        
+
         pdata = (drivelink_private_data_t *) object->private_data;
-  
+
         for (i=0,rc=EINVAL; i < pdata->drive_link_count; i++) {
 
                 if (pdata->drive_link[i].object == child){
-                        
+
                         dot = &pdata->ordering_table[i];
 
                         if (new_child != NULL) {
@@ -619,7 +632,7 @@ int dl_create( list_anchor_t input_objects, option_array_t *options,  list_ancho
                 strncat(drivelink->name, "/", EVMS_NAME_SIZE-strlen(drivelink->name));
         }
         strncat(drivelink->name, drive_link_name, EVMS_NAME_SIZE-strlen(drivelink->name));
-        
+
         strncpy(pdata->parent_object_name, drive_link_name, EVMS_NAME_SIZE);
 
         // no links yet
@@ -645,26 +658,25 @@ int dl_create( list_anchor_t input_objects, option_array_t *options,  list_ancho
                 }
         }
 
-        // Now, consume objects from the input objects list        
+        // Now, consume objects from the input objects list
         rc = dl_expand_drivelink(drivelink, input_objects );
 
-        if (rc==0) {                                      
+        if (rc==0) {
                 iter = EngFncs->insert_thing(output_objects, drivelink,
                                              INSERT_BEFORE, NULL);
                 if (!iter) {
                         rc = ENOMEM;
                 }
         }
- 
+
         if (rc==0) {
-                drivelink->flags |= SOFLAG_DIRTY + SOFLAG_NEEDS_ACTIVATE;
-                dl_build_linear_mapping(drivelink);       
+                dl_build_linear_mapping(drivelink);
                 dl_setup_geometry(drivelink);   // call routine to figure out a geometry for the aggregate
         }
         else {
                 EngFncs->unregister_name(drivelink->name);
                 dl_unregister_serial_number(pdata->parent_serial_number);
-                dl_free_drivelink_object(drivelink);   
+                dl_free_drivelink_object(drivelink);
         }
 
         LOG_EXIT_INT(rc);
@@ -689,7 +701,6 @@ int dl_w_delete( storage_object_t * object, list_anchor_t  child_objects, boolea
 	LOG_ENTRY();
 
 	REQUIRE(dl_isa_drivelink(object)==TRUE);
-	REQUIRE(child_objects != NULL);
 
 	pdata = (drivelink_private_data_t *) object->private_data;
 
@@ -697,7 +708,7 @@ int dl_w_delete( storage_object_t * object, list_anchor_t  child_objects, boolea
 
 		child = pdata->drive_link[i].object;
 
-		if (child==NULL) 
+		if (child==NULL)
 			continue;
 
 		if ( dl_isa_missing_child(child)==TRUE ) {
@@ -758,7 +769,7 @@ int dl_delete(storage_object_t *object, list_anchor_t child_objects)
 
 /*
  * Function: dl_discard
- * 
+ *
  * This function is similar to delete.  Just call delete to free all
  * data structures related to the DriveLink objects.
  */
@@ -795,7 +806,7 @@ int dl_discard(list_anchor_t objects)
  *  added at the end of the drive link object ... expanding by adding
  *  sectors at the end of the drive link.
  */
-int dl_expand( storage_object_t * drivelink, 
+int dl_expand( storage_object_t * drivelink,
                storage_object_t * expand_object,
                list_anchor_t      objects,
                option_array_t   * options )
@@ -814,12 +825,12 @@ int dl_expand( storage_object_t * drivelink,
         REQUIRE(dl_isa_drivelink(drivelink)==TRUE);
         REQUIRE(expand_object != NULL);
         REQUIRE(objects != NULL);
- 
+
         LOG_DEBUG("expanding drivelink %s\n", drivelink->name);
         LOG_DEBUG("expand object is %s\n", expand_object->name);
-        
+
         pdata = (drivelink_private_data_t *) drivelink->private_data;
-        
+
         if (drivelink == expand_object) {
 
                 expand_link_count = EngFncs->list_count(objects);
@@ -828,14 +839,14 @@ int dl_expand( storage_object_t * drivelink,
 
                 REQUIRE( pdata->drive_link_count + expand_link_count <= EVMS_DRIVELINK_MAX_ENTRIES );
 
-                rc = dl_expand_drivelink( drivelink, objects );       
+                rc = dl_expand_drivelink( drivelink, objects );
         }
         else {
                 rc = ENODEV;
                 child = dl_get_last_child( drivelink );
                 if (child) {
                         funcs = (struct plugin_functions_s *)child->plugin->functions.plugin;
-                        if (funcs) {                    
+                        if (funcs) {
                                 rc = funcs->expand( child,
                                                     expand_object,
                                                     objects,
@@ -860,11 +871,14 @@ int dl_expand( storage_object_t * drivelink,
         }
 
         if (rc==0) {
-                drivelink->flags |= SOFLAG_DIRTY + SOFLAG_NEEDS_ACTIVATE;
-                dl_build_linear_mapping(drivelink);       
-                dl_setup_geometry(drivelink);        
-        }       
-    
+                drivelink->flags |= SOFLAG_DIRTY;
+		if (drivelink->flags & SOFLAG_ACTIVE) {
+			drivelink->flags |= SOFLAG_NEEDS_ACTIVATE;
+		}
+                dl_build_linear_mapping(drivelink);
+                dl_setup_geometry(drivelink);
+        }
+
         LOG_EXIT_INT(rc);
         return rc;
 }
@@ -902,6 +916,7 @@ int dl_shrink( storage_object_t * drivelink,
         uint                       selected_objects_count=0;
         sector_count_t             child_useable_size;
         sector_count_t             padding_sectors;
+        sector_count_t             shrink_size = 0;
         list_element_t             iter;
 
 
@@ -943,11 +958,22 @@ int dl_shrink( storage_object_t * drivelink,
                         }
                 }
 
+		// Check with the engine before starting the expand.
+		for (i = 1; i <= selected_objects_count; i++) {
+			shrink_size += pdata->drive_link[pdata->drive_link_count - i].sector_count;
+		}
+		rc = EngFncs->can_shrink_by(drivelink, &shrink_size);
+		if (rc) {
+			LOG_ERROR("Shrink of object %s rejected by the engine.\n", drivelink->name);
+			LOG_EXIT_INT(rc);
+			return rc;
+		}
+
                 // now do the shrink ... always removing last child in drivelink
                 for (i=0,rc=0; i<selected_objects_count && rc==0; i++) {
 
                         child = dl_get_last_child(drivelink);
-                        if (child) {                    
+                        if (child) {
                                 rc = dl_shrink_drivelink( drivelink, child );
                         }
                         else {
@@ -965,7 +991,7 @@ int dl_shrink( storage_object_t * drivelink,
         else {
 
                 child = dl_get_last_child(drivelink);
-                if (child) {            
+                if (child) {
                         struct plugin_functions_s * fncs = (struct plugin_functions_s *) child->plugin->functions.plugin;
                         rc = fncs->shrink(child, shrink_object, objects, options );
                 }
@@ -987,15 +1013,18 @@ int dl_shrink( storage_object_t * drivelink,
                         pdata->drive_link[index].sector_count       = child_useable_size;
                         pdata->drive_link[index].padding            = padding_sectors;
                         pdata->ordering_table[index].child_vsize    = child_useable_size;
- 
+
                 }
 
         }
- 
-        if ( rc==0 ) {             
-                dl_build_linear_mapping(drivelink);     
-                dl_setup_geometry(drivelink);   
-                drivelink->flags |= SOFLAG_DIRTY + SOFLAG_NEEDS_ACTIVATE;
+
+        if ( rc==0 ) {
+                dl_build_linear_mapping(drivelink);
+                dl_setup_geometry(drivelink);
+                drivelink->flags |= SOFLAG_DIRTY;
+		if (drivelink->flags & SOFLAG_ACTIVE) {
+			drivelink->flags |= SOFLAG_NEEDS_ACTIVATE;
+		}
         }
 
         LOG_EXIT_INT(rc);
@@ -1007,7 +1036,7 @@ int dl_shrink( storage_object_t * drivelink,
  *  Function:  dl_replace_child
  *
  *  At this point the replace plugin has already committed its replace
- *  object and the copy has been attempted.  If the copy failed, then 
+ *  object and the copy has been attempted.  If the copy failed, then
  *  the new_child object will be the original drivelink child.  If the
  *  copy was successful, then the new_child object will indeed be the
  *  new child object chosen for the replace.
@@ -1035,7 +1064,7 @@ int dl_replace_child( storage_object_t * object,
 
         // find the child in our drivelink
         for (i=0,index=-1; i<pdata->drive_link_count; i++) {
-                if (pdata->drive_link[i].object == child) {                     
+                if (pdata->drive_link[i].object == child) {
                         index=i;
                         break;
                 }
@@ -1046,9 +1075,9 @@ int dl_replace_child( storage_object_t * object,
         if (new_child->feature_header == NULL) {
                 new_child->feature_header = (evms_feature_header_t *) EngFncs->engine_alloc( sizeof(evms_feature_header_t) );
         }
-        
+
         REQUIRE(new_child->feature_header != NULL);
-        
+
         rc = dl_build_feature_header( object, &(pdata->drive_link[index]), new_child->feature_header );
 
         if (rc) {
@@ -1066,8 +1095,11 @@ int dl_replace_child( storage_object_t * object,
         if (rc==0) {
                 pdata->drive_link[index].object = new_child;
                 dl_build_ordered_child_object_list( object, &object->child_objects );
-                object->flags |= SOFLAG_DIRTY + SOFLAG_NEEDS_ACTIVATE;
-        
+                object->flags |= SOFLAG_DIRTY;
+		if (object->flags & SOFLAG_ACTIVE) {
+			object->flags |= SOFLAG_NEEDS_ACTIVATE;
+		}
+
                 // remove parent from child parent list
                 EngFncs->remove_thing(child->parent_objects, object);
 
@@ -1076,7 +1108,7 @@ int dl_replace_child( storage_object_t * object,
                 fh   = child->feature_header;
 
                 if (dl_isa_missing_child(child)==FALSE) {
-                        
+
                         fncs->add_sectors_to_kill_list( child,
                                                         fh->feature_data1_start_lsn,
                                                         fh->feature_data1_size );
@@ -1198,7 +1230,7 @@ int dl_read(storage_object_t *object, lsn_t lsn, sector_count_t count, void *buf
         storage_object_t          *child;
 
         LOG_ENTRY();
-        LOG_DEBUG("drivelink= %s  size = %"PRIu64"  lsn= %"PRIu64"  count= %"PRIu64"\n", 
+        LOG_DEBUG("drivelink= %s  size = %"PRIu64"  lsn= %"PRIu64"  count= %"PRIu64"\n",
                     object->name, object->size, lsn, count);
 
         REQUIRE(buffer != NULL);
@@ -1285,7 +1317,7 @@ int dl_write(storage_object_t *object, lsn_t lsn, sector_count_t count, void *bu
 
 
         LOG_ENTRY();
-        LOG_DEBUG("drivelink= %s  size = %"PRIu64"  lsn= %"PRIu64"  count= %"PRIu64"\n", 
+        LOG_DEBUG("drivelink= %s  size = %"PRIu64"  lsn= %"PRIu64"  count= %"PRIu64"\n",
                    object->name, object->size, lsn, count);
 
         REQUIRE(buffer != NULL);
@@ -1295,17 +1327,17 @@ int dl_write(storage_object_t *object, lsn_t lsn, sector_count_t count, void *bu
                 LOG_EXIT_INT(0);
                 return 0;
         }
-        
+
         REQUIRE(dl_isa_drivelink(object)==TRUE);
 
         if ( object->flags & SOFLAG_READ_ONLY ) {
-                rc = EROFS;  
+                rc = EROFS;
                 LOG_EXIT_INT(rc);
                 return rc;
         }
 
         pdata = (drivelink_private_data_t *) object->private_data;
-        
+
         for (i=0; i < pdata->drive_link_count; i++) {
 
                 if ( pdata->drive_link[i].end_lsn >= io_lsn ) {
@@ -1324,7 +1356,7 @@ int dl_write(storage_object_t *object, lsn_t lsn, sector_count_t count, void *bu
                         }
 
                         link_lsn = io_lsn - pdata->drive_link[i].start_lsn;
-                        
+
                         if ( dl_isa_missing_child(child)==TRUE ) {
                                 rc = EIO;
                         }
@@ -1340,7 +1372,7 @@ int dl_write(storage_object_t *object, lsn_t lsn, sector_count_t count, void *bu
                 }
 
         }
-  
+
         LOG_EXIT_INT(rc);
         return rc;
 }
@@ -1367,13 +1399,13 @@ int dl_plugin_function( storage_object_t * missing_child,
         storage_object_t *drivelink=NULL;
 
         LOG_ENTRY();
-        
+
         REQUIRE(dl_isa_missing_child(missing_child)==TRUE);
         REQUIRE(objects != NULL);
         REQUIRE(options != NULL);
 
         drivelink = dl_get_parent(missing_child);
-        
+
         REQUIRE(drivelink != NULL);
 
         switch (action) {
@@ -1422,10 +1454,10 @@ int dl_get_plugin_functions( storage_object_t        * object,
 
                 func_info->info[DL_REPLACE_MISSING_CHILD_OPTION_INDEX].function = EVMS_Replace_Missing_DL_Child;
 
-                SET_STRING( func_info->info[DL_REPLACE_MISSING_CHILD_OPTION_INDEX].title, "Replace Missing Child" );
-                SET_STRING( func_info->info[DL_REPLACE_MISSING_CHILD_OPTION_INDEX].verb,  "Replace" );
-                SET_STRING( func_info->info[DL_REPLACE_MISSING_CHILD_OPTION_INDEX].name, "Replace Missing Child" );
-                SET_STRING( func_info->info[DL_REPLACE_MISSING_CHILD_OPTION_INDEX].help,  "Use this function to replace a missing drivelink child with an available storage object." );
+                func_info->info[DL_REPLACE_MISSING_CHILD_OPTION_INDEX].title = EngFncs->engine_strdup( "Replace Missing Child" );
+                func_info->info[DL_REPLACE_MISSING_CHILD_OPTION_INDEX].verb = EngFncs->engine_strdup( _("Replace") );
+                func_info->info[DL_REPLACE_MISSING_CHILD_OPTION_INDEX].name = EngFncs->engine_strdup( _("Replace Missing Child") );
+                func_info->info[DL_REPLACE_MISSING_CHILD_OPTION_INDEX].help = EngFncs->engine_strdup( _("Use this function to replace a missing drivelink child with an available storage object.") );
 
                 rc = 0;
         }
