@@ -238,7 +238,7 @@ static int GPT_CanDestroy( storage_object_t * seg )
  *
  */
 static int GPT_CanExpand( storage_object_t       *seg,        // data segment
-                          sector_count_t  *expand_limit,      // ?
+                          sector_count_t         expand_limit,      // ?
                           list_anchor_t          expansion_points ) // list to place expand object on
 {
         DISKSEG              *freespace;
@@ -266,14 +266,15 @@ static int GPT_CanExpand( storage_object_t       *seg,        // data segment
                         cylinder_size = get_cylinder_size(ld);
 
                         // we can only expand in cylinder size amounts
-                        if ( freespace->size >= cylinder_size ) {
+                        if ( freespace->size >= cylinder_size &&
+                             expand_limit >= cylinder_size ) {
 
                                 expand_object = (expand_object_info_t *) EngFncs->engine_alloc( sizeof(expand_object_info_t) );
                                 if (expand_object) {
                                         list_element_t e;
 
                                         expand_object->object          = seg;
-                                        expand_object->max_expand_size = freespace->size;
+                                        expand_object->max_expand_size = min(freespace->size, expand_limit);
 
                                         e = EngFncs->insert_thing( expansion_points,
                                                                    expand_object,
@@ -404,7 +405,7 @@ static int GPT_CanExpandBy(storage_object_t * seg, sector_count_t *size)
  *  If not exact set new_size to closest higher value possible.
  */
 static int GPT_CanShrink( storage_object_t * seg,            // object to shrink
-                          sector_count_t   * shrink_limit,   // a delta size
+                          sector_count_t     shrink_limit,   // a delta size
                           list_anchor_t            shrink_points ) // of type shrink_object_info_t
 {
         int                   rc = EINVAL;
@@ -431,7 +432,8 @@ static int GPT_CanShrink( storage_object_t * seg,            // object to shrink
                         cylinder_size = get_cylinder_size(ld);
 
                         // see if caller is trying to shrink smaller than min partition size
-                        if ( seg->size > cylinder_size ) {
+                        if ( seg->size > cylinder_size &&
+                             shrink_limit >= cylinder_size ) {
 
                                 if (shrink_points) {
 
@@ -444,7 +446,7 @@ static int GPT_CanShrink( storage_object_t * seg,            // object to shrink
                                                 // minimum partition size is 1 cylinder ... then anything more than
                                                 // a cylinder is the max we can shrink.
                                                 shrink_object->object          = seg;
-                                                shrink_object->max_shrink_size = seg->size - cylinder_size;
+                                                shrink_object->max_shrink_size = min(seg->size - cylinder_size, shrink_limit);
 
                                                 e = EngFncs->insert_thing( shrink_points,
                                                                            shrink_object,
@@ -1250,17 +1252,19 @@ static int CreateDiskSegment( storage_object_t      * freespace,            /* a
                         get_device_name(ld, devname);
 
                         if ( isa_disk_object(ld) == FALSE ) {
-                                sprintf(seg->name, "%s.%d", devname, next_minor);
+				char last_char = devname[strlen(devname)-1];
+
+				if ((last_char >= '1') && (last_char <= '9')) {
+					// Must be an embedded GPT partition.
+					strcat(devname, ".");
+				}
                         }
                         else {
                                 if (devname[strlen(devname)-1] == '/') {
                                         strcat(devname, "part");
                                 }
-                                sprintf(seg->name, "%s%d", devname, next_minor);
                         }
-
-                        // mark seg dirty for commit and needs DM activate
-                        seg->flags |= SOFLAG_DIRTY + SOFLAG_NEEDS_ACTIVATE;
+			sprintf(seg->name, "%s%d", devname, next_minor);
 
                         if ( insert_gpt_segment_into_list( ld->parent_objects, seg ) == NULL ) {
 
@@ -1322,7 +1326,7 @@ static int GetCreateOptions(  storage_object_t  * seg,
         LOG_ENTRY();
 
         // init the options before beginning
-        *ptype = unknown_partition;
+        *ptype = basic_data_partition;
         *sector_offset = 0;
         *size = 0;
 
@@ -1341,24 +1345,6 @@ static int GetCreateOptions(  storage_object_t  * seg,
                                 *sector_offset = options->option[i].value.ui64;
                                 break;
 
-                        case SEG_CREATE_OPTION_TYPE_INDEX:
-                                strcpy( partition_type, options->option[i].value.s );
-
-                                if ( strncmp(partition_type, BASIC_DATA_PARTITION_STRING, strlen(BASIC_DATA_PARTITION_STRING))==0) {
-                                        *ptype = basic_data_partition;
-                                }
-                                else if ( strncmp(partition_type, EFI_SYSTEM_PARTITION_STRING, strlen(EFI_SYSTEM_PARTITION_STRING))==0) {
-                                        *ptype = efi_system_partition;
-                                }
-                                else if ( strncmp(partition_type, GPT_SWAP_PARTITION_STRING, strlen(GPT_SWAP_PARTITION_STRING))==0) {
-                                        *ptype = swap_partition;
-                                }
-                                else if ( strncmp(partition_type, LEGACY_MBR_PARTITION_STRING, strlen(LEGACY_MBR_PARTITION_STRING))==0) {
-                                        *ptype = legacy_mbr_partition;
-                                }
-
-                                break;
-
                         default:
                                 break;
 
@@ -1373,23 +1359,6 @@ static int GetCreateOptions(  storage_object_t  * seg,
                         else if (strcmp(options->option[i].name, SEG_CREATE_OPTION_OFFSET_NAME) == 0) {
                                 *sector_offset = options->option[i].value.ui64;
                         }
-                        else if (strcmp(options->option[i].name, SEG_CREATE_OPTION_TYPE_NAME) == 0) {
-                                strcpy( partition_type, options->option[i].value.s );
-
-                                if ( strncmp(partition_type, BASIC_DATA_PARTITION_STRING, strlen(BASIC_DATA_PARTITION_STRING))==0) {
-                                        *ptype = basic_data_partition;
-                                }
-                                else if ( strncmp(partition_type, EFI_SYSTEM_PARTITION_STRING, strlen(EFI_SYSTEM_PARTITION_STRING))==0) {
-                                        *ptype = efi_system_partition;
-                                }
-                                else if ( strncmp(partition_type, GPT_SWAP_PARTITION_STRING, strlen(GPT_SWAP_PARTITION_STRING))==0) {
-                                        *ptype = swap_partition;
-                                }
-                                else if ( strncmp(partition_type, LEGACY_MBR_PARTITION_STRING, strlen(LEGACY_MBR_PARTITION_STRING))==0) {
-                                        *ptype = legacy_mbr_partition;
-                                }
-
-                        }
 
                 }
         }
@@ -1586,12 +1555,12 @@ static int GPT_Discard(list_anchor_t objects)
 			if (ld) {
 				remove_gpt_segment_from_list( ld->parent_objects, seg );
 
-				if (EngFncs->list_empty(ld->parent_objects) == TRUE) {
-					disk_pdata = get_gpt_disk_private_data(ld);
-					if ( disk_pdata ) {
+				disk_pdata = get_gpt_disk_private_data(ld);
+				if ( disk_pdata ) {
 
+					if (seg == disk_pdata->md1) {
 						// unregister the disk uuid identifier
-						gh = ((SEG_PRIVATE_DATA *)disk_pdata->md1->private_data)->gh;
+						gh = ((SEG_PRIVATE_DATA *)seg->private_data)->gh;
 						if (gh) {
 
 							uuid_string = guid_to_string( &gh->disk_id );
@@ -1602,7 +1571,9 @@ static int GPT_Discard(list_anchor_t objects)
 
 						}
 					}
+				}
 
+				if (EngFncs->list_empty(ld->parent_objects) == TRUE) {
 					// toss our disk private data area
 					delete_gpt_disk_private_data( ld );
 				}
@@ -1749,6 +1720,15 @@ static int GPT_Expand( storage_object_t *seg,
                                 // calc actual expand sector count
                                 expand_sectors = end_lba - freespace->start + 1;
 
+				// ask the engine if this expand amount is ok.
+				rc = EngFncs->can_expand_by(seg, &expand_sectors);
+				if (rc) {
+					LOG_ERROR("Shrink of segment %s rejected by "
+						  "the engine.\n", seg->name);
+					LOG_EXIT_INT(rc);
+					return rc;
+				}
+
                                 // expand the data segment
                                 old_seg_size = seg->size;
                                 seg->size   += expand_sectors;
@@ -1758,7 +1738,10 @@ static int GPT_Expand( storage_object_t *seg,
                                 freespace->size   -= expand_sectors;
 
                                 // mark seg dirty for commit and needs DM activate
-                                seg->flags |= SOFLAG_DIRTY + SOFLAG_NEEDS_ACTIVATE;
+				seg->flags |= SOFLAG_DIRTY;
+				if (seg->flags & SOFLAG_ACTIVE) {
+					seg->flags |= SOFLAG_NEEDS_ACTIVATE;
+				}
 
                                 LOG_DEBUG("     Data Seg  Name: %s\n", seg->name);
                                 LOG_DEBUG("           New Start: %"PRIu64"\n", seg->start);
@@ -1883,6 +1866,15 @@ static int GPT_Shrink( storage_object_t * seg,
                                 shrink_sector_count = (shrink_sector_count/cylinder_size)*cylinder_size;
                         }
 
+			// ask the engine if it's ok to shrink by this amount.
+			rc = EngFncs->can_shrink_by(seg, &shrink_sector_count);
+			if (rc) {
+				LOG_ERROR("Shrink of segment %s rejected by "
+					  "the engine.\n", seg->name);
+				LOG_EXIT_INT(rc);
+				return rc;
+			}
+
                         // resulting seg size
                         new_seg_size = seg->size - shrink_sector_count;
 
@@ -1910,7 +1902,10 @@ static int GPT_Shrink( storage_object_t * seg,
                                 find_freespace_on_gpt_disk( ld );
 
                                 // mark seg dirty for commit and needs DM activate
-                                seg->flags |= SOFLAG_DIRTY + SOFLAG_NEEDS_ACTIVATE;
+				seg->flags |= SOFLAG_DIRTY;
+				if (seg->flags & SOFLAG_ACTIVE) {
+					seg->flags |= SOFLAG_NEEDS_ACTIVATE;
+				}
 
                                 // success.
                                 rc = 0;
@@ -2011,12 +2006,13 @@ static int GPT_CommitChanges( storage_object_t *seg, uint phase )
                              disk_pdata->flags & DISK_HAS_CHANGES_PENDING &&
                              !(disk_pdata->flags & DISK_HAS_MOVE_PENDING) ) {
 
-                                rc = commit_guid_partition_tables( ld, 1 );
+                                rc = commit_guid_partition_tables( ld, seg, 1, FALSE );
 
                                 if ( rc == 0 ) {
 
-                                        rc2 = commit_guid_partition_tables( ld, 2 );  // dont care ... discovery will fixup
-                                                                                      // any problems with 2nd copy of metadata
+                                        rc2 = commit_guid_partition_tables( ld, seg,
+                                                                            2, FALSE );  // dont care ... discovery will fixup
+                                                                                         // any problems with 2nd copy of metadata
 
                                         // mark all seg storage objects clean on this disk
                                         LIST_FOR_EACH( ld->parent_objects, iter, clean_seg ) {
@@ -2217,10 +2213,10 @@ static int GPT_get_plugin_functions( storage_object_t        * object,
 
                         func_info->info[SEG_MOVE_OPTION_INDEX].function = EVMS_Task_GPT_Move_Segment;
 
-                        SET_STRING( func_info->info[SEG_MOVE_OPTION_INDEX].title, "Move" );
-                        SET_STRING( func_info->info[SEG_MOVE_OPTION_INDEX].verb,  "Move" );
-                        SET_STRING( func_info->info[SEG_MOVE_OPTION_INDEX].name, "Move" );
-                        SET_STRING( func_info->info[SEG_MOVE_OPTION_INDEX].help,  "Use this function to move a data segment." );
+                        func_info->info[SEG_MOVE_OPTION_INDEX].title = EngFncs->engine_strdup( "Move" );
+                        func_info->info[SEG_MOVE_OPTION_INDEX].verb = EngFncs->engine_strdup( _("Move") );
+                        func_info->info[SEG_MOVE_OPTION_INDEX].name = EngFncs->engine_strdup( _("Move") );
+                        func_info->info[SEG_MOVE_OPTION_INDEX].help = EngFncs->engine_strdup( _("Use this function to move a data segment.") );
 
                 }
 
@@ -2237,6 +2233,34 @@ static int GPT_get_plugin_functions( storage_object_t        * object,
 }
 
 
+/*
+ * Save the metadata needed to build this segment. Call the Engine's
+ * save_metadata() service for each contiguous chunk of metadata that
+ * we need to write to the underlying disk.
+ */
+static int GPT_backup_metadata(storage_object_t *object)
+{
+	LOGICALDISK *ld;
+	int rc = 0;
+
+	LOG_ENTRY();
+
+	if (object->data_type != FREE_SPACE_TYPE) {
+                ld = get_logical_disk(object);
+                if (ld) {
+                        rc = commit_guid_partition_tables(ld, object, 1, TRUE);
+                        if (!rc) {
+                                rc = commit_guid_partition_tables(ld, object,
+                                                                  2, TRUE);
+                        }
+                }
+	}
+
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
+
 /*-------------------------------------------------------------------------------------+
 +                                                                                      +
 +                              PLUGIN FUNCTION TABLE                                   +
@@ -2267,10 +2291,13 @@ static struct plugin_functions_s sft={
         read:                                GPT_Read,
         write:                               GPT_Write,
         set_volume:                          GPT_set_volume,
+        can_activate:                        GPT_can_activate,
         activate:                            GPT_activate,
+        can_deactivate:                      GPT_can_deactivate,
         deactivate:                          GPT_deactivate,
         get_plugin_functions:                GPT_get_plugin_functions,
         plugin_function:                     GPT_plugin_function,
+        backup_metadata:                     GPT_backup_metadata,
 
         // the following routines found in options.c
         get_option_count:                    GPT_GetOptionCount,
@@ -2293,8 +2320,8 @@ static plugin_record_t gpt_plugin_record = {
 
         version:                            {MAJOR_VERSION, MINOR_VERSION, PATCH_LEVEL},
 
-        required_engine_api_version:        {13,0,0},
-        required_plugin_api_version:        {plugin: {12,0,0}},
+        required_engine_api_version:        {15,0,0},
+        required_plugin_api_version:        {plugin: {13,1,0}},
 
         short_name:                         "GptSegMgr",
         long_name:                          "GPT Segment Manager",
