@@ -366,7 +366,7 @@ DISKSEG * allocate_gpt_freespace_disk_segment( LOGICALDISK *ld )
                                 }
                                 else {
                                         sprintf(freeseg->name, "%s_freespace%d", devname, get_next_avail_freespace_number(ld) );
-                                }                       
+                                }
                         }
 
                 }
@@ -477,7 +477,7 @@ static int  merge_freespace_segments( list_anchor_t seglist )
 
                                 }
 
-                        } while (this != NULL);  
+                        } while (this != NULL);
                 }
         }
 
@@ -501,7 +501,7 @@ int  merge_adjacent_freedisksegs_in_gpt_seglist( list_anchor_t seglist )
 
         do {
                 rc = merge_freespace_segments(seglist);
-        } while (rc==0);  
+        } while (rc==0);
 
 
         rc = 0;
@@ -715,13 +715,13 @@ storage_object_t *  allocate_gpt_disk_segment( storage_object_t *object )
         rc = EngFncs->allocate_segment( NULL, &seg );
         if (rc==0) {
 
-                // insert logical disk into new segment's child list            
+                // insert logical disk into new segment's child list
                 if (EngFncs->insert_thing(seg->child_objects,object,INSERT_AFTER,NULL)) {
                         rc = 0;
                 }
                 else {
                         rc = EPERM;
-                } 
+                }
 
                 if (rc == 0) {
 
@@ -814,7 +814,7 @@ storage_object_t * create_gpt_metadata_segment( storage_object_t   *object,
                         }
                         else {
                                 sprintf(metadata->name, "%s_%s", devname, name );
-                        }                   
+                        }
                 }
 
         }
@@ -846,7 +846,7 @@ storage_object_t * build_gpt_segment_from_partition_record(  storage_object_t *o
                 return NULL;
         }
 
-        // get device name 
+        // get device name
         get_device_name(object, devname);
 
         // get gpt seg private data area
@@ -879,7 +879,7 @@ storage_object_t * build_gpt_segment_from_partition_record(  storage_object_t *o
         if ( pdata->type == legacy_mbr_partition ) {
 
                 if ( isa_disk_object(object) == FALSE ) {
-                        sprintf(seg->name, "%s.legacy_mbr_partition", devname );        
+                        sprintf(seg->name, "%s.legacy_mbr_partition", devname );
                 }
                 else {
                         if (devname[strlen(devname)-1] == '/') {
@@ -896,14 +896,19 @@ storage_object_t * build_gpt_segment_from_partition_record(  storage_object_t *o
         else {
 
                 if ( isa_disk_object(object) == FALSE ) {
-                        sprintf(seg->name, "%s.%d", devname, minor);
+			char last_char = devname[strlen(devname)-1];
+
+			if ((last_char >= '1') && (last_char <= '9')) {
+				// Must be an embedded GPT partition.
+				strcat(devname, ".");
+			}
                 }
                 else {
                         if (devname[strlen(devname)-1] == '/') {
                                 strcat(devname, "part");
                         }
-                        sprintf(seg->name, "%s%d", devname, minor);
                 }
+		sprintf(seg->name, "%s%d", devname, minor);
 
                 seg->data_type = DATA_TYPE;
                 pdata->minor   = (u_int32_t) minor;
@@ -1080,12 +1085,12 @@ void *  insert_gpt_segment_into_ordered_list( list_anchor_t seglist, storage_obj
 
 
         switch (rc) {
-        
+
         case 0:  /* Ok, found a segment we should insert in front of */
-                e = EngFncs->insert_thing( seglist, 
+                e = EngFncs->insert_thing( seglist,
                                            seg,
                                            INSERT_BEFORE|EXCLUSIVE_INSERT,
-                                           EngFncs->find_in_list(seglist, seg2, NULL) );
+                                           EngFncs->find_in_list(seglist, seg2, NULL, NULL) );
                 if (e != NULL) {
                         rc = 0;
                 }
@@ -1098,7 +1103,7 @@ void *  insert_gpt_segment_into_ordered_list( list_anchor_t seglist, storage_obj
         case -1: /* This new segment lays out on the disk at a higher */
                 /* LBA than any other existing segment.  So, just    */
                 /* insert at end of the segment list.                */
-                e = EngFncs->insert_thing( seglist, 
+                e = EngFncs->insert_thing( seglist,
                                            seg,
                                            INSERT_AFTER|EXCLUSIVE_INSERT,
                                            NULL );
