@@ -197,7 +197,7 @@ static int create_move_target( DISKSEG  *seg,        // source data segment
 
                 rc = EFBIG;
 
-                // calculate the starting lba                                                           
+                // calculate the starting lba
                 if (starts_on_cylinder_boundary(ld,seg->start)==TRUE) {
                         if (starts_on_cylinder_boundary(ld,freespace->start)==TRUE) {
                                 start = freespace->start;
@@ -437,8 +437,8 @@ int gpt_move_segment( DISKSEG *src, DISKSEG *freespace )
  *
  *  Called to update partition tables after moving a segment.
  */
-static int do_move_segment_finish( DISKSEG      *src, 
-                                   DISKSEG      *trg, 
+static int do_move_segment_finish( DISKSEG      *src,
+                                   DISKSEG      *trg,
                                    int           copy_rc,
                                    boolean       offline )
 {
@@ -459,18 +459,18 @@ static int do_move_segment_finish( DISKSEG      *src,
                 trg_pdata  = (SEG_PRIVATE_DATA *)trg->private_data;
 
                 if (copy_rc==0) {
-                        memcpy(&saved_seg, src, sizeof(DISKSEG));                       
+                        memcpy(&saved_seg, src, sizeof(DISKSEG));
                         rc = gpt_update_segment_info(src,trg);
                         if (rc==0) {
-                                rc = commit_guid_partition_tables( ld, 1 );
+                                rc = commit_guid_partition_tables( ld, trg, 1, FALSE );
                                 if (rc==0) {
-                                        rc = commit_guid_partition_tables( ld, 2 );
+                                        rc = commit_guid_partition_tables( ld, trg, 2, FALSE );
                                 }
                         }
                         if (rc) {
                                 memcpy(src, &saved_seg, sizeof(DISKSEG));
-                                commit_guid_partition_tables( ld, 1 );
-                                commit_guid_partition_tables( ld, 2 );
+                                commit_guid_partition_tables( ld, src, 1, FALSE );
+                                commit_guid_partition_tables( ld, src, 2, FALSE );
                         }
                 }
                 else {
@@ -530,20 +530,20 @@ static int do_online_copy(storage_object_t *src, storage_object_t *tgt, copy_job
                                 EngFncs->dm_suspend(src, FALSE);
                         }
                         else {
-                                LOG_SERIOUS("Error code %d when resuming object %s: %s\n", 
+                                LOG_SERIOUS("Error code %d when resuming object %s: %s\n",
                                             rc, src->name, EngFncs->strerror(rc));
                         }
 
-                        EngFncs->dm_set_suspended_flag(FALSE);                    
+                        EngFncs->dm_set_suspended_flag(FALSE);
                 }
                 else {
-                        LOG_SERIOUS("Error code %d when loading the new mirror target for the object %s: %s\n", 
+                        LOG_SERIOUS("Error code %d when loading the new mirror target for the object %s: %s\n",
                                     rc, src->name, EngFncs->strerror(rc));
                 }
 
         }
         else {
-                LOG_SERIOUS("Error code %d when getting the target for the object %s: %s\n", 
+                LOG_SERIOUS("Error code %d when getting the target for the object %s: %s\n",
                             rc, src->name, EngFncs->strerror(rc));
         }
 
@@ -585,9 +585,9 @@ static int do_offline_copy( storage_object_t *src, storage_object_t *tgt, copy_j
  */
 int gpt_move_segment_commit( DISKSEG *src, DISKSEG *tgt, copy_job_t *job )
 {
-        int rc=0;       
-        char * choices[] = {"Yes", "No", NULL};        
-        int answer = 0;    
+        int rc=0;
+        char * choices[] = {_("Yes"), _("No"), NULL};
+        int answer = 0;
         logical_volume_t *vol=NULL;
 
         LOG_ENTRY();
