@@ -146,7 +146,7 @@ int fsim_mkfs(logical_volume_t * volume, option_array_t * options )
 {
 	int     rc;
 	char   *argv[MKFS_JFS_OPTIONS_COUNT + 7];
-	char    logsize[sizeof(unsigned int) + 1];
+	char    logsize[25];
 	pid_t   pidm;
 	int     status;
 	int     fds2[2];
@@ -358,7 +358,7 @@ int fsim_fsck(logical_volume_t * volume, option_array_t * options )
 			bytes_read = read(fds2[0],buffer,MAX_USER_MESSAGE_LEN);
 			if (bytes_read > 0) {
 				/* display fsck.jfs output */
-				MESSAGE("fsck.jfs output: \n\n%s",buffer);
+				MESSAGE(_("fsck.jfs output: \n\n%s"),buffer);
 				memset(buffer,0,bytes_read); //clear out message
 			}
 			usleep(10000); // don't hog all the cpu
@@ -367,7 +367,7 @@ int fsim_fsck(logical_volume_t * volume, option_array_t * options )
 			/* get fsck.jfs exit code */
 			bytes_read = read(fds2[0],buffer,MAX_USER_MESSAGE_LEN);
 			if (bytes_read > 0) {
-				MESSAGE("fsck output: \n\n%s",buffer);
+				MESSAGE(_("fsck output: \n\n%s"),buffer);
 			}
 			rc = WEXITSTATUS(status);
 			if (rc == 0) {
