@@ -3,16 +3,16 @@
  *
  *   This program is free software;  you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or 
+ *   the Free Software Foundation; either version 2 of the License, or
  *   (at your option) any later version.
- * 
+ *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  *   the GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program;  if not, write to the Free Software 
+ *   along with this program;  if not, write to the Free Software
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
  * Module: LVM Region Manager
@@ -602,7 +602,7 @@ int lvm_check_vg_name(char * vg_name)
 	LIST_FOR_EACH(lvm_group_list, itr, group) {
 		lvm_translate_container_name_to_vg_name(group, this_vg_name);
 		if (! strncmp(vg_name, this_vg_name, NAME_LEN)) {
-			MESSAGE("%s is already in use as a container name\n",
+			MESSAGE(_("%s is already in use as a container name\n"),
 				vg_name);
 			rc = EEXIST;
 			goto out;
@@ -899,3 +899,428 @@ void lvm_fix_group_after_pv_removal(lvm_volume_group_t * group)
 	LOG_EXIT_VOID();
 }
 
+
+/*** Volume Group Expand/Shrink Functions ***/
+
+
+/**
+ * lvm_can_add_object
+ *
+ * Can the specified object be added to the specified container.
+ **/
+int lvm_can_add_object(storage_object_t * object,
+		       storage_container_t * container)
+{
+	lvm_volume_group_t * group = container->private_data;
+	u_int32_t pe_size;
+	int rc;
+
+	LOG_ENTRY();
+
+	/* Check that the segment can be added to a group. */
+	rc = lvm_check_segment_for_group_inclusion(object, group);
+	if (rc) {
+		goto out;
+	}
+
+	/* Make sure the group has space for a new PV. */
+	if (group->vg->pv_cur >= MAX_PV) {
+		LOG_WARNING("Container %s already has maximum number of PVs (%d).\n",
+			    container->name, MAX_PV);
+		rc = EINVAL;
+		goto out;
+	}
+
+	/* Check that the segment is big enough to add to this group. */
+	pe_size = group->vg->pe_size;
+	rc = lvm_check_segment_for_pe_size(object, &pe_size);
+
+out:
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
+/**
+ * lvm_can_remove_object
+ *
+ * Can the specified object be removed from its container.
+ **/
+int lvm_can_remove_object(storage_object_t * object)
+{
+	storage_container_t * container = object->consuming_container;
+	lvm_volume_group_t * group;
+	lvm_physical_volume_t * pv_entry;
+	int rc = 0;
+
+	LOG_ENTRY();
+
+	/* Check that the segment is part of an LVM container. */
+	if (!container) {
+		LOG_WARNING("Object %s is not in a container.\n", object->name);
+		rc = EINVAL;
+		goto out;
+	}
+	if (container->plugin != my_plugin_record) {
+		LOG_WARNING("Object %s is in non-LVM container %s.\n",
+			    object->name, container->name);
+		rc = EINVAL;
+		goto out;
+	}
+	group = container->private_data;
+	
+	/* Make sure this PV isn't the last one in the group. To take the last
+	 * PV out, use the lvm_destroy_container function. We don't want empty
+	 * groups hanging around.
+	 */
+	if (group->pv_count == 1) {
+		LOG_WARNING("Cannot remove the last object from container %s\n",
+			    container->name);
+		LOG_WARNING("Use delete_container to remove this container.\n");
+		rc = EINVAL;
+		goto out;
+	}
+
+	/* Get the PV for this segment. */
+	pv_entry = lvm_get_pv_for_segment(object);
+	if (!pv_entry) {
+		LOG_ERROR("Could not find PV entry for object %s\n",
+			  object->name);
+		rc = EINVAL;
+		goto out;
+	}
+
+	/* Make sure the PV is not in use by any volumes. */
+	if (pv_entry->pv->lv_cur) {
+		LOG_WARNING("Object %s is in use by %d regions\n",
+			    object->name, pv_entry->pv->lv_cur);
+		rc = EINVAL;
+		goto out;
+	}
+
+	/* Make sure no extents are being moved to this PV. */
+	if (pv_entry->move_extents > 0) {
+		LOG_WARNING("Object %s is scheduled to have %d extents moved to it\n",
+			    object->name, pv_entry->move_extents);
+		rc = EINVAL;
+	}
+
+out:
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
+/**
+ * lvm_can_shrink_pv
+ *
+ * The specified PV can be shrunk if it has available extents at the end of
+ * the object. If the PV is shrinkable, set the shrink_limit to the amount
+ * of space that could be freed.
+ **/
+int lvm_can_shrink_pv(lvm_physical_volume_t *pv_entry, u_int64_t *shrink_limit)
+{
+	lvm_volume_group_t *group = pv_entry->group;
+	u_int32_t i, unused_extents = 0;
+	int rc = EBUSY;
+
+	LOG_ENTRY();
+	LOG_DEBUG("Checking if PV %s can be shrunk.\n",
+		  pv_entry->segment->name);
+
+	/* Work backwards from the end of the PE-map
+	 * and see how many extents are unused.
+	 */
+	for (i = pv_entry->pv->pe_total; i > 0; i--) {
+		if (lvm_pe_is_available(pv_entry->pe_map + (i-1))) {
+			unused_extents++;
+		} else {
+			break;
+		}
+	}
+
+	if (unused_extents) {
+		if (unused_extents == pv_entry->pv->pe_total) {
+			/* Always need at least one extent per PV. */
+			unused_extents--;
+		}
+		LOG_DEBUG("PV %s can shrink by %u extents.\n",
+			  pv_entry->segment->name, unused_extents);
+		*shrink_limit = unused_extents * group->vg->pe_size;
+		rc = 0;
+	}
+
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
+/**
+ * check_extra_extents_count
+ *
+ * Verify that there is sufficient space in the metadata PE table to hold the
+ * specified number of additional extents. If not enough space is available,
+ * reduce new_pes appropriately.
+ **/
+void check_extra_extents_count(lvm_physical_volume_t *pv_entry,
+			       u_int32_t *new_pes)
+{
+	u_int64_t pe_start, pe_map_start, pe_map_size;
+	u_int32_t total_pes, available_pes;
+
+	LOG_ENTRY();
+
+	pe_map_start = bytes_to_sectors(pv_entry->pv->pe_on_disk.base);
+	pe_start = lvm_get_pe_start(pv_entry);
+	pe_map_size = pe_start - pe_map_start;
+
+	total_pes = (pe_map_size << EVMS_VSECTOR_SIZE_SHIFT) /
+		    sizeof(pe_disk_t);
+	available_pes = total_pes - pv_entry->pv->pe_total;
+
+	if (*new_pes > available_pes) {
+		LOG_WARNING("Requested expanding PV %s by %u PEs, but only "
+			    "have metadata space available for %u new PEs.\n",
+			    pv_entry->segment->name, *new_pes, available_pes);
+		*new_pes = available_pes;
+	}
+
+	LOG_EXIT_VOID();
+}
+
+/**
+ * lvm_expand_pv_in_container
+ *
+ * Tell the specified PV in this container to expand using the input-objects
+ * and options. After it expands, update the PV's internal data to reflect
+ * the increased space.
+ **/
+int lvm_expand_pv_in_container(storage_container_t *container,
+			       storage_object_t *consumed_object,
+			       storage_object_t *expand_object,
+			       list_anchor_t input_objects,
+			       option_array_t *options)
+{
+	lvm_volume_group_t *group = container->private_data;
+	u_int32_t new_extents, old_pe_count, new_pe_count;
+	lvm_physical_extent_t *old_pe_map, *new_pe_map;
+	u_int64_t old_size, new_size, delta_size;
+	lvm_physical_volume_t *pv_entry;
+	u_int32_t i;
+	int rc;
+
+	LOG_ENTRY();
+	LOG_DEBUG("Expanding object %s in container %s.\n",
+		  consumed_object->name, container->name);
+
+	/* Make sure the consumed-object is really a PV. */
+	if (consumed_object->consuming_container != container) {
+		LOG_ERROR("Attempt to expand object %s which isn't a PV in "
+			  "container %s.\n", consumed_object->name,
+			  container->name);
+		rc = EINVAL;
+		goto out;
+	}
+
+	pv_entry = lvm_get_pv_for_segment(consumed_object);
+	if (!pv_entry) {
+		rc = EINVAL;
+		goto out;
+	}
+
+	/* Tell the PV object to expand. Save the old size. */
+	old_size = consumed_object->size;
+	rc = consumed_object->plugin->functions.plugin->expand(consumed_object,
+							       expand_object,
+							       input_objects,
+							       options);
+	if (rc) {
+		LOG_ERROR("Error expanding object %s in container %s.\n",
+			  consumed_object->name, container->name);
+		goto out;
+	}
+
+	/* Calculate the number of new extents after the object expands. */
+	new_size = consumed_object->size;
+	delta_size = new_size - old_size;
+	new_extents = delta_size / group->vg->pe_size;
+
+	/* Check that there's enough metadata space for the number
+	 * of new extents. It will round down if necessary.
+	 */
+	check_extra_extents_count(pv_entry, &new_extents);
+
+	/* Save the old PE map and allocate a new PE map. */
+	old_pe_map = pv_entry->pe_map;
+	old_pe_count = pv_entry->pv->pe_total;
+	new_pe_count = old_pe_count + new_extents;
+	pv_entry->pv->pe_total = new_pe_count;
+	pv_entry->pv->pv_size = new_size;
+	pv_entry->pe_map = NULL;
+
+	rc = lvm_allocate_pe_map(pv_entry);
+	if (rc) {
+		pv_entry->pv->pe_total = old_pe_count;
+		pv_entry->pv->pv_size = old_size;
+		pv_entry->pe_map = old_pe_map;
+		goto out;
+	}
+	new_pe_map = pv_entry->pe_map;
+
+	/* Update the group information. */
+	group->vg->pe_total += new_extents;
+	container->size += new_extents * group->vg->pe_size;
+
+	/* Copy the contents of the old PE map to the new PE map.
+	 * For each old PE, point that PE's LE at the new PE.
+	 */
+	for (i = 0; i < old_pe_count; i++) {
+		new_pe_map[i].pe = old_pe_map[i].pe;
+		new_pe_map[i].le = old_pe_map[i].le;
+		new_pe_map[i].new_le = old_pe_map[i].new_le;
+
+		if (old_pe_map[i].le &&
+		    old_pe_map[i].le->pe == &(old_pe_map[i])) {
+			old_pe_map[i].le->pe = &(new_pe_map[i]);
+		}
+		if (old_pe_map[i].new_le &&
+		    old_pe_map[i].new_le->new_pe == &(old_pe_map[i])) {
+			old_pe_map[i].new_le->new_pe = &(new_pe_map[i]);
+		}
+	}
+
+	/* The freespace volume needs to be adjusted. */
+	rc = lvm_update_freespace_volume(group);
+	if (rc) {
+		LOG_SERIOUS("Error updating freespace for container %s\n",
+			    container->name);
+		goto out;
+	}
+
+	EngFncs->engine_free(old_pe_map);
+	container->flags |= SCFLAG_DIRTY;
+
+out:
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
+/**
+ * lvm_shrink_pv_in_container
+ *
+ * Tell the specified PV in this container to shrink using the input-objects
+ * and options. After it shrinks, update the PV's internal data to reflect
+ * the decreased space.
+ **/
+int lvm_shrink_pv_in_container(storage_container_t *container,
+			       storage_object_t *consumed_object,
+			       storage_object_t *shrink_object,
+			       list_anchor_t input_objects,
+			       option_array_t *options)
+{
+	u_int64_t shrink_limit, old_size, new_size, delta_size;
+	u_int32_t removed_extents, old_pe_count, new_pe_count;
+	lvm_volume_group_t *group = container->private_data;
+	lvm_physical_extent_t *old_pe_map, *new_pe_map;
+	lvm_physical_volume_t *pv_entry;
+	u_int32_t i;
+	int rc;
+
+	LOG_ENTRY();
+	LOG_DEBUG("Shrinking object %s in container %s.\n",
+		  consumed_object->name, container->name);
+
+	/* Make sure the consumed-object is really a PV. */
+	if (consumed_object->consuming_container != container) {
+		LOG_ERROR("Attempt to shrink object %s which isn't a PV in "
+			  "container %s.\n", consumed_object->name,
+			  container->name);
+		rc = EINVAL;
+		goto out;
+	}
+
+	pv_entry = lvm_get_pv_for_segment(consumed_object);
+	if (!pv_entry) {
+		rc = EINVAL;
+		goto out;
+	}
+
+	/* Check that we can allow this PV to shrink. */
+	rc = lvm_can_shrink_pv(pv_entry, &shrink_limit);
+	if (rc) {
+		LOG_ERROR("Cannot shrink object %s.\n", consumed_object->name);
+		goto out;
+	}
+
+	/* Tell the PV object to shrink. Save the old size. */
+	old_size = consumed_object->size;
+	rc = consumed_object->plugin->functions.plugin->shrink(consumed_object,
+							       shrink_object,
+							       input_objects,
+							       options);
+	if (rc) {
+		LOG_ERROR("Error shrinking object %s in container %s.\n",
+			  consumed_object->name, container->name);
+		goto out;
+	}
+
+	/* Calculate the number of extents to remove after the object shrunk. */
+	new_size = consumed_object->size;
+	delta_size = old_size - new_size;
+	removed_extents = (delta_size / group->vg->pe_size) + 
+			  ((delta_size % group->vg->pe_size) ? 1 : 0);
+
+	/* Save the old PE map and allocate a new PE map. */
+	old_pe_map = pv_entry->pe_map;
+	old_pe_count = pv_entry->pv->pe_total;
+	new_pe_count = old_pe_count - removed_extents;
+	pv_entry->pv->pe_total = new_pe_count;
+	pv_entry->pv->pv_size = new_size;
+	pv_entry->pe_map = NULL;
+
+	rc = lvm_allocate_pe_map(pv_entry);
+	if (rc) {
+		pv_entry->pv->pe_total = old_pe_count;
+		pv_entry->pv->pv_size = old_size;
+		pv_entry->pe_map = old_pe_map;
+		goto out;
+	}
+	new_pe_map = pv_entry->pe_map;
+
+	/* Update the group information. */
+	group->vg->pe_total -= removed_extents;
+	container->size -= removed_extents * group->vg->pe_size;
+
+	/* Copy the contents of the old PE map to the new PE map.
+	 * For each old PE, point that PE's LE at the new PE.
+	 */
+	for (i = 0; i < new_pe_count; i++) {
+		new_pe_map[i].pe = old_pe_map[i].pe;
+		new_pe_map[i].le = old_pe_map[i].le;
+		new_pe_map[i].new_le = old_pe_map[i].new_le;
+
+		if (old_pe_map[i].le &&
+		    old_pe_map[i].le->pe == &(old_pe_map[i])) {
+			old_pe_map[i].le->pe = &(new_pe_map[i]);
+		}
+		if (old_pe_map[i].new_le &&
+		    old_pe_map[i].new_le->new_pe == &(old_pe_map[i])) {
+			old_pe_map[i].new_le->new_pe = &(new_pe_map[i]);
+		}
+	}
+
+	/* The freespace volume needs to be adjusted. */
+	rc = lvm_update_freespace_volume(group);
+	if (rc) {
+		LOG_SERIOUS("Error updating freespace for container %s\n",
+			    container->name);
+		goto out;
+	}
+
+
+	/* Update the container's size. */
+	EngFncs->engine_free(old_pe_map);
+	container->flags |= SCFLAG_DIRTY;
+
+out:
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
