@@ -18,7 +18,7 @@
  * Module: LVM Region Manager
  * File: evms2/engine/plugins/lvm/lvm_info.c
  *
- * Description: 
+ * Description:
  */
 
 #include <stdlib.h>
@@ -43,7 +43,7 @@ int lvm_get_volume_info(lvm_logical_volume_t * volume,
 
 	/* Get memory for the info array. */
 	info = EngFncs->engine_alloc(sizeof(extended_info_array_t) +
-				     sizeof(extended_info_t) * 8);
+				     sizeof(extended_info_t) * 9);
 	if (!info) {
 		LOG_CRITICAL("Memory error creating info array\n");
 		LOG_EXIT_INT(ENOMEM);
@@ -51,76 +51,76 @@ int lvm_get_volume_info(lvm_logical_volume_t * volume,
 	}
 
 	/* Region Name */
-	SET_STRING(info->info[i].name, "LV_Name");
-	SET_STRING(info->info[i].title, "Region Name");
-	SET_STRING(info->info[i].desc, "Name of LVM Region (LV)");
+	info->info[i].name = EngFncs->engine_strdup("LV_Name");
+	info->info[i].title = EngFncs->engine_strdup(_("Region Name"));
+	info->info[i].desc = EngFncs->engine_strdup(_("Name of LVM Region (LV)"));
 	info->info[i].type = EVMS_Type_String;
-	SET_STRING(info->info[i].value.s, volume->region->name);
+	info->info[i].value.s = EngFncs->engine_strdup(volume->region->name);
 	i++;
 
 	/* Container Name */
-	SET_STRING(info->info[i].name, "VG_Name");
-	SET_STRING(info->info[i].title, "Container Name");
-	SET_STRING(info->info[i].desc, "Name of LVM Container (VG)");
+	info->info[i].name = EngFncs->engine_strdup("VG_Name");
+	info->info[i].title = EngFncs->engine_strdup(_("Container Name"));
+	info->info[i].desc = EngFncs->engine_strdup(_("Name of LVM Container (VG)"));
 	info->info[i].type = EVMS_Type_String;
-	SET_STRING(info->info[i].value.s, volume->group->container->name);
+	info->info[i].value.s = EngFncs->engine_strdup(volume->group->container->name);
 	i++;
 
 	/* LV Number */
-	SET_STRING(info->info[i].name, "LV_Number");
-	SET_STRING(info->info[i].title, "Region Number");
-	SET_STRING(info->info[i].desc, "ID number for this region in this container");
+	info->info[i].name = EngFncs->engine_strdup("LV_Number");
+	info->info[i].title = EngFncs->engine_strdup(_("Region Number"));
+	info->info[i].desc = EngFncs->engine_strdup(_("ID number for this region in this container"));
 	info->info[i].type = EVMS_Type_Int;
 	info->info[i].value.i = volume->number;
 	i++;
 
 	/* Region Size */
-	SET_STRING(info->info[i].name, "LV_Size");
-	SET_STRING(info->info[i].title, "Region Size");
-	SET_STRING(info->info[i].desc, "Total space for this region");
+	info->info[i].name = EngFncs->engine_strdup("LV_Size");
+	info->info[i].title = EngFncs->engine_strdup(_("Region Size"));
+	info->info[i].desc = EngFncs->engine_strdup(_("Total space for this region"));
 	info->info[i].type = EVMS_Type_Unsigned_Int32;
 	info->info[i].unit = EVMS_Unit_Sectors;
 	info->info[i].value.ui32 = volume->region->size;
 	i++;
 
 	/* Allocated LEs */
-	SET_STRING(info->info[i].name, "Extents");
-	SET_STRING(info->info[i].title, "Logical Extents");
-	SET_STRING(info->info[i].desc, "Number of logical extents used by this region");
+	info->info[i].name = EngFncs->engine_strdup("Extents");
+	info->info[i].title = EngFncs->engine_strdup(_("Logical Extents"));
+	info->info[i].desc = EngFncs->engine_strdup(_("Number of logical extents used by this region"));
 	info->info[i].type = EVMS_Type_Unsigned_Int32;
 	info->info[i].value.ui32 = volume->lv->lv_allocated_le;
 	info->info[i].flags = EVMS_EINFO_FLAGS_MORE_INFO_AVAILABLE;
 	i++;
 
 	/* Write Permissions */
-	SET_STRING(info->info[i].name, "Permissions");
-	SET_STRING(info->info[i].title, "Access Permissions");
+	info->info[i].name = EngFncs->engine_strdup("Permissions");
+	info->info[i].title = EngFncs->engine_strdup(_("Access Permissions"));
 	info->info[i].type = EVMS_Type_String;
 	if (volume->lv->lv_access & LV_WRITE &&
 	    volume->lv->lv_access & LV_READ ) {
-		SET_STRING(info->info[i].value.s, "Read-Write");
+		info->info[i].value.s = EngFncs->engine_strdup(_("Read-Write"));
 	} else if (volume->lv->lv_access & LV_WRITE) {
-		SET_STRING(info->info[i].value.s, "Write-Only");
+		info->info[i].value.s = EngFncs->engine_strdup(_("Write-Only"));
 	} else if (volume->lv->lv_access & LV_READ) {
-		SET_STRING(info->info[i].value.s, "Read-Only");
+		info->info[i].value.s = EngFncs->engine_strdup(_("Read-Only"));
 	} else {
-		SET_STRING(info->info[i].value.s, "No-Access");
+		info->info[i].value.s = EngFncs->engine_strdup(_("No-Access"));
 	}
 	i++;
 
 	/* Striping Info */
 	if (volume->lv->lv_stripes > 1) {
 		/* Number of Stripes */
-		SET_STRING(info->info[i].name, "Stripes");
-		SET_STRING(info->info[i].title, "Stripes");
-		SET_STRING(info->info[i].desc, "Number of objects this region is striped across");
+		info->info[i].name = EngFncs->engine_strdup("Stripes");
+		info->info[i].title = EngFncs->engine_strdup(_("Stripes"));
+		info->info[i].desc = EngFncs->engine_strdup(_("Number of objects this region is striped across"));
 		info->info[i].type = EVMS_Type_Unsigned_Int32;
 		info->info[i].value.ui32 = volume->lv->lv_stripes;
 		i++;
 
 		/* Stripe Size */
-		SET_STRING(info->info[i].name, "Stripe_Size");
-		SET_STRING(info->info[i].title, "Stripe Size");
+		info->info[i].name = EngFncs->engine_strdup("Stripe_Size");
+		info->info[i].title = EngFncs->engine_strdup(_("Stripe Size"));
 		info->info[i].type = EVMS_Type_Unsigned_Int32;
 		info->info[i].unit = EVMS_Unit_Sectors;
 		info->info[i].value.ui32 = volume->lv->lv_stripesize;
@@ -128,11 +128,11 @@ int lvm_get_volume_info(lvm_logical_volume_t * volume,
 	}
 	
 	if (volume->flags & LVM_LV_FLAG_INCOMPLETE) {
-		SET_STRING(info->info[i].name, "Incomplete_LV");
-		SET_STRING(info->info[i].title, "INCOMPLETE REGION!!!");
-		SET_STRING(info->info[i].desc, "This region has extents which are not mapped! One or more objects are probably missing from this container!");
+		info->info[i].name = EngFncs->engine_strdup("Incomplete_LV");
+		info->info[i].title = EngFncs->engine_strdup(_("INCOMPLETE REGION!!!"));
+		info->info[i].desc = EngFncs->engine_strdup(_("This region has extents which are not mapped! One or more objects are probably missing from this container!"));
 		info->info[i].type = EVMS_Type_String;
-		SET_STRING(info->info[i].value.s, "INCOMPLETE REGION!!!");
+		info->info[i].value.s = EngFncs->engine_strdup(_("INCOMPLETE REGION!!!"));
 		i++;
 	}
 
@@ -206,7 +206,7 @@ int lvm_get_volume_extent_info(lvm_logical_volume_t * volume,
 	LOG_ENTRY();
 
 	/* Get memory for the info array. */
-	info = EngFncs->engine_alloc(sizeof(extended_info_array_t));
+	info = EngFncs->engine_alloc(sizeof(extended_info_array_t) + sizeof(extended_info_t));
 	if (!info) {
 		LOG_CRITICAL("Memory error creating info array\n");
 		LOG_EXIT_INT(ENOMEM);
@@ -214,16 +214,16 @@ int lvm_get_volume_extent_info(lvm_logical_volume_t * volume,
 	}
 
 	/* Extent mappings */
-	SET_STRING(info->info[i].name, "Extent_Map");
-	SET_STRING(info->info[i].title, "Logical Extents");
-	SET_STRING(info->info[i].desc, "LE Number : PV Name : PE Number");
+	info->info[i].name = EngFncs->engine_strdup("Extent_Map");
+	info->info[i].title = EngFncs->engine_strdup(_("Logical Extents"));
+	info->info[i].desc = EngFncs->engine_strdup(_("LE Number : PV Name : PE Number"));
 	info->info[i].type = EVMS_Type_String;
 	info->info[i].collection_type = EVMS_Collection_List;
 	info->info[i].collection.list = EngFncs->engine_alloc(sizeof(value_list_t) +
 							      sizeof(value_t) *
-							      volume->lv->lv_allocated_le);
-	snprintf(buffer, 150, "%-5s : %-15s : %-5s", "LE #", "PV Name", "PE #");
-	SET_STRING(info->info[i].collection.list->value[info_line].s, buffer);
+							      (volume->lv->lv_allocated_le + 1));
+	snprintf(buffer, 150, "%-5s : %-15s : %-5s", "LE #", _("PV Name"), _("PE Number"));
+	info->info[i].collection.list->value[info_line].s = EngFncs->engine_strdup(buffer);
 	info_line++;
 
 	/* Print info for each LE. Condense runs of consecutive
@@ -232,7 +232,7 @@ int lvm_get_volume_extent_info(lvm_logical_volume_t * volume,
 	for (j = 0; j < volume->lv->lv_allocated_le; j++) {
 		if (!volume->le_map[j].pe) {
 			snprintf(buffer, 150, "%-5d : %-15s : %-5s",
-				 j, "Missing PV", "");
+				 j, _("Missing PV"), "");
 			consecutive_run = FALSE;
 		}
 		/* Are we at the end of the LV or at the end of a run
@@ -262,7 +262,7 @@ int lvm_get_volume_extent_info(lvm_logical_volume_t * volume,
 		else {
 			continue;
 		}
-		SET_STRING(info->info[i].collection.list->value[info_line].s, buffer);
+		info->info[i].collection.list->value[info_line].s = EngFncs->engine_strdup(buffer);
 		info_line++;
 	}
 	info->info[i].collection.list->count = info_line;
@@ -289,7 +289,7 @@ int lvm_get_group_info(lvm_volume_group_t * group,
 
 	/* Get memory for the info array. */
 	info = EngFncs->engine_alloc(sizeof(extended_info_array_t) +
-				     sizeof(extended_info_t) * 12);
+				     sizeof(extended_info_t) * 13);
 	if (!info) {
 		LOG_CRITICAL("Memory error creating info array\n");
 		LOG_EXIT_INT(ENOMEM);
@@ -297,43 +297,43 @@ int lvm_get_group_info(lvm_volume_group_t * group,
 	}
 
 	/* Container Name */
-	SET_STRING(info->info[i].name, "VG_Name");
-	SET_STRING(info->info[i].title, "Container Name");
-	SET_STRING(info->info[i].desc, "Name of LVM Container (VG)");
+	info->info[i].name = EngFncs->engine_strdup("VG_Name");
+	info->info[i].title = EngFncs->engine_strdup(_("Container Name"));
+	info->info[i].desc = EngFncs->engine_strdup(_("Name of LVM Container (VG)"));
 	info->info[i].type = EVMS_Type_String;
-	SET_STRING(info->info[i].value.s, group->container->name);
+	info->info[i].value.s = EngFncs->engine_strdup(group->container->name);
 	i++;
 
 	/* VG Number */
-	SET_STRING(info->info[i].name, "VG_Number");
-	SET_STRING(info->info[i].title, "Container Number");
-	SET_STRING(info->info[i].desc, "ID number for this container in the LVM plugin");
+	info->info[i].name = EngFncs->engine_strdup("VG_Number");
+	info->info[i].title = EngFncs->engine_strdup(_("Container Number"));
+	info->info[i].desc = EngFncs->engine_strdup(_("ID number for this container in the LVM plugin"));
 	info->info[i].type = EVMS_Type_Int;
 	info->info[i].value.i = group->vg->vg_number;
 	i++;
 
 	/* VG Size */
-	SET_STRING(info->info[i].name, "VG_Size");
-	SET_STRING(info->info[i].title, "Container Size");
-	SET_STRING(info->info[i].desc, "Total accumulated space in this container");
+	info->info[i].name = EngFncs->engine_strdup("VG_Size");
+	info->info[i].title = EngFncs->engine_strdup(_("Container Size"));
+	info->info[i].desc = EngFncs->engine_strdup(_("Total accumulated space in this container"));
 	info->info[i].type = EVMS_Type_Unsigned_Int32;
 	info->info[i].unit = EVMS_Unit_Sectors;
 	info->info[i].value.ui32 = group->container->size;
 	i++;
 
 	/* Available VG Size */
-	SET_STRING(info->info[i].name, "VG_Free_Size");
-	SET_STRING(info->info[i].title, "Available Size");
-	SET_STRING(info->info[i].desc, "Amount of space currently available for allocating to regions");
+	info->info[i].name = EngFncs->engine_strdup("VG_Free_Size");
+	info->info[i].title = EngFncs->engine_strdup(_("Available Size"));
+	info->info[i].desc = EngFncs->engine_strdup(_("Amount of space currently available for allocating to regions"));
 	info->info[i].type = EVMS_Type_Unsigned_Int32;
 	info->info[i].unit = EVMS_Unit_Sectors;
 	info->info[i].value.ui32 = group->freespace->lv->lv_size;
 	i++;
 
 	/* Percent Allocated */
-	SET_STRING(info->info[i].name, "VG_Percent_Allocated");
-	SET_STRING(info->info[i].title, "Percent Allocated");
-	SET_STRING(info->info[i].desc, "Percentage of space currently allocated to regions");
+	info->info[i].name = EngFncs->engine_strdup("VG_Percent_Allocated");
+	info->info[i].title = EngFncs->engine_strdup(_("Percent Allocated"));
+	info->info[i].desc = EngFncs->engine_strdup(_("Percentage of space currently allocated to regions"));
 	info->info[i].type = EVMS_Type_Real32;
 	info->info[i].unit = EVMS_Unit_Percent;
 	info->info[i].value.r32 = ((float)(group->vg->pe_allocated) /
@@ -341,71 +341,71 @@ int lvm_get_group_info(lvm_volume_group_t * group,
 	i++;
 
 	/* PE Size */
-	SET_STRING(info->info[i].name, "PE_Size");
-	SET_STRING(info->info[i].title, "Extent Size");
-	SET_STRING(info->info[i].desc, "Size of each extent available for allocating to regions");
+	info->info[i].name = EngFncs->engine_strdup("PE_Size");
+	info->info[i].title = EngFncs->engine_strdup(_("Extent Size"));
+	info->info[i].desc = EngFncs->engine_strdup(_("Size of each extent available for allocating to regions"));
 	info->info[i].type = EVMS_Type_Unsigned_Int32;
 	info->info[i].unit = EVMS_Unit_Sectors;
 	info->info[i].value.ui32 = group->vg->pe_size;
 	i++;
 
 	/* Total PEs */
-	SET_STRING(info->info[i].name, "Total_PEs");
-	SET_STRING(info->info[i].title, "Total PEs");
-	SET_STRING(info->info[i].desc, "Total number of extents in the container");
+	info->info[i].name = EngFncs->engine_strdup("Total_PEs");
+	info->info[i].title = EngFncs->engine_strdup(_("Total PEs"));
+	info->info[i].desc = EngFncs->engine_strdup(_("Total number of extents in the container"));
 	info->info[i].type = EVMS_Type_Unsigned_Int32;
 	info->info[i].value.ui32 = group->vg->pe_total;
 	i++;
 
 	/* Available PEs */
-	SET_STRING(info->info[i].name, "Available_PEs");
-	SET_STRING(info->info[i].title, "Available PEs");
-	SET_STRING(info->info[i].desc, "Number of extents available for allocating to regions");
+	info->info[i].name = EngFncs->engine_strdup("Available_PEs");
+	info->info[i].title = EngFncs->engine_strdup(_("Available PEs"));
+	info->info[i].desc = EngFncs->engine_strdup(_("Number of extents available for allocating to regions"));
 	info->info[i].type = EVMS_Type_Unsigned_Int32;
 	info->info[i].value.ui32 = group->freespace->lv->lv_allocated_le;
 	i++;
 
 	/* Number of PVs */
-	SET_STRING(info->info[i].name, "Current_PVs");
-	SET_STRING(info->info[i].title, "Number of Objects (PVs)");
-	SET_STRING(info->info[i].desc, "Number of objects consumed by this container");
+	info->info[i].name = EngFncs->engine_strdup("Current_PVs");
+	info->info[i].title = EngFncs->engine_strdup(_("Number of Objects (PVs)"));
+	info->info[i].desc = EngFncs->engine_strdup(_("Number of objects consumed by this container"));
 	info->info[i].type = EVMS_Type_Unsigned_Int32;
 	info->info[i].value.ui32 = group->pv_count;
 	info->info[i].flags = EVMS_EINFO_FLAGS_MORE_INFO_AVAILABLE;
 	i++;
 
 	/* Number of LVs */
-	SET_STRING(info->info[i].name, "Current_LVs");
-	SET_STRING(info->info[i].title, "Number of Regions (LVs)");
-	SET_STRING(info->info[i].desc, "Number of regions produced by this container");
+	info->info[i].name = EngFncs->engine_strdup("Current_LVs");
+	info->info[i].title = EngFncs->engine_strdup(_("Number of Regions (LVs)"));
+	info->info[i].desc = EngFncs->engine_strdup(_("Number of regions produced by this container"));
 	info->info[i].type = EVMS_Type_Unsigned_Int32;
 	info->info[i].value.ui32 = group->volume_count;
 	info->info[i].flags = EVMS_EINFO_FLAGS_MORE_INFO_AVAILABLE;
 	i++;
 
 	/* Max LV Size */
-	SET_STRING(info->info[i].name, "Max_LV_Size");
-	SET_STRING(info->info[i].title, "Maximum Region Size");
-	SET_STRING(info->info[i].desc, "Maximum possible size of any region in this container for the given PE size");
+	info->info[i].name = EngFncs->engine_strdup("Max_LV_Size");
+	info->info[i].title = EngFncs->engine_strdup(_("Maximum Region Size"));
+	info->info[i].desc = EngFncs->engine_strdup(_("Maximum possible size of any region in this container for the given PE size"));
 	info->info[i].type = EVMS_Type_Unsigned_Int32;
 	info->info[i].unit = EVMS_Unit_Sectors;
 	info->info[i].value.ui32 = group->vg->pe_size * 64 * 1024;
 	i++;
 
 	/* VG UUID */
-	SET_STRING(info->info[i].name, "VG_UUID");
-	SET_STRING(info->info[i].title, "Container UUID");
+	info->info[i].name = EngFncs->engine_strdup("VG_UUID");
+	info->info[i].title = EngFncs->engine_strdup(_("Container UUID"));
 	info->info[i].type = EVMS_Type_String;
 	buf = lvm_print_uuid(group->vg->vg_uuid);
-	SET_STRING(info->info[i].value.s, buf);
+	info->info[i].value.s = EngFncs->engine_strdup(buf);
 	i++;
 
 	if (group->pv_count < group->vg->pv_cur) {
-		SET_STRING(info->info[i].name, "Incomplete_VG");
-		SET_STRING(info->info[i].title, "INCOMPLETE CONTAINER!!!");
-		SET_STRING(info->info[i].desc, "This container seems to be missing one or more objects!");
+		info->info[i].name = EngFncs->engine_strdup("Incomplete_VG");
+		info->info[i].title = EngFncs->engine_strdup(_("INCOMPLETE CONTAINER!!!"));
+		info->info[i].desc = EngFncs->engine_strdup(_("This container seems to be missing one or more objects!"));
 		info->info[i].type = EVMS_Type_String;
-		SET_STRING(info->info[i].value.s, "INCOMPLETE CONTAINER!!!");
+		info->info[i].value.s = EngFncs->engine_strdup(_("INCOMPLETE CONTAINER!!!"));
 		i++;
 	}
 
@@ -430,8 +430,8 @@ int lvm_get_group_pv_list_info(lvm_volume_group_t * group,
 
 	/* Get memory for the info array. */
 	info = EngFncs->engine_alloc(sizeof(extended_info_array_t) +
-				     sizeof(extended_info_t) * 
-				     (group->pv_count-1));
+				     sizeof(extended_info_t) *
+				     group->pv_count);
 	if (!info) {
 		LOG_CRITICAL("Memory error creating info array\n");
 		LOG_EXIT_INT(ENOMEM);
@@ -441,12 +441,12 @@ int lvm_get_group_pv_list_info(lvm_volume_group_t * group,
 	for (j = 1; j <= MAX_PV; j++) {
 		if (group->pv_list[j]) {
 			sprintf(buf, "PV%d", j);
-			SET_STRING(info->info[i].name, buf);
-			sprintf(buf, "Object (PV) %d", j);
-			SET_STRING(info->info[i].title, buf);
+			info->info[i].name = EngFncs->engine_strdup(buf);
+			sprintf(buf, _("Object (PV) %d"), j);
+			info->info[i].title = EngFncs->engine_strdup(buf);
 			info->info[i].type = EVMS_Type_String;
-			SET_STRING(info->info[i].value.s,
-				   group->pv_list[j]->segment->name);
+			info->info[i].value.s =
+				EngFncs->engine_strdup(group->pv_list[j]->segment->name);
 			info->info[i].flags = EVMS_EINFO_FLAGS_MORE_INFO_AVAILABLE;
 			i++;
 		}
@@ -474,7 +474,7 @@ int lvm_get_group_lv_list_info(lvm_volume_group_t * group,
 	/* Get memory for the info array. */
 	info = EngFncs->engine_alloc(sizeof(extended_info_array_t) +
 				     sizeof(extended_info_t) *
-				     (group->volume_count-1));
+				     group->volume_count);
 	if (!info) {
 		LOG_CRITICAL("Memory error creating info array\n");
 		LOG_EXIT_INT(ENOMEM);
@@ -484,12 +484,12 @@ int lvm_get_group_lv_list_info(lvm_volume_group_t * group,
 	for (j = 1; j <= MAX_LV; j++) {
 		if (group->volume_list[j]) {
 			sprintf(buf, "LV%d", j);
-			SET_STRING(info->info[i].name, buf);
-			sprintf(buf, "Region (LV) %d", j);
-			SET_STRING(info->info[i].title, buf);
+			info->info[i].name = EngFncs->engine_strdup(buf);
+			sprintf(buf, _("Region (LV) %d"), j);
+			info->info[i].title = EngFncs->engine_strdup(buf);
 			info->info[i].type = EVMS_Type_String;
-			SET_STRING(info->info[i].value.s,
-				   group->volume_list[j]->region->name);
+			info->info[i].value.s =
+				EngFncs->engine_strdup(group->volume_list[j]->region->name);
 			i++;
 		}
 	}
@@ -516,7 +516,7 @@ int lvm_get_pv_info(lvm_physical_volume_t * pv_entry,
 
 	/* Get memory for the info array. */
 	info = EngFncs->engine_alloc(sizeof(extended_info_array_t) +
-				     sizeof(extended_info_t) * 9);
+				     sizeof(extended_info_t) * 10);
 	if (!info) {
 		LOG_CRITICAL("Memory error creating info array\n");
 		LOG_EXIT_INT(ENOMEM);
@@ -524,58 +524,58 @@ int lvm_get_pv_info(lvm_physical_volume_t * pv_entry,
 	}
 
 	/* PV Name */
-	SET_STRING(info->info[i].name, "PV_Name");
-	SET_STRING(info->info[i].title, "Object Name");
-	SET_STRING(info->info[i].desc, "Name of LVM Object (PV)");
+	info->info[i].name = EngFncs->engine_strdup("PV_Name");
+	info->info[i].title = EngFncs->engine_strdup(_("Object Name"));
+	info->info[i].desc = EngFncs->engine_strdup(_("Name of LVM Object (PV)"));
 	info->info[i].type = EVMS_Type_String;
-	SET_STRING(info->info[i].value.s, pv_entry->segment->name);
+	info->info[i].value.s = EngFncs->engine_strdup(pv_entry->segment->name);
 	i++;
 
 	/* VG Name */
-	SET_STRING(info->info[i].name, "VG_Name");
-	SET_STRING(info->info[i].title, "Container Name");
-	SET_STRING(info->info[i].desc, "Name of LVM Container (VG)");
+	info->info[i].name = EngFncs->engine_strdup("VG_Name");
+	info->info[i].title = EngFncs->engine_strdup(_("Container Name"));
+	info->info[i].desc = EngFncs->engine_strdup(_("Name of LVM Container (VG)"));
 	info->info[i].type = EVMS_Type_String;
-	SET_STRING(info->info[i].value.s, pv_entry->group->container->name);
+	info->info[i].value.s = EngFncs->engine_strdup(pv_entry->group->container->name);
 	i++;
 
 	/* PV Number */
-	SET_STRING(info->info[i].name, "PV_Number");
-	SET_STRING(info->info[i].title, "Object Number");
-	SET_STRING(info->info[i].desc, "ID number for this object (PV) in this container");
+	info->info[i].name = EngFncs->engine_strdup("PV_Number");
+	info->info[i].title = EngFncs->engine_strdup(_("Object Number"));
+	info->info[i].desc = EngFncs->engine_strdup(_("ID number for this object (PV) in this container"));
 	info->info[i].type = EVMS_Type_Int;
 	info->info[i].value.i = pv_entry->number;
 	i++;
 
 	/* PV Size */
-	SET_STRING(info->info[i].name, "PV_Size");
-	SET_STRING(info->info[i].title, "Object Size");
+	info->info[i].name = EngFncs->engine_strdup("PV_Size");
+	info->info[i].title = EngFncs->engine_strdup(_("Object Size"));
 	info->info[i].type = EVMS_Type_Unsigned_Int32;
 	info->info[i].unit = EVMS_Unit_Sectors;
 	info->info[i].value.ui32 = pv_entry->segment->size;
 	i++;
 
 	/* Usable Size */
-	SET_STRING(info->info[i].name, "Usable_PV_Size");
-	SET_STRING(info->info[i].title, "Usable Space");
-	SET_STRING(info->info[i].desc, "PE Size * Number of PEs. Some object space is used for metadata, some is unusable due to PE size");
+	info->info[i].name = EngFncs->engine_strdup("Usable_PV_Size");
+	info->info[i].title = EngFncs->engine_strdup(_("Usable Space"));
+	info->info[i].desc = EngFncs->engine_strdup(_("PE Size * Number of PEs. Some object space is used for metadata, some is unusable due to PE size"));
 	info->info[i].type = EVMS_Type_Unsigned_Int32;
 	info->info[i].unit = EVMS_Unit_Sectors;
 	info->info[i].value.ui32 = pv_entry->pv->pe_size * pv_entry->pv->pe_total;
 	i++;
 
 	/* Number of Regions */
-	SET_STRING(info->info[i].name, "Current_Regions");
-	SET_STRING(info->info[i].title, "Current Regions");
-	SET_STRING(info->info[i].desc, "Number of regions currently using space on this object");
+	info->info[i].name = EngFncs->engine_strdup("Current_Regions");
+	info->info[i].title = EngFncs->engine_strdup(_("Current Regions"));
+	info->info[i].desc = EngFncs->engine_strdup(_("Number of regions currently using space on this object"));
 	info->info[i].type = EVMS_Type_Unsigned_Int32;
 	info->info[i].value.ui32 = pv_entry->pv->lv_cur;
 	i++;
 
 	/* PE Size */
-	SET_STRING(info->info[i].name, "PE_Size");
-	SET_STRING(info->info[i].title, "Extent Size");
-	SET_STRING(info->info[i].desc, "Size of each extent available for allocating to regions");
+	info->info[i].name = EngFncs->engine_strdup("PE_Size");
+	info->info[i].title = EngFncs->engine_strdup(_("Extent Size"));
+	info->info[i].desc = EngFncs->engine_strdup(_("Size of each extent available for allocating to regions"));
 	info->info[i].type = EVMS_Type_Unsigned_Int32;
 	info->info[i].unit = EVMS_Unit_Sectors;
 	info->info[i].value.ui32 = pv_entry->pv->pe_size;
@@ -583,18 +583,18 @@ int lvm_get_pv_info(lvm_physical_volume_t * pv_entry,
 
 	/* Total PEs */
 	sprintf(buf2, "PEMapPV%ld", pv_entry->number);
-	SET_STRING(info->info[i].name, buf2);
-	SET_STRING(info->info[i].title, "Total PEs");
-	SET_STRING(info->info[i].desc, "Total number of extents in this object");
+	info->info[i].name = EngFncs->engine_strdup(buf2);
+	info->info[i].title = EngFncs->engine_strdup(_("Total PEs"));
+	info->info[i].desc = EngFncs->engine_strdup(_("Total number of extents in this object"));
 	info->info[i].type = EVMS_Type_Unsigned_Int32;
 	info->info[i].value.ui32 = pv_entry->pv->pe_total;
 	info->info[i].flags = EVMS_EINFO_FLAGS_MORE_INFO_AVAILABLE;
 	i++;
 
 	/* Available PEs */
-	SET_STRING(info->info[i].name, "Available_PEs");
-	SET_STRING(info->info[i].title, "Available PEs");
-	SET_STRING(info->info[i].desc, "Number of extents available for allocating to regions");
+	info->info[i].name = EngFncs->engine_strdup("Available_PEs");
+	info->info[i].title = EngFncs->engine_strdup(_("Available PEs"));
+	info->info[i].desc = EngFncs->engine_strdup(_("Number of extents available for allocating to regions"));
 	info->info[i].type = EVMS_Type_Unsigned_Int32;
 	info->info[i].value.ui32 = pv_entry->pv->pe_total -
 				   pv_entry->pv->pe_allocated -
@@ -602,11 +602,11 @@ int lvm_get_pv_info(lvm_physical_volume_t * pv_entry,
 	i++;
 
 	/* PV UUID */
-	SET_STRING(info->info[i].name, "PV_UUID");
-	SET_STRING(info->info[i].title, "Object (PV) UUID");
+	info->info[i].name = EngFncs->engine_strdup("PV_UUID");
+	info->info[i].title = EngFncs->engine_strdup(_("Object (PV) UUID"));
 	info->info[i].type = EVMS_Type_String;
 	buf1 = lvm_print_uuid(pv_entry->pv->pv_uuid);
-	SET_STRING(info->info[i].value.s, buf1);
+	info->info[i].value.s = EngFncs->engine_strdup(buf1);
 	i++;
 
 	info->count = i;
@@ -669,7 +669,7 @@ int lvm_get_pv_extent_info(lvm_physical_volume_t * pv_entry,
 	LOG_ENTRY();
 
 	/* Get memory for the info array. */
-	info = EngFncs->engine_alloc(sizeof(extended_info_array_t));
+	info = EngFncs->engine_alloc(sizeof(extended_info_array_t) + sizeof(extended_info_t));
 	if (!info) {
 		LOG_CRITICAL("Memory error creating info array\n");
 		LOG_EXIT_INT(ENOMEM);
@@ -677,16 +677,16 @@ int lvm_get_pv_extent_info(lvm_physical_volume_t * pv_entry,
 	}
 
 	/* Extent mappings */
-	SET_STRING(info->info[i].name, "Extent_Map");
-	SET_STRING(info->info[i].title, "Physical Extents");
-	SET_STRING(info->info[i].desc, "PE Number : LV Name : LE Number: Sector");
+	info->info[i].name = EngFncs->engine_strdup("Extent_Map");
+	info->info[i].title = EngFncs->engine_strdup(_("Physical Extents"));
+	info->info[i].desc = EngFncs->engine_strdup(_("PE Number : LV Name : LE Number: Sector"));
 	info->info[i].type = EVMS_Type_String;
 	info->info[i].collection_type = EVMS_Collection_List;
 	info->info[i].collection.list = EngFncs->engine_alloc(sizeof(value_list_t) +
 							      sizeof(value_t) *
-							      pv_entry->pv->pe_total);
-	snprintf(buffer, 150, "%-5s : %-20s : %-5s : %-5s", "PE #", "LV Name", "LE #", "Sector");
-	SET_STRING(info->info[i].collection.list->value[0].s, buffer);
+							      (pv_entry->pv->pe_total + 1));
+	snprintf(buffer, 150, "%-5s : %-20s : %-5s : %-5s", _("PE Number"), _("LV Name"), _("LE Number"), _("Sector"));
+	info->info[i].collection.list->value[0].s = EngFncs->engine_strdup(buffer);
 
 	/* Print info for each PE. Condense runs of consecutive PEs into
 	 * the first and last, with a ... in between.
@@ -698,13 +698,13 @@ int lvm_get_pv_extent_info(lvm_physical_volume_t * pv_entry,
 				if (! is_next_pe_consecutive(pv_entry, j)) {
 					snprintf(buffer, 150,
 						 "%-5d : %-20s : %-5s : %-5"PRIu64"",
-						 j, "none", "n/a",
+						 j, _("none"), "n/a",
 						 pv_entry->pe_map[j].sector);
 					consecutive_run = FALSE;
 				} else if (!consecutive_run) {
 					snprintf(buffer, 150,
 						 "%-5d : %-20s : %-5s : %-5"PRIu64"",
-						 j, "none", "n/a",
+						 j, _("none"), "n/a",
 						 pv_entry->pe_map[j].sector);
 					consecutive_run = TRUE;
 					printed_dots = FALSE;
@@ -744,7 +744,7 @@ int lvm_get_pv_extent_info(lvm_physical_volume_t * pv_entry,
 				 pe_map[j].sector);
 		}
 
-		SET_STRING(info->info[i].collection.list->value[info_line].s, buffer);
+		info->info[i].collection.list->value[info_line].s = EngFncs->engine_strdup(buffer);
 		info_line++;
 	}
 	info->info[i].collection.list->count = info_line;
