@@ -28,6 +28,8 @@ extern int lvm_translate_container_name_to_vg_name(lvm_volume_group_t * group,
 extern int lvm_translate_lv_name_to_region_name(char * lv_name,
 						char * disk_group_name,
 						char * region_name);
+extern void lvm_translate_region_name_to_lv_name(lvm_logical_volume_t *volume,
+						 char *lv_name);
 extern void lvm_make_lv_name(char * simple_name,
 			     lvm_volume_group_t * group,
 			     char * lv_name);
