@@ -69,7 +69,7 @@ int lvm_create_container_parse_option_array(option_array_t * options,
 			}
 		}
 
-		LOG_EXTRA("Parsing option %d\n", options->option[i].number);
+		LOG_DEBUG("Parsing option %d\n", options->option[i].number);
 
 		/* Use the number-based index to record the option. */
 		switch (options->option[i].number) {
@@ -126,18 +126,18 @@ int lvm_create_container_allocate_option_descriptor(option_desc_array_t * od)
 	od->count = LVM_OPTION_PE_SIZE_INDEX + 1;
 
 	/* Option 0 is VG name. */
-	SET_STRING(od->option[LVM_OPTION_VG_NAME_INDEX].name, LVM_OPTION_VG_NAME_STR);
-	SET_STRING(od->option[LVM_OPTION_VG_NAME_INDEX].title, "Name for new LVM container");
+	od->option[LVM_OPTION_VG_NAME_INDEX].name = EngFncs->engine_strdup(LVM_OPTION_VG_NAME_STR);
+	od->option[LVM_OPTION_VG_NAME_INDEX].title = EngFncs->engine_strdup(_("Name for new LVM container"));
 	od->option[LVM_OPTION_VG_NAME_INDEX].type = EVMS_Type_String;
 	od->option[LVM_OPTION_VG_NAME_INDEX].min_len = 1;
 	od->option[LVM_OPTION_VG_NAME_INDEX].max_len = EVMS_VOLUME_NAME_SIZE;
 	od->option[LVM_OPTION_VG_NAME_INDEX].flags = EVMS_OPTION_FLAGS_NO_INITIAL_VALUE;
-	od->option[LVM_OPTION_VG_NAME_INDEX].value.s = EngFncs->engine_alloc(EVMS_VOLUME_NAME_SIZE);
+	od->option[LVM_OPTION_VG_NAME_INDEX].value.s = EngFncs->engine_alloc(EVMS_VOLUME_NAME_SIZE + 1);
 
 	/* Option 1 is PE size. */
-	SET_STRING(od->option[LVM_OPTION_PE_SIZE_INDEX].name, LVM_OPTION_PE_SIZE_STR);
-	SET_STRING(od->option[LVM_OPTION_PE_SIZE_INDEX].title, "PE size for new container");
-	SET_STRING(od->option[LVM_OPTION_PE_SIZE_INDEX].tip, "Acceptable range: 8kB to 16GB. Must be a power of 2.");
+	od->option[LVM_OPTION_PE_SIZE_INDEX].name = EngFncs->engine_strdup(LVM_OPTION_PE_SIZE_STR);
+	od->option[LVM_OPTION_PE_SIZE_INDEX].title = EngFncs->engine_strdup(_("PE size for new container"));
+	od->option[LVM_OPTION_PE_SIZE_INDEX].tip = EngFncs->engine_strdup(_("Acceptable range: 8kB to 16GB. Must be a power of 2."));
 	od->option[LVM_OPTION_PE_SIZE_INDEX].type = EVMS_Type_Unsigned_Int32;
 	od->option[LVM_OPTION_PE_SIZE_INDEX].unit = EVMS_Unit_Sectors;
 	od->option[LVM_OPTION_PE_SIZE_INDEX].flags = EVMS_OPTION_FLAGS_NOT_REQUIRED|EVMS_OPTION_FLAGS_AUTOMATIC;
@@ -162,13 +162,10 @@ int lvm_create_container_get_acceptable(list_anchor_t acceptable_segments)
 
 	LOG_ENTRY();
 
-	/* Find all top-level disks, segments, and regions in the system. This
-	 * will also retrieve LVM regions, so let the user beware. :)
-	 */
-	rc = EngFncs->get_object_list(DISK|SEGMENT|REGION, DATA_TYPE, NULL, NULL,
-				      VALID_INPUT_OBJECT, &object_list);
+	/* Find all available objects. */
+	rc = lvm_get_available_objects(NULL, &object_list);
 	if (!rc) {
-		rc = EngFncs->merge_lists(acceptable_segments, object_list, NULL);
+		rc = EngFncs->merge_lists(acceptable_segments, object_list, NULL, NULL);
 	}
 
 	LOG_EXIT_INT(rc);
@@ -241,7 +238,7 @@ int lvm_create_container_set_option(task_context_t * context,
 	int rc = 0;
 
 	LOG_ENTRY();
-	LOG_EXTRA("Setting option %d\n", index);
+	LOG_DEBUG("Setting option %d\n", index);
 
 	switch (index) {
 
@@ -271,7 +268,7 @@ int lvm_create_container_set_option(task_context_t * context,
 			rc = lvm_check_segment_for_pe_size(segment,
 							   &(value->ui32));
 			if (rc) {
-				MESSAGE("One or more objects too small for PE size.\n");
+				MESSAGE(_("One or more objects too small for PE size.\n"));
 				rc = ENOSPC;
 				break;
 			}
@@ -379,7 +376,7 @@ int lvm_create_region_parse_option_array(option_array_t * options,
 			}
 		}
 
-		LOG_EXTRA("Parsing option %d\n", options->option[i].number);
+		LOG_DEBUG("Parsing option %d\n", options->option[i].number);
 
 		/* Use the number-based index to record the option. */
 		switch (options->option[i].number) {
@@ -474,7 +471,7 @@ int lvm_create_region_verify_options(lvm_lv_create_options_t * lv_opts,
 		if (rc) {
 			lv_opts->extents += (lv_opts->stripes - rc);
 			lv_opts->lv_size = lv_opts->extents * group->vg->pe_size;
-			MESSAGE("Rounding size up to stripes boundary: %d\n",
+			MESSAGE(_("Rounding size up to stripes boundary: %d\n"),
 				lv_opts->lv_size);
 			rc = 0;
 		}
@@ -530,44 +527,44 @@ int lvm_create_region_allocate_option_descriptor(option_desc_array_t * od)
 	od->count = LVM_OPTION_PV_NAMES_INDEX + 1;
 
 	/* Option 0 is LV Name. */
-	SET_STRING(od->option[LVM_OPTION_LV_NAME_INDEX].name, LVM_OPTION_LV_NAME_STR);
-	SET_STRING(od->option[LVM_OPTION_LV_NAME_INDEX].title, "Name for new LVM Region (LV)");
+	od->option[LVM_OPTION_LV_NAME_INDEX].name = EngFncs->engine_strdup(LVM_OPTION_LV_NAME_STR);
+	od->option[LVM_OPTION_LV_NAME_INDEX].title = EngFncs->engine_strdup(_("Name for new LVM Region (LV)"));
 	od->option[LVM_OPTION_LV_NAME_INDEX].type	= EVMS_Type_String;
 	od->option[LVM_OPTION_LV_NAME_INDEX].min_len	= 1;
 	od->option[LVM_OPTION_LV_NAME_INDEX].max_len	= EVMS_VOLUME_NAME_SIZE;
 	od->option[LVM_OPTION_LV_NAME_INDEX].flags	= EVMS_OPTION_FLAGS_NO_INITIAL_VALUE;
-	od->option[LVM_OPTION_LV_NAME_INDEX].value.s	= EngFncs->engine_alloc(EVMS_VOLUME_NAME_SIZE);
+	od->option[LVM_OPTION_LV_NAME_INDEX].value.s	= EngFncs->engine_alloc(EVMS_VOLUME_NAME_SIZE + 1);
 
 	/* Option 1 is Extents. */
-	SET_STRING(od->option[LVM_OPTION_EXTENTS_INDEX].name, LVM_OPTION_EXTENTS_STR);
-	SET_STRING(od->option[LVM_OPTION_EXTENTS_INDEX].title, "Number of logical extents");
-	SET_STRING(od->option[LVM_OPTION_EXTENTS_INDEX].tip, "Extents are the unit of allocation of space for LVM regions. Specify number of Extents, and Size will be adjusted accordingly.");
+	od->option[LVM_OPTION_EXTENTS_INDEX].name = EngFncs->engine_strdup(LVM_OPTION_EXTENTS_STR);
+	od->option[LVM_OPTION_EXTENTS_INDEX].title = EngFncs->engine_strdup(_("Number of logical extents"));
+	od->option[LVM_OPTION_EXTENTS_INDEX].tip = EngFncs->engine_strdup(_("Extents are the unit of allocation of space for LVM regions. Specify number of Extents, and Size will be adjusted accordingly."));
 	od->option[LVM_OPTION_EXTENTS_INDEX].type	= EVMS_Type_Unsigned_Int32;
 	od->option[LVM_OPTION_EXTENTS_INDEX].flags	= EVMS_OPTION_FLAGS_NOT_REQUIRED |
 							  EVMS_OPTION_FLAGS_NO_INITIAL_VALUE |
 							  EVMS_OPTION_FLAGS_AUTOMATIC;
 
 	/* Option 2 is LV Size. */
-	SET_STRING(od->option[LVM_OPTION_LV_SIZE_INDEX].name, LVM_OPTION_LV_SIZE_STR);
-	SET_STRING(od->option[LVM_OPTION_LV_SIZE_INDEX].title, "Size of new region");
-	SET_STRING(od->option[LVM_OPTION_LV_SIZE_INDEX].tip, "Specify Size, and the number of Extents will be adjusted accordingly.");
+	od->option[LVM_OPTION_LV_SIZE_INDEX].name = EngFncs->engine_strdup(LVM_OPTION_LV_SIZE_STR);
+	od->option[LVM_OPTION_LV_SIZE_INDEX].title = EngFncs->engine_strdup(_("Size of new region"));
+	od->option[LVM_OPTION_LV_SIZE_INDEX].tip = EngFncs->engine_strdup(_("Specify Size, and the number of Extents will be adjusted accordingly."));
 	od->option[LVM_OPTION_LV_SIZE_INDEX].type	= EVMS_Type_Unsigned_Int32;
 	od->option[LVM_OPTION_LV_SIZE_INDEX].unit	= EVMS_Unit_Sectors;
 	od->option[LVM_OPTION_LV_SIZE_INDEX].flags	= EVMS_OPTION_FLAGS_NO_INITIAL_VALUE;
 
 	/* Option 3 is Stripes. */
-	SET_STRING(od->option[LVM_OPTION_STRIPES_INDEX].name, LVM_OPTION_STRIPES_STR);
-	SET_STRING(od->option[LVM_OPTION_STRIPES_INDEX].title, "Number of stripes");
-	SET_STRING(od->option[LVM_OPTION_STRIPES_INDEX].tip, "Number of objects (PVs) to use in a striped volume. Leave at 1 for a linear volume.");
+	od->option[LVM_OPTION_STRIPES_INDEX].name = EngFncs->engine_strdup(LVM_OPTION_STRIPES_STR);
+	od->option[LVM_OPTION_STRIPES_INDEX].title = EngFncs->engine_strdup(_("Number of stripes"));
+	od->option[LVM_OPTION_STRIPES_INDEX].tip = EngFncs->engine_strdup(_("Number of objects (PVs) to use in a striped volume. Leave at 1 for a linear volume."));
 	od->option[LVM_OPTION_STRIPES_INDEX].type	= EVMS_Type_Unsigned_Int32;
 	od->option[LVM_OPTION_STRIPES_INDEX].flags	= EVMS_OPTION_FLAGS_NOT_REQUIRED |
 							  EVMS_OPTION_FLAGS_AUTOMATIC;
 	od->option[LVM_OPTION_STRIPES_INDEX].value.ui32	= 1;
 
 	/* Option 4 is Stripe Size. */
-	SET_STRING(od->option[LVM_OPTION_STRIPE_SIZE_INDEX].name, LVM_OPTION_STRIPE_SIZE_STR);
-	SET_STRING(od->option[LVM_OPTION_STRIPE_SIZE_INDEX].title, "Stripe Size");
-	SET_STRING(od->option[LVM_OPTION_STRIPE_SIZE_INDEX].tip, "Granularity at which data is striped across the underlying objects.");
+	od->option[LVM_OPTION_STRIPE_SIZE_INDEX].name = EngFncs->engine_strdup(LVM_OPTION_STRIPE_SIZE_STR);
+	od->option[LVM_OPTION_STRIPE_SIZE_INDEX].title = EngFncs->engine_strdup(_("Stripe Size"));
+	od->option[LVM_OPTION_STRIPE_SIZE_INDEX].tip = EngFncs->engine_strdup(_("Granularity at which data is striped across the underlying objects."));
 	od->option[LVM_OPTION_STRIPE_SIZE_INDEX].type	= EVMS_Type_Unsigned_Int32;
 	od->option[LVM_OPTION_STRIPE_SIZE_INDEX].unit	= EVMS_Unit_Sectors;
 	od->option[LVM_OPTION_STRIPE_SIZE_INDEX].flags	= EVMS_OPTION_FLAGS_NOT_REQUIRED |
@@ -576,9 +573,9 @@ int lvm_create_region_allocate_option_descriptor(option_desc_array_t * od)
 							  EVMS_OPTION_FLAGS_INACTIVE;
 
 	/* Option 5 is PV Names. */
-	SET_STRING(od->option[LVM_OPTION_PV_NAMES_INDEX].name, LVM_OPTION_PV_NAMES_STR);
-	SET_STRING(od->option[LVM_OPTION_PV_NAMES_INDEX].title, "Objects (PVs) to place the region on");
-	SET_STRING(od->option[LVM_OPTION_PV_NAMES_INDEX].tip, "Region will be allocated on only these objects. Leave blank for automatic allocation.");
+	od->option[LVM_OPTION_PV_NAMES_INDEX].name = EngFncs->engine_strdup(LVM_OPTION_PV_NAMES_STR);
+	od->option[LVM_OPTION_PV_NAMES_INDEX].title = EngFncs->engine_strdup(_("Objects (PVs) to place the region on"));
+	od->option[LVM_OPTION_PV_NAMES_INDEX].tip = EngFncs->engine_strdup(_("Region will be allocated on only these objects. Leave blank for automatic allocation."));
 	od->option[LVM_OPTION_PV_NAMES_INDEX].type		= EVMS_Type_String;
 	od->option[LVM_OPTION_PV_NAMES_INDEX].min_len		= 1;
 	od->option[LVM_OPTION_PV_NAMES_INDEX].max_len		= EVMS_VOLUME_NAME_SIZE;
@@ -655,7 +652,7 @@ int lvm_create_region_set_objects(task_context_t * context,
 		  freespace->lv->lv_allocated_le : LVM_PE_T_MAX;
 	stripes = lvm_get_available_stripes(group);
 
-	LOG_EXTRA("Setting object %s\n", freespace->region->name);
+	LOG_DEBUG("Setting object %s\n", freespace->region->name);
 
 	/* Now that we have a selected freespace, we can update the option
 	 * descriptor appropriately.
@@ -689,7 +686,7 @@ int lvm_create_region_set_objects(task_context_t * context,
 	for (i = 1, j = 0; i < MAX_PV; i++) {
 		if (group->pv_list[i] &&
 		    lvm_pv_has_available_extents(group->pv_list[i])) {
-			SET_STRING(od->option[LVM_OPTION_PV_NAMES_INDEX].constraint.list->value[j].s, group->pv_list[i]->segment->name);
+			od->option[LVM_OPTION_PV_NAMES_INDEX].constraint.list->value[j].s = EngFncs->engine_strdup(group->pv_list[i]->segment->name);
 			j++;
 		}
 	}
@@ -733,7 +730,7 @@ int lvm_create_region_set_option(task_context_t * context,
 	}
 	group = freespace->group;
 
-	LOG_EXTRA("Setting option %d\n", index);
+	LOG_DEBUG("Setting option %d\n", index);
 
 	switch (index) {
 
@@ -831,7 +828,7 @@ int lvm_create_region_set_option(task_context_t * context,
 				EngFncs->engine_free(od->option[index].value.list->value[i].s);
 				od->option[index].value.list->value[i].s = NULL;
 			}
-			SET_STRING(od->option[index].value.list->value[i].s, value->list->value[i].s);
+			od->option[index].value.list->value[i].s = EngFncs->engine_strdup(value->list->value[i].s);
 		}
 		for (; i < od->option[index].value.list->count; i++) {
 			if (od->option[index].value.list->value[i].s) {
@@ -894,7 +891,7 @@ int lvm_expand_region_parse_option_array(option_array_t * options,
 			}
 		}
 
-		LOG_EXTRA("Parsing option %d\n", options->option[i].number);
+		LOG_DEBUG("Parsing option %d\n", options->option[i].number);
 
 		/* Use the number-based index to record the option. */
 		switch (options->option[i].number) {
@@ -985,7 +982,7 @@ int lvm_expand_region_verify_options(lvm_lv_expand_options_t * lv_opts,
 	}
 
 	/* Check for sufficient space in the group. This is only a
-	 * rough check. Space constraints on striping 
+	 * rough check. Space constraints on striping
 	 * will happen later.
 	 */
 	if (lv_opts->add_extents > group->freespace->lv->lv_allocated_le) {
@@ -1017,26 +1014,26 @@ int lvm_expand_region_allocate_option_descriptor(option_desc_array_t * od)
 	od->count = LVM_OPTION_EXPAND_PV_NAMES_INDEX + 1;
 
 	/* Option 0 is expansion extents. */
-	SET_STRING(od->option[LVM_OPTION_EXPAND_EXTENTS_INDEX].name, LVM_OPTION_EXPAND_EXTENTS_STR);
-	SET_STRING(od->option[LVM_OPTION_EXPAND_EXTENTS_INDEX].title, "Additional Extents");
-	SET_STRING(od->option[LVM_OPTION_EXPAND_EXTENTS_INDEX].tip, "Number of extents to add to the selected LVM region. Only specify extents or size!");
+	od->option[LVM_OPTION_EXPAND_EXTENTS_INDEX].name = EngFncs->engine_strdup(LVM_OPTION_EXPAND_EXTENTS_STR);
+	od->option[LVM_OPTION_EXPAND_EXTENTS_INDEX].title = EngFncs->engine_strdup(_("Additional Extents"));
+	od->option[LVM_OPTION_EXPAND_EXTENTS_INDEX].tip = EngFncs->engine_strdup(_("Number of extents to add to the selected LVM region. Only specify extents or size!"));
 	od->option[LVM_OPTION_EXPAND_EXTENTS_INDEX].type = EVMS_Type_Unsigned_Int32;
 	od->option[LVM_OPTION_EXPAND_EXTENTS_INDEX].flags = EVMS_OPTION_FLAGS_NOT_REQUIRED |
 							    EVMS_OPTION_FLAGS_NO_INITIAL_VALUE |
 							    EVMS_OPTION_FLAGS_AUTOMATIC;
 
 	/* Option 1 is expansion size. */
-	SET_STRING(od->option[LVM_OPTION_EXPAND_SIZE_INDEX].name, LVM_OPTION_EXPAND_SIZE_STR);
-	SET_STRING(od->option[LVM_OPTION_EXPAND_SIZE_INDEX].title, "Additional Size");
-	SET_STRING(od->option[LVM_OPTION_EXPAND_SIZE_INDEX].tip, "Amount of space to add to the selected LVM region");
+	od->option[LVM_OPTION_EXPAND_SIZE_INDEX].name = EngFncs->engine_strdup(LVM_OPTION_EXPAND_SIZE_STR);
+	od->option[LVM_OPTION_EXPAND_SIZE_INDEX].title = EngFncs->engine_strdup(_("Additional Size"));
+	od->option[LVM_OPTION_EXPAND_SIZE_INDEX].tip = EngFncs->engine_strdup(_("Amount of space to add to the selected LVM region"));
 	od->option[LVM_OPTION_EXPAND_SIZE_INDEX].type = EVMS_Type_Unsigned_Int32;
 	od->option[LVM_OPTION_EXPAND_SIZE_INDEX].unit = EVMS_Unit_Sectors;
 	od->option[LVM_OPTION_EXPAND_SIZE_INDEX].flags = EVMS_OPTION_FLAGS_NO_INITIAL_VALUE;
 
 	/* Option 2 is PV Names. */
-	SET_STRING(od->option[LVM_OPTION_EXPAND_PV_NAMES_INDEX].name, LVM_OPTION_EXPAND_PV_NAMES_STR);
-	SET_STRING(od->option[LVM_OPTION_EXPAND_PV_NAMES_INDEX].title, "Objects (PVs) to expand the region onto");
-	SET_STRING(od->option[LVM_OPTION_EXPAND_PV_NAMES_INDEX].tip, "Region will be expanded only onto these objects. Leave blank for automatic allocation.");
+	od->option[LVM_OPTION_EXPAND_PV_NAMES_INDEX].name = EngFncs->engine_strdup(LVM_OPTION_EXPAND_PV_NAMES_STR);
+	od->option[LVM_OPTION_EXPAND_PV_NAMES_INDEX].title = EngFncs->engine_strdup(_("Objects (PVs) to expand the region onto"));
+	od->option[LVM_OPTION_EXPAND_PV_NAMES_INDEX].tip = EngFncs->engine_strdup(_("Region will be expanded only onto these objects. Leave blank for automatic allocation."));
 	od->option[LVM_OPTION_EXPAND_PV_NAMES_INDEX].type = EVMS_Type_String;
 	od->option[LVM_OPTION_EXPAND_PV_NAMES_INDEX].min_len = 1;
 	od->option[LVM_OPTION_EXPAND_PV_NAMES_INDEX].max_len = EVMS_VOLUME_NAME_SIZE;
@@ -1109,7 +1106,7 @@ int lvm_expand_region_init_options(task_context_t * context)
 		goto out;
 	}
 
-	LOG_EXTRA("Setting selected object %s\n", freespace->region->name);
+	LOG_DEBUG("Setting selected object %s\n", freespace->region->name);
 
 	/* Now that we have a selected freespace, we can update the option
 	 * descriptor appropriately.
@@ -1133,7 +1130,7 @@ int lvm_expand_region_init_options(task_context_t * context)
 	for (i = 1, j = 0; i < MAX_PV; i++) {
 		if (group->pv_list[i] &&
 		    lvm_pv_has_available_extents(group->pv_list[i])) {
-			SET_STRING(od->option[LVM_OPTION_EXPAND_PV_NAMES_INDEX].constraint.list->value[j].s, group->pv_list[i]->segment->name);
+			od->option[LVM_OPTION_EXPAND_PV_NAMES_INDEX].constraint.list->value[j].s = EngFncs->engine_strdup(group->pv_list[i]->segment->name);
 			j++;
 		}
 	}
@@ -1174,7 +1171,7 @@ int lvm_expand_region_set_option(task_context_t * context,
 		add_extents = LVM_PE_T_MAX - volume->lv->lv_allocated_le;
 	}
 
-	LOG_EXTRA("Setting option %d\n", index);
+	LOG_DEBUG("Setting option %d\n", index);
 
 	switch (index) {
 
@@ -1293,7 +1290,7 @@ int lvm_expand_region_set_option(task_context_t * context,
 
 	case LVM_OPTION_EXPAND_PV_NAMES_INDEX:
 		for ( i = 0; i < value->list->count; i++ ) {
-			SET_STRING(od->option[index].value.list->value[i].s, value->list->value[i].s);
+			od->option[index].value.list->value[i].s = EngFncs->engine_strdup(value->list->value[i].s);
 		}
 		for ( ; i < od->option[index].value.list->count; i++ ) {
 			if ( od->option[index].value.list->value[i].s ) {
@@ -1356,7 +1353,7 @@ int lvm_shrink_region_parse_option_array(option_array_t *options,
 			}
 		}
 
-		LOG_EXTRA("Parsing option %d\n", options->option[i].number);
+		LOG_DEBUG("Parsing option %d\n", options->option[i].number);
 
 		/* Use the number-based index to record the option. */
 		switch ( options->option[i].number ) {
@@ -1457,18 +1454,18 @@ int lvm_shrink_region_allocate_option_descriptor(option_desc_array_t * od)
 	od->count = LVM_OPTION_SHRINK_SIZE_INDEX + 1;
 
 	/* Option 0 is shrink extents. */
-	SET_STRING(od->option[LVM_OPTION_SHRINK_EXTENTS_INDEX].name, LVM_OPTION_SHRINK_EXTENTS_STR);
-	SET_STRING(od->option[LVM_OPTION_SHRINK_EXTENTS_INDEX].title, "Shrink by Extents");
-	SET_STRING(od->option[LVM_OPTION_SHRINK_EXTENTS_INDEX].tip, "Number of extents to remove from the selected LVM region. Only specify extents or size!");
+	od->option[LVM_OPTION_SHRINK_EXTENTS_INDEX].name = EngFncs->engine_strdup(LVM_OPTION_SHRINK_EXTENTS_STR);
+	od->option[LVM_OPTION_SHRINK_EXTENTS_INDEX].title = EngFncs->engine_strdup(_("Shrink by Extents"));
+	od->option[LVM_OPTION_SHRINK_EXTENTS_INDEX].tip = EngFncs->engine_strdup(_("Number of extents to remove from the selected LVM region. Only specify extents or size!"));
 	od->option[LVM_OPTION_SHRINK_EXTENTS_INDEX].type	= EVMS_Type_Unsigned_Int32;
 	od->option[LVM_OPTION_SHRINK_EXTENTS_INDEX].flags	= EVMS_OPTION_FLAGS_NOT_REQUIRED |
 								  EVMS_OPTION_FLAGS_NO_INITIAL_VALUE |
 								  EVMS_OPTION_FLAGS_AUTOMATIC;
 
 	/* Option 1 is shrink size. */
-	SET_STRING(od->option[LVM_OPTION_SHRINK_SIZE_INDEX].name, LVM_OPTION_SHRINK_SIZE_STR);
-	SET_STRING(od->option[LVM_OPTION_SHRINK_SIZE_INDEX].title, "Shrink by Size");
-	SET_STRING(od->option[LVM_OPTION_SHRINK_SIZE_INDEX].tip, "Amount of space to remove from the selected LVM region");
+	od->option[LVM_OPTION_SHRINK_SIZE_INDEX].name = EngFncs->engine_strdup(LVM_OPTION_SHRINK_SIZE_STR);
+	od->option[LVM_OPTION_SHRINK_SIZE_INDEX].title = EngFncs->engine_strdup(_("Shrink by Size"));
+	od->option[LVM_OPTION_SHRINK_SIZE_INDEX].tip = EngFncs->engine_strdup(_("Amount of space to remove from the selected LVM region"));
 	od->option[LVM_OPTION_SHRINK_SIZE_INDEX].type	= EVMS_Type_Unsigned_Int32;
 	od->option[LVM_OPTION_SHRINK_SIZE_INDEX].unit	= EVMS_Unit_Sectors;
 	od->option[LVM_OPTION_SHRINK_SIZE_INDEX].flags	= EVMS_OPTION_FLAGS_NO_INITIAL_VALUE;
@@ -1558,7 +1555,7 @@ int lvm_shrink_region_set_option(task_context_t * context,
 	/* Calculate maximum number of extents that can be removed. */
 	remove_extents = volume->lv->lv_allocated_le - 1;
 
-	LOG_EXTRA("Setting option %d\n", index);
+	LOG_DEBUG("Setting option %d\n", index);
 
 	switch (index) {
 
@@ -1704,7 +1701,6 @@ int lvm_expand_container_get_acceptable(storage_container_t * container,
 {
 	lvm_volume_group_t * group = container->private_data;
 	storage_object_t * segment;
-	object_search_flags_t flags;
 	list_anchor_t object_list;
 	list_element_t itr;
 	u_int32_t pe_size;
@@ -1712,12 +1708,8 @@ int lvm_expand_container_get_acceptable(storage_container_t * container,
 
 	LOG_ENTRY();
 
-	/* Find all top-level disks, segments, and regions in the system. This
-	 * will also retrieve LVM regions, so let the user beware. :)
-	 */
-	flags = VALID_INPUT_OBJECT | (container->disk_group ? 0 : NO_DISK_GROUP);
-	rc = EngFncs->get_object_list(DISK|SEGMENT|REGION, DATA_TYPE, NULL,
-				      container->disk_group, flags, &object_list);
+	/* Find all available objects. */
+	rc = lvm_get_available_objects(container, &object_list);
 	if (rc) {
 		goto out;
 	}
@@ -1847,7 +1839,7 @@ void lvm_shrink_container_get_acceptable(storage_container_t * container,
 	 */
 	for ( i = 1; i <= MAX_PV; i++ ) {
 		if ( group->pv_list[i] ) {
-			rc = my_plugin_record->container_functions->can_remove_object(group->pv_list[i]->segment);
+			rc = lvm_can_remove_object(group->pv_list[i]->segment);
 			if (!rc) {
 				EngFncs->insert_thing(acceptable_segments,
 						      group->pv_list[i]->segment,
@@ -1889,7 +1881,7 @@ int lvm_shrink_container_set_objects(task_context_t * context,
 	 * the selected group.
 	 */
 	LIST_FOR_EACH(context->selected_objects, itr, segment) {
-		rc = my_plugin_record->container_functions->can_remove_object(segment);
+		rc = lvm_can_remove_object(segment);
 		if (rc) {
 			/* FIXME: Add this segment to declined objects. */
 			LOG_ERROR("One or more objects are invalid for container shrink.\n");
@@ -1918,3 +1910,274 @@ void lvm_shrink_container_set_option(task_context_t * context,
 	LOG_EXIT_VOID();
 }
 
+
+/****** Volume Rename Option Handlers ******/
+
+
+/**
+ * lvm_set_volume_info_init_task
+ **/
+int lvm_set_volume_info_init_task(task_context_t *context)
+{
+	option_desc_array_t *od = context->option_descriptors;
+	storage_object_t *region = context->object;
+	u_int32_t i;
+
+	LOG_ENTRY();
+	LOG_DEBUG("Initializing set-info task for region %s.\n", region->name);
+
+	/* No info to set for freespace regions. */
+	if (region->data_type != DATA_TYPE) {
+		LOG_ERROR("No information to set for freespace region %s.\n",
+			  region->name);
+		LOG_EXIT_INT(EINVAL);
+		return EINVAL;
+	}
+
+	/* Initialize the "Name" option. */
+	i = LVM_OPTION_LV_NAME_INDEX;
+	od->option[i].name = EngFncs->engine_strdup(LVM_OPTION_LV_NAME_STR);
+	od->option[i].title = EngFncs->engine_strdup(_("New name for this LVM region."));
+	od->option[i].type = EVMS_Type_String;
+	od->option[i].min_len = 1;
+	od->option[i].max_len = EVMS_NAME_SIZE;
+	od->option[i].flags = EVMS_OPTION_FLAGS_NO_INITIAL_VALUE;
+	od->option[i].value.s = EngFncs->engine_alloc(EVMS_NAME_SIZE+1);
+
+	od->count = ++i;
+	context->min_selected_objects = 0;
+	context->max_selected_objects = 0;
+
+	LOG_EXIT_INT(0);
+	return 0;
+}
+
+/**
+ * lvm_set_volume_info_set_option
+ **/
+int lvm_set_volume_info_set_option(task_context_t * context,
+				   u_int32_t index,
+				   value_t * value,
+				   task_effect_t * effect)
+{
+	option_desc_array_t *od = context->option_descriptors;
+	storage_object_t *region = context->object;
+	lvm_volume_group_t *group = region->producing_container->private_data;
+	int rc;
+
+	LOG_ENTRY();
+
+	switch (index) {
+
+	case LVM_OPTION_LV_NAME_INDEX:
+		/* Make sure this name is not in use. */
+		rc = lvm_check_lv_name(value->s, group);
+		if (!rc) {
+			LOG_DEBUG("Setting name option: %s\n", value->s);
+			strncpy(od->option[index].value.s, value->s, EVMS_NAME_SIZE);
+			od->option[index].flags &= ~EVMS_OPTION_FLAGS_NO_INITIAL_VALUE;
+		}
+		break;
+
+	default:
+		rc = EINVAL;
+		break;
+	}
+
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
+/**
+ * lvm_set_volume_info_parse_options
+ *
+ * Parse the option-values from the option-array.
+ **/
+void lvm_set_volume_info_parse_options(lvm_logical_volume_t *volume,
+				       option_array_t *options,
+				       char **lv_name)
+{
+	u_int i;
+
+	LOG_ENTRY();
+
+	/* Default values. */
+	*lv_name = NULL;
+
+	for (i = 0; i < options->count; i++) {
+		/* If only the name-based index is specified, get the number. */
+		if (!options->option[i].is_number_based) {
+			if (!strcmp(options->option[i].name,
+				    LVM_OPTION_LV_NAME_STR)) {
+				options->option[i].number = LVM_OPTION_LV_NAME_INDEX;
+			} else {
+				continue;
+			}
+		}
+
+		/* Use the number-based index to record the option. */
+		switch (options->option[i].number) {
+
+		case LVM_OPTION_LV_NAME_INDEX:
+			*lv_name = options->option[i].value.s;
+			LOG_DEBUG("Name option: %s\n", *lv_name);
+			break;
+
+		default:
+			break;
+		}
+	}
+
+	LOG_EXIT_VOID();
+}
+
+/**
+ * lvm_set_volume_info_validate_options
+ *
+ * Verify that these options are valid for setting info for this volume.
+ **/
+int lvm_set_volume_info_validate_options(lvm_logical_volume_t *volume,
+					 char *lv_name)
+{
+	int rc;
+
+	LOG_ENTRY();
+
+	/* Validate the new region name. */
+	rc = lvm_check_lv_name(lv_name, volume->group);
+
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
+
+/****** Group Rename Option Handlers ******/
+
+
+/**
+ * lvm_set_group_info_init_task
+ **/
+int lvm_set_group_info_init_task(task_context_t *context)
+{
+	option_desc_array_t *od = context->option_descriptors;
+	storage_container_t *container = context->container;
+	u_int32_t i;
+
+	LOG_ENTRY();
+	LOG_DEBUG("Initializing set-info task for container %s.\n",
+		  container->name);
+
+	/* Initialize the "Name" option. */
+	i = LVM_OPTION_VG_NAME_INDEX;
+	od->option[i].name = EngFncs->engine_strdup(LVM_OPTION_VG_NAME_STR);
+	od->option[i].title = EngFncs->engine_strdup(_("New name for this LVM container."));
+	od->option[i].type = EVMS_Type_String;
+	od->option[i].min_len = 1;
+	od->option[i].max_len = EVMS_NAME_SIZE;
+	od->option[i].flags = EVMS_OPTION_FLAGS_NO_INITIAL_VALUE;
+	od->option[i].value.s = EngFncs->engine_alloc(EVMS_NAME_SIZE+1);
+
+	od->count = ++i;
+	context->min_selected_objects = 0;
+	context->max_selected_objects = 0;
+
+	LOG_EXIT_INT(0);
+	return 0;
+}
+
+/**
+ * lvm_set_group_info_set_option
+ **/
+int lvm_set_group_info_set_option(task_context_t * context,
+				  u_int32_t index,
+				  value_t * value,
+				  task_effect_t * effect)
+{
+	option_desc_array_t *od = context->option_descriptors;
+	int rc;
+
+	LOG_ENTRY();
+
+	switch (index) {
+
+	case LVM_OPTION_VG_NAME_INDEX:
+		/* Make sure this name is not in use. */
+		rc = lvm_check_vg_name(value->s);
+		if (!rc) {
+			LOG_DEBUG("Setting name option: %s\n", value->s);
+			strncpy(od->option[index].value.s, value->s, EVMS_NAME_SIZE);
+			od->option[index].flags &= ~EVMS_OPTION_FLAGS_NO_INITIAL_VALUE;
+		}
+		break;
+
+	default:
+		rc = EINVAL;
+		break;
+	}
+
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
+/**
+ * lvm_set_group_info_parse_options
+ *
+ * Parse the option-values from the option-array.
+ **/
+void lvm_set_group_info_parse_options(lvm_volume_group_t *group,
+				      option_array_t *options,
+				      char **vg_name)
+{
+	u_int i;
+
+	LOG_ENTRY();
+
+	/* Default values. */
+	*vg_name = NULL;
+
+	for (i = 0; i < options->count; i++) {
+		/* If only the name-based index is specified, get the number. */
+		if (!options->option[i].is_number_based) {
+			if (!strcmp(options->option[i].name,
+				    LVM_OPTION_VG_NAME_STR)) {
+				options->option[i].number = LVM_OPTION_VG_NAME_INDEX;
+			} else {
+				continue;
+			}
+		}
+
+		/* Use the number-based index to record the option. */
+		switch (options->option[i].number) {
+
+		case LVM_OPTION_VG_NAME_INDEX:
+			*vg_name = options->option[i].value.s;
+			LOG_DEBUG("Name option: %s\n", *vg_name);
+			break;
+
+		default:
+			break;
+		}
+	}
+
+	LOG_EXIT_VOID();
+}
+
+/**
+ * lvm_set_group_info_validate_options
+ *
+ * Verify that these options are valid for setting info for this volume.
+ **/
+int lvm_set_group_info_validate_options(lvm_volume_group_t *group,
+					char *vg_name)
+{
+	int rc;
+
+	LOG_ENTRY();
+
+	/* Validate the new container name. */
+	rc = lvm_check_vg_name(vg_name);
+
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
