@@ -49,7 +49,7 @@
  * - Mark the PV's group's container dirty so all of this will get
  *   written to disk on the next commit.
  **/
-static u_int64_t lvm_get_pe_start(lvm_physical_volume_t * pv_entry)
+u_int64_t lvm_get_pe_start(lvm_physical_volume_t * pv_entry)
 {
 	pv_disk_t * pv = pv_entry->pv;
 	u_int64_t first_pe_sector = 0;
@@ -93,7 +93,7 @@ static u_int64_t lvm_get_pe_start(lvm_physical_volume_t * pv_entry)
  * allocate the memory to hold the PE map. The pv_entry must already have
  * its PV structure filled in.
  **/
-static int lvm_allocate_pe_map(lvm_physical_volume_t * pv_entry)
+int lvm_allocate_pe_map(lvm_physical_volume_t * pv_entry)
 {
 	u_int64_t pe_start;
 	u_int32_t i;
@@ -744,3 +744,28 @@ inline int lvm_pe_is_available(lvm_physical_extent_t * pe)
 	return (!pe->pe.lv_num && !pe->new_le);
 }
 
+/**
+ * lvm_get_available_objects
+ *
+ * Find all top-level disks, segments, and regions in the system. This
+ * will also retrieve LVM regions, so let the user beware. :)
+ **/
+int lvm_get_available_objects(storage_container_t *container,
+			      list_anchor_t *objects)
+{
+	storage_container_t *disk_group;
+	object_search_flags_t flags;
+	int rc;
+
+	LOG_ENTRY();
+
+	disk_group = (container) ? container->disk_group : NULL;
+	flags = VALID_INPUT_OBJECT |
+		((container && !disk_group) ? NO_DISK_GROUP : 0);
+	rc = EngFncs->get_object_list(DISK | SEGMENT | REGION, DATA_TYPE,
+				      NULL, disk_group, flags, objects);
+
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
