@@ -195,8 +195,9 @@ void lvm_deallocate_logical_volume(lvm_logical_volume_t * volume)
 	if (region) {
 		lvm_clear_child_list(region);
 		lvm_remove_region_from_container(region);
-		EngFncs->free_region(region);
+		region->private_data = NULL;
 		volume->region = NULL;
+		EngFncs->free_region(region);
 	}
 
 	/* Only the freespace volume has an lv field that was allocated. All
@@ -500,8 +501,8 @@ void lvm_check_le_maps(lvm_volume_group_t * group,
 		}
 		if (count) {
 			if (final_call) {
-				MESSAGE("Region %s has an incomplete LE map.\n"
-					"Missing %d out of %d LEs.\n",
+				MESSAGE(_("Region %s has an incomplete LE map.\n"
+					  "Missing %d out of %d LEs.\n"),
 					volume->region->name, count,
 					volume->lv->lv_allocated_le);
 			}
@@ -533,6 +534,16 @@ static void lvm_compare_volume_targets(lvm_logical_volume_t * volume,
 
 	LOG_ENTRY();
 
+	/* Don't bother comparing if the LE-map is incomplete and
+	 * it's not final-discovery.
+	 */
+	if ((volume->flags & LVM_LV_FLAG_INCOMPLETE) && !final_call) {
+		LOG_DEBUG("Delaying target comparison on incomplete region "
+			  "%s.\n", volume->region->name);
+		LOG_EXIT_VOID();
+		return;
+	}
+
 	/* Get the kernel's mapping for this region. */
 	rc = EngFncs->dm_get_targets(volume->region, &dm_target_list);
 	if (rc) {
@@ -607,10 +618,9 @@ static void lvm_compare_volume_targets(lvm_logical_volume_t * volume,
 
 compare_done:
 	if (final_call && needs_reactivate) {
-		MESSAGE("Error comparing kernel mappings for active region %s.\n",
-			volume->region->name);
-		MESSAGE("Region %s will be reactivated at the next commit.\n",
-			volume->region->name);
+		MESSAGE(_("Error comparing kernel mappings for active region %s.\n"
+			  "Region %s will be reactivated at the next commit.\n"),
+			volume->region->name, volume->region->name);
 		volume->region->flags |= SOFLAG_NEEDS_ACTIVATE;
 	}
 
@@ -648,7 +658,6 @@ void lvm_check_for_active_volumes(lvm_volume_group_t * group,
 		}
 
 		if (!(volume->region->flags & SOFLAG_ACTIVE)) {
-			volume->region->flags |= SOFLAG_NEEDS_ACTIVATE;
 			continue;
 		}
 
@@ -812,7 +821,7 @@ int lvm_check_lv_name(char * name,
 	for (i = 1; i <= MAX_LV; i++) {
 		if (group->volume_list[i] &&
 		    ! strncmp(lv_name, group->volume_list[i]->lv->lv_name, NAME_LEN)) {
-			MESSAGE("LV name %s already exists in container %s\n",
+			MESSAGE(_("LV name %s already exists in container %s\n"),
 				name, group->container->name);
 			rc = EEXIST;
 			goto out;
@@ -1970,3 +1979,140 @@ boolean lvm_volume_is_online(lvm_logical_volume_t * volume)
 	return online;
 }
 
+
+/**
+ * Volume-rename routines.
+ **/
+
+
+/**
+ * switch_region_name
+ *
+ * If a region was renamed and then needs to be deactivated, the deactivation
+ * must be done with the previous name, since that is the name that DM is
+ * currently using for that region. Switch the current name and the old name
+ * so the deactivation will succeed.
+ **/
+void switch_region_name(storage_object_t *region)
+{
+	char temp[EVMS_NAME_SIZE+1];
+
+	LOG_ENTRY();
+
+	if (LVM_LV_RENAMED(region)) {
+		strncpy(temp, region->dev_name, EVMS_NAME_SIZE);
+		strncpy(region->dev_name, region->name, EVMS_NAME_SIZE);
+		strncpy(region->name, temp, EVMS_NAME_SIZE);
+	}
+
+	LOG_EXIT_VOID();
+}
+
+/**
+ * remove_old_region_name
+ *
+ * This region has been renamed, and now we no longer need the old name. Clear
+ * the old name and the rename flag.
+ **/
+void remove_old_region_name(storage_object_t *region)
+{
+
+	LOG_ENTRY();
+
+	if (LVM_LV_RENAMED(region)) {
+		EngFncs->unregister_name(region->dev_name);
+		memset(region->dev_name, 0, EVMS_NAME_SIZE+1);
+	}
+
+	LOG_EXIT_VOID();
+}
+
+/**
+ * rename_region
+ *
+ * Check if this region has been renamed. If so, tell DM to change the name
+ * of the device. If the rename succeeds, remove the old name.
+ **/
+int rename_region(storage_object_t *region)
+{
+	int rc = 0;
+
+	LOG_ENTRY();
+
+	if (LVM_LV_RENAMED(region)) {
+		LOG_DEBUG("Renaming region from %s to %s.\n",
+			  region->dev_name, region->name);
+
+		rc = EngFncs->dm_rename(region, region->dev_name, region->name);
+		if (!rc) {
+			remove_old_region_name(region);
+		} else {
+			LOG_ERROR("Error renaming region from %s to %s.\n",
+				  region->dev_name, region->name);
+		}
+	}
+
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
+/**
+ * lvm_set_new_volume_name
+ *
+ * Set a new name in the specified volume. If necessary, save the old name so
+ * we can rename or deactivate the DM device during commit.
+ **/
+void lvm_set_new_volume_name(lvm_logical_volume_t *volume, char *name)
+{
+	lvm_volume_group_t *group = volume->group;
+	storage_object_t *region = volume->region;
+	char new_name[EVMS_NAME_SIZE+1];
+	char old_name[EVMS_NAME_SIZE+1];
+	char lv_name[EVMS_NAME_SIZE+1];
+	char *dg_name = group->container->disk_group ?
+			group->container->disk_group->name : NULL;
+
+	LOG_ENTRY();
+	LOG_DEBUG("Change name of region %s to %s.\n", region->name, name);
+
+	/* Save the current name. */
+	strncpy(old_name, region->name, EVMS_NAME_SIZE);
+
+	/* Copy the new name into the region. */
+	lvm_make_lv_name(name, group, lv_name);
+	lvm_translate_lv_name_to_region_name(lv_name, dg_name, new_name);
+	EngFncs->register_name(new_name);
+	strncpy(region->name, new_name, EVMS_NAME_SIZE);
+
+	/* Copy the new LV-name into the metadata. */
+	memset(volume->lv->lv_name, 0, NAME_LEN);
+	strncpy(volume->lv->lv_name, lv_name, NAME_LEN-1);
+
+	if (LVM_LV_RENAMED(region)) {
+		/* If we've already renamed this volume, we can simply
+		 * unregister the old name. We need the *first* name in
+		 * order to properly rename or deactivate the DM device.
+		 */
+		EngFncs->unregister_name(old_name);
+	} else {
+		/* If this is the first rename, save the old name in the
+		 * volume's private data.
+		 */
+		strncpy(region->dev_name, old_name, EVMS_NAME_SIZE);
+
+		if (region->flags & SOFLAG_ACTIVE) {
+			/* If the region is active, we need to tell DM
+			 * to rename the device during activation.
+			 */
+			region->flags |= SOFLAG_NEEDS_ACTIVATE;
+		} else {
+			/* If the region is inactive, we
+			 * can unregister the old_name.
+			 */
+			remove_old_region_name(region);
+		}
+	}
+
+	LOG_EXIT_VOID();
+}
+
