@@ -23,8 +23,12 @@
 #ifndef _MAC_DEV_MAPPER_HEADER_
 #define _MAC_DEV_MAPPER_HEADER_ 1
 
+int    mac_can_activate( storage_object_t *object );
+
 int    mac_activate( storage_object_t *object );
 
+int    mac_can_deactivate( storage_object_t *object );
+
 int    mac_deactivate( storage_object_t *object );
 
 #endif
