@@ -145,6 +145,7 @@ static int md_find_volume_for_object(storage_object_t * obj, md_sb_ver_t *sb_ver
 				LOG_WARNING("Volume %s has been discovered already."
 					    "  Object %s comes in late.\n",
 					    vol->name, obj->name);
+				md_append_region_to_object(vol->region, member->obj);
 				vol->flags |= MD_NEEDS_REDISCOVER;
 			}
 			break;
@@ -570,6 +571,23 @@ int md_analyze_active_region(md_volume_t *vol)
 		goto out;
 	}
 
+	for (i=0; i<md_info.nr_disks && !rc; i++) {
+		d.number = i;
+		rc = md_ioctl_get_disk_info(region, &d);
+		if (rc) {
+			LOG_WARNING("Can't get info for disk[%d].\n", i);
+			rc = 0;
+			continue;
+		}
+		member = md_volume_find_member_from_major_minor(vol, d.major, d.minor);
+		if (member && (member->dev_number != d.number)) {
+			LOG_WARNING("The kernel said disk(%d:%d) is at index[%d].\n",
+				    d.major, d.minor, d.number);
+			member->dev_number = d.number;
+			EngFncs->sort_list(vol->members, md_volume_sort_members, NULL);
+		}
+	}
+
 	md_volume_get_super_info(vol, &sb_info);
 	if (md_info.raid_disks != sb_info.raid_disks) {
 		if (vol->flags & MD_ARRAY_RESIZE_PENDING) {
@@ -724,9 +742,27 @@ int md_analyze_active_region(md_volume_t *vol)
 			/* This disk is not in the MD volume. */
 			if (d.state & (1<<MD_DISK_REMOVED)) {
 				LOG_DEBUG("Disk[%d] was removed.\n", d.number);
+			} else if (d.state & (1<<MD_DISK_FAULTY)) {
+			       length = sprintf(message_buffer,
+						_("%s region %s contains a failed disk with major %d and minor %d."
+						  "  The Kernel MD driver still has a reference to this disk."
+						  "  However, EVMS did not find MD superblock on this disk.\n\n"
+						  "  Would you like to request the kernel to release the reference to this disk?\n"),
+						level_to_string(md_info.level), vol->name, 
+						d.major, d.minor);
+			
+				if (md_get_yes_no_response(message_buffer) == TRUE) {
+					md_ioctl_hot_remove_disk(region, makedev(d.major, d.minor));
+				}
+
 			} else {
-				LOG_MD_BUG();
-				rc = EINVAL;
+				LOG_WARNING("Missing disk[%d] info:"
+					    " raid_disk(%d), major(%d), minor(%d) state(0x%08X).\n",
+					    d.number, d.raid_disk, d.major, d.minor, d.state);
+				if (EngFncs->is_2_4_kernel() == FALSE) {
+					LOG_MD_BUG();
+					rc = EINVAL;
+				}
 			}
 		}
 	}
