@@ -49,6 +49,7 @@ static int md_get_volume_info( md_volume_t            * vol,
 	extended_info_t *cur_info;
 	md_member_t *member;
 	list_element_t iter;
+	char resync_status[64];
 
 	LOG_ENTRY();
 	
@@ -60,7 +61,7 @@ static int md_get_volume_info( md_volume_t            * vol,
 	}
 	
 	if (region->flags & SOFLAG_ACTIVE) {
-		rc = md_analyze_active_region(vol);
+		md_analyze_active_region(vol);
 	}
 
 	info_entries = NUM_VOLUME_INFO_ENTRIES + EngFncs->list_count(vol->members);
@@ -70,7 +71,6 @@ static int md_get_volume_info( md_volume_t            * vol,
 	info = EngFncs->engine_alloc(sizeof(extended_info_array_t) +
 				     sizeof(extended_info_t) * info_entries);
 	if (info != NULL) {
-		info->count = info_entries;
 		cur_info = info->info;
 
 		cur_info->name = EngFncs->engine_strdup("name");
@@ -126,11 +126,14 @@ static int md_get_volume_info( md_volume_t            * vol,
 				}
 				strcat(message_buffer, _("Active"));
 			}
-			if (vol->flags & MD_ARRAY_SYNCING) {
-				if (message_buffer[0] != '\0') {
-					strcat(message_buffer, ", ");
+			if (mdstat_check_recovery(vol->md_minor, resync_status)) {
+				if (resync_status) {
+					if (message_buffer[0] != '\0') {
+						strcat(message_buffer, ", ");
+					}
+					strcat(message_buffer, _("Syncing"));
+					strcat(message_buffer, resync_status);
 				}
-				strcat(message_buffer, _("Syncing"));
 			}
 		}
 
@@ -289,7 +292,7 @@ static int set_disk_info(md_disk_info_t *info, extended_info_t *cur_info)
 	int info_count = 0;
 	char buf[64];
 
-	sprintf(buf, _("number %d"), info->number);
+	sprintf(buf, "number %d", info->number);
 	cur_info->name = EngFncs->engine_strdup(buf);
 	cur_info->title = EngFncs->engine_strdup(_("Number"));
 	cur_info->desc = EngFncs->engine_strdup(_("Disk number in the array"));
@@ -302,7 +305,7 @@ static int set_disk_info(md_disk_info_t *info, extended_info_t *cur_info)
 	cur_info->group.group_number = 0;
 	cur_info->group.group_level = 0;
 	cur_info->group.group_name = NULL;
-	cur_info->flags = EVMS_EINFO_FLAGS_MORE_INFO_AVAILABLE;
+	cur_info->flags = 0;
 	cur_info++;
 	info_count++;
 
@@ -414,7 +417,7 @@ static int get_superblock_disk_info(md_volume_t *vol, int index, extended_info_t
 	}
 	
 	// If none of the flags are set, it must be a spare disk.
-	if (message_buffer[0] == '\0') {
+	if (buffer[0] == '\0') {
 		strcpy(buffer, _("Spare"));
 	}
 
@@ -476,10 +479,40 @@ static int md_get_child_disk_info(md_member_t *member, extended_info_array_t **
 
 	int rc = 0;
 	extended_info_array_t * info;
+	extended_info_t *cur_info;
+	char buf[64];
 
-	info = EngFncs->engine_alloc(sizeof(extended_info_array_t) + sizeof(extended_info_t) * NUM_DISK_INFO_ENTRIES);
+	LOG_ENTRY();
+
+	info = EngFncs->engine_alloc(sizeof(extended_info_array_t) + sizeof(extended_info_t) * (NUM_DISK_INFO_ENTRIES + 1));
 	if (info != NULL) {
+
 		info->count = get_member_disk_info(member, info->info);
+
+		if (info->count != NUM_DISK_INFO_ENTRIES) {
+			LOG_MD_BUG();
+			EngFncs->engine_free(info);
+			return EINVAL;
+		}
+
+		cur_info = &info->info[info->count];
+
+		sprintf(buf, "superblock_child_object_%d", member->dev_number);
+		cur_info->name = EngFncs->engine_strdup(buf);
+		cur_info->title = EngFncs->engine_strdup(_("Superblock Index"));
+		cur_info->desc = EngFncs->engine_strdup(_("Superblock information on the child object"));
+		cur_info->type = EVMS_Type_Unsigned_Int32;
+		cur_info->unit = EVMS_Unit_None;
+		cur_info->format = EVMS_Format_Normal;
+		cur_info->value.ui32 = member->dev_number;
+		cur_info->collection_type = EVMS_Collection_None;
+		cur_info->collection.list = NULL;
+		cur_info->group.group_number = 0;
+		cur_info->group.group_level = 0;
+		cur_info->group.group_name = NULL;
+		cur_info->flags = EVMS_EINFO_FLAGS_MORE_INFO_AVAILABLE;
+
+		info->count++;
 		*info_array = info;
 	} else {
 		LOG_CRITICAL("Error getting memory for an extended_info_array./n");
@@ -494,10 +527,40 @@ static int md_get_stale_disk_info(md_member_t *member, extended_info_array_t **
 {
 	int rc = 0;
 	extended_info_array_t * info;
+	extended_info_t *cur_info;
+	char buf[64];
 
-	info = EngFncs->engine_alloc(sizeof(extended_info_array_t) + sizeof(extended_info_t) * NUM_DISK_INFO_ENTRIES);
+	LOG_ENTRY();
+
+	info = EngFncs->engine_alloc(sizeof(extended_info_array_t) + sizeof(extended_info_t) * (NUM_DISK_INFO_ENTRIES + 1));
 	if (info != NULL) {
+		
 		info->count = get_superblock_disk_info(member->vol, member->dev_number, info->info);
+
+		if (info->count != NUM_DISK_INFO_ENTRIES) {
+			LOG_MD_BUG();
+			EngFncs->engine_free(info);
+			return EINVAL;
+		}
+
+		cur_info = &info->info[info->count];
+
+		sprintf(buf, "superblock_stale_object_%d", member->dev_number);
+		cur_info->name = EngFncs->engine_strdup(buf);
+		cur_info->title = EngFncs->engine_strdup(_("Superblock Index"));
+		cur_info->desc = EngFncs->engine_strdup(_("Superblock information on the stale object"));
+		cur_info->type = EVMS_Type_Unsigned_Int32;
+		cur_info->unit = EVMS_Unit_None;
+		cur_info->format = EVMS_Format_Normal;
+		cur_info->value.ui32 = member->dev_number;
+		cur_info->collection_type = EVMS_Collection_None;
+		cur_info->collection.list = NULL;
+		cur_info->group.group_number = 0;
+		cur_info->group.group_level = 0;
+		cur_info->group.group_name = NULL;
+		cur_info->flags = EVMS_EINFO_FLAGS_MORE_INFO_AVAILABLE;
+
+		info->count++;
 		*info_array = info;
 	} else {
 		LOG_CRITICAL("Error getting memory for an extended_info_array./n");
@@ -512,32 +575,26 @@ static int md_get_stale_disk_info(md_member_t *member, extended_info_array_t **
 /*
  * md_get_superblock_info
  *
- * if child_index is -1, the master superblock will be used.
+ * if member is NULL, the master superblock will be used.
  */
 static int md_get_superblock_info(
 	md_volume_t *vol,
-	int child_index,
+	md_member_t *member,
 	extended_info_array_t ** info_array )
 {
 	extended_info_array_t * info;
 	extended_info_t *cur_info;
 	int info_entries;
 	md_super_info_t super;
-	md_member_t *member;
 	int i;
 
-	if (child_index == -1) {
+	LOG_ENTRY();
+
+	if (member == NULL) {
 		// Request to display volume's master superblock information
 		md_volume_get_super_info(vol, &super);
 	} else {
-		member = md_volume_find_member(vol, child_index);
-		if (member) {
-			LOG_CRITICAL("Could not locate member index %d in %s.\n",
-				     child_index, vol->name);
-			md_member_get_super_info(member, &super);
-		} else {
-			return EINVAL;
-		}
+		md_member_get_super_info(member, &super);
 	}
 
 	info_entries = NUM_SUPER_INFO_ENTRIES + super.nr_disks*NUM_DISK_INFO_ENTRIES;
@@ -546,6 +603,7 @@ static int md_get_superblock_info(
 				     sizeof(extended_info_t) * info_entries);
 	if (info == NULL) {
 		LOG_CRITICAL("Error getting memory for an extended_info_array.\n");
+		LOG_EXIT_INT(ENOMEM);
 		return ENOMEM;
 	}
 	cur_info = info->info;
@@ -1002,6 +1060,9 @@ int md_get_info(md_volume_t *vol, char *name, extended_info_array_t **info_array
 	int rc = 0;
 	int idx = -1;
 	md_member_t *member;
+	list_element_t iter;
+
+	LOG_ENTRY();
 
 	if (!name) {
 		// Default case. Return all basic info about the region.
@@ -1010,7 +1071,11 @@ int md_get_info(md_volume_t *vol, char *name, extended_info_array_t **info_array
 	else if ( ! strncmp(name, "child_object", 12) ) {
                 // "Extra" information about the child's superblock
 		idx = atoi(name + 12);
-		member = md_volume_find_member(vol, idx);
+		LIST_FOR_EACH(vol->members, iter, member) {
+			if ((member->dev_number == idx) && !(member->flags & MD_MEMBER_STALE)) {
+				break;
+			}
+		}
 		if (member) {
 			rc = md_get_child_disk_info(member, info_array);
 		} else {
@@ -1023,6 +1088,11 @@ int md_get_info(md_volume_t *vol, char *name, extended_info_array_t **info_array
                 // "Extra" information about the stale disk's superblock
 		idx = atoi(name + 12);
 		member = md_volume_find_member(vol, idx);
+		LIST_FOR_EACH(vol->members, iter, member) {
+			if ((member->dev_number == idx) && (member->flags & MD_MEMBER_STALE)) {
+				break;
+			}
+		}
 		if (member) {
 			rc = md_get_stale_disk_info(member, info_array);
 		} else {
@@ -1031,11 +1101,20 @@ int md_get_info(md_volume_t *vol, char *name, extended_info_array_t **info_array
 			return EINVAL;
 		}
 	}
-	else if ( ! strncmp(name, "number", 6) ) {
-                // "Extra" information about a disk in the array
-		idx = atoi(name + 6);
-		if (idx >= 0) {
-			rc = md_get_superblock_info(vol, idx, info_array);
+	else if ( ! strncmp(name, "superblock", 10) ) {
+                // "Extra" information about the master superblock
+		rc = md_get_superblock_info(vol, NULL, info_array);
+	}	
+	else if ( ! strncmp(name, "superblock_child_object_", 24) ) {
+                // "Extra" information about the child superblock
+		idx = atoi(name + 24);
+		LIST_FOR_EACH(vol->members, iter, member) {
+			if ((member->dev_number == idx) && !(member->flags & MD_MEMBER_STALE)) {
+				break;
+			}
+		}
+		if (member) {
+			rc = md_get_superblock_info(vol, member, info_array);
 		}
 		else {
 			LOG_ERROR("No support for extra region information about \"%s\"\n", name);
@@ -1043,9 +1122,22 @@ int md_get_info(md_volume_t *vol, char *name, extended_info_array_t **info_array
 			return EINVAL;
 		}
 	}
-	else if ( ! strncmp(name, "superblock", 10) ) {
-                // "Extra" information about the master superblock
-		rc = md_get_superblock_info(vol, -1, info_array);
+	else if ( ! strncmp(name, "superblock_stale_object_", 24) ) {
+                // "Extra" information about the stale superblock
+		idx = atoi(name + 24);
+		LIST_FOR_EACH(vol->members, iter, member) {
+			if ((member->dev_number == idx) && (member->flags & MD_MEMBER_STALE)) {
+				break;
+			}
+		}
+		if (member) {
+			rc = md_get_superblock_info(vol, member, info_array);
+		}
+		else {
+			LOG_ERROR("No support for extra region information about \"%s\"\n", name);
+			LOG_EXIT_INT(EINVAL);
+			return EINVAL;
+		}
 	}
 	else {
 		LOG_ERROR("No support for extra region information about \"%s\"\n", name);
