@@ -97,6 +97,13 @@ int md_ioctl_flush_buffer_cache(storage_object_t *obj)
 	int rc = 0, fd;
 	LOG_ENTRY();
 
+	if (obj->dev_major == MD_MAJOR) {
+		LOG_DEBUG("%s (%d:%d) is an MD device, skip flushing buffer cache.\n",
+			  obj->name, obj->dev_major, obj->dev_minor);
+		LOG_EXIT_INT(0);
+		return 0;
+	}
+
 	fd = EngFncs->open_object(obj, O_RDONLY);
 	if (fd <= 0) {
 		LOG_DEBUG("Unable to open object %s to send ioctl\n", obj->name);
