@@ -145,7 +145,7 @@ static void md_analyze_sb(md_volume_t *vol, int level, int raid_disks, u_int32_t
 			goto queue_corrupt_message;
 		}
 		
-		if ( (1 << ffz(~(chunksize_in_bytes))) != chunksize_in_bytes) {
+		if ( (1 << (ffs(chunksize_in_bytes)-1)) != chunksize_in_bytes) {
 			length = sprintf(message_buffer,
 					 _("%s region %s is corrupt."
 					   "  The chunk size (%d bytes) is not a power of 2."),
@@ -2909,6 +2909,7 @@ int md_member_get_raid_disk(md_member_t *member)
 		return -1;
 	}
 	member->vol->sb_func->get_sb_disk_info(member, &info);
+	LOG_EXIT_INT(info.raid_disk);
 	return info.raid_disk;
 }
 
