@@ -768,6 +768,7 @@ static int raid1_init_region(
 static int raid1_create_region(md_volume_t * vol, list_anchor_t output_list, boolean final_call)
 {
 	int rc = 0;
+	int length;
 	storage_object_t * region;
 
 	LOG_ENTRY();
@@ -781,9 +782,12 @@ static int raid1_create_region(md_volume_t * vol, list_anchor_t output_list, boo
 	if (!vol->active_disks) {
 		if (final_call) {
 			vol->flags |= MD_CORRUPT;
-			LOG_CRITICAL("Volume %s does not have any active disks."
-				    "  This is final discovery call, the volume is corrupt.\n",
-				    vol->name);
+			length = sprintf(message_buffer,
+					 _("RAID1 region %s is corrupt. "
+					   "  The number of raid disks for a full functional array is %d."
+					   "  The number of active disks is %d."),
+					 vol->name, vol->raid_disks, vol->active_disks);
+			md_queue_corrupt_message(vol->personality, message_buffer, length);
 		} else {
 			LOG_DEBUG("Volume %s does not have any active disks, delaying discovery.\n",
 				  vol->name);
