@@ -34,8 +34,8 @@
 #include "ogfsfsim.h"
 #include "cluster_config.h"
 
-static plugin_record_t *my_plugin_record = &ogfs_plugin_record;
-static engine_functions_t *EngFncs = NULL;
+plugin_record_t *my_plugin_record = &ogfs_plugin_record;
+engine_functions_t *EngFncs = NULL;
 static u_int32_t ogfsutils_support = 0;
 
 /**
@@ -106,7 +106,7 @@ int ogfs_exec_utility(logical_volume_t *volume, char **argv)
 			bytes_read = read(fds2[0], buffer, MAX_USER_MESSAGE_LEN);
 			if (bytes_read > 0) {
 				if (display_to_user) {
-					MESSAGE("%s output: \n%s\n", argv[0], buffer);
+					MESSAGE(_("%s output: \n%s\n"), argv[0], buffer);
 				} else {
 					LOG_DETAILS("%s output: \n%s\n", argv[0], buffer);
 				}
@@ -122,7 +122,7 @@ int ogfs_exec_utility(logical_volume_t *volume, char **argv)
 				bytes_read = read(fds2[0], buffer, MAX_USER_MESSAGE_LEN);
 				if (bytes_read > 0) {
 					if (display_to_user) {
-						MESSAGE("%s output: \n%s\n", argv[0], buffer);
+						MESSAGE(_("%s output: \n%s\n"), argv[0], buffer);
 					} else {
 						LOG_DETAILS("%s output: \n%s\n", argv[0], buffer);
 					}
@@ -1030,7 +1030,7 @@ static int ogfs_read_jindex(logical_volume_t *volume, ogfs_jindex_t *jindex, u_i
 static inline int set_arg(char **argv, int index, char *str)
 {
 	argv[index] = NULL;
-	SET_STRING(argv[index], str);
+	argv[index] = EngFncs->engine_strdup(str);
 	return 0;
 }
 
@@ -1952,9 +1952,9 @@ static int set_mkfs_args(option_array_t *options, char **argv, int *argc, char *
 						 * cidev or claim it, remind the user
 						 * that (s)he needs to do that themselves.
 						 */
-						 MESSAGE("Please configure lock table device"
-						 	 " %s with ogfsconf before mounting"
-						 	 " the new file system",
+						 MESSAGE(_("Please configure lock table device "
+							   "%s with ogfsconf before mounting "
+							   "the new file system"),
 						 	 options->option[i].value.s);
 					}
 				}
@@ -2412,10 +2412,10 @@ static int ogfs_init_mkfs_option_descriptors(task_context_t *context)
 	LOG_ENTRY();
 	
 	/* set blocksize */
-	SET_STRING(od->option[OGFS_MKFS_BLOCKSIZE_INDEX].name, OGFS_MKFS_BLOCKSIZE_NAME);
-	SET_STRING(od->option[OGFS_MKFS_BLOCKSIZE_INDEX].title, "Block size");
-	SET_STRING(od->option[OGFS_MKFS_BLOCKSIZE_INDEX].tip,
-			"Acceptable range: 512 to 65536 bytes. Must be a power of 2.");
+	od->option[OGFS_MKFS_BLOCKSIZE_INDEX].name = EngFncs->engine_strdup(OGFS_MKFS_BLOCKSIZE_NAME);
+	od->option[OGFS_MKFS_BLOCKSIZE_INDEX].title = EngFncs->engine_strdup(_("Block size"));
+	od->option[OGFS_MKFS_BLOCKSIZE_INDEX].tip =
+		EngFncs->engine_strdup(_("Acceptable range: 512 to 65536 bytes. Must be a power of 2."));
 	od->option[OGFS_MKFS_BLOCKSIZE_INDEX].type = EVMS_Type_Unsigned_Int32;
 	od->option[OGFS_MKFS_BLOCKSIZE_INDEX].unit = EVMS_Unit_Bytes;
 	od->option[OGFS_MKFS_BLOCKSIZE_INDEX].flags = EVMS_OPTION_FLAGS_NOT_REQUIRED | EVMS_OPTION_FLAGS_AUTOMATIC;
@@ -2425,9 +2425,9 @@ static int ogfs_init_mkfs_option_descriptors(task_context_t *context)
 	od->option[OGFS_MKFS_BLOCKSIZE_INDEX].value.ui32 = OGFS_DEFAULT_FS_BLOCKSIZE;
 			
 	/* set locking protocol*/
-	SET_STRING(od->option[OGFS_MKFS_PROTOCOL_INDEX].name, OGFS_MKFS_PROTOCOL_NAME);
-	SET_STRING(od->option[OGFS_MKFS_PROTOCOL_INDEX].title, "Locking Protocol");
-	SET_STRING(od->option[OGFS_MKFS_PROTOCOL_INDEX].tip, "Name of the locking protocol");
+	od->option[OGFS_MKFS_PROTOCOL_INDEX].name = EngFncs->engine_strdup(OGFS_MKFS_PROTOCOL_NAME);
+	od->option[OGFS_MKFS_PROTOCOL_INDEX].title = EngFncs->engine_strdup(_("Locking Protocol"));
+	od->option[OGFS_MKFS_PROTOCOL_INDEX].tip = EngFncs->engine_strdup(_("Name of the locking protocol"));
 	od->option[OGFS_MKFS_PROTOCOL_INDEX].type = EVMS_Type_String;
 	od->option[OGFS_MKFS_PROTOCOL_INDEX].min_len = 1;
 	od->option[OGFS_MKFS_PROTOCOL_INDEX].max_len = EVMS_VOLUME_NAME_SIZE;
@@ -2441,9 +2441,9 @@ static int ogfs_init_mkfs_option_descriptors(task_context_t *context)
 	}
 	
 	/* set lock table device */
-	SET_STRING(od->option[OGFS_MKFS_LOCKDEV_INDEX].name, OGFS_MKFS_LOCKDEV_NAME);
-	SET_STRING(od->option[OGFS_MKFS_LOCKDEV_INDEX].title, "Lock Table Volume");
-	SET_STRING(od->option[OGFS_MKFS_LOCKDEV_INDEX].tip, "Shared volume containing locking metadata");
+	od->option[OGFS_MKFS_LOCKDEV_INDEX].name = EngFncs->engine_strdup(OGFS_MKFS_LOCKDEV_NAME);
+	od->option[OGFS_MKFS_LOCKDEV_INDEX].title = EngFncs->engine_strdup(_("Lock Table Volume"));
+	od->option[OGFS_MKFS_LOCKDEV_INDEX].tip = EngFncs->engine_strdup(_("Shared volume containing locking metadata"));
 	od->option[OGFS_MKFS_LOCKDEV_INDEX].type = EVMS_Type_String;
 	od->option[OGFS_MKFS_LOCKDEV_INDEX].min_len = 1;
 	od->option[OGFS_MKFS_LOCKDEV_INDEX].max_len = EVMS_VOLUME_NAME_SIZE;
@@ -2460,9 +2460,9 @@ static int ogfs_init_mkfs_option_descriptors(task_context_t *context)
 	 */
 	
 	/* set external journals multi-selection list */
-	SET_STRING(od->option[OGFS_MKFS_JOURNALS_INDEX].name, OGFS_MKFS_JOURNALS_NAME);
-	SET_STRING(od->option[OGFS_MKFS_JOURNALS_INDEX].title, "External Journals");
-	SET_STRING(od->option[OGFS_MKFS_JOURNALS_INDEX].tip, "Journal volumes (one for each node).");
+	od->option[OGFS_MKFS_JOURNALS_INDEX].name = EngFncs->engine_strdup(OGFS_MKFS_JOURNALS_NAME);
+	od->option[OGFS_MKFS_JOURNALS_INDEX].title = EngFncs->engine_strdup(_("External Journals"));
+	od->option[OGFS_MKFS_JOURNALS_INDEX].tip = EngFncs->engine_strdup(_("Journal volumes (one for each node)."));
 	od->option[OGFS_MKFS_JOURNALS_INDEX].type = EVMS_Type_String;
 	od->option[OGFS_MKFS_JOURNALS_INDEX].min_len = 1;
 	od->option[OGFS_MKFS_JOURNALS_INDEX].max_len = EVMS_VOLUME_NAME_SIZE;
@@ -2577,7 +2577,7 @@ static int ogfs_set_mkfs_option(task_context_t *context,
 				EngFncs->engine_free(od->option[index].value.list->value[i].s);
 				od->option[index].value.list->value[i].s = NULL;
 			}
-			SET_STRING(od->option[index].value.list->value[i].s, value->list->value[i].s);
+			od->option[index].value.list->value[i].s = EngFncs->engine_strdup(value->list->value[i].s);
 		}
 		for (; i < od->option[index].value.list->count; i++) {
 			if (od->option[index].value.list->value[i].s) {
@@ -2761,9 +2761,9 @@ static int ogfs_get_cluster_extended_info(logical_volume_t *volume,
 		einfo->count = OGFS_CG_INFO_COUNT;
 
 		/* version */
-		SET_STRING(einfo->info[0].name,  "Version");
-		SET_STRING(einfo->info[0].title, "Version Number");
-		SET_STRING(einfo->info[0].desc,  "Version number of cluster configuration data");
+		einfo->info[0].name = EngFncs->engine_strdup("Version");
+		einfo->info[0].title = EngFncs->engine_strdup(_("Version Number"));
+		einfo->info[0].desc = EngFncs->engine_strdup(_("Version number of cluster configuration data"));
 		einfo->info[0].type              = EVMS_Type_Unsigned_Int32;
 		einfo->info[0].unit              = EVMS_Unit_None;
 		einfo->info[0].value.ui32        = ogfs_cg->cg_version;
@@ -2771,29 +2771,29 @@ static int ogfs_get_cluster_extended_info(logical_volume_t *volume,
 		memset(&einfo->info[0].group, 0, sizeof(group_info_t));
 		
 		/* lock device */
-		SET_STRING(einfo->info[1].name,  "LockDev");
-		SET_STRING(einfo->info[1].title, "Lock Device");
-		SET_STRING(einfo->info[1].desc,  "Name of lock device");
+		einfo->info[1].name = EngFncs->engine_strdup("LockDev");
+		einfo->info[1].title = EngFncs->engine_strdup(_("Lock Device"));
+		einfo->info[1].desc = EngFncs->engine_strdup(_("Name of lock device"));
 		einfo->info[1].type              = EVMS_Type_String;
 		einfo->info[1].unit              = EVMS_Unit_None;
-		SET_STRING(einfo->info[1].value.s, ogfs_cg->cg_lockdev);
+		einfo->info[1].value.s = EngFncs->engine_strdup(ogfs_cg->cg_lockdev);
 		einfo->info[1].collection_type   = EVMS_Collection_None;
 		memset(&einfo->info[1].group, 0, sizeof(group_info_t));
 
 		/* data device */
-		SET_STRING(einfo->info[2].name,  "DataDev");
-		SET_STRING(einfo->info[2].title, "Data Device");
-		SET_STRING(einfo->info[2].desc,  "Name of data device");
+		einfo->info[2].name = EngFncs->engine_strdup("DataDev");
+		einfo->info[2].title = EngFncs->engine_strdup(_("Data Device"));
+		einfo->info[2].desc = EngFncs->engine_strdup(_("Name of data device"));
 		einfo->info[2].type              = EVMS_Type_String;
 		einfo->info[2].unit              = EVMS_Unit_None;
-		SET_STRING(einfo->info[2].value.s, ogfs_cg->cg_datadev);
+		einfo->info[2].value.s = EngFncs->engine_strdup(ogfs_cg->cg_datadev);
 		einfo->info[2].collection_type   = EVMS_Collection_None;
 		memset(&einfo->info[2].group, 0, sizeof(group_info_t));
 
 		/* callback port */
-		SET_STRING(einfo->info[3].name,  "Port");
-		SET_STRING(einfo->info[3].title, "Callback Port");
-		SET_STRING(einfo->info[3].desc,  "Port number used for callbacks between nodes");
+		einfo->info[3].name = EngFncs->engine_strdup("Port");
+		einfo->info[3].title = EngFncs->engine_strdup(_("Callback Port"));
+		einfo->info[3].desc = EngFncs->engine_strdup(_("Port number used for callbacks between nodes"));
 		einfo->info[3].type              = EVMS_Type_Unsigned_Int32;
 		einfo->info[3].unit              = EVMS_Unit_None;
 		einfo->info[3].value.ui32        = ogfs_cg->cg_cbport;
@@ -2801,9 +2801,9 @@ static int ogfs_get_cluster_extended_info(logical_volume_t *volume,
 		memset(&einfo->info[3].group, 0, sizeof(group_info_t));
 		
 		/* node timeout */
-		SET_STRING(einfo->info[4].name,  "Timeout");
-		SET_STRING(einfo->info[4].title, "Node Timeout");
-		SET_STRING(einfo->info[4].desc,  "Time in seconds before node is considered 'dead'");
+		einfo->info[4].name = EngFncs->engine_strdup("Timeout");
+		einfo->info[4].title = EngFncs->engine_strdup(_("Node Timeout"));
+		einfo->info[4].desc = EngFncs->engine_strdup(_("Time in seconds before node is considered \"dead\""));
 		einfo->info[4].type              = EVMS_Type_Unsigned_Int32;
 		einfo->info[4].unit              = EVMS_Unit_None;
 		einfo->info[4].value.ui32        = ogfs_cg->cg_node_timeout;
@@ -2848,9 +2848,9 @@ static int ogfs_get_sb_extended_info(logical_volume_t *volume,
 		einfo->count = OGFS_SB_INFO_COUNT;
 
 		/* generation number */
-		SET_STRING(einfo->info[0].name,  "Generation");
-		SET_STRING(einfo->info[0].title, "Generation Number");
-		SET_STRING(einfo->info[0].desc,  "Number incremented each time a change occurs");
+		einfo->info[0].name = EngFncs->engine_strdup("Generation");
+		einfo->info[0].title = EngFncs->engine_strdup(_("Generation Number"));
+		einfo->info[0].desc = EngFncs->engine_strdup(_("Number incremented each time a change occurs"));
 		einfo->info[0].type              = EVMS_Type_Unsigned_Int64;
 		einfo->info[0].unit              = EVMS_Unit_None;
 		einfo->info[0].value.ui64        = ogfs_sb->sb_header.mh_generation;
@@ -2858,9 +2858,9 @@ static int ogfs_get_sb_extended_info(logical_volume_t *volume,
 		memset(&einfo->info[0].group, 0, sizeof(group_info_t));
 
 		/* filesystem block size */
-		SET_STRING(einfo->info[1].name,  "BlockSize");
-		SET_STRING(einfo->info[1].title, "Block Size");
-		SET_STRING(einfo->info[1].desc,  "File system block size in bytes");
+		einfo->info[1].name = EngFncs->engine_strdup("BlockSize");
+		einfo->info[1].title = EngFncs->engine_strdup(_("Block Size"));
+		einfo->info[1].desc = EngFncs->engine_strdup(_("File system block size in bytes"));
 		einfo->info[1].type              = EVMS_Type_Unsigned_Int32;
 		einfo->info[1].unit              = EVMS_Unit_Bytes;
 		einfo->info[1].value.ui32        = ogfs_sb->sb_bsize;
@@ -2868,22 +2868,22 @@ static int ogfs_get_sb_extended_info(logical_volume_t *volume,
 		memset(&einfo->info[1].group, 0, sizeof(group_info_t));
 		
 		/* locking protocol name */
-		SET_STRING(einfo->info[2].name,  "LockProtocol");
-		SET_STRING(einfo->info[2].title, "Locking Protocol");
-		SET_STRING(einfo->info[2].desc,  "Name of locking protocol filesystem is using");
+		einfo->info[2].name = EngFncs->engine_strdup("LockProtocol");
+		einfo->info[2].title = EngFncs->engine_strdup(_("Locking Protocol"));
+		einfo->info[2].desc = EngFncs->engine_strdup(_("Name of locking protocol filesystem is using"));
 		einfo->info[2].type              = EVMS_Type_String;
 		einfo->info[2].unit              = EVMS_Unit_None;
-		SET_STRING(einfo->info[2].value.s, ogfs_sb->sb_lockproto);
+		einfo->info[2].value.s = EngFncs->engine_strdup(ogfs_sb->sb_lockproto);
 		einfo->info[2].collection_type   = EVMS_Collection_None;
 		memset(&einfo->info[2].group, 0, sizeof(group_info_t));
 
 		/* lock table name */
-		SET_STRING(einfo->info[3].name,  "LockTable");
-		SET_STRING(einfo->info[3].title, "Lock Table");
-		SET_STRING(einfo->info[3].desc,  "Name of lock table for this filesystem");
+		einfo->info[3].name = EngFncs->engine_strdup("LockTable");
+		einfo->info[3].title = EngFncs->engine_strdup(_("Lock Table"));
+		einfo->info[3].desc = EngFncs->engine_strdup(_("Name of lock table for this filesystem"));
 		einfo->info[3].type              = EVMS_Type_String;
 		einfo->info[3].unit              = EVMS_Unit_None;
-		SET_STRING(einfo->info[3].value.s, ogfs_sb->sb_locktable);
+		einfo->info[3].value.s = EngFncs->engine_strdup(ogfs_sb->sb_locktable);
 		einfo->info[3].collection_type   = EVMS_Collection_None;
 		memset(&einfo->info[3].group, 0, sizeof(group_info_t));
 		
@@ -2975,57 +2975,59 @@ static int ogfs_get_plugin_info(char *descriptor_name,
 					my_plugin_record->required_plugin_api_version.fsim.minor,
 					my_plugin_record->required_plugin_api_version.fsim.patchlevel);
 
-				SET_STRING(einfo->info[0].name,  "Short Name");
-				SET_STRING(einfo->info[0].title, "Short Name");
-				SET_STRING(einfo->info[0].desc,  "A short name given to this plugin");
+				einfo->info[0].name = EngFncs->engine_strdup("Short Name");
+				einfo->info[0].title = EngFncs->engine_strdup(_("Short Name"));
+				einfo->info[0].desc = EngFncs->engine_strdup(_("A short name given to this plug-in"));
 				einfo->info[0].type              = EVMS_Type_String;
 				einfo->info[0].unit              = EVMS_Unit_None;
-				SET_STRING(einfo->info[0].value.s, my_plugin_record->short_name);
+				einfo->info[0].value.s = EngFncs->engine_strdup(my_plugin_record->short_name);
 				einfo->info[0].collection_type    = EVMS_Collection_None;
 				memset(&einfo->info[0].group, 0, sizeof(group_info_t));
 
-				SET_STRING(einfo->info[1].name,  "Long Name");
-				SET_STRING(einfo->info[1].title, "Long Name");
-				SET_STRING(einfo->info[1].desc,  "A long name given to this plugin");
+				einfo->info[1].name = EngFncs->engine_strdup("Long Name");
+				einfo->info[1].title = EngFncs->engine_strdup(_("Long Name"));
+				einfo->info[1].desc = EngFncs->engine_strdup(_("A longer, more descriptive name for this plug-in"));
 				einfo->info[1].type              = EVMS_Type_String;
 				einfo->info[1].unit              = EVMS_Unit_None;
-				SET_STRING(einfo->info[1].value.s, my_plugin_record->long_name);
+				einfo->info[1].value.s = EngFncs->engine_strdup(my_plugin_record->long_name);
 				einfo->info[1].collection_type   = EVMS_Collection_None;
 				memset(&einfo->info[1].group, 0, sizeof(group_info_t));
 
-				SET_STRING(einfo->info[2].name,  "Type");
-				SET_STRING(einfo->info[2].title, "Plugin Type");
-				SET_STRING(einfo->info[2].desc,  "There are various types of plugins; each responsible for some kind of storage object.");
+				einfo->info[2].name = EngFncs->engine_strdup("Type");
+				einfo->info[2].title = EngFncs->engine_strdup(_("Plug-in Type"));
+				einfo->info[2].desc = EngFncs->engine_strdup(_("There are various types of plug-ins, each responsible for some kind of storage object or logical volume."));
 				einfo->info[2].type              = EVMS_Type_String;
 				einfo->info[2].unit              = EVMS_Unit_None;
-				SET_STRING(einfo->info[2].value.s, "File System Interface Module");
+				einfo->info[2].value.s = EngFncs->engine_strdup(_("File System Interface Module"));
 				einfo->info[2].collection_type   = EVMS_Collection_None;
 				memset(&einfo->info[2].group, 0, sizeof(group_info_t));
 
-				SET_STRING(einfo->info[3].name,  "Version");
-				SET_STRING(einfo->info[3].title, "Plugin Version");
-				SET_STRING(einfo->info[3].desc,  "This is the version number of the plugin.");
+				einfo->info[3].name = EngFncs->engine_strdup("Version");
+				einfo->info[3].title = EngFncs->engine_strdup(_("Plug-in Version"));
+				einfo->info[3].desc = EngFncs->engine_strdup(_("This is the version number of the plug-in."));
 				einfo->info[3].type              = EVMS_Type_String;
 				einfo->info[3].unit              = EVMS_Unit_None;
-				SET_STRING(einfo->info[3].value.s, version_string);
+				einfo->info[3].value.s = EngFncs->engine_strdup(version_string);
 				einfo->info[3].collection_type   = EVMS_Collection_None;
 				memset(&einfo->info[3].group, 0, sizeof(group_info_t));
 
-				SET_STRING(einfo->info[4].name,  "Required Engine Services Version");
-				SET_STRING(einfo->info[4].title, "Required Engine Services Version");
-				SET_STRING(einfo->info[4].desc,  "This is the version of the Engine services that this plug-in requires.  It will not run on older versions of the Engine services.");
+				einfo->info[4].name = EngFncs->engine_strdup("Required Engine Services Version");
+				einfo->info[4].title = EngFncs->engine_strdup(_("Required Engine Services Version"));
+				einfo->info[4].desc = EngFncs->engine_strdup(_("This is the version of the Engine services that this plug-in requires.  "
+									       "It will not run on older versions of the Engine services."));
 				einfo->info[4].type              = EVMS_Type_String;
 				einfo->info[4].unit              = EVMS_Unit_None;
-				SET_STRING(einfo->info[4].value.s, required_engine_api_version_string);
+				einfo->info[4].value.s = EngFncs->engine_strdup(required_engine_api_version_string);
 				einfo->info[4].collection_type   = EVMS_Collection_None;
 				memset(&einfo->info[4].group, 0, sizeof(group_info_t));
 
-				SET_STRING(einfo->info[5].name,  "Required Engine FSIM API Version");
-				SET_STRING(einfo->info[5].title, "Required Engine FSIM API Version");
-				SET_STRING(einfo->info[5].desc,  "This is the version of the Engine FSIM API that this plug-in requires.  It will not run on older versions of the Engine FSIM API.");
+				einfo->info[5].name = EngFncs->engine_strdup("Required Engine FSIM API Version");
+				einfo->info[5].title = EngFncs->engine_strdup(_("Required Engine FSIM API Version"));
+				einfo->info[5].desc = EngFncs->engine_strdup(_("This is the version of the Engine FSIM API that this plug-in requires.  "
+									       "It will not run on older versions of the Engine FSIM API."));
 				einfo->info[5].type              = EVMS_Type_String;
 				einfo->info[5].unit              = EVMS_Unit_None;
-				SET_STRING(einfo->info[5].value.s, required_fsim_api_version_string);
+				einfo->info[5].value.s = EngFncs->engine_strdup(required_fsim_api_version_string);
 				einfo->info[5].collection_type   = EVMS_Collection_None;
 				memset(&einfo->info[5].group, 0, sizeof(group_info_t));
 
@@ -3072,7 +3074,7 @@ static int ogfs_can_expand_by(logical_volume_t *volume, sector_count_t *delta)
 		 * the delta isn't at least 100 ogfs blocks in size.
 		 */
 		if (*delta >= (100 * (ogfs_sb->sb_bsize >> EVMS_VSECTOR_SIZE_SHIFT))) {
-			if (volume->fs_size + *delta > volume->max_fs_size) {
+			if (*delta > volume->max_fs_size - volume->fs_size) {
 				*delta = volume->max_fs_size - volume->fs_size;
 			}
 			rc = 0;
@@ -3136,13 +3138,13 @@ plugin_record_t ogfs_plugin_record = {
 		.patchlevel	= PATCH_LEVEL
 	},
 	.required_engine_api_version = {
-		.major		= 13,
+		.major		= 15,
 		.minor		= 0,
 		.patchlevel	= 0
 	},
 	.required_plugin_api_version = {
 		.fsim = {
-			.major	= 10,
+			.major	= 11,
 			.minor	= 0,
 			.patchlevel = 0
 		}
