@@ -32,8 +32,6 @@
 #include "reiserfsim.h"
 
 static int fsim_rw_diskblocks( logical_volume_t *,int, int64_t, int32_t, void *, int );
-static int set_mkfs_options( option_array_t *, char **, char * );
-static int set_fsck_options( option_array_t *, char **, logical_volume_t * );
 static int reiser_version = 0;	// version of reiserfs utils.
 static char reiser_version_text[32];  // version of reiserfs utils.
 
@@ -66,15 +64,80 @@ static int fsim_get_volume_limits(     struct reiserfs_super_block * sb_ptr,
 	return rc;
 }
 
-static inline void free_args(char **argv)
+/*
+ * NAME: set_mkfs_options
+ *
+ * FUNCTION: Build options array (argv) for mkreiserfs
+ *
+ * PARAMETERS:
+ *      options   - options array passed from EVMS engine
+ *      argv      - mkfs options array
+ *      vol_name  - volume name on which program will be executed
+ *
+ */
+static void set_mkfs_options( option_array_t * options, char ** argv, char * vol_name )
 {
-	int i;
+	int i, opt_count = 0;
+	char buffer[EVMS_VOLUME_NAME_SIZE + 10];
 
-	for (i = 0; argv[i] != NULL; i++) {
-		EngFncs->engine_free(argv[i]);
+	LOG_ENTRY();
+
+	argv[opt_count++] = "mkreiserfs";
+
+	    /* 'force/quiet' option -ff*/
+	argv[opt_count++] = "-ff";
+
+	for ( i=0; i<options->count; i++ ) {
+
+		if (!options->option[i].is_number_based) {
+			if ( !strcmp(options->option[i].name, "vollabel")
+			     && (reiser_version >= 2)
+                             && options->option[i].value.s ) {
+				/* 'set volume name' option */
+				options->option[i].number = MKFS_SETVOL_INDEX;
+                        } else if (!strcmp(options->option[i].name, "logsize")
+				   && options->option[i].value.ui ) {
+				/* 'set log size' option */
+				options->option[i].number = MKFS_SETLOGSIZE_INDEX;
+                        } else {
+				/* unknown. ignore. */
+				continue;
+                        }
+                }
+
+		switch (options->option[i].number) {
+			
+			case MKFS_SETVOL_INDEX:
+				/* 'set volume name' option */
+				if ( options->option[i].value.s ) {
+					argv[opt_count++] = "-l";
+					argv[opt_count++] = options->option[i].value.s;
+                                }
+				break;
+
+			case MKFS_SETLOGSIZE_INDEX:
+				/* 'set log size' option */
+				if ( options->option[i].value.ui ) {
+					sprintf( buffer, "%s%u", "-s", options->option[i].value.ui/4 );
+					argv[opt_count++] = buffer;
+                                }
+				break;
+
+			default:
+				break;
+				
+		}
 	}
+
+	argv[opt_count++] = vol_name;
+	/* NULL terminate the argument array */
+	argv[opt_count] = NULL;
+
+	LOG_EXIT_VOID();
+	return;
 }
 
+
 /*
  * Format the volume.
  */
@@ -103,39 +166,35 @@ static int fsim_mkfs(logical_volume_t * volume, option_array_t * options )
 		return(rc);
 	}
 
-	rc = set_mkfs_options( options, argv, volume->dev_node );
+	set_mkfs_options( options, argv, volume->dev_node );
 
-	if (rc == 0) {
-		pidm = EngFncs->fork_and_execvp(volume, argv, NULL, fds2, fds2);
-		if (pidm != -1) {
-			fcntl(fds2[0], F_SETFL, fcntl(fds2[0], F_GETFL,0) | O_NONBLOCK);
-			while (!(waitpid( pidm, &status, WNOHANG ))) {
+	pidm = EngFncs->fork_and_execvp(volume, argv, NULL, fds2, fds2);
+	if (pidm != -1) {
+		fcntl(fds2[0], F_SETFL, fcntl(fds2[0], F_GETFL,0) | O_NONBLOCK);
+		while (!(waitpid( pidm, &status, WNOHANG ))) {
+			bytes_read = read(fds2[0],buffer,MAX_USER_MESSAGE_LEN);
+			if (bytes_read > 0) {
+				LOG_DEFAULT("mkfs output: \n%s",buffer); // on prompt if there is one
+				memset(buffer,0,bytes_read); //clear out message
+			}
+			usleep(10000); // don't hog all the cpu
+		}
+		if (WIFEXITED(status)) {
+			do {
 				bytes_read = read(fds2[0],buffer,MAX_USER_MESSAGE_LEN);
 				if (bytes_read > 0) {
-					LOG_DEFAULT("mkfs output: \n%s",buffer); // on prompt if there is one
-					memset(buffer,0,bytes_read); //clear out message
+					LOG_DEFAULT("mkfs output: \n%s",buffer);
 				}
-				usleep(10000); // don't hog all the cpu
-			}
-			if (WIFEXITED(status) && (WEXITSTATUS(status) != ENOENT)) {
-				do {
-					bytes_read = read(fds2[0],buffer,MAX_USER_MESSAGE_LEN);
-					if (bytes_read > 0) {
-						LOG_DEFAULT("mkfs output: \n%s",buffer);
-					}
-				} while (bytes_read > 0);
-				LOG_DEFAULT("mkfs completed with rc = %d \n",status);
-				rc = WEXITSTATUS(status);
-			} else {
-				rc = EINTR;
-			}
+			} while (bytes_read > 0);
+			LOG_DEFAULT("mkfs completed with rc = %d \n",status);
+			rc = WEXITSTATUS(status);
 		} else {
-			rc = EIO;
+			rc = EINTR;
 		}
+	} else {
+		rc = EIO;
 	}
 
-	free_args(argv);
-
 	EngFncs->engine_free(buffer);
 	close(fds2[0]);
 	close(fds2[1]);
@@ -144,92 +203,58 @@ static int fsim_mkfs(logical_volume_t * volume, option_array_t * options )
 }
 
 
-static inline int set_arg(char **argv, int index, char *str)
-{
-        argv[index] = NULL;
-	SET_STRING(argv[index], str);
-	return 0;
-}
-
 /*
- * NAME: set_mkfs_options
+ * NAME: set_fsck_options
  *
- * FUNCTION: Build options array (argv) for mkreiserfs
+ * FUNCTION: Build options array (argv) for reiserfsck
  *
  * PARAMETERS:
  *      options   - options array passed from EVMS engine
- *      argv      - mkfs options array
- *      vol_name  - volume name on which program will be executed
+ *      argv      - fsck options array
+ *      volume    - volume on which program will be executed
  *
  */
-static int set_mkfs_options( option_array_t * options, char ** argv, char * vol_name )
+static void set_fsck_options( option_array_t * options, char ** argv, logical_volume_t * volume )
 {
-	int i, rc, opt_count = 0;
-	char buffer[EVMS_VOLUME_NAME_SIZE + 10];
-
+	int i, opt_count = 0;
 	LOG_ENTRY();
 
-        rc = set_arg(argv, opt_count++, "mkreiserfs");
-	if (rc == 0) {
-                /* 'force/quiet' option -ff*/
-	        rc = set_arg(argv, opt_count++, "-ff");
+	argv[opt_count++] = "reiserfsck";
+	argv[opt_count++] = "-q";
 
-                if (rc == 0) {
-	                for ( i=0; i<options->count && rc == 0; i++ ) {
+	for ( i=0; i<options->count; i++) {
+		if ( !options->option[i].is_number_based ) {
+			if ( !strcmp(options->option[i].name, "mode") &&
+			     !EngFncs->is_mounted(volume->dev_node, NULL)) {
+				options->option[i].number = FSCK_MODE_INDEX;
+                        } else {
+				/* unknown. ignore. */
+				continue;
+                        }
+                }
 
-		                if (!options->option[i].is_number_based) {
-                			if ( !strcmp(options->option[i].name, "vollabel")
-                                                && (reiser_version >= 2)
-						&& options->option[i].value.s ) {
-		                		/* 'set volume name' option */
-						options->option[i].number = MKFS_SETVOL_INDEX;
-		                	} else if (!strcmp(options->option[i].name, "logsize")
-							&& options->option[i].value.ui ) {
-						/* 'set log size' option */
-						options->option[i].number = MKFS_SETLOGSIZE_INDEX;
-		                	 } else {
-                                                 /* unknown. ignore. */
-                                                 continue;
-                                        }
+		switch (options->option[i].number) {
+			case FSCK_MODE_INDEX:
+                                if (!(strcmp(options->option[i].value.s,FSCK_FIX))) {
+					argv[opt_count++] = "--fix-fixable";
+                                } else if (!(strcmp(options->option[i].value.s,FSCK_REBUILD_TREE))) {
+					argv[opt_count++] = "--rebuild-tree";
+                                } else if (!(strcmp(options->option[i].value.s,FSCK_READONLY))) {
+					argv[opt_count++] = "--check";
                                 }
-
-				switch (options->option[i].number) {
-			
-				case MKFS_SETVOL_INDEX:
-					/* 'set volume name' option */
-					if ( options->option[i].value.s ) {
-						rc = set_arg(argv, opt_count++, "-l");
-						if (rc == 0) {
-							rc = set_arg(argv, opt_count++, options->option[i].value.s);
-						}
-					}
-					break;
-
-				case MKFS_SETLOGSIZE_INDEX:
-					/* 'set log size' option */
-					if ( options->option[i].value.ui ) {
-						sprintf( buffer, "%s%u", "-s", options->option[i].value.ui/4 );
-						rc = set_arg(argv, opt_count++, buffer);
-					}
-					break;
-
-				default:
-					break;
-				}
+				break;
+				
+			default:
+				break;
 			}
-
-			if (rc == 0) {
-				rc = set_arg(argv, opt_count++, vol_name);
-				if (rc == 0) {
-					/* NULL terminate the argument array */
-					argv[opt_count] = NULL;
-				}
-			}
-		}
 	}
+	
+	argv[opt_count++] = volume->dev_node;
+	/* NULL terminate the pointer array */
+	argv[opt_count] = NULL;
 
-	LOG_EXIT_INT(rc);
-	return rc;
+	LOG_EXIT_VOID();
+	return;
 }
 
 
@@ -273,47 +298,43 @@ static int fsim_fsck(logical_volume_t * volume, option_array_t * options )
 		return(rc);
 	}
 
-	rc = set_fsck_options( options, argv, volume );
+	set_fsck_options( options, argv, volume );
 
-	if (rc == 0) {
-		pidf = EngFncs->fork_and_execvp(volume, argv, fds1, fds2, fds2);
-		if (pidf != -1) {
-			/* Fork and execute the correct program. */
-			// WARNING: Do Not close read handle of stdin or
-			// you will cause a SIGPIPE if you write after the
-			// child process has gone away.
-			write(fds1[1], "Yes\n",4);
+	pidf = EngFncs->fork_and_execvp(volume, argv, fds1, fds2, fds2);
+	if (pidf != -1) {
+		/* Fork and execute the correct program. */
+		// WARNING: Do Not close read handle of stdin or
+		// you will cause a SIGPIPE if you write after the
+		// child process has gone away.
+		write(fds1[1], "Yes\n",4);
 
-			/* wait for child to complete */
-			fcntl(fds2[0], F_SETFL, fcntl(fds2[0], F_GETFL,0) | O_NONBLOCK);
-			while (!(waitpid( pidf, &status, WNOHANG ))) {
+		/* wait for child to complete */
+		fcntl(fds2[0], F_SETFL, fcntl(fds2[0], F_GETFL,0) | O_NONBLOCK);
+		while (!(waitpid( pidf, &status, WNOHANG ))) {
+			bytes_read = read(fds2[0],buffer,MAX_USER_MESSAGE_LEN);
+			if (bytes_read > 0) {
+				write(fds1[1], "\n",1);	// send newline to take default
+				MESSAGE(_("fsck output: \n%s"),buffer);	// on prompt if there is one
+				memset(buffer,0,bytes_read); //clear out message
+			}
+			usleep(10000); // don't hog all the cpu
+		}
+		if (WIFEXITED(status)) {
+			do {
 				bytes_read = read(fds2[0],buffer,MAX_USER_MESSAGE_LEN);
 				if (bytes_read > 0) {
-					write(fds1[1], "\n",1);	// send newline to take default
-					MESSAGE("fsck output: \n%s",buffer); // on prompt if there is one
-					memset(buffer,0,bytes_read); //clear out message
+					MESSAGE(_("fsck output: \n\n%s"),buffer);
 				}
-				usleep(10000); // don't hog all the cpu
-			}
-			if ( WIFEXITED(status) && (WEXITSTATUS(status) != ENOENT)) {
-				do {
-					bytes_read = read(fds2[0],buffer,MAX_USER_MESSAGE_LEN);
-					if (bytes_read > 0) {
-						MESSAGE("fsck output: \n\n%s",buffer);
-					}
-				} while (bytes_read > 0);
-				LOG_DEFAULT("fsck completed with rc = %d \n",status);
-				rc = WEXITSTATUS(status);
-			} else {
-				rc = EINTR;
-			}
+			} while (bytes_read > 0);
+			LOG_DEFAULT("fsck completed with rc = %d \n",status);
+			rc = WEXITSTATUS(status);
 		} else {
-			rc = EIO;
+			rc = EINTR;
 		}
+	} else {
+		rc = EIO;
 	}
 
-	free_args(argv);
-
 	EngFncs->engine_free(buffer);
 
 	close(fds1[0]);
@@ -325,68 +346,6 @@ static int fsim_fsck(logical_volume_t * volume, option_array_t * options )
 }
 
 
-/*
- * NAME: set_fsck_options
- *
- * FUNCTION: Build options array (argv) for reiserfsck
- *
- * PARAMETERS:
- *      options   - options array passed from EVMS engine
- *      argv      - fsck options array
- *      volume    - volume on which program will be executed
- *
- */
-static int set_fsck_options( option_array_t * options, char ** argv, logical_volume_t * volume )
-{
-	int i, rc, opt_count = 0;
-	LOG_ENTRY();
-
-	rc = set_arg(argv, opt_count++, "reiserfsck");
-	if (rc == 0) {
-		rc = set_arg(argv, opt_count++, "-q");
-
-		if (rc == 0) {
-			for ( i=0; i<options->count && rc == 0; i++) {
-				if ( !options->option[i].is_number_based ) {
-					if ( !strcmp(options->option[i].name, "mode") &&
-					     !EngFncs->is_mounted(volume->dev_node, NULL)) {
-						options->option[i].number = FSCK_MODE_INDEX;
-					} else {
-						/* unknown. ignore. */
-						continue;
-					}
-				}
-
-				switch (options->option[i].number) {
-				case FSCK_MODE_INDEX:
-					if (!(strcmp(options->option[i].value.s,FSCK_FIX))) {
-						rc = set_arg(argv, opt_count++, "--fix-fixable");
-					} else if (!(strcmp(options->option[i].value.s,FSCK_REBUILD_TREE))) {
-						rc = set_arg(argv, opt_count++, "--rebuild-tree");
-					} else if (!(strcmp(options->option[i].value.s,FSCK_READONLY))) {
-						rc = set_arg(argv, opt_count++, "--check");
-					}
-					break;
-				default:
-					break;
-				}
-			}
-
-			if (rc == 0) {
-				rc = set_arg(argv, opt_count++, volume->dev_node);
-				if (rc == 0) {
-					/* NULL terminate the pointer array */
-					argv[opt_count] = NULL;
-				}
-			}
-		}
-	}
-
-	LOG_EXIT_INT(rc);
-	return rc;
-}
-
-
 /*
  * NAME: fsim_get_reiser_reiserfs_super_block
  *
@@ -818,13 +777,13 @@ static int reiser_expand(logical_volume_t *volume,
 	LOG_ENTRY();
 
 	if (!reiser_version) {
-		MESSAGE("ReiserFS utils were either not found or not at the "
-			"proper version. The fsck and mkfs utilities must be "
-			"installed using the standard names of mkreiserfs and "
-			"reiserfsck. ReiserFS utils must be version 3.x.0 or "
-			"later to function properly with this FSIM.Please get "
-			"the current version of ReiserFS utils from "
-			"http://www.namesys.com\n");
+		MESSAGE(_("ReiserFS utils were either not found or not at the "
+			  "proper version. The fsck and mkfs utilities must be "
+			  "installed using the standard names of mkreiserfs and "
+			  "reiserfsck. ReiserFS utils must be version 3.x.0 or "
+			  "later to function properly with this FSIM.Please get "
+			  "the current version of ReiserFS utils from "
+			  "http://www.namesys.com\n"));
 		LOG_EXIT_INT(E_CANCELED);
 		return E_CANCELED;
 	}
@@ -853,7 +812,7 @@ static int reiser_expand(logical_volume_t *volume,
 	}
 
 	argv[0] = "resize_reiserfs";
-	SET_STRING(argv[1],volume->dev_node);
+	argv[1] = volume->dev_node;
 	argv[2] = NULL;
 
 	pidf = EngFncs->fork_and_execvp(volume, argv, fds1, fds2, fds2);
@@ -867,7 +826,7 @@ static int reiser_expand(logical_volume_t *volume,
 		while (!(waitpid( pidf, &status, WNOHANG ))) {
 			bytes_read = read(fds2[0],buffer,MAX_USER_MESSAGE_LEN);
 			if (bytes_read > 0) {
-				MESSAGE("expand output: \n\n%s",buffer);
+				MESSAGE(_("expand output: \n\n%s"),buffer);
 				memset(buffer,0,bytes_read); //clear out message
 			}
 			usleep(10000); // don't hog all the cpu
@@ -875,7 +834,7 @@ static int reiser_expand(logical_volume_t *volume,
 		// do final read, just in case we missed some
 		bytes_read = read(fds2[0],buffer,MAX_USER_MESSAGE_LEN);
 		if (bytes_read > 0) {
-			MESSAGE("expand output: \n\n%s",buffer);
+			MESSAGE(_("expand output: \n\n%s"),buffer);
 		}
 		if ( WIFEXITED(status) ) {
 			/* get expand exit code */
@@ -888,7 +847,6 @@ static int reiser_expand(logical_volume_t *volume,
 		rc = EIO;
 	}
 	EngFncs->engine_free(buffer);
-	EngFncs->engine_free(argv[1]);
 	reiser_get_fs_size(volume, new_size);
 
 	close(fds1[0]);
@@ -963,13 +921,13 @@ static int reiser_shrink(logical_volume_t *volume,
 	LOG_ENTRY();
 
 	if (!reiser_version) {
-		MESSAGE("ReiserFS utils were either not found or not at the "
-			"proper version. The fsck and mkfs utilities must be "
-			"installed using the standard names of mkreiserfs and "
-			"reiserfsck. ReiserFS utils must be version 3.x.0 or "
-			"later to function properly with this FSIM. Please get "
-			"the current version of ReiserFS utils from "
-			"http://www.namesys.com\n");
+		MESSAGE(_("ReiserFS utils were either not found or not at the "
+			  "proper version. The fsck and mkfs utilities must be "
+			  "installed using the standard names of mkreiserfs and "
+			  "reiserfsck. ReiserFS utils must be version 3.x.0 or "
+			  "later to function properly with this FSIM. Please get "
+			  "the current version of ReiserFS utils from "
+			  "http://www.namesys.com\n"));
 		LOG_EXIT_INT(E_CANCELED);
 		return E_CANCELED;
 	}
@@ -1008,7 +966,7 @@ static int reiser_shrink(logical_volume_t *volume,
 	argv[1] = "-q";
 	sprintf(size_buf,"-s%"PRIu64"", (sector_count_t)requested_size * (sector_count_t)EVMS_VSECTOR_SIZE);	//set newsize in bytes
 	argv[2] = size_buf;
-	SET_STRING(argv[3],volume->dev_node);
+	argv[3] = volume->dev_node;
 	argv[4] = NULL;
 
 	pidf = EngFncs->fork_and_execvp(volume, argv, fds1, fds2, fds2);
@@ -1023,7 +981,7 @@ static int reiser_shrink(logical_volume_t *volume,
 		while (!(waitpid( pidf, &status, WNOHANG ))) {
 			bytes_read = read(fds2[0],buffer,MAX_USER_MESSAGE_LEN);
 			if (bytes_read > 0) {
-				MESSAGE("Shrink output: \n%s",buffer);
+				MESSAGE(_("Shrink output: \n%s"),buffer);
 				memset(buffer,0,bytes_read); //clear out message
 			}
 			usleep(10000); // don't hog all the cpu
@@ -1031,7 +989,7 @@ static int reiser_shrink(logical_volume_t *volume,
 		// do final read, just in case we missed some
 		bytes_read = read(fds2[0],buffer,MAX_USER_MESSAGE_LEN);
 		if (bytes_read > 0) {
-			MESSAGE("Shrink output: \n%s",buffer);
+			MESSAGE(_("Shrink output: \n%s"),buffer);
 		}
 		if ( WIFEXITED(status) ) {
 			/* get shrink exit code */
@@ -1044,7 +1002,6 @@ static int reiser_shrink(logical_volume_t *volume,
 		rc = EIO;
 	}
 	EngFncs->engine_free(buffer);
-	EngFncs->engine_free(argv[3]);
 	reiser_get_fs_size(volume, new_size);
 
 	close(fds1[0]);
@@ -1073,13 +1030,13 @@ static int reiser_mkfs(logical_volume_t *volume, option_array_t *options)
 		return rc;
 	}
 	if (!reiser_version) {
-		MESSAGE("ReiserFS utils were either not found or not at the "
-			"proper version. The fsck and mkfs utilities must be "
-			"installed using the standard names of mkreiserfs and "
-			"reiserfsck. ReiserFS utils must be version 3.x.0 or "
-			"later to function properly with this FSIM. Please get "
-			"the current version of ReiserFS utils from "
-			"http://www.namesys.com\n");
+		MESSAGE(_("ReiserFS utils were either not found or not at the "
+			  "proper version. The fsck and mkfs utilities must be "
+			  "installed using the standard names of mkreiserfs and "
+			  "reiserfsck. ReiserFS utils must be version 3.x.0 or "
+			  "later to function properly with this FSIM. Please get "
+			  "the current version of ReiserFS utils from "
+			  "http://www.namesys.com\n"));
 		LOG_EXIT_INT(E_CANCELED);
 		return E_CANCELED;
 	}
@@ -1121,13 +1078,13 @@ static int reiser_fsck(logical_volume_t *volume, option_array_t *options)
 
 	LOG_ENTRY();
 	if (!reiser_version) {
-		MESSAGE("ReiserFS utils were either not found or not at the "
-			"proper version. The fsck and mkfs utilities must be "
-			"installed using the standard names of mkreiserfs and "
-			"reiserfsck. ReiserFS utils must be version 3.x.0 or "
-			"later to function properly with this FSIM. Please get "
-			"the current version of ReiserFS utils from "
-			"http://www.namesys.com\n");
+		MESSAGE(_("ReiserFS utils were either not found or not at the "
+			  "proper version. The fsck and mkfs utilities must be "
+			  "installed using the standard names of mkreiserfs and "
+			  "reiserfsck. ReiserFS utils must be version 3.x.0 or "
+			  "later to function properly with this FSIM. Please get "
+			  "the current version of ReiserFS utils from "
+			  "http://www.namesys.com\n"));
 		LOG_EXIT_INT(E_CANCELED);
 		return E_CANCELED;
 	}
@@ -1154,9 +1111,10 @@ static int reiser_fsck(logical_volume_t *volume, option_array_t *options)
 	 * and errors were found, let the user know how to fix them.
 	 */
 	if (EngFncs->is_mounted(volume->dev_node, NULL) && rc) {
-		MESSAGE( "%s is mounted.", volume->dev_node );
-		MESSAGE( "reiserfsck (fsck.reiserfs) checked the volume READ ONLY and found errors but did not fix them." );
-		MESSAGE( "Unmount %s and run reiserfsck (fsck.reiserfs) again to repair the file system.", volume->dev_node );
+		MESSAGE( _("%s is mounted."
+			   "reiserfsck (fsck.reiserfs) checked the volume READ ONLY and found errors but did not fix them."
+			   "Unmount %s and run reiserfsck (fsck.reiserfs) again to repair the file system."),
+			 volume->dev_node, volume->dev_node );
 	}
 
 	LOG_EXIT_INT(rc);
@@ -1263,14 +1221,14 @@ static int reiser_init_task(task_context_t *context)
 		}
 
 		if (!reiser_version) {
-			MESSAGE("ReiserFS utils were either not found or not "
-				"at the proper version. The fsck and mkfs "
-				"utilities must be installed using the "
-				"standard names of mkreiserfs and reiserfsck. "
-				"ReiserFS utils must be version 3.x.0 or "
-				"later to function properly with this FSIM. "
-				"Please get the current version of ReiserFS "
-				"utils from http://www.namesys.com\n");
+			MESSAGE(_("ReiserFS utils were either not found or not "
+				  "at the proper version. The fsck and mkfs "
+				  "utilities must be installed using the "
+				  "standard names of mkreiserfs and reiserfsck. "
+				  "ReiserFS utils must be version 3.x.0 or "
+				  "later to function properly with this FSIM. "
+				  "Please get the current version of ReiserFS "
+				  "utils from http://www.namesys.com\n"));
 			rc = E_CANCELED;
 			break;
 		}
@@ -1279,9 +1237,9 @@ static int reiser_init_task(task_context_t *context)
 
 		context->option_descriptors->count = MKFS_OPTIONS_COUNT;
 		/* Set version display option */
-		SET_STRING(context->option_descriptors->option[MKFS_VERSION_INDEX].name, "version" );
-		SET_STRING(context->option_descriptors->option[MKFS_VERSION_INDEX].title, "Version of mkreiserfs" );
-		SET_STRING(context->option_descriptors->option[MKFS_VERSION_INDEX].tip, "Informational only: this is the current version of ResierFS utitlities found on your system." );
+		context->option_descriptors->option[MKFS_VERSION_INDEX].name = EngFncs->engine_strdup("version" );
+		context->option_descriptors->option[MKFS_VERSION_INDEX].title = EngFncs->engine_strdup(_("Version of mkreiserfs"));
+		context->option_descriptors->option[MKFS_VERSION_INDEX].tip = EngFncs->engine_strdup(_("Informational only: this is the current version of ResierFS utitlities found on your system."));
 		context->option_descriptors->option[MKFS_VERSION_INDEX].help = NULL;
 		context->option_descriptors->option[MKFS_VERSION_INDEX].type = EVMS_Type_String;
 		context->option_descriptors->option[MKFS_VERSION_INDEX].unit = EVMS_Unit_None;
@@ -1294,9 +1252,9 @@ static int reiser_init_task(task_context_t *context)
 
 
 		/* Set Volume Label option */
-		SET_STRING(context->option_descriptors->option[MKFS_SETVOL_INDEX].name, "vollabel" );
-		SET_STRING(context->option_descriptors->option[MKFS_SETVOL_INDEX].title, "Volume Label" );
-		SET_STRING(context->option_descriptors->option[MKFS_SETVOL_INDEX].tip, "Set the volume label for the file system." );
+		context->option_descriptors->option[MKFS_SETVOL_INDEX].name = EngFncs->engine_strdup("vollabel" );
+		context->option_descriptors->option[MKFS_SETVOL_INDEX].title = EngFncs->engine_strdup(_("Volume Label"));
+		context->option_descriptors->option[MKFS_SETVOL_INDEX].tip = EngFncs->engine_strdup(_("Set the volume label for the file system."));
 		context->option_descriptors->option[MKFS_SETVOL_INDEX].help = NULL;
 		context->option_descriptors->option[MKFS_SETVOL_INDEX].type = EVMS_Type_String;
 		context->option_descriptors->option[MKFS_SETVOL_INDEX].unit = EVMS_Unit_None;
@@ -1310,9 +1268,9 @@ static int reiser_init_task(task_context_t *context)
 		}
 
 		/* Log size option */
-		SET_STRING(context->option_descriptors->option[MKFS_SETLOGSIZE_INDEX].name, "logsize" );
-		SET_STRING(context->option_descriptors->option[MKFS_SETLOGSIZE_INDEX].title, "Log Size" );
-		SET_STRING(context->option_descriptors->option[MKFS_SETLOGSIZE_INDEX].tip, "Set log size (in kilobytes).  Default log size is 8193 4k Blocks." );
+		context->option_descriptors->option[MKFS_SETLOGSIZE_INDEX].name = EngFncs->engine_strdup("logsize" );
+		context->option_descriptors->option[MKFS_SETLOGSIZE_INDEX].title = EngFncs->engine_strdup(_("Log Size"));
+		context->option_descriptors->option[MKFS_SETLOGSIZE_INDEX].tip = EngFncs->engine_strdup(_("Set log size (in kilobytes).  Default log size is 8193 4k Blocks."));
 		context->option_descriptors->option[MKFS_SETLOGSIZE_INDEX].help = NULL;
 		context->option_descriptors->option[MKFS_SETLOGSIZE_INDEX].type = EVMS_Type_Unsigned_Int32;
 		context->option_descriptors->option[MKFS_SETLOGSIZE_INDEX].unit = EVMS_Unit_Kilobytes;
@@ -1335,14 +1293,14 @@ static int reiser_init_task(task_context_t *context)
 
 	case EVMS_Task_fsck:
 		if (!reiser_version) {
-			MESSAGE("ReiserFS utils were either not found or not "
-				"at the proper version. The fsck and mkfs "
-				"utilities must be installed using the "
-				"standard names of mkreiserfs and "
-				"reiserfsck. ReiserFS utils must be version "
-				"3.x.0 or later to function properly with this "
-				"FSIM. Please get the current version of "
-				"ReiserFS utils from http://www.namesys.com\n");
+			MESSAGE(_("ReiserFS utils were either not found or not "
+				  "at the proper version. The fsck and mkfs "
+				  "utilities must be installed using the "
+				  "standard names of mkreiserfs and "
+				  "reiserfsck. ReiserFS utils must be version "
+				  "3.x.0 or later to function properly with this "
+				  "FSIM. Please get the current version of "
+				  "ReiserFS utils from http://www.namesys.com\n"));
 			rc = E_CANCELED;
 			break;
 		}
@@ -1358,12 +1316,12 @@ static int reiser_init_task(task_context_t *context)
 		if (value_list) {
 			if (EngFncs->is_mounted(context->volume->dev_node, NULL)) {
 				value_list->count = 1;
-				SET_STRING(value_list->value[0].s,"Check Read-Only");
+				value_list->value[0].s = EngFncs->engine_strdup(_("Check Read-Only"));
 			} else {
 				value_list->count = 3;
-				SET_STRING(value_list->value[0].s,"Check Read-Only");
-				SET_STRING(value_list->value[1].s,"Fix");
-				SET_STRING(value_list->value[2].s,"Rebuild Tree");
+				value_list->value[0].s = EngFncs->engine_strdup(_("Check Read-Only"));
+				value_list->value[1].s = EngFncs->engine_strdup(_("Fix"));
+				value_list->value[2].s = EngFncs->engine_strdup(_("Rebuild Tree"));
 			}
 		} else {
 			LOG_EXIT_INT(rc);
@@ -1371,9 +1329,9 @@ static int reiser_init_task(task_context_t *context)
 		}
 
 		/* MODE option */
-		SET_STRING(context->option_descriptors->option[FSCK_MODE_INDEX].name, "mode" );
-		SET_STRING(context->option_descriptors->option[FSCK_MODE_INDEX].title, "Mode" );
-		SET_STRING(context->option_descriptors->option[FSCK_MODE_INDEX].tip, "Mode of operation" );
+		context->option_descriptors->option[FSCK_MODE_INDEX].name = EngFncs->engine_strdup("mode");
+		context->option_descriptors->option[FSCK_MODE_INDEX].title = EngFncs->engine_strdup(_("Mode"));
+		context->option_descriptors->option[FSCK_MODE_INDEX].tip = EngFncs->engine_strdup(_("Mode of operation"));
 		context->option_descriptors->option[FSCK_MODE_INDEX].help = NULL;
 		context->option_descriptors->option[FSCK_MODE_INDEX].max_len = EVMS_VOLUME_NAME_SIZE;
 		context->option_descriptors->option[FSCK_MODE_INDEX].min_len = 1;
@@ -1461,7 +1419,7 @@ static int reiser_set_option(task_context_t *context,
 			if (EngFncs->is_mounted(context->volume->dev_node, NULL)) {
 				if (!(strcmp(value->s,FSCK_FIX)) || !(strcmp(value->s,FSCK_REBUILD_TREE))) {
 					strcpy(context->option_descriptors->option[FSCK_MODE_INDEX].value.s,FSCK_READONLY);
-					MESSAGE("Only Check mode is alllowed when the volume is mounted.");
+					MESSAGE(_("Only Check mode is alllowed when the volume is mounted."));
 					*effect |= EVMS_Effect_Reload_Options;
 				}
 			} else {
@@ -1470,10 +1428,10 @@ static int reiser_set_option(task_context_t *context,
 					strcpy(context->option_descriptors->option[FSCK_MODE_INDEX].value.s,value->s);
 				}
 				if (strcmp(value->s,FSCK_READONLY)) {
-					MESSAGE("Selecting a mode other than read only will cause "
-						"default responses to be given to any questions asked by the reiserfsck (fsck.reiserfs) "
-						"utitlity.  If you do not wish to run in this mode, you must run reiserfsck (fsck.reiserfs) "
-						"from the command line.");
+					MESSAGE(_("Selecting a mode other than read only will cause "
+						  "default responses to be given to any questions asked by the reiserfsck (fsck.reiserfs) "
+						  "utitlity.  If you do not wish to run in this mode, you must run reiserfsck (fsck.reiserfs) "
+						  "from the command line."));
 				}
 			}
 			break;
@@ -1523,8 +1481,12 @@ static int reiser_set_volumes(task_context_t *context,
 				rc = EBUSY;
 			} else {
 				if ( vol->vol_size < MINREISERFS) {
-					MESSAGE( "The size of volume %s is %"PRIu64" bytes.", vol->name, vol->vol_size << EVMS_VSECTOR_SIZE_SHIFT );
-					MESSAGE( "mkfs.reiserfs requires a minimum of %u bytes to build the ReiserFS file system.", MINREISERFS );
+					char number_buffer[64];
+
+					sprintf(number_buffer, "%"PRIu64, vol->vol_size << EVMS_VSECTOR_SIZE_SHIFT );
+					MESSAGE( _("The size of volume %s is %s bytes.  "
+						   "mkfs.reiserfs requires a minimum of %u bytes to build the ReiserFS file system."),
+						   vol->name, number_buffer, MINREISERFS );
 					rc = EPERM;
 				}
 			}
@@ -1588,36 +1550,36 @@ static int reiser_get_volume_info(logical_volume_t *volume,
 
 			Info->count = 5;
 
-			SET_STRING( Info->info[0].name, "MagicNumber" );
-			SET_STRING( Info->info[0].title, "Magic Number" );
-			SET_STRING( Info->info[0].desc, "Magic Number");
+			Info->info[0].name = EngFncs->engine_strdup( "MagicNumber" );
+			Info->info[0].title = EngFncs->engine_strdup( _("Magic Number") );
+			Info->info[0].desc = EngFncs->engine_strdup( _("Magic Number") );
 			Info->info[0].type               = EVMS_Type_String;
 			Info->info[0].unit               = EVMS_Unit_None;
-			SET_STRING(Info->info[0].value.s, sb_ptr->s_v1.s_magic);
+			Info->info[0].value.s = EngFncs->engine_strdup(sb_ptr->s_v1.s_magic);
 			Info->info[0].collection_type    = EVMS_Collection_None;
 			memset( &Info->info[0].group, 0, sizeof(group_info_t));
 
-			SET_STRING( Info->info[1].name, "Version" );
-			SET_STRING( Info->info[1].title, "Version Number" );
-			SET_STRING( Info->info[1].desc, "Version Number");
+			Info->info[1].name = EngFncs->engine_strdup( "Version" );
+			Info->info[1].title = EngFncs->engine_strdup( _("Version Number") );
+			Info->info[1].desc = EngFncs->engine_strdup( _("Version Number") );
 			Info->info[1].type               = EVMS_Type_Unsigned_Int16;
 			Info->info[1].unit               = EVMS_Unit_None;
 			Info->info[1].value.ui16         = get_sb_version(sb_ptr);
 			Info->info[1].collection_type    = EVMS_Collection_None;
 			memset( &Info->info[1].group, 0, sizeof(group_info_t));
 
-			SET_STRING( Info->info[2].name, "VolLabel" );
-			SET_STRING( Info->info[2].title, "Volume Label" );
-			SET_STRING( Info->info[2].desc, "File system volume label");
+			Info->info[2].name = EngFncs->engine_strdup( "VolLabel" );
+			Info->info[2].title = EngFncs->engine_strdup( _("Volume Label") );
+			Info->info[2].desc = EngFncs->engine_strdup( _("File system volume label") );
 			Info->info[2].type               = EVMS_Type_String;
 			Info->info[2].unit               = EVMS_Unit_None;
-			SET_STRING(Info->info[2].value.s, sb_ptr->s_label);
+			Info->info[2].value.s = EngFncs->engine_strdup(sb_ptr->s_label);
 			Info->info[2].collection_type    = EVMS_Collection_None;
 			memset( &Info->info[2].group, 0, sizeof(group_info_t));
 
-			SET_STRING( Info->info[3].name, "UsableSize" );
-			SET_STRING( Info->info[3].title, "File System Size" );
-			SET_STRING( Info->info[3].desc, "Size of the file system (not including journal)");
+			Info->info[3].name = EngFncs->engine_strdup( "UsableSize" );
+			Info->info[3].title = EngFncs->engine_strdup( _("File System Size") );
+			Info->info[3].desc = EngFncs->engine_strdup( _("Size of the file system (not including journal)") );
 			Info->info[3].type               = EVMS_Type_Unsigned_Int64;
 			Info->info[3].unit               = EVMS_Unit_Sectors;
 			Info->info[3].value.ui64         = (get_sb_block_count(sb_ptr)
@@ -1625,9 +1587,9 @@ static int reiser_get_volume_info(logical_volume_t *volume,
 			Info->info[3].collection_type    = EVMS_Collection_None;
 			memset( &Info->info[3].group, 0, sizeof(group_info_t));
 
-			SET_STRING( Info->info[4].name, "LogSize" );
-			SET_STRING( Info->info[4].title, "Journal Size" );
-			SET_STRING( Info->info[4].desc, "Size of the file system journal");
+			Info->info[4].name = EngFncs->engine_strdup( "LogSize" );
+			Info->info[4].title = EngFncs->engine_strdup( _("Journal Size") );
+			Info->info[4].desc = EngFncs->engine_strdup( _("Size of the file system journal") );
 			Info->info[4].type               = EVMS_Type_Unsigned_Int64;
 			Info->info[4].unit               = EVMS_Unit_Sectors;
 			Info->info[4].value.ui64         = get_jp_journal_size(sb_ptr) * (get_sb_block_size(sb_ptr)/EVMS_VSECTOR_SIZE);
@@ -1691,57 +1653,57 @@ static int reiser_get_plugin_info(char *descriptor_name,
 					my_plugin_record->required_plugin_api_version.fsim.minor,
 					my_plugin_record->required_plugin_api_version.fsim.patchlevel );
 
-				SET_STRING( Info->info[0].name, "Short Name" );
-				SET_STRING( Info->info[0].title, "Short Name" );
-				SET_STRING( Info->info[0].desc, "A short name given to this plugin");
+				Info->info[0].name = EngFncs->engine_strdup( "Short Name" );
+				Info->info[0].title = EngFncs->engine_strdup( _("Short Name") );
+				Info->info[0].desc = EngFncs->engine_strdup( _("A short name given to this plug-in") );
 				Info->info[0].type               = EVMS_Type_String;
 				Info->info[0].unit               = EVMS_Unit_None;
-				SET_STRING( Info->info[0].value.s, my_plugin_record->short_name );
+				Info->info[0].value.s = EngFncs->engine_strdup( my_plugin_record->short_name );
 				Info->info[0].collection_type    = EVMS_Collection_None;
 				memset( &Info->info[0].group, 0, sizeof(group_info_t));
 
-				SET_STRING( Info->info[1].name, "Long Name" );
-				SET_STRING( Info->info[1].title, "Long Name" );
-				SET_STRING( Info->info[1].desc, "A long name given to this plugin");
+				Info->info[1].name = EngFncs->engine_strdup( "Long Name" );
+				Info->info[1].title = EngFncs->engine_strdup( _("Long Name") );
+				Info->info[1].desc = EngFncs->engine_strdup( _("A longer, more descriptive name for this plug-in") );
 				Info->info[1].type               = EVMS_Type_String;
 				Info->info[1].unit               = EVMS_Unit_None;
-				SET_STRING( Info->info[1].value.s, my_plugin_record->long_name );
+				Info->info[1].value.s = EngFncs->engine_strdup( my_plugin_record->long_name );
 				Info->info[1].collection_type    = EVMS_Collection_None;
 				memset( &Info->info[1].group, 0, sizeof(group_info_t));
 
-				SET_STRING( Info->info[2].name, "Type" );
-				SET_STRING( Info->info[2].title, "Plugin Type" );
-				SET_STRING( Info->info[2].desc, "There are various types of plugins; each responsible for some kind of storage object.");
+				Info->info[2].name = EngFncs->engine_strdup( "Type" );
+				Info->info[2].title = EngFncs->engine_strdup( _("Plug-in Type") );
+				Info->info[2].desc = EngFncs->engine_strdup( _("There are various types of plug-ins, each responsible for some kind of storage object or logical volume.") );
 				Info->info[2].type               = EVMS_Type_String;
 				Info->info[2].unit               = EVMS_Unit_None;
-				SET_STRING( Info->info[2].value.s, "File System Interface Module" );
+				Info->info[2].value.s = EngFncs->engine_strdup( _("File System Interface Module") );
 				Info->info[2].collection_type    = EVMS_Collection_None;
 				memset( &Info->info[2].group, 0, sizeof(group_info_t));
 
-				SET_STRING( Info->info[3].name, "Version" );
-				SET_STRING( Info->info[3].title, "Plugin Version" );
-				SET_STRING( Info->info[3].desc, "This is the version number of the plugin.");
+				Info->info[3].name = EngFncs->engine_strdup( "Version" );
+				Info->info[3].title = EngFncs->engine_strdup( _("Plug-in Version") );
+				Info->info[3].desc = EngFncs->engine_strdup( _("This is the version number of the plug-in."));
 				Info->info[3].type               = EVMS_Type_String;
 				Info->info[3].unit               = EVMS_Unit_None;
-				SET_STRING( Info->info[3].value.s, version_string );
+				Info->info[3].value.s = EngFncs->engine_strdup( version_string );
 				Info->info[3].collection_type    = EVMS_Collection_None;
 				memset( &Info->info[3].group, 0, sizeof(group_info_t));
 
-				SET_STRING( Info->info[4].name, "Required Engine Services Version" );
-				SET_STRING( Info->info[4].title, "Required Engine Services Version" );
-				SET_STRING( Info->info[4].desc, "This is the version of the Engine services that this plug-in requires.  It will not run on older versions of the Engine services.");
+				Info->info[4].name = EngFncs->engine_strdup( "Required Engine Services Version" );
+				Info->info[4].title = EngFncs->engine_strdup( _("Required Engine Services Version") );
+				Info->info[4].desc = EngFncs->engine_strdup( _("This is the version of the Engine services that this plug-in requires.  It will not run on older versions of the Engine services.") );
 				Info->info[4].type               = EVMS_Type_String;
 				Info->info[4].unit               = EVMS_Unit_None;
-				SET_STRING( Info->info[4].value.s, required_engine_api_version_string );
+				Info->info[4].value.s = EngFncs->engine_strdup( required_engine_api_version_string );
 				Info->info[4].collection_type    = EVMS_Collection_None;
 				memset( &Info->info[4].group, 0, sizeof(group_info_t));
 
-				SET_STRING( Info->info[5].name, "Required Engine FSIM API Version" );
-				SET_STRING( Info->info[5].title, "Required Engine FSIM API Version" );
-				SET_STRING( Info->info[5].desc, "This is the version of the Engine FSIM API that this plug-in requires.  It will not run on older versions of the Engine FSIM API.");
+				Info->info[5].name = EngFncs->engine_strdup( "Required Engine FSIM API Version" );
+				Info->info[5].title = EngFncs->engine_strdup( _("Required Engine FSIM API Version") );
+				Info->info[5].desc = EngFncs->engine_strdup( _("This is the version of the Engine FSIM API that this plug-in requires.  It will not run on older versions of the Engine FSIM API.") );
 				Info->info[5].type               = EVMS_Type_String;
 				Info->info[5].unit               = EVMS_Unit_None;
-				SET_STRING( Info->info[5].value.s, required_fsim_api_version_string );
+				Info->info[5].value.s = EngFncs->engine_strdup( required_fsim_api_version_string );
 				Info->info[5].collection_type    = EVMS_Collection_None;
 				memset( &Info->info[5].group, 0, sizeof(group_info_t));
 
@@ -1780,7 +1742,7 @@ static int reiser_can_expand_by(logical_volume_t *volume,
 				     &volume->min_fs_size,
 				     &volume->max_vol_size,
 				     &volume->max_fs_size);
-		if (volume->fs_size + *delta > volume->max_fs_size) {
+		if (*delta > volume->max_fs_size - volume->fs_size) {
 			*delta = volume->max_fs_size - volume->fs_size;
 		}
 	}
@@ -1807,10 +1769,10 @@ static int reiser_can_shrink_by(logical_volume_t *volume,
 				     &volume->min_fs_size,
 				     &volume->max_vol_size,
 				     &volume->max_fs_size);
-		if (volume->fs_size - *delta < volume->min_fs_size) {
+		if (*delta > volume->fs_size - volume->min_fs_size) {
 			*delta = volume->fs_size - volume->min_fs_size;
 		}
-		if (volume->min_fs_size >= volume->vol_size) {
+		if (volume->min_fs_size >= volume->fs_size) {
 			rc = ENOSPC;
 		}
 	}
@@ -1853,13 +1815,13 @@ plugin_record_t reiser_plugin_record = {
 		.patchlevel	= PATCH_LEVEL
 	},
 	.required_engine_api_version = {
-		.major		= 13,
+		.major		= 15,
 		.minor		= 0,
 		.patchlevel	= 0
 	},
 	.required_plugin_api_version = {
 		.fsim = {
-			.major		= 10,
+			.major		= 11,
 			.minor		= 0,
 			.patchlevel	= 0
 		}
