@@ -44,7 +44,7 @@
 /*******************macros*******************/
 #define LOCAL_NAME                   "RSCT plugin"
 #define GROUP_NAME                   "RSCT_plugin_group"
-#define MAX_MSG_COUNT                 100000  
+#define MAX_MSG_COUNT                 100000
 
 /*******************gloable variables**********************/
 GSList*                  gl_cb_list;
@@ -69,7 +69,7 @@ unsigned                 gl_rsct_thread_activate;
 
 unsigned                 gl_ack_thread_dispatch_once;
 
-pthread_mutex_t          gl_event_queue_lock; 
+pthread_mutex_t          gl_event_queue_lock;
 
 
 pthread_t                gl_rsct_thread;
@@ -96,7 +96,7 @@ extern int gl_transid;
 /*******************internal static functions***************/
 
 
-static void 
+static void
 unregister_all()
 {
 	
@@ -142,7 +142,7 @@ static int get_msg_count()
 }
 
 //only called by the act and rsct  thread
-static void 
+static void
 RSCT_non_blocking_dispatch()
 {
 
@@ -158,7 +158,7 @@ RSCT_non_blocking_dispatch()
 	return;
 }
 
-static int 
+static int
 get_local_instance_number()
 {
 	
@@ -205,12 +205,12 @@ rsct_dispatch(void *arg)
 			LOG_DEBUG("there are data in coming\n");
 			RSCT_non_blocking_dispatch();
 		}
-	} 
+	}
 	LOG_EXIT_PTR(NULL);
 	return(NULL);
 }
 
-static gint 
+static gint
 cb_compare (gconstpointer a,
 	    gconstpointer b)
 {
@@ -231,7 +231,7 @@ cb_compare (gconstpointer a,
 	
 }
 
-static int 
+static int
 frag_send_msg(ece_msg_t* ecemsg)
 {
 	
@@ -279,7 +279,7 @@ frag_send_msg(ece_msg_t* ecemsg)
 		total_frag_num=(tlen-1)/flen +1;
 	
 	
-	LOG_DEBUG("tlen=%d, flen=%d,total_frag_num=%d\n", 
+	LOG_DEBUG("tlen=%d, flen=%d,total_frag_num=%d\n",
 		  tlen,
 		  flen,
 		  total_frag_num);
@@ -309,13 +309,13 @@ frag_send_msg(ece_msg_t* ecemsg)
 		do{
 			
 			//give some rest to rsct between the interval
-			usleep(sleep_time); 
+			usleep(sleep_time);
 			
 			proposal_info.gs_message_request.gs_num_phases  = HA_GS_1_PHASE;
 			proposal_info.gs_message_request.gs_time_limit  = TIME_LIMIT;
-			proposal_info.gs_message_request.gs_message.gs_length  = 
+			proposal_info.gs_message_request.gs_message.gs_length  =
 				length+sizeof(rsct_msg_t);
-			proposal_info.gs_message_request.gs_message.gs_message = (char*)sendBuf;;
+			proposal_info.gs_message_request.gs_message.gs_message = (char*)sendBuf;
 			
 			pthread_mutex_lock(&gl_send_lock);	
 			
@@ -334,20 +334,20 @@ frag_send_msg(ece_msg_t* ecemsg)
 					LOG_ERROR("warning:msg sending collide:"
 						  " msg_count=%d,seqno=%d,length=%d\n",
 						  rsctmsg->msg_count,
-						  rsctmsg->seqno, 
+						  rsctmsg->seqno,
 						  proposal_info.gs_message_request.
 						  gs_message.gs_length);
 				}
 				else {
-					LOG_ERROR("ERROR:" 
+					LOG_ERROR("ERROR:"
 						     "ha_gs_send_message() failed rc=%d\n", gs_rc);
 					free(sendBuf);
 					RETURN(EINVAL);
 				}
 			}
 		}while(gs_rc !=HA_GS_OK);
-	} 
-  
+	}
+
 	LOG_DEBUG("frag_send success\n");
 	free(sendBuf);
 	LOG_EXIT_INT(0);
@@ -358,7 +358,7 @@ frag_send_msg(ece_msg_t* ecemsg)
 
 /*******************RSCT plugin interface functions **********/
 
-static void 
+static void
 RSCT_cleanup_evms_plugin(void)
 {
 	
@@ -384,7 +384,7 @@ RSCT_cleanup_evms_plugin(void)
 	gs_rc = ha_gs_leave( gl_provider_token,
 			     &proposal_info);
 	if(gs_rc != HA_GS_OK) {
-		LOG_ERROR("ERROR: leaving the group failed, rc=%d\n", 
+		LOG_ERROR("ERROR: leaving the group failed, rc=%d\n",
 			     gs_rc);
 	}
 	
@@ -405,7 +405,7 @@ RSCT_cleanup_evms_plugin(void)
 	return;
 }
 
-static int 
+static int
 RSCT_setup_evms_plugin(engine_functions_t *func)
 {
 	
@@ -431,7 +431,7 @@ RSCT_setup_evms_plugin(engine_functions_t *func)
 		LOG_EXIT_INT(EINVAL);
 		return(EINVAL);
 	}
-  
+
 	gl_ece_engine_funcs=func;
 	
 	//initialize mutex, lock and others for ack
@@ -449,7 +449,7 @@ RSCT_setup_evms_plugin(engine_functions_t *func)
 	pthread_mutex_init(&gl_rsct_thread_lock,NULL);
 	pthread_cond_init(&gl_rsct_thread_cond,NULL);
 	gl_rsct_thread_activate=1;
-  
+
 	//initialize lock for event queue
 	pthread_mutex_init(&gl_event_queue_lock,NULL);
 	
@@ -461,7 +461,7 @@ RSCT_setup_evms_plugin(engine_functions_t *func)
 	
 	//initialize gl_msg_count_lock
 	pthread_mutex_init(&gl_msg_count_lock,NULL);
-  
+
 	//initialize gl_send_lock and gl_send_func_lock
 	pthread_mutex_init(&gl_send_lock,NULL);
 	pthread_mutex_init(&gl_send_func_lock,NULL);
@@ -501,7 +501,7 @@ RSCT_setup_evms_plugin(engine_functions_t *func)
 	
 	/*initialize membership information*/
 	RSCT_mem_init();
-  
+
 	/* initiazlize internal callback functions*/
 	RSCT_INTERNAL_cb_init();
 	
@@ -527,7 +527,7 @@ RSCT_setup_evms_plugin(engine_functions_t *func)
 	proposal_info.gs_join_request.gs_announcement_callback
 		= RSCT_INTERNAL_announcement_cb;
 	proposal_info.gs_join_request.gs_merge_callback      = NULL;
-  
+
 	group_attributes.gs_version                      = 1;
 	group_attributes.gs_sizeof_group_attributes
 		= sizeof(ha_gs_group_attributes_t);
@@ -542,7 +542,7 @@ RSCT_setup_evms_plugin(engine_functions_t *func)
 	group_attributes.gs_source_reflection_time_limit = 0;
 	group_attributes.gs_group_name                   = GROUP_NAME;
 	group_attributes.gs_source_group_name            = NULL;
-  
+
 	gs_rc = ha_gs_join(&gl_provider_token,
 			   &proposal_info);
 	if(gs_rc != HA_GS_OK) {
@@ -553,7 +553,7 @@ RSCT_setup_evms_plugin(engine_functions_t *func)
 	
 	//subscribe node membership group
 	
-	subscriber_proposal_info =   
+	subscriber_proposal_info =
 		(ha_gs_proposal_info_t*)malloc(sizeof(ha_gs_proposal_info_t));
 	
 	subscriber_proposal_info->gs_subscribe_request.gs_subscription_control
@@ -581,7 +581,7 @@ RSCT_setup_evms_plugin(engine_functions_t *func)
 		local_instance_number = SLAVE_INSTANCE_NUM;
 	ha_gs_get_node_number(&local_node_number);
 	RSCT_mem_get_mynode(&local_nodeid);
-	RSCT_plugin_add(local_node_number, 
+	RSCT_plugin_add(local_node_number,
 			local_instance_number,
 			&local_nodeid);
 	
@@ -593,7 +593,7 @@ RSCT_setup_evms_plugin(engine_functions_t *func)
 	}
 	
 	
-	//spawn a thread to handle membership and message callback for 
+	//spawn a thread to handle membership and message callback for
 	//application
 	if(pthread_create(&gl_appcb_thread, NULL, appcb, NULL)) {
 		LOG_ERROR("appcb thread: Thread_create() error!\n");
@@ -615,8 +615,8 @@ RSCT_setup_evms_plugin(engine_functions_t *func)
 
 
 
-static int 
-RSCT_register_callback(ece_callback_type_t type, 
+static int
+RSCT_register_callback(ece_callback_type_t type,
 		       ece_cb_t cb)
 {
 	
@@ -640,7 +640,7 @@ RSCT_register_callback(ece_callback_type_t type,
 	do{
 		num_entries= RSCT_mem_get_num_config_nodes();
 		size= sizeof(ece_event_t) +sizeof(ece_nodeid_t)*num_entries;
-    
+
 		event=(ece_event_t*) malloc(size);
 		event->num_entries=num_entries;
 		
@@ -651,14 +651,14 @@ RSCT_register_callback(ece_callback_type_t type,
 	
 	leave_event=(ece_event_t*)malloc(sizeof(ece_event_t));
 	
-	real_size = sizeof(ece_event_t) 
+	real_size = sizeof(ece_event_t)
 		+ sizeof(ece_nodeid_t)*event->num_entries;
 	
 	//get quorum info
 	event->quorum_flag= RSCT_mem_has_quorum();
 	leave_event->quorum_flag= RSCT_mem_has_quorum();
 	
-	if(type == DELTAS){ 
+	if(type == DELTAS){
 		//call it twice: one for join with all members
 		//               the other for leave with none
 		
@@ -684,12 +684,12 @@ RSCT_register_callback(ece_callback_type_t type,
 	}
 	
 	
-	//deliver the first callback  
+	//deliver the first callback
 	LOG_EXIT_INT(0);
 	return(0);
 }
 
-static int 
+static int
 RSCT_unregister_callback(ece_cb_t cb)
 {
 	
@@ -716,7 +716,7 @@ RSCT_unregister_callback(ece_cb_t cb)
 
 
 
-static int 
+static int
 RSCT_send_msg(ece_msg_t *ecemsg)
 {
 	
@@ -729,7 +729,7 @@ RSCT_send_msg(ece_msg_t *ecemsg)
 	
 	int                out_instance_number=0;
 	ece_nodeid_t*      out_nodeid;
-  
+
 	LOG_ENTRY();
 	
 	ha_gs_get_node_number(&local_node_number);
@@ -739,7 +739,7 @@ RSCT_send_msg(ece_msg_t *ecemsg)
 	
 	local_instance_number=get_local_instance_number();
 	
-	if(strncmp((char*)&ecemsg->node, 
+	if(strncmp((char*)&ecemsg->node,
 		   (char*)&all_nodes,
 		   sizeof(ece_nodeid_t) )  == 0)
 		isBroadcast=TRUE;
@@ -810,7 +810,7 @@ RSCT_send_msg(ece_msg_t *ecemsg)
 		pthread_mutex_unlock(&gl_ack_lock);
 		LOG_DEBUG("get the ACK!\n");
 		
-	}  
+	}
 	pthread_mutex_unlock(&gl_send_func_lock);
 	
 	LOG_DEBUG("RSCT_send_msg done\n");
@@ -820,7 +820,7 @@ RSCT_send_msg(ece_msg_t *ecemsg)
 
 
 
-static int 
+static int
 RSCT_get_clusterid(ece_clusterid_t *clusterid)
 {	
 	const char* cid=RSCT_mem_get_clusterid();
@@ -828,25 +828,25 @@ RSCT_get_clusterid(ece_clusterid_t *clusterid)
 	LOG_ENTRY();
 	
 	memset(clusterid, 0, sizeof(ece_clusterid_t));
-	strcpy((char *)clusterid, cid);  
+	strcpy((char *)clusterid, cid);
 	
 	LOG_EXIT_INT(0);
 	return(0);
 }
 
-static int 
+static int
 RSCT_get_my_nodeid(ece_nodeid_t *nodeid)
 {	
 	
-	LOG_ENTRY();  
+	LOG_ENTRY();
 	RSCT_mem_get_mynode(nodeid);
 	LOG_EXIT_INT(0);
 	return(0);
 }
 
-static int 
+static int
 RSCT_get_num_config_nodes(uint* num)
-{  
+{
 	int count;
 	
 	LOG_ENTRY();
@@ -862,8 +862,8 @@ RSCT_get_num_config_nodes(uint* num)
 	return(-1);
 }
 
-static int 
-RSCT_get_all_nodes(unsigned int *num_entries, 
+static int
+RSCT_get_all_nodes(unsigned int *num_entries,
 		   ece_nodeid_t *nodes)
 {
 	int count;
@@ -890,7 +890,7 @@ RSCT_get_all_nodes(unsigned int *num_entries,
 	return(0);
 }
 
-static int 
+static int
 RSCT_get_membership(ece_event_t* event)
 {
 	
@@ -903,9 +903,9 @@ RSCT_get_membership(ece_event_t* event)
 	return(ret);
 }
 
-static int 
-RSCT_nodeid_to_string(const ece_nodeid_t *nodeid, 
-		      char *str, 
+static int
+RSCT_nodeid_to_string(const ece_nodeid_t *nodeid,
+		      char *str,
 		      uint *str_len)
 {
 	
@@ -934,7 +934,7 @@ RSCT_nodeid_to_string(const ece_nodeid_t *nodeid,
 	return(0);
 }
 
-static int 
+static int
 RSCT_string_to_nodeid(const char* str,
 		      ece_nodeid_t* nodeid)
 {
@@ -953,7 +953,7 @@ RSCT_string_to_nodeid(const char* str,
 	return(0);
 }
 
-static int 
+static int
 RSCT_get_plugin_info(char  * info_name,
 		     extended_info_array_t** info_array){
 	
@@ -970,71 +970,69 @@ RSCT_get_plugin_info(char  * info_name,
 	}
 	
 	if (!info_name) {
-		if (!(info = ENGINE_ALLOC(sizeof(extended_info_array_t) + 
-					  sizeof(extended_info_t)*5))) {
+		if (!(info = ENGINE_ALLOC(sizeof(extended_info_array_t) +
+					  sizeof(extended_info_t)*6))) {
 			LOG_ERROR("Error allocating memory for info array\n");
 			LOG_EXIT_INT(ENOMEM);
 			return(ENOMEM);
 		}
 		
-		SET_STRING(info->info[i].name, "ShortName");
-		SET_STRING(info->info[i].title, "Short Name");
-		SET_STRING(info->info[i].desc, "A short name given to this plugin");
+		info->info[i].name = EngFncs->engine_strdup("ShortName");
+		info->info[i].title = EngFncs->engine_strdup(_("Short Name"));
+		info->info[i].desc = EngFncs->engine_strdup(_("A short name given to this plug-in"));
 		info->info[i].type = EVMS_Type_String;
-		SET_STRING(info->info[i].value.s, ece_plugin_record.short_name);
+		info->info[i].value.s = EngFncs->engine_strdup(ece_plugin_record.short_name);
 		i++;
 		
-		SET_STRING(info->info[i].name, "LongName");
-		SET_STRING(info->info[i].title, "Long Name");
-		SET_STRING(info->info[i].desc, "A long name given to this plugin");
+		info->info[i].name = EngFncs->engine_strdup("LongName");
+		info->info[i].title = EngFncs->engine_strdup(_("Long Name"));
+		info->info[i].desc = EngFncs->engine_strdup(_("A longer, more descriptive name for this plug-in"));
 		info->info[i].type = EVMS_Type_String;
-		SET_STRING(info->info[i].value.s, ece_plugin_record.long_name);
+		info->info[i].value.s = EngFncs->engine_strdup(ece_plugin_record.long_name);
 		i++;
 		
-		SET_STRING(info->info[i].name, "Type");
-		SET_STRING(info->info[i].title, "Plugin Type");
-		SET_STRING(info->info[i].desc, "There are various "
-	       "types of plugins; each responsible for "
-			   "some kind of storage object.");
+		info->info[i].name = EngFncs->engine_strdup("Type");
+		info->info[i].title = EngFncs->engine_strdup(_("Plug-in Type"));
+		info->info[i].desc =
+			EngFncs->engine_strdup("There are various types of plug-ins, each responsible for some kind of storage object or logical volume."));
 		info->info[i].type = EVMS_Type_String;
-		SET_STRING(info->info[i].value.s, "Cluster Manager");
+		info->info[i].value.s = EngFncs->engine_strdup(_("Cluster Manager"));
 		i++;
 		
-		SET_STRING(info->info[i].name, "Version");
-		SET_STRING(info->info[i].title, "Plugin Version");
-		SET_STRING(info->info[i].desc, 
-			   "This is the version number of the plugin.");
+		info->info[i].name = EngFncs->engine_strdup("Version");
+		info->info[i].title = EngFncs->engine_strdup(_("Plug-in Version"));
+		info->info[i].desc =
+			EngFncs->engine_strdup(_("This is the version number of the plug-in."));
 		info->info[i].type = EVMS_Type_String;
-		snprintf(buffer, 50, "%d.%d.%d", MAJOR_VERSION, 
+		snprintf(buffer, 50, "%d.%d.%d", MAJOR_VERSION,
 			 MINOR_VERSION, PATCH_LEVEL);
-		SET_STRING(info->info[i].value.s, buffer);
+		info->info[i].value.s = EngFncs->engine_strdup(buffer);
 		i++;
 		
-		SET_STRING(info->info[i].name, "Required_Engine_Version");
-		SET_STRING(info->info[i].title, "Required Engine Services Version");
-		SET_STRING(info->info[i].desc, "This is the version of the Engine "
-			   "services that this plug-in requires.  It will not "
-			   "run on older versions of the Engine services.");
+		info->info[i].name = EngFncs->engine_strdup("Required_Engine_Version");
+		info->info[i].title = EngFncs->engine_strdup(_("Required Engine Services Version"));
+		info->info[i].desc =
+			EngFncs->engine_strdup(_("This is the version of the Engine services that this plug-in requires.  "
+						 "It will not run on older versions of the Engine services."));
 		info->info[i].type = EVMS_Type_String;
-		snprintf(buffer, 50, "%d.%d.%d", 
-			 ece_plugin_record.required_engine_api_version.major, 
-			 ece_plugin_record.required_engine_api_version.minor, 
+		snprintf(buffer, 50, "%d.%d.%d",
+			 ece_plugin_record.required_engine_api_version.major,
+			 ece_plugin_record.required_engine_api_version.minor,
 			 ece_plugin_record.required_engine_api_version.patchlevel);
-		SET_STRING(info->info[i].value.s, buffer);
+		info->info[i].value.s = EngFncs->engine_strdup(buffer);
 		i++;
 		
-		SET_STRING(info->info[i].name, "Required_Cluster_Version");
-		SET_STRING(info->info[i].title, "Required Cluster API Version");
-		SET_STRING(info->info[i].desc, "This is the version of the "
-			   "Engine's cluster plug-in API that this plug-in "
-			   "requires.  It will not run on older versions of "
-			   "the Engine's cluster plug-in API.");
+		info->info[i].name = EngFncs->engine_strdup("Required_Cluster_Version");
+		info->info[i].title = EngFncs->engine_strdup(_("Required Cluster API Version"));
+		info->info[i].desc =
+			EngFncs->engine_strdup(_("This is the version of the Engine's cluster plug-in API that this plug-in requires.  "
+						 "It will not run on older versions of the Engine's cluster plug-in API."));
 		info->info[i].type = EVMS_Type_String;
-		snprintf(buffer, 50, "%d.%d.%d", 
-			 ece_plugin_record.required_plugin_api_version.cluster.major, 
-			 ece_plugin_record.required_plugin_api_version.cluster.minor, 
+		snprintf(buffer, 50, "%d.%d.%d",
+			 ece_plugin_record.required_plugin_api_version.cluster.major,
+			 ece_plugin_record.required_plugin_api_version.cluster.minor,
 			 ece_plugin_record.required_plugin_api_version.cluster.patchlevel);
-		SET_STRING(info->info[i].value.s, buffer);
+		info->info[i].value.s = EngFncs->engine_strdup(buffer);
 		i++;
 	} else {
 		LOG_ERROR("No support for extra plugin information about \"%s\"\n", info_name);
@@ -1049,7 +1047,7 @@ RSCT_get_plugin_info(char  * info_name,
 	
 }
 
-static int 
+static int
 RSCT_get_plugin_functions(ece_nodeid_t * nodeid,
 			  function_info_array_t** actions)
 {
@@ -1062,12 +1060,12 @@ RSCT_get_plugin_functions(ece_nodeid_t * nodeid,
 }
 
 
-static int 
+static int
 RSCT_plugin_function(ece_nodeid_t * nodeid,
 		     task_action_t  action,
 		     list_anchor_t  objects,
 		     option_array_t * options)
-{ 
+{
 	
 	/* not sure what to fill here TOBEDONE*/
 	LOG_ENTRY();
@@ -1098,15 +1096,15 @@ static cluster_functions_t ft={
 
 plugin_record_t ece_plugin_record = {
 	
-	id:			SetPluginID(EVMS_OEM_IBM, 
-					    EVMS_CLUSTER_MANAGER_INTERFACE_MODULE, 
+	id:			SetPluginID(EVMS_OEM_IBM,
+					    EVMS_CLUSTER_MANAGER_INTERFACE_MODULE,
 					    2),
 	
 	version:		{major:      MAJOR_VERSION,
 				 minor:      MINOR_VERSION,
 				 patchlevel: PATCH_LEVEL},
 	
-	required_engine_api_version: {major:      13,
+	required_engine_api_version: {major:      15,
 				      minor:      0,
 				      patchlevel: 0},
 	
