@@ -255,7 +255,7 @@ send_back_ack(ece_nodeid_t* sender_id,int msg_count)
 	proposal_info.gs_message_request.gs_num_phases         = HA_GS_1_PHASE;
 	proposal_info.gs_message_request.gs_time_limit         = TIME_LIMIT;
 	proposal_info.gs_message_request.gs_message.gs_length  = sizeof(rsct_msg_t);
-	proposal_info.gs_message_request.gs_message.gs_message = (char*)&ack_rsctmsg;;
+	proposal_info.gs_message_request.gs_message.gs_message = (char*)&ack_rsctmsg;
 	
 	
 	pthread_mutex_lock(&gl_send_lock);
