@@ -311,9 +311,9 @@ static int S390_CanDestroy( storage_object_t * seg )
  *  (4) a freespace segment immediately follows it
  *
  */
-static int S390_CanExpand( storage_object_t    *seg,        
-                           sector_count_t      *expand_limit,      
-                           list_anchor_t        expansion_points ) 
+static int S390_CanExpand( storage_object_t    *seg,
+                           sector_count_t       expand_limit,
+                           list_anchor_t        expansion_points )
 {
         DISKSEG              *freespace;
         LOGICALDISK          *ld;
@@ -342,13 +342,14 @@ static int S390_CanExpand( storage_object_t    *seg,
                                 cylinder_size = get_cylinder_size(ld);
 
                                 // we can only expand in cylinder size amounts
-                                if ( freespace->size >= cylinder_size ) {
+                                if ( freespace->size >= cylinder_size &&
+                                     expand_limit >= cylinder_size ) {
 
                                         expand_object = (expand_object_info_t *) EngFncs->engine_alloc( sizeof(expand_object_info_t) );
                                         if (expand_object) {
 
                                                 expand_object->object          = seg;
-                                                expand_object->max_expand_size = freespace->size;
+                                                expand_object->max_expand_size = min(freespace->size, expand_limit);
 
                                                 e=EngFncs->insert_thing( expansion_points,
                                                                          expand_object,
@@ -359,7 +360,7 @@ static int S390_CanExpand( storage_object_t    *seg,
                                                 }
                                                 else {
                                                         EngFncs->engine_free( expand_object );
-                                                        rc = EPERM; 
+                                                        rc = EPERM;
                                                 }
 
                                         }
@@ -484,7 +485,7 @@ static int S390_CanExpandBy(storage_object_t * seg, sector_count_t *size)
  *  If not exact set new_size to closest higher value possible.
  */
 static int S390_CanShrink( storage_object_t * seg,       // object to shrink
-                           sector_count_t   * shrink_limit,  // a delta size
+                           sector_count_t     shrink_limit,  // a delta size
                            list_anchor_t            shrink_points ) // of type shrink_object_info_t
 {
         int                   rc = EINVAL;
@@ -512,7 +513,8 @@ static int S390_CanShrink( storage_object_t * seg,       // object to shrink
                                 cylinder_size = get_cylinder_size(ld);
 
                                 // see if caller is trying to shrink smaller than min partition size
-                                if ( seg->size > cylinder_size ) {
+                                if ( seg->size > cylinder_size &&
+                                     shrink_limit >= cylinder_size ) {
 
                                         if (shrink_points) {
 
@@ -523,7 +525,7 @@ static int S390_CanShrink( storage_object_t * seg,       // object to shrink
                                                         // minimum partition size is 1 cylinder ... then anything more than
                                                         // a cylinder is the max we can shrink.
                                                         shrink_object->object          = seg;
-                                                        shrink_object->max_shrink_size = seg->size - cylinder_size;
+                                                        shrink_object->max_shrink_size = min(seg->size - cylinder_size, shrink_limit);
 
                                                         e=EngFncs->insert_thing( shrink_points,
                                                                                  shrink_object,
@@ -534,7 +536,7 @@ static int S390_CanShrink( storage_object_t * seg,       // object to shrink
                                                         }
                                                         else {
                                                                 EngFncs->engine_free( shrink_object );
-                                                                rc = EPERM; 
+                                                                rc = EPERM;
                                                         }
 
                                                 }
@@ -653,7 +655,7 @@ static DISKSEG * create_metadata_segment( list_anchor_t   output_list,
         if (metadata) {
 
                 if ( insert_s390_segment_into_list( ld->parent_objects, metadata ) != NULL ) {
-                        
+
                         e = EngFncs->insert_thing( output_list,
                                                    metadata,
                                                    INSERT_AFTER,
@@ -1225,10 +1227,10 @@ static int get_devmap_info( DISKSEG *seg )
 		goto out;
 	}
 
+	/* Everything matches. We can copy the segment name to the dev_name. */
+	strncpy(seg->dev_name, seg->name, EVMS_NAME_SIZE);
+
 out:
-	if (rc) {
-		seg->flags |= SOFLAG_NEEDS_ACTIVATE;
-	}
 	EngFncs->dm_deallocate_targets(targets);
         LOG_EXIT_INT(0);
         return 0;
@@ -1405,7 +1407,7 @@ static int CreateDiskSegment( storage_object_t      * freespace,            /* a
          */
         if ( disk_pdata->partition_count >= 3 ) {
 
-                char * choices[] = {"Yes", "No", NULL};
+                char * choices[] = {_("Yes"), _("No"), NULL};
                 int answer = 0;
 
                 QUESTION( &answer, choices,
@@ -1525,9 +1527,6 @@ static int CreateDiskSegment( storage_object_t      * freespace,            /* a
                         rc = init_format1_dscb( ld, seg );
                         if (rc == 0) {
 
-                                // mark seg dirty for commit and needs DM activate
-                                seg->flags |= (SOFLAG_DIRTY | SOFLAG_NEEDS_ACTIVATE);
-
                                 if ( insert_s390_segment_into_list( ld->parent_objects, seg ) == NULL ) {
 
                                         LOG_ERROR("error, some kind of list insert error");
@@ -1597,7 +1596,7 @@ static int GetCreateOptions(  storage_object_t  * seg,
                 if (options->option[i].is_number_based) {
 
                         switch (options->option[i].number) {
-                        
+
                         case SEG_CREATE_OPTION_SIZE_INDEX:
                                 *size = options->option[i].value.ui64;
                                 break;
@@ -1971,6 +1970,15 @@ static int S390_Expand( storage_object_t *seg, storage_object_t *expand_object,
                                 // calc actual expand sector count
                                 expand_sectors = end_lba - freespace->start + 1;
 
+				// ask the engine if this expand amount is ok.
+				rc = EngFncs->can_expand_by(seg, &expand_sectors);
+				if (rc) {
+					LOG_ERROR("Shrink of segment %s rejected by "
+						  "the engine.\n", seg->name);
+					LOG_EXIT_INT(rc);
+					return rc;
+				}
+
                                 // expand the data segment
                                 old_seg_size = seg->size;
                                 seg->size   += expand_sectors;
@@ -2103,6 +2111,15 @@ static int S390_Shrink( storage_object_t * seg,
                                 shrink_sector_count = (shrink_sector_count/cylinder_size)*cylinder_size;
                         }
 
+			// ask the engine if it's ok to shrink by this amount.
+			rc = EngFncs->can_shrink_by(seg, &shrink_sector_count);
+			if (rc) {
+				LOG_ERROR("Shrink of segment %s rejected by "
+					  "the engine.\n", seg->name);
+				LOG_EXIT_INT(rc);
+				return rc;
+			}
+
                         // resulting seg size
                         new_seg_size = seg->size - shrink_sector_count;
 
@@ -2208,20 +2225,9 @@ static int S390_CommitChanges( storage_object_t *object, uint phase )
                                                 }
                                                 disk_pdata->flags &= ~DISK_HAS_FORMAT_PENDING;
                                         }
-                                }
-                                else {
-                                        LIST_FOR_EACH(ld->parent_objects, iter, seg ){
-                                                if (seg->data_type == DATA_TYPE) {
-                                                        pdata = (SEG_PRIVATE_DATA *)seg->private_data;
-                                                        if (pdata) {
-                                                                if (pdata->cflags & SEG_NEEDS_DM_RENAME) {
-                                                                        s390_activate(seg);
-                                                                }
-                                                        }
-                                                }
-                                        }
-                                        rc = 0;
-                                }
+                                } else {
+					rc = 0;
+				}
                         }
                         else if (phase == MOVE) {
 
@@ -2243,7 +2249,7 @@ static int S390_CommitChanges( storage_object_t *object, uint phase )
                                   !(disk_pdata->flags & DISK_HAS_MOVE_PENDING)) {
 
                                 if ( disk_pdata->flags & DISK_HAS_CHANGES_PENDING ) {
-                                        rc = commit_390_partition_table( ld, phase );
+                                        rc = commit_390_partition_table( ld, object, phase, FALSE );
                                         if (!rc) {
                                                 LIST_FOR_EACH( ld->parent_objects, iter, seg ){
                                                         seg->flags &= ~SOFLAG_DIRTY;
@@ -2269,6 +2275,21 @@ static int S390_CommitChanges( storage_object_t *object, uint phase )
 }
 
 
+static int s390_backup_metadata( storage_object_t *seg )
+{
+        LOGICALDISK *ld = get_logical_disk(seg);
+	int rc = 0;
+
+	LOG_ENTRY();
+
+	if (seg->data_type != FREE_SPACE_TYPE) {
+		rc = commit_390_partition_table( ld, seg, 1, TRUE );
+	}
+
+	LOG_EXIT_INT(rc);
+	return rc;
+}
+
 static int S390_Read( storage_object_t  *seg,
                       lsn_t              offset,
                       sector_count_t     count,
@@ -2348,7 +2369,7 @@ static int GetFormatOptions( option_array_t * options, ibm_label_type_t *disk_la
                 if (options->option[i].is_number_based) {
 
                         switch (options->option[i].number) {
-                        
+
                         case SEG_FORMAT_OPTION_TYPENAME_INDEX:
                                 if ( strstr( options->option[i].value.s, "CDL") ) {
                                         *disk_layout = ibm_cdl;
@@ -2458,7 +2479,7 @@ static int S390_Format( storage_object_t * input_object,
                                 else {
                                         LOG_ERROR("error, unable to create segment for %s\n", ld->name );
                                         rc = ENOMEM;
-                                }                               
+                                }
 
                         }
 
@@ -2491,7 +2512,7 @@ static  int S390_PluginPrivateFunctions( storage_object_t * object,
 {
         int rc=EINVAL;
         DISKSEG *freespace=NULL;
-       
+
 
         LOG_ENTRY();
 
@@ -2533,7 +2554,7 @@ static int S390_GetPluginPrivateFunctions( storage_object_t        * object,
 
         LOG_ENTRY();
 
-        func_info = EngFncs->engine_alloc( sizeof(function_info_array_t) + (sizeof(function_info_t)* 3) );
+        func_info = EngFncs->engine_alloc( sizeof(function_info_array_t) + (sizeof(function_info_t)* 2) );
         if (func_info) {
 
                 func_info->count = 0;
@@ -2545,11 +2566,11 @@ static int S390_GetPluginPrivateFunctions( storage_object_t        * object,
 
                         func_info->info[index].function = EVMS_Task_Format_Disk;
 
-                        sprintf(title,"Format Disk %s", ld->name );
-                        SET_STRING( func_info->info[index].title, title );
-                        SET_STRING( func_info->info[index].verb,  "Format" );
-                        SET_STRING( func_info->info[index].name, "Format" );
-                        SET_STRING( func_info->info[index].help,  "Use this function to format a dasd device prior to use." );
+                        sprintf(title, _("Format Disk %s"), ld->name );
+                        func_info->info[index].title = EngFncs->engine_strdup( title );
+                        func_info->info[index].verb = EngFncs->engine_strdup( _("Format") );
+                        func_info->info[index].name = EngFncs->engine_strdup( _("Format") );
+                        func_info->info[index].help = EngFncs->engine_strdup( _("Use this function to format a dasd device prior to use.") );
 
                         ++func_info->count;
                         ++index;
@@ -2559,10 +2580,10 @@ static int S390_GetPluginPrivateFunctions( storage_object_t        * object,
 
                         func_info->info[index].function = EVMS_Task_Move_Segment;
 
-                        SET_STRING( func_info->info[index].title, "Move" );
-                        SET_STRING( func_info->info[index].verb,  "Move" );
-                        SET_STRING( func_info->info[index].name, "Move" );
-                        SET_STRING( func_info->info[index].help,  "Use this function to move a data segment." );
+                        func_info->info[index].title = EngFncs->engine_strdup( "Move" );
+                        func_info->info[index].verb = EngFncs->engine_strdup( _("Move") );
+                        func_info->info[index].name = EngFncs->engine_strdup( _("Move") );
+                        func_info->info[index].help = EngFncs->engine_strdup( _("Use this function to move a data segment.") );
 
                         ++func_info->count;
                         ++index;
@@ -2621,8 +2642,11 @@ static struct plugin_functions_s sft={
         get_plugin_info:                     S390_GetPluginInfo,
         get_plugin_functions:                S390_GetPluginPrivateFunctions,
         plugin_function:                     S390_PluginPrivateFunctions,
+        can_activate:                        s390_can_activate,
         activate:                            s390_activate,
-        deactivate:                          s390_deactivate
+        can_deactivate:                      s390_can_deactivate,
+        deactivate:                          s390_deactivate,
+        backup_metadata:                     s390_backup_metadata,
 };
 
 /*-------------------------------------------------------------------------------------+
@@ -2637,8 +2661,8 @@ static plugin_record_t plugin_record = {
 
         version:                          {MAJOR_VERSION, MINOR_VERSION, PATCH_LEVEL},
 
-        required_engine_api_version:      {13, 0, 0},
-        required_plugin_api_version:      {plugin: {12, 0, 0}},
+        required_engine_api_version:      {15, 0, 0},
+        required_plugin_api_version:      {plugin: {13, 1, 0}},
 
         short_name:                       "S390SegMgr",
         long_name:                        "S390 Segment Manager",
