@@ -165,22 +165,16 @@ static int update_logical_disk_info( LOGICALDISK *ld, int fd )
 
 #ifdef BLKGETSIZE64
 	rc = EngFncs->ioctl_object(ld,fd,BLKGETSIZE64,&size);
+	size >>= EVMS_VSECTOR_SIZE_SHIFT;  // This size is in bytes.
 #else
-	rc = EngFncs->ioctl_object(ld,fd,BLKGETSIZE,&size);
+	rc = EngFncs->ioctl_object(ld,fd,BLKGETSIZE,&size); // This size is in sectors.
 #endif
 	if (!rc) {
-		size >>= EVMS_VSECTOR_SIZE_SHIFT;
-
 		rc = EngFncs->ioctl_object(ld,fd,HDIO_GETGEO,&geo);
 		if (!rc) {
 
 			rc = EngFncs->ioctl_object(ld,fd,BLKSSZGET,&hardsector_size);
-			if (!rc) {
-				size >>= EVMS_VSECTOR_SIZE_SHIFT;
-			}
-
 		}
-
 	}
 
 	// Note ... a valid disk object starts at 0 and has some size
@@ -302,20 +296,22 @@ static int llformat( LOGICALDISK *ld, int blocksize,  int intensity )
 
 	// setup evms progress indicator
 	sprintf(title, "Formatting %d cylinders on disk ... %s\n", cylinder_count, ld->name );
-	progress.total_count = cylinder_count;
 	progress.id = 0;
 	progress.title = title;
 	progress.description = "";
 	progress.type = DISPLAY_PERCENT;
 	progress.count = 0;
+	progress.total_count = cylinder_count;
 	progress.remaining_seconds = 0;
 	progress.plugin_private_data = NULL;
 	progress.ui_private_data = NULL;
+	progress.timer_data = NULL;
+	progress.flags = 0;
 
 	use_progress = EngFncs->progress(&progress) == 0;
 
 	if (!use_progress) {
-		MESSAGE("FORMATTING %d CYLINDERS ... \n", cylinder_count);
+		MESSAGE(_("Formatting %d cylinders ... \n"), cylinder_count);
 	}
 
 	// format 1 cylinder at a time
@@ -331,7 +327,8 @@ static int llformat( LOGICALDISK *ld, int blocksize,  int intensity )
 
 		rc = EngFncs->ioctl_object(ld,fd, BIODASDFMT, (void *) &format_packet);
 		if (rc) {
-			MESSAGE("error, dasd driver returned error (%s) while formating tracks.\n", strerror(errno) );
+			MESSAGE(_("Error:  DASD driver returned error (%s) while formating tracks.\n"),
+				strerror(errno) );
 		}
 		else {
 
@@ -343,15 +340,15 @@ static int llformat( LOGICALDISK *ld, int blocksize,  int intensity )
 			}
 			else {
 				if ( i >= (cylinder_count/4) && quarter_way == FALSE ) {
-					MESSAGE(".... 1/4 done\n");
+					MESSAGE(_(".... 1/4 done\n"));
 					quarter_way = TRUE;
 				}
 				else if ( i >= (cylinder_count/2) && half_way == FALSE ) {
-					MESSAGE(".... 1/2 done\n");
+					MESSAGE(_(".... 1/2 done\n"));
 					half_way = TRUE;
 				}
 				else if ( i >= (cylinder_count-(cylinder_count/4)) && three_quarters == FALSE ) {
-					MESSAGE(".... 3/4 done\n");
+					MESSAGE(_(".... 3/4 done\n"));
 					three_quarters = TRUE;
 				}
 			}
@@ -368,12 +365,12 @@ static int llformat( LOGICALDISK *ld, int blocksize,  int intensity )
 	}
 
 	if (use_progress) {
-		progress.title = "Finished.";
+		progress.title = "Finished";
 		progress.count = progress.total_count;
 		EngFncs->progress(&progress);
 	}
 	else {
-		MESSAGE("FINISHED!\n\n");
+		MESSAGE(_("Finished\n"));
 	}
 
 
