@@ -354,7 +354,7 @@ DISKSEG *  allocate_s390_disk_segment( LOGICALDISK *ld )
                         }
                 }
                 else {
-                        rc = EPERM;             
+                        rc = EPERM;
                 }
 
         }
@@ -541,7 +541,7 @@ DISKSEG * get_freespace_following_s390_segment(DISKSEG *seg)
         list_element_t  iter;
 
         // get logical disk and the entire segment list for the disk
-        ld = get_logical_disk( seg );   
+        ld = get_logical_disk( seg );
         if ( ld ) {
 
                 prev = EngFncs->first_thing( ld->parent_objects, &iter );
@@ -682,7 +682,7 @@ int  find_freespace_on_s390_disk( LOGICALDISK *ld )
                                         free_s390_disk_segment( seg );
                                         rc = ENOMEM;
                                         LOG_EXIT_INT(rc);
-                                        return rc;                                      
+                                        return rc;
                                 }
 
                         }
@@ -789,7 +789,7 @@ static int  merge_freespace_segments( list_anchor_t seglist )
                                         }
 
                                         if (get_freespace_number(prev) > get_freespace_number(this) ) {
-                                                EngFncs->remove_thing( seglist, prev );                                         
+                                                EngFncs->remove_thing( seglist, prev );
                                                 this->start -= prev->size;
                                                 this->size  += prev->size;
                                                 free_s390_disk_segment( prev );
@@ -797,7 +797,7 @@ static int  merge_freespace_segments( list_anchor_t seglist )
                                                 return 0;
                                         }
                                         else {
-                                                EngFncs->remove_thing( seglist, this );                                         
+                                                EngFncs->remove_thing( seglist, this );
                                                 prev->size  += this->size;
                                                 free_s390_disk_segment( this );
                                                 LOG_DEBUG("        kept seg: %s  start: %"PRIu64"  size: %"PRIu64"\n", prev->name, prev->start, prev->size );
@@ -806,7 +806,7 @@ static int  merge_freespace_segments( list_anchor_t seglist )
 
                                 }
 
-                        } while (this != NULL);  
+                        } while (this != NULL);
 
                 }
         }
@@ -955,12 +955,12 @@ void *  insert_s390_segment_into_ordered_list( list_anchor_t seglist, storage_ob
 
 
         switch (rc) {
-        
+
         case 0:  /* Ok, found a segment we should insert in front of */
-                e = EngFncs->insert_thing( seglist, 
+                e = EngFncs->insert_thing( seglist,
                                            seg,
                                            INSERT_BEFORE|EXCLUSIVE_INSERT,
-                                           EngFncs->find_in_list(seglist, seg2, NULL) );
+                                           EngFncs->find_in_list(seglist, seg2, NULL, NULL) );
                 if (e != NULL) {
                         rc = 0;
                 }
@@ -972,7 +972,7 @@ void *  insert_s390_segment_into_ordered_list( list_anchor_t seglist, storage_ob
         case -1: /* This new segment lays out on the disk at a higher */
                 /* LBA than any other existing segment.  So, just    */
                 /* insert at end of the segment list.                */
-                e = EngFncs->insert_thing( seglist, 
+                e = EngFncs->insert_thing( seglist,
                                            seg,
                                            INSERT_AFTER|EXCLUSIVE_INSERT,
                                            NULL );
@@ -1282,13 +1282,13 @@ int   fixup_390_partition_names( LOGICALDISK *ld )
                         // build new evms segment name
                         get_name_for_disk_segment( seg );
 
-                        // test if name has changed and then mark the object so we
-                        // can do a dm rename of the kernel object at commit time.
+                        // If the name has changed and the segment is active, we
+                        // need to re-activate it so it can get renamed in DM.
                         if (  (seg->flags & SOFLAG_ACTIVE) &&
                               strlen(old_name) > 0  &&
                               strncmp(seg->name, old_name, EVMS_NAME_SIZE)!=0) {
 
-                                s390_schedule_dm_rename(old_name,seg); 
+                                seg->flags |= SOFLAG_NEEDS_ACTIVATE;
 
                         }
 
@@ -1337,7 +1337,7 @@ boolean i_can_format_disk( LOGICALDISK *ld )
                                 return FALSE;
                         }
                 }
-                LOG_EXIT_BOOL(FALSE);
+                LOG_EXIT_BOOL(TRUE);
                 return TRUE;
         }
 
