@@ -196,7 +196,7 @@ static int create_move_target( DISKSEG  *seg,        // source data segment
 
                 rc = EFBIG;
 
-                // calculate the starting lba                                                           
+                // calculate the starting lba
                 if (starts_on_cylinder_boundary(ld,seg->start)==TRUE) {
                         if (starts_on_cylinder_boundary(ld,freespace->start)==TRUE) {
                                 start = freespace->start;
@@ -338,13 +338,13 @@ int s390_can_move_segment( DISKSEG *seg )
  *  table metadata info. This means we need to update
  *  the source segment info, i.e. where it is now located
  *  on the disk.
- */        
+ */
 static int s390_update_segment_info( DISKSEG *src, DISKSEG *trg)
 {
-        int rc=EINVAL;             
+        int rc=EINVAL;
         LOGICALDISK *ld=NULL;
 
-        LOG_ENTRY(); 
+        LOG_ENTRY();
 
         if (src && trg) {
 
@@ -436,7 +436,7 @@ int s390_move_segment( DISKSEG *src, DISKSEG *freespace )
  *  Called after a move to update partition tables.
  */
 static int do_move_segment_finish( DISKSEG      *src,
-                                   DISKSEG      *trg, 
+                                   DISKSEG      *trg,
                                    int           copy_rc,
                                    boolean       offline )
 {
@@ -448,28 +448,28 @@ static int do_move_segment_finish( DISKSEG      *src,
         SEG_PRIVATE_DATA    saved_pdata;
         DISKSEG             saved_seg;
 
-        LOG_ENTRY(); 
+        LOG_ENTRY();
 
         if (src && trg) {
 
                 ld         = get_logical_disk(src);
-                disk_pdata = get_s390_disk_private_data(ld);    
+                disk_pdata = get_s390_disk_private_data(ld);
                 src_pdata  = (SEG_PRIVATE_DATA *)src->private_data;
-                trg_pdata  = (SEG_PRIVATE_DATA *)trg->private_data;        
+                trg_pdata  = (SEG_PRIVATE_DATA *)trg->private_data;
 
                 if (copy_rc==0) {
                         memcpy(&saved_seg, src, sizeof(DISKSEG));
                         memcpy(&saved_pdata, src_pdata, sizeof(SEG_PRIVATE_DATA));
-                        rc = s390_update_segment_info(src,trg);                       
+                        rc = s390_update_segment_info(src,trg);
                         if (rc==0) {
-                                rc = commit_390_partition_table(ld,FIRST_METADATA_WRITE);
+                                rc = commit_390_partition_table(ld,trg,FIRST_METADATA_WRITE,FALSE);
                         }
                         // if any errors ... restore src seg and recommit vtoc in case
                         // it was left in inconsistant state.
                         if (rc) {
                                 memcpy(src, &saved_seg, sizeof(DISKSEG));
                                 memcpy(src_pdata, &saved_pdata, sizeof(SEG_PRIVATE_DATA));
-                                commit_390_partition_table(ld,FIRST_METADATA_WRITE);
+                                commit_390_partition_table(ld,src,FIRST_METADATA_WRITE,FALSE);
                         }
                 }
                 else {
@@ -477,9 +477,9 @@ static int do_move_segment_finish( DISKSEG      *src,
                 }
 
                 free_s390_disk_segment( trg );
-                find_freespace_on_s390_disk(ld); 
+                find_freespace_on_s390_disk(ld);
                 src_pdata->move_target = NULL;
-                src->flags |= SOFLAG_NEEDS_ACTIVATE; 
+                src->flags |= SOFLAG_NEEDS_ACTIVATE;
                 s390_activate(src);
 
         }
@@ -529,20 +529,20 @@ static int do_online_copy(storage_object_t *src, storage_object_t *tgt, copy_job
                                 EngFncs->dm_suspend(src, FALSE);
                         }
                         else {
-                                LOG_SERIOUS("Error code %d when resuming object %s: %s\n", 
+                                LOG_SERIOUS("Error code %d when resuming object %s: %s\n",
                                             rc, src->name, EngFncs->strerror(rc));
                         }
 
                         EngFncs->dm_set_suspended_flag(FALSE);
                 }
                 else {
-                        LOG_SERIOUS("Error code %d when loading the new mirror target for the object %s: %s\n", 
+                        LOG_SERIOUS("Error code %d when loading the new mirror target for the object %s: %s\n",
                                     rc, src->name, EngFncs->strerror(rc));
                 }
 
         }
         else {
-                LOG_SERIOUS("Error code %d when getting the target for the object %s: %s\n", 
+                LOG_SERIOUS("Error code %d when getting the target for the object %s: %s\n",
                             rc, src->name, EngFncs->strerror(rc));
         }
 
@@ -585,9 +585,9 @@ static int do_offline_copy( storage_object_t *src, storage_object_t *tgt, copy_j
  */
 int s390_move_segment_commit( DISKSEG *src, DISKSEG *tgt, copy_job_t *job )
 {
-        int rc=0;       
-        char * choices[] = {"Yes", "No", NULL};        
-        int answer = 0;    
+        int rc=0;
+        char * choices[] = {_("Yes"), _("No"), NULL};
+        int answer = 0;
         logical_volume_t *vol=NULL;
 
         LOG_ENTRY();
