@@ -22,9 +22,13 @@
 extern int get_snapshot_metadata(storage_object_t * snap_child,
 				 snapshot_metadata_t ** metadata);
 extern storage_object_t * find_origin(char * origin_name);
-extern int write_snapshot_metadata(snapshot_volume_t * snap_volume);
+extern int write_snapshot_metadata(snapshot_volume_t * snap_volume,
+				   boolean backup);
 extern void erase_snapshot_metadata(storage_object_t * snap_child);
-extern int check_snapshot_header(snapshot_volume_t * snap_volume);
+extern int erase_snapshot_header(snapshot_volume_t * snap_volume,
+				 boolean backup);
+extern void get_snapshot_state(snapshot_volume_t * snap_volume);
+extern void get_origin_state(snapshot_volume_t * org_volume);
 
 #endif
 
