@@ -160,21 +160,26 @@ static int template_can_unassign(storage_object_t *object)
  * template_can_expand
  *
  * Can you expand this object?  If yes, build an expand_object_info_t and add
- * it to the expand_points list.  If you can't expand directly, but can allow
- * one of your child objects to expand, call can_expand on whichever child you
- * will allow to expand.  If you can not handle expanding below you, do not
- * pass the command down to your child.
+ * it to the expand_points list. The specified expand_limit is the maximum
+ * change that is allowed for this object. If you cannot expand by an amount
+ * less than or equal to this limit, then your object should not be added to
+ * the expand_points list.
+ *
+ * If you can't expand directly, but can allow one of your child objects to
+ * expand, call can_expand on whichever child you will allow to expand. If you
+ * can not handle expanding below you, do not pass the command down to your
+ * child.
  *
  * Return:
- * - 0 for yes. If expand_limit is greater than the maximum *change* in size
- *   you will allow, then set it to that maximum delta-value. Create an
- *   expand_object_info_t and add to the expand_points.
- * - Non-zero error-code for no.
+ * - Return 0, even if you can't expand and didn't add anything to the list.
+ *   The engine will determine success or failure based on the contents of the
+ *   expand_points list.  Return non-zero if something went wrong in the
+ *   processing of the command, such as a failure to allocate memory.
  *
  * OPTIONAL
  **/
 static int template_can_expand(storage_object_t *object,
-			       sector_count_t *expand_limit,	/* a delta size */
+			       sector_count_t expand_limit,	/* a delta size */
 			       list_anchor_t expand_points)	/* of type expand_object_info_t */
 {
 	LOG_ENTRY();
@@ -209,20 +214,26 @@ static int template_can_expand_by(storage_object_t *object,
  * template_can_shrink
  *
  * Can you shrink this object?  If yes, build a shrink_object_info_t and add
- * it to the shrink_points list.  If you can't shrink directly, but can allow
- * one of your child objects to shrink, call can_shrink on whichever child you
- * will allow to shrink.  If you can not handle shrinking below you, do not
- * pass the command down to your child.
+ * it to the shrink_points list. The specified shrink_limit is the maximum
+ * change that is allowed for this object. If you cannot shrink by an amount
+ * less than or equal to this limit, then your object should not be added to
+ * the shrink_points list.
+ *
+ * If you can't shrink directly, but can allow one of your child objects to
+ * shrink, call can_shrink on whichever child you will allow to shrink. If you
+ * can not handle shrinking below you, do not pass the command down to your
+ * child.
  *
  * Return:
- * - 0 for yes. Set the shrink_limit to the maximum *change* in size that you
- *   will allow. Create an expand_object_info_t and add to the expand_points.
- * - Non-zero error-code for no.
+ * - Return 0, even if you can't shrink and didn't add anything to the list.
+ *   The engine will determine success or failure based on the contents of the
+ *   shrink_points list.  Return non-zero if something went wrong in the
+ *   processing of the command, such as a failure to allocate memory.
  *
  * OPTIONAL
  **/
 static int template_can_shrink(storage_object_t *object,
-			       sector_count_t *shrink_limit,	/* a delta size */
+			       sector_count_t shrink_limit,	/* a delta size */
 			       list_anchor_t shrink_points)	/* of type shrink_object_info_t */
 {
 	LOG_ENTRY();
@@ -609,6 +620,25 @@ static int template_commit_changes(storage_object_t *object,
 	return ENOSYS;
 }
 
+/**
+ * template_can_activate
+ *
+ * Can you activate this object?
+ *
+ * Return:
+ * - 0 for success.
+ * - Non-zero error-code for failure.
+ *
+ * OPTIONAL
+ **/
+static int template_can_activate(storage_object_t *object)
+{
+	LOG_ENTRY();
+
+	LOG_EXIT_INT(0);
+	return 0;
+}
+
 /**
  * template_activate
  *
@@ -632,6 +662,25 @@ static int template_activate(storage_object_t *object)
 	return 0;
 }
 
+/**
+ * template_can_deactivate
+ *
+ * Can you deactivate this object?
+ *
+ * Return:
+ * - 0 for success.
+ * - Non-zero error-code for failure.
+ *
+ * OPTIONAL
+ **/
+static int template_can_deactivate(storage_object_t *object)
+{
+	LOG_ENTRY();
+
+	LOG_EXIT_INT(0);
+	return 0;
+}
+
 /**
  * template_deactivate
  *
@@ -952,28 +1001,30 @@ static int template_function(storage_object_t *object,
 	return ENOSYS;
 }
 
+/**
+ * template_backup__metadata
+ *
+ * Save the metadata needed to build this object. The plug-in can call
+ * the Engine's save_metadata() service for each contiguous chunk of
+ * metadata that it writes to each child object.
+ *
+ * Return:
+ * - 0 for success
+ * - Non-zero error-code for failure.
+ **/
+static int template_backup_metadata(storage_object_t *object)
+{
+	LOG_ENTRY();
+
+	LOG_EXIT_INT(0);
+	return 0;
+}
+
 /*
  * Container APIs. If your plugin doesn't use containers, delete all of
  * the following stubs until the plugin_functions table.
  */
 
-/**
- * template_can_create_container
- *
- * Can you create a container from this list of data segments?
- *
- * Return:
- * - 0 for yes
- * - Non-zero error-code for no.
- **/
-static int template_can_create_container(list_anchor_t objects)
-{
-	LOG_ENTRY();
-
-	LOG_EXIT_INT(ENOSYS);
-	return ENOSYS;
-}
-
 /**
  * template_can_delete_container
  *
@@ -993,16 +1044,22 @@ static int template_can_delete_container(storage_container_t *container)
 }
 
 /**
- * template_can_add_object
+ * template_can_expand_container
  *
- * Can you add this object to the container (as a consumed-object)?
+ * Can you expand this container?  If yes, build an expand_object_info_t
+ * and add it to the expand_points list.  If you allow your consumed
+ * objects to expand, call can_expand() on whichever consumed objects
+ * you will allow to expand.  If you can not handle expanding below you,
+ * do not pass the command down to your consumed objects.
  *
  * Return:
- * - 0 for yes
- * - Non-zero error-code for no.
+ * - Return 0, even if you can't expand and didn't add anything to the list.
+ *   The engine will determine success or failure based on the contents of the
+ *   expand_points list.  Return non-zero if something went wrong in the
+ *   processing of the command, such as a failure to allocate memory.
  **/
-static int template_can_add_object(storage_object_t *object,
-				   storage_container_t *container)
+static int template_can_expand_container(storage_container_t *container,
+					 list_anchor_t expand_points);	/* of type expand_object_info_t, */
 {
 	LOG_ENTRY();
 
@@ -1011,15 +1068,68 @@ static int template_can_add_object(storage_object_t *object,
 }
 
 /**
- * template_can_remove_object
+ * template_can_expand_container_by
  *
- * Can you remove this object from the container that currently consumes it?
+ * Can you allow the specified consumed objects to expand by "size"?
+ * Return 0 if yes, else an error code.  "size" is the delta expand BY
+ * size, not the resulting size.  Update the "size" if your container
+ * would expand by a different delta size when your consumed object
+ * expanded by the given size.
  *
  * Return:
  * - 0 for yes
  * - Non-zero error-code for no.
  **/
-static int template_can_remove_object(storage_object_t *object)
+static int template_can_expand_container_by(storage_container_t *container,
+					    storage_object_t *consumed_object,
+					    sector_count_t *size)
+{
+	LOG_ENTRY();
+
+	LOG_EXIT_INT(ENOSYS);
+	return ENOSYS;
+}
+
+/**
+ * template_can_shrink_container
+ *
+ * Can you shrink this container?  If yes, build a shrink_object_info_t
+ * and add it to the shrink_points list.  If you allow your consumed
+ * objects to shrink, call can_shrink() on whichever consumed objects
+ * you will allow to shrink.  If you can not handle shrinking below
+ * you, do not pass the command down to your consumed objects.
+ *
+ * Return:
+ * - Return 0, even if you can't shrink and didn't add anything to the list.
+ *   The engine will determine success or failure based on the contents of the
+ *   shrink_points list.  Return non-zero if something went wrong in the
+ *   processing of the command, such as a failure to allocate memory.
+ **/
+static int template_can_shrink_container(storage_container_t *container,
+					 list_anchor_t shrink_points);	/* of type shrink_object_info_t, */
+{
+	LOG_ENTRY();
+
+	LOG_EXIT_INT(ENOSYS);
+	return ENOSYS;
+}
+
+/**
+ * template_can_shrink_container_by
+ *
+ * Can you allow the specified consumed objects to shrink by "size"?
+ * Return 0 if yes, else an error code.  "size" is the delta shrink BY
+ * size, not the resulting size.  Update the "size" if your object would
+ * shrink by a different delta size when your consumed object shrunk by
+ * the given size.
+ *
+ * Return:
+ * - 0 for yes
+ * - Non-zero error-code for no.
+ **/
+static int template_can_shrink_container_by(storage_container_t *container,
+					    storage_object_t *consumed_object,
+					    sector_count_t *size)
 {
 	LOG_ENTRY();
 
@@ -1035,8 +1145,8 @@ static int template_can_remove_object(storage_object_t *object)
  * allocate the container structure.
  *
  * Return:
- * - 0 for yes
- * - Non-zero error-code for no.
+ * - 0 for success
+ * - Non-zero error-code for failure.
  **/
 static int template_create_container(list_anchor_t objects,
 				     option_array_t *options,
@@ -1049,17 +1159,17 @@ static int template_create_container(list_anchor_t objects,
 }
 
 /**
- * template_add_object
- *
- * Claim this object and add it to this container's consumed_objects list.
+ * template_expand_container
  *
  * Return:
- * - 0 for yes
- * - Non-zero error-code for no.
+ * - 0 for success
+ * - Non-zero error-code for failure.
  **/
-static int template_add_object(storage_object_t *object,
-			       storage_container_t *container,
-			       option_array_t *options)
+static int template_expand_container(storage_container_t *container,
+				     storage_object_t *consumed_object,
+				     storage_object_t *expand_object,
+				     list_anchor_t input_objects,
+				     option_array_t *options)
 {
 	LOG_ENTRY();
 
@@ -1068,18 +1178,17 @@ static int template_add_object(storage_object_t *object,
 }
 
 /**
- * template_transfer_object
- *
- * Transfer this consumed object from its current container to the specified
- * container.
+ * template_shrink_container
  *
  * Return:
- * - 0 for yes
- * - Non-zero error-code for no.
+ * - 0 for success
+ * - Non-zero error-code for failure.
  **/
-static int template_transfer_object(storage_object_t * object,
-				    storage_container_t * container,
-				    option_array_t * options)
+static int template_shrink_container(storage_container_t *container,
+				     storage_object_t *consumed_object,
+				     storage_object_t *shrink_object,
+				     list_anchor_t input_objects,
+				     option_array_t *options)
 {
 	LOG_ENTRY();
 
@@ -1094,8 +1203,8 @@ static int template_transfer_object(storage_object_t * object,
  * structures you may have associated with it.
  *
  * Return:
- * - 0 for yes
- * - Non-zero error-code for no.
+ * - 0 for success
+ * - Non-zero error-code for failure.
  **/
 static int template_discard_container(storage_container_t *container)
 {
@@ -1105,25 +1214,6 @@ static int template_discard_container(storage_container_t *container)
 	return ENOSYS;
 }
 
-/**
- * template_remove_object
- *
- * Remove this consumed object from its current container.  Make sure there
- * are no allocated objects produced by the container that are using space in
- * the object. Does not destroy the object.
- *
- * Return:
- * - 0 for yes
- * - Non-zero error-code for no.
- **/
-static int template_remove_object(storage_object_t *object)
-{
-	LOG_ENTRY();
-
-	LOG_EXIT_INT(ENOSYS);
-	return ENOSYS;
-}
-
 /**
  * template_delete_container
  *
@@ -1134,8 +1224,8 @@ static int template_remove_object(storage_object_t *object)
  * container structure.
  *
  * Return:
- * - 0 for yes
- * - Non-zero error-code for no.
+ * - 0 for success
+ * - Non-zero error-code for failure.
  **/
 static int template_delete_container(storage_container_t *container,
 				     list_anchor_t objects_consumed)
@@ -1158,8 +1248,8 @@ static int template_delete_container(storage_container_t *container,
  * one) during the SECOND_METADATA_WRITE phase.
  *
  * Return:
- * - 0 for yes
- * - Non-zero error-code for no.
+ * - 0 for success
+ * - Non-zero error-code for failure.
  **/
 static int template_commit_container_changes(storage_container_t *container,
 					     commit_phase_t phase)
@@ -1197,8 +1287,8 @@ static int template_commit_container_changes(storage_container_t *container,
  * caller just wants top level information about the object.
  *
  * Return:
- * - 0 for yes
- * - Non-zero error-code for no.
+ * - 0 for success
+ * - Non-zero error-code for failure.
  **/
 static int template_get_container_info(storage_container_t *container,
 				       char *info_name,
@@ -1216,8 +1306,8 @@ static int template_get_container_info(storage_container_t *container,
  * Apply the settings of the options to the given container.
  *
  * Return:
- * - 0 for yes
- * - Non-zero error-code for no.
+ * - 0 for success
+ * - Non-zero error-code for failure.
  **/
 static int template_set_container_info(storage_container_t *container,
 				       option_array_t *options)
@@ -1234,8 +1324,8 @@ static int template_set_container_info(storage_container_t *container,
  * Return an array of plugin functions that you support for this container.
  *
  * Return:
- * - 0 for yes
- * - Non-zero error-code for no.
+ * - 0 for success
+ * - Non-zero error-code for failure.
  *
  * OPTIONAL
  **/
@@ -1254,8 +1344,8 @@ static int template_get_container_functions(storage_container_t *container,
  * Execute the plug-in function on the container.
  *
  * Return:
- * - 0 for yes
- * - Non-zero error-code for no.
+ * - 0 for success
+ * - Non-zero error-code for failure.
  *
  * OPTIONAL
  **/
@@ -1270,6 +1360,25 @@ static int template_container_function(storage_container_t *container,
 	return ENOSYS;
 }
 
+/**
+ * template_backup_container_metadata
+ *
+ * Save the metadata needed to build this container. The plug-in can call
+ * the Engine's save_metadata() service for each contiguous chunk of
+ * metadata that it writes to each child object.
+ *
+ * Return:
+ * - 0 for success
+ * - Non-zero error-code for failure.
+ **/
+static int template_backup_container_metadata(storage_container_t *container)
+{
+	LOG_ENTRY();
+
+	LOG_EXIT_INT(0);
+	return 0;
+}
+
 /*
  * Table of standard plugin APIs for the Template plugin.
  */
@@ -1297,7 +1406,9 @@ static plugin_functions_t template_functions = {
 	.set_volume			= template_set_volume,
 	.add_sectors_to_kill_list	= template_add_sectors_to_kill_list,
 	.commit_changes			= template_commit_changes,
+	.can_activate			= template_can_activate,
 	.activate			= template_activate,
+	.can_deactivate			= template_can_deactivate,
 	.deactivate			= template_deactivate,
 	.get_option_count		= template_get_option_count,
 	.init_task			= template_init_task,
@@ -1310,6 +1421,7 @@ static plugin_functions_t template_functions = {
 	.write				= template_write,
 	.get_plugin_functions		= template_get_functions,
 	.plugin_function		= template_function,
+	.backup_metadata		= template_backup_metadata,
 };
 
 /*
@@ -1317,21 +1429,22 @@ static plugin_functions_t template_functions = {
  * containers, simply delete this table.
  */
 static container_functions_t template_container_functions = {
-	.can_create_container		= template_can_create_container,
 	.can_delete_container		= template_can_delete_container,
-	.can_add_object			= template_can_add_object,
-	.can_remove_object		= template_can_remove_object,
+	.can_expand_container		= template_can_expand_container,
+	.can_expand_container_by	= template_can_expand_container_by,
+	.can_shrink_container		= template_can_shrink_container,
+	.can_shrink_container_by	= template_can_shrink_container_by,
 	.create_container		= template_create_container,
-	.add_object			= template_add_object,
-	.transfer_object		= template_transfer_object,
+	.expand_container		= template_expand_container,
+	.shrink_container		= template_shrink_container,
 	.discard_container		= template_discard_container,
-	.remove_object			= template_remove_object,
 	.delete_container		= template_delete_container,
 	.commit_container_changes	= template_commit_container_changes,
 	.get_container_info		= template_get_container_info,
 	.set_container_info		= template_set_container_info,
 	.get_plugin_functions		= template_get_container_functions,
-	.plugin_function		= template_container_function
+	.plugin_function		= template_container_function,
+	.backup_container_metadata	= template_backup_container_metadata,
 };
 
 /*
@@ -1356,21 +1469,21 @@ plugin_record_t template_plugin = {
 	},
 	/* Don't change these API versions. */
 	.required_engine_api_version = {
-		.major		= 13,
+		.major		= 15,
 		.minor		= 0,
 		.patchlevel	= 0
 	},
 	.required_plugin_api_version = {
 		.plugin	= {
-			.major		= 12,
-			.minor		= 0,
+			.major		= 13,
+			.minor		= 1,
 			.patchlevel	= 0
 		}
 	},
 	/* Delete this field if the plugin doesn't use containers. */
 	.required_container_api_version = {
-		.major		= 9,
-		.minor		= 0,
+		.major		= 10,
+		.minor		= 1,
 		.patchlevel	= 0
 	},
 	.short_name = "Template",
