@@ -134,8 +134,8 @@ void Report_Parser_Error(Parser_Errors Error, char * Filename, TokenType * Token
     case Expecting_Create_Or_Check_Command:
       My_Printf("Expecting the 'Create' or 'Check' commands.\n");
       break;
-    case Expecting_Defrag_Or_Delete_Command:
-      My_Printf("Expecting the 'Defragment' or 'Delete' commands.\n");
+    case Expecting_Delete_Command:
+      My_Printf("Expecting the 'Delete' command.\n");
       break;
     case Expecting_Format_Command:
       My_Printf("Expecting the 'Format' command.\n");
