@@ -83,7 +83,7 @@ ui_callbacks_t  Callback_Functions;                  /* Used to pass call back f
  * Private Constants
  --------------------------------------------------*/
 #define VERSION_NUMBER			VERSION
-#define REQUIRED_ENGINE_VERSION_MAJOR   9
+#define REQUIRED_ENGINE_VERSION_MAJOR   10
 #define REQUIRED_ENGINE_VERSION_MINOR   0
 #define REQUIRED_ENGINE_VERSION_PATCH   0
 #define REPLACEMENT_PARAMETER           73
@@ -1294,9 +1294,6 @@ static int Map_Command_List_Entries (ADDRESS   Object,
     case Check_Node:
       Node_Name = CheckStr;
       break;
-    case Defragment_Node:
-      Node_Name = DefragmentStr;
-      break;
     case Add_Feature_Node:
       Node_Name = AFStr;
       break;
