@@ -54,54 +54,6 @@ gboolean is_commit_in_progress(void)
 	return commit_in_progress;
 }
 
-/**
- *	enum_changed_objects - enumerates volumes, storage objects, and containers that have changes
- *	@handles: the address of where to store the handle array
- *
- *	This routine returns a handle array containing volumes, storage objects, and
- *	containers that may need to be activated, re-activated, deleted, need to re-write
- *	meta-data or whatever.
- */
-int enum_changed_objects(handle_array_t ** handles)
-{
-	int rc = 0;
-	handle_array_t *volumes = NULL;
-	handle_array_t *objects = NULL;
-	handle_array_t *containers = NULL;
-
-	*handles = NULL;
-
-	evms_get_volume_list(0, 0, VOL_CHANGES_PENDING, &volumes);
-	evms_get_container_list(0, 0, CON_CHANGES_PENDING, &containers);
-
-	if (volumes != NULL && containers != NULL) {
-		*handles = merge_handle_array(volumes, containers);
-		evms_free(volumes);
-		evms_free(containers);
-	} else if (volumes != NULL) {
-		*handles = duplicate_handle_array(volumes);
-		evms_free(volumes);
-	} else if (containers != NULL) {
-		*handles = duplicate_handle_array(containers);
-		evms_free(containers);
-	}
-
-	evms_get_object_list(0, 0, 0, 0, OBJ_CHANGES_PENDING, &objects);
-
-	if (objects != NULL && *handles != NULL) {
-		handle_array_t *tmp = *handles;
-
-		*handles = merge_handle_array(tmp, objects);
-		g_free(tmp);
-		evms_free(objects);
-	} else if (objects != NULL) {
-		*handles = objects;
-	} else if (*handles == NULL) {
-		rc = ENODATA;
-	}
-	return rc;
-}
-
 /*
  *
  *   void commit_cleanup (evmsgui_commit_info_t *)
