@@ -4,16 +4,16 @@
  *
  *   This program is free software;  you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or 
+ *   the Free Software Foundation; either version 2 of the License, or
  *   (at your option) any later version.
- * 
+ *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  *   the GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program;  if not, write to the Free Software 
+ *   along with this program;  if not, write to the Free Software
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
  * Module: create.c
@@ -40,12 +40,12 @@
  *      API call using the handle of the object associated with the last row
  *      selected. It then creates and displays the results popup which destroys
  *      both windows when dismissed.
- * 
+ *
  *   Entry:
  *      button        - address of the GtkButton widget
  *      error_msg     - string to use for additional error message on failure
  *      success_msg   - string to use as additional message on success
- *      compat_create - boolean that indicates if we are creating a 
+ *      compat_create - boolean that indicates if we are creating a
  *                      compatability volume
  *
  *   Exit:
@@ -99,14 +99,14 @@ void on_create_volume_button_clicked(GtkButton * button, gchar * error_msg, gcha
 /*
  *
  *   void on_create_volume_clist_realize (GtkWidget *, gboolean)
- *   
+ *
  *   Description:
  *      This routine populates the given GtkCList with the list
  *      of objects that can be used on a create volume operation.
- * 
+ *
  *   Entry:
  *      widget        - address of the selections GtkCList widget
- *      compat_create - boolean that indicates if we are creating a 
+ *      compat_create - boolean that indicates if we are creating a
  *                      compatability volume
  *
  *   Exit:
@@ -150,11 +150,11 @@ void on_create_volume_clist_realize(GtkWidget * widget, gboolean compat_create)
 /*
  *
  *   gboolean is_create_task_acceptable_object (plugin_handle_t, object_handle_t)
- *   
+ *
  *   Description:
  *      This routine checks to see if the given object is an
  *      acceptable object for the create task of the given plugin.
- * 
+ *
  *   Entry:
  *      plugin - the plugin to check against its create task
  *      handle - the handle of the object to see if acceptable
@@ -179,11 +179,11 @@ gboolean is_create_task_acceptable_object(plugin_handle_t plugin, object_handle_
 /*
  *
  *   gboolean can_create_from_freespace_object (object_handle_t)
- *   
+ *
  *   Description:
  *      This routine determines if a create task on the plugin for
  *      the given freespace storage object could work.
- * 
+ *
  *   Entry:
  *      handle - the handle of a freespace object to see if suitable for create
  *
@@ -211,11 +211,11 @@ gboolean can_create_from_freespace_object(object_handle_t handle)
 /*
  *
  *   plugin_handle_t get_plugin_handle_for_object (object_handle_t)
- *   
+ *
  *   Description:
  *      This routine returns the plugin handle of the plugin that
  *      manages the supplied storage object.
- * 
+ *
  *   Entry:
  *      handle - the handle of a storage object
  *
@@ -239,11 +239,11 @@ plugin_handle_t get_plugin_handle_for_object(object_handle_t handle)
 /*
  *
  *   gchar *get_create_menu_item_text (object_type_t)
- *   
+ *
  *   Description:
  *      This routine returns a string containing the menu
  *      item for a create from freespace.
- * 
+ *
  *   Entry:
  *      type - the object type (SEGMENT or REGION)
  *
@@ -258,10 +258,10 @@ gchar *get_create_menu_item_text(object_type_t type)
 
 	switch (type) {
 	case REGION:
-		text = g_strdup("Create Region...");
+		text = g_strdup(_("Create Region..."));
 		break;
 	case SEGMENT:
-		text = g_strdup("Create Segment...");
+		text = g_strdup(_("Create Segment..."));
 		break;
 	default:
 		log_warning("%s: %d is not a valid create from freespace type.\n", __FUNCTION__,
@@ -275,11 +275,11 @@ gchar *get_create_menu_item_text(object_type_t type)
 /*
  *
  *   gchar *get_create_from_freespace_window_title (object_handle_t)
- *   
+ *
  *   Description:
- *      This routine returns a string containing the title for 
+ *      This routine returns a string containing the title for
  *      the window for the create from freespace action.
- * 
+ *
  *   Entry:
  *      handle - the handle of the free space storage object
  *
@@ -296,10 +296,10 @@ gchar *get_create_from_freespace_window_title(object_handle_t handle)
 	if (evms_get_handle_object_type(handle, &type) == SUCCESS) {
 		switch (type) {
 		case REGION:
-			text = g_strdup("Create Region Configuration Options");
+			text = g_strdup(_("Create Region Configuration Options"));
 			break;
 		case SEGMENT:
-			text = g_strdup("Create Segment Configuration Options");
+			text = g_strdup(_("Create Segment Configuration Options"));
 			break;
 		default:
 			log_warning("%s: %d is not a valid create from freespace type.\n",
@@ -314,11 +314,11 @@ gchar *get_create_from_freespace_window_title(object_handle_t handle)
 /*
  *
  *   void on_create_from_freespace_menu_item_activate (GtkMenuItem *, gpointer)
- *   
+ *
  *   Description:
  *      This routine initiates the create task to create a new object from
  *      the supplied freespace storage object.
- * 
+ *
  *   Entry:
  *      handle - the handle of a freespace object to create a new object from
  *
@@ -369,11 +369,11 @@ void on_create_from_freespace_menu_item_activate(GtkMenuItem * menuitem, gpointe
 /*
  *
  *   handle_array_t *get_all_feature_plugins_list (void)
- *   
+ *
  *   Description:
  *      This routine returns a single, merged handle array of all
  *      the different feature type plugins.
- * 
+ *
  *   Entry:
  *      None
  *
@@ -413,11 +413,11 @@ handle_array_t *get_all_feature_plugins_list(void)
 /*
  *
  *   gboolean can_create_feature_object (object_handle_t)
- *   
+ *
  *   Description:
  *      This routine determines if the given object can be
  *      used to create any feature object.
- * 
+ *
  *   Entry:
  *      handle - the handle of the storage object to check
  *
@@ -457,12 +457,12 @@ gboolean can_create_feature_object(object_handle_t handle)
 /*
  *
  *   void on_feature_create_clist_realize (GtkWidget *, gpointer)
- *   
+ *
  *   Description:
  *      This routine populates the given GtkCList with the list
  *      of feature plugins that say they can create something with
  *      the given object handle.
- * 
+ *
  *   Entry:
  *      widget    - address of the selections GtkCList widget
  *      user_data - contains object handle
@@ -511,7 +511,7 @@ void on_feature_create_clist_realize(GtkWidget * widget, gpointer user_data)
  *      This routine responds to feature plugin selection with creating the
  *      task given and creating the acceptable object selection dialog
  *      window to proceed with acceptable parameter and option selections.
- * 
+ *
  *   Entry:
  *      button    - address of the GtkButton widget
  *      user_data - contains the handle of the context object create initiated on
@@ -573,7 +573,7 @@ void on_feature_plugin_selection_button_clicked(GtkButton * button, gpointer use
 
 			/*
 			 * If we already created the next window for selecting
-			 * the acceptable objects (and maybe even other follow-on 
+			 * the acceptable objects (and maybe even other follow-on
 			 * windows like for options), make sure we destroy them
 			 * all and clean up the list since our plugin selections
 			 * have apperently changed and so all the following panels
@@ -660,11 +660,11 @@ void on_feature_plugin_selection_button_clicked(GtkButton * button, gpointer use
 /*
  *
  *   void on_create_feature_object_menu_item_activate (GtkMenuItem *, gpointer)
- *   
+ *
  *   Description:
  *      This routine initiates process to create a new feature object
  *      starting with the given selected object.
- * 
+ *
  *   Entry:
  *      menuitem  - the menu item that initiated the action
  *      user_data - contains the handle of a object to use in creating a feature object
