@@ -51,6 +51,8 @@
 #include "help.h"
 #include "main_window_cb.h"
 #include "logging.h"
+#include "activation.h"
+#include "backup.h"
 
 #define EVMS_HOWTO_URL "http://evms.sourceforge.net/users_guide/"
 #define EVMS_FAQ_URL "http://evms.sourceforge.net/faq.html"
@@ -384,9 +386,9 @@ void on_expand_container_menu_item_activate(GtkMenuItem * menuitem, gpointer use
 						  NULL,
 						  get_expand_container_help_text(),
 						  on_expand_container_clist_realize,
-						  on_resize_point_selection_button_clicked,
+						  on_resize_container_button_clicked,
 						  NULL, NULL, NULL, NULL,
-						  GUINT_TO_POINTER(EVMS_Task_Expand_Container));
+						  GUINT_TO_POINTER(EVMS_Task_Expand));
 
 	gtk_widget_show(window);
 }
@@ -483,9 +485,9 @@ void on_shrink_container_menu_item_activate(GtkMenuItem * menuitem, gpointer use
 						  NULL,
 						  get_shrink_container_help_text(),
 						  on_shrink_container_clist_realize,
-						  on_resize_point_selection_button_clicked,
+						  on_resize_container_button_clicked,
 						  NULL, NULL, NULL, NULL,
-						  GUINT_TO_POINTER(EVMS_Task_Shrink_Container));
+						  GUINT_TO_POINTER(EVMS_Task_Shrink));
 
 	gtk_widget_show(window);
 }
@@ -719,6 +721,62 @@ void on_convert_to_evms_volume_menu_item_activate(GtkMenuItem * menuitem, gpoint
 	gtk_widget_show(window);
 }
 
+/*
+ *
+ *   void on_activatee_menu_item_activate (GtkMenuItem *, gpointer)
+ *
+ *   Description:
+ *      This routine creates the generic selection dialog
+ *      and connects the appropriate signal handlers that will
+ *      handle the actions for activating a volume or object.
+ *
+ *   Entry:
+ *      menuitem  - address of the GtkMenuItem widget
+ *      user_data - address of user data bound with signal (not used)
+ *
+ *   Exit:
+ *      Generic selection dialog is displayed and signal handlers
+ *      have been connected for the corresponding actions.
+ *
+ */
+void on_activate_menu_item_activate(GtkMenuItem * menuitem, gpointer user_data)
+{
+	GtkWidget *window;
+
+	window = create_activate_window(on_activate_clist_realize,
+					user_data);
+
+	gtk_widget_show(window);
+}
+
+/*
+ *
+ *   void on_deactivate_menu_item_activate (GtkMenuItem *, gpointer)
+ *
+ *   Description:
+ *      This routine creates the generic selection dialog
+ *      and connects the appropriate signal handlers that will
+ *      handle the actions for deactivating a volume or object.
+ *
+ *   Entry:
+ *      menuitem  - address of the GtkMenuItem widget
+ *      user_data - address of user data bound with signal (not used)
+ *
+ *   Exit:
+ *      Generic selection dialog is displayed and signal handlers
+ *      have been connected for the corresponding actions.
+ *
+ */
+void on_deactivate_menu_item_activate(GtkMenuItem * menuitem, gpointer user_data)
+{
+	GtkWidget *window;
+
+	window = create_deactivate_window(on_deactivate_clist_realize,
+					  user_data);
+
+	gtk_widget_show(window);
+}
+
 /*
  *
  *   void on_add_feature_to_volume_menu_item_activate (GtkMenuItem *, gpointer)
@@ -957,39 +1015,6 @@ void on_fsck_menu_item_activate(GtkMenuItem * menuitem, gpointer user_data)
 	gtk_widget_show(window);
 }
 
-/*
- *
- *   void on_defrag_menu_item_activate (GtkMenuItem *, gpointer)
- *
- *   Description:
- *      This routine creates the generic selection dialog
- *      and connects the appropriate signal handlers that will
- *      handle the actions for defragmenting a filesystem located
- *      on a volume.
- *
- *   Entry:
- *      menuitem  - address of the GtkMenuItem widget
- *      user_data - address of user data bound with signal (not used)
- *
- *   Exit:
- *      Generic selection dialog is displayed and signal handlers
- *      have been connected for the corresponding actions.
- *
- */
-void on_defrag_menu_item_activate(GtkMenuItem * menuitem, gpointer user_data)
-{
-	GtkWidget *window;
-
-	window = create_standard_selection_window(_("Defragment File System"),
-						  NULL,
-						  get_defrag_help_text(),
-						  on_filesys_operation_clist_realize,
-						  on_initiate_filesys_operation_button_clicked,
-						  NULL, NULL, NULL, NULL, GUINT_TO_POINTER(DEFRAG));
-
-	gtk_widget_show(window);
-}
-
 /*
  *
  *   void on_obliterate_filesystem_menu_item_activate (GtkMenuItem *, gpointer)
@@ -1121,6 +1146,59 @@ void process_request_to_commit_changes(void)
 	}
 }
 
+/*
+ *
+ *   void process_request_to_backup_changes (void)
+ *
+ *   Description:
+ *      This routine call the function that displays a dialog
+ *      confirming that the user wants to backup changes or
+ *      not. If so, the dialog will initiate the backup process.
+ *
+ *   Entry:
+ *      Nothing
+ *
+ *   Exit:
+ *      If non-activateion changes are pending, a dialog is shown saying that
+ *      the metadata cannot be backed up.  Otherwise a confirmation dialog is
+ *      displayed.
+ *
+ */
+void process_request_to_backup_changes(void)
+{
+	int rc;
+	int i;
+	boolean changes_pending;
+	change_record_array_t * change_records = NULL;
+
+	rc = evms_changes_pending(&changes_pending, &change_records);
+	if (rc == 0) {
+		if (changes_pending) {
+			/*
+			 * The Engine can backup metadata if all that needs
+			 * to be done is activation.
+			 */
+			changes_pending = FALSE;
+			for (i = 0; !changes_pending && (i < change_records->count); i++) {
+				if (change_records->changes_pending[i].changes & ~(CHANGE_ACTIVATE | CHANGE_REACTIVATE | CHANGE_DEACTIVATE)) {
+					changes_pending = TRUE;
+				}
+			}
+		}
+		if (changes_pending) {
+			display_cannot_backup_metadata_dialog();
+		} else {
+			display_backup_metadata_confirmation_dialog();
+		}
+
+		evms_free(change_records);
+	} else {
+		log_error("%s: evms_changes_pending() returned error code %d.\n", __FUNCTION__, rc);
+		display_results_window(rc, NULL, _("Unable to determine if changes are pending!"),
+				       NULL, FALSE, get_main_window_id());
+	}
+}
+
 /*
  *
  *   void on_commit_menu_item_activate (GtkMenuItem *, gpointer)
@@ -1144,6 +1222,29 @@ void on_commit_menu_item_activate(GtkMenuItem * menuitem, gpointer user_data)
 	process_request_to_commit_changes();
 }
 
+/*
+ *
+ *   void on_backup_menu_item_activate (GtkMenuItem *, gpointer)
+ *
+ *   Description:
+ *      This routine call the function that displays a dialog
+ *      confirming that the user wants to backup changes or
+ *      not. If so, the dialog will initiate the backup process.
+ *
+ *   Entry:
+ *      menuitem  - address of the GtkMenuItem widget
+ *      user_data - address of user data bound with signal (not used)
+ *
+ *   Exit:
+ *      If changes are pending, a confirmation dialog is displayed
+ *      otherwise a popup is displayed indicating nothing to be done.
+ *
+ */
+void on_backup_menu_item_activate(GtkMenuItem * menuitem, gpointer user_data)
+{
+	process_request_to_backup_changes();
+}
+
 /*
  *
  *   void process_request_to_exit (void)
@@ -1912,9 +2013,6 @@ void disable_filesys_action_menu_items_if_no_fsims(void)
 			widget = GTK_WIDGET(gtk_object_get_data(main_window, "fsck_menu_item"));
 			gtk_widget_set_sensitive(widget, FALSE);
 
-			widget = GTK_WIDGET(gtk_object_get_data(main_window, "defrag_menu_item"));
-			gtk_widget_set_sensitive(widget, FALSE);
-
 			widget =
 			    GTK_WIDGET(gtk_object_get_data
 				       (main_window, "obliterate_filesystem_menu_item"));
