@@ -343,40 +343,41 @@ void on_resize_point_selection_button_clicked(GtkButton * button, task_action_t
 		gint rc;
 		gchar *verb;
 		gchar *window_title;
+		handle_object_info_t *object = NULL;
+
+		evms_get_info(resize_point, &object);
 
 		if (next_window != NULL)
 			destroy_window_list(g_list_find(window_list, next_window));
 
 		switch (action) {
 		case EVMS_Task_Expand:
-			{
-				handle_object_info_t *object = NULL;
-
-				evms_get_info(resize_point, &object);
-
-				if ((object != NULL) &&
-				    (object->type == VOLUME)) {
+			if (object != NULL) {
+				switch (object->type) {
+				case VOLUME:
 					window_title = _("Expand a Volume");
-				} else {
+					break;
+				case CONTAINER:
+					window_title = _("Expand a Storage Container");
+					break;
+				default:
 					window_title = _("Expand a Storage Object");
 				}
-				verb = _("Expand");
-				if (object != NULL) {
-					evms_free(object);
-				}
+			} else {
+				window_title = _("Expand a Storage Object");
 			}
-			break;
-		case EVMS_Task_Expand_Container:
-			window_title = _("Expand a Storage Container");
 			verb = _("Expand");
-			break;
-		case EVMS_Task_Shrink_Container:
-			window_title = _("Shrink a Storage Container");
-			verb = _("Shrink");
+			if (object != NULL) {
+				evms_free(object);
+			}
 			break;
 		case EVMS_Task_Shrink:
 		default:
-			window_title = _("Shrink a Storage Object");
+			if (object->type == CONTAINER) {
+				window_title = _("Shrink a Storage Container");
+			} else {
+				window_title = _("Shrink a Storage Object");
+			}
 			verb = _("Shrink");
 			break;
 		}
@@ -728,7 +729,7 @@ void on_expand_thing_button_clicked(GtkButton * button, gpointer user_data)
  */
 void on_expand_container_button_clicked(GtkButton * button, gpointer user_data)
 {
-	on_resize_point_selection_button_clicked(button, EVMS_Task_Expand_Container);
+	on_resize_point_selection_button_clicked(button, EVMS_Task_Expand);
 }
 
 /*
@@ -749,7 +750,7 @@ void on_expand_container_button_clicked(GtkButton * button, gpointer user_data)
  */
 void on_shrink_container_button_clicked(GtkButton * button, gpointer user_data)
 {
-	on_resize_point_selection_button_clicked(button, EVMS_Task_Shrink_Container);
+	on_resize_point_selection_button_clicked(button, EVMS_Task_Shrink);
 }
 
 /*
@@ -921,10 +922,7 @@ void on_expand_thing_menu_item_activate(GtkMenuItem * menuitem, gpointer user_da
 							  VOLUME ?
 							  add_volume_as_selected_expansion_item :
 							  add_thing_as_selected_list_item,
-							  type ==
-							  CONTAINER ?
-							  on_expand_container_button_clicked :
-							  on_expand_thing_button_clicked, NULL,
+                                                          on_expand_thing_button_clicked, NULL,
 							  NULL, NULL, NULL, user_data);
 
 		gtk_widget_show(window);
@@ -995,9 +993,6 @@ void on_shrink_thing_menu_item_activate(GtkMenuItem * menuitem, gpointer user_da
 							  VOLUME ?
 							  add_volume_as_selected_shrink_item :
 							  add_thing_as_selected_list_item,
-							  type ==
-							  CONTAINER ?
-							  on_shrink_container_button_clicked :
 							  on_shrink_thing_button_clicked, NULL,
 							  NULL, NULL, NULL, user_data);
 
