@@ -355,12 +355,10 @@ gchar *get_task_action_string(task_handle_t task)
 			break;
 
 		case EVMS_Task_Expand:
-		case EVMS_Task_Expand_Container:
 			action_text = g_strdup(_("Expand"));
 			break;
 
 		case EVMS_Task_Shrink:
-		case EVMS_Task_Shrink_Container:
 			action_text = g_strdup(_("Shrink"));
 			break;
 
@@ -380,10 +378,6 @@ gchar *get_task_action_string(task_handle_t task)
 			action_text = g_strdup(_("Make"));
 			break;
 
-		case EVMS_Task_defrag:
-			action_text = g_strdup(_("Defragment"));
-			break;
-
 		default:
 			log_debug("%s: Uknown task action %d received.\n", __FUNCTION__, action);
 			break;
@@ -461,7 +455,7 @@ gboolean are_handle_arrays_equal(handle_array_t * array1, handle_array_t * array
 		gint array_size;
 
 		array_size =
-		    sizeof(handle_array_t) + ((array1->count - 1) * sizeof(object_handle_t));
+		    sizeof(handle_array_t) + (array1->count * sizeof(object_handle_t));
 		arrays_equal = memcmp(array1, array2, array_size) == 0;
 	}
 
@@ -1338,7 +1332,6 @@ void on_invoke_task_button_clicked(GtkButton * button, gpointer user_data)
 			switch (action) {
 			case EVMS_Task_mkfs:
 			case EVMS_Task_fsck:
-			case EVMS_Task_defrag:
 				message =
 				    _
 				    ("The operation has been scheduled for completion when changes are saved.");
@@ -1480,7 +1473,7 @@ void on_acceptable_objects_button_clicked(GtkButton * button, gpointer user_data
 				if (next_button_text == NULL)
 					next_button_text = get_task_action_string(task);
 
-				next_window = create_standard_task_window("Configuration Options",
+				next_window = create_standard_task_window(_("Configuration Options"),
 									  next_button_text,
 									  NULL,
 									  on_button_clicked_display_prev_window,
@@ -2565,13 +2558,15 @@ gboolean can_plugin_do_action(plugin_handle_t plugin, task_action_t action)
 gint set_selected_object(task_handle_t task, object_handle_t handle)
 {
 	gint rc;
-	handle_array_t ha;
+	gchar mem[sizeof(handle_array_t) + sizeof(object_handle_t)];
+	handle_array_t * ha = (handle_array_t *) mem;
 	declined_handle_array_t *declined_list;
 
-	ha.count = 1;
-	ha.handle[0] = handle;
 
-	rc = evms_set_selected_objects(task, &ha, &declined_list, NULL);
+	ha->count = 1;
+	ha->handle[0] = handle;
+
+	rc = evms_set_selected_objects(task, ha, &declined_list, NULL);
 
 	if (rc == SUCCESS && declined_list) {
 		if (declined_list->count > 0)
