@@ -4,16 +4,16 @@
  *
  *   This program is free software;  you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or 
+ *   the Free Software Foundation; either version 2 of the License, or
  *   (at your option) any later version.
- * 
+ *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  *   the GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program;  if not, write to the Free Software 
+ *   along with this program;  if not, write to the Free Software
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
  * Module: transfer.h
@@ -23,4 +23,6 @@ void on_remove_thing_from_container_button_clicked(GtkButton * button, gpointer
 
 void on_remove_thing_from_container_menu_item_activate(GtkMenuItem * menuitem, gpointer user_data);
 
+gboolean can_remove_from_container(object_handle_t handle);
+
 void on_remove_thing_from_container_clist_realize(GtkWidget * widget, gpointer user_data);
