@@ -4,19 +4,19 @@
  *
  *   This program is free software;  you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or 
+ *   the Free Software Foundation; either version 2 of the License, or
  *   (at your option) any later version.
- * 
+ *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  *   the GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program;  if not, write to the Free Software 
+ *   along with this program;  if not, write to the Free Software
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
- */ 
+ */
 
 #include <glib.h>
 #include <ncurses.h>
@@ -72,7 +72,7 @@ gboolean can_volume_be_formatted(object_handle_t volume)
 	int rc;
 	gboolean result = FALSE;
 	handle_array_t  *plugins;
-    
+
 	rc = evms_get_plugin_list(EVMS_FILESYSTEM_INTERFACE_MODULE, 0, &plugins);
 	if (rc != 0) {
 		log_error("%s: evms_get_plugin_list() returned error code %d.\n", __FUNCTION__, rc);
@@ -113,19 +113,6 @@ int filter_fsckable_volume(engine_handle_t handle, void *user_data)
 	return evms_can_fsck(handle);
 }
 
-/**
- *	filter_defragable_volume - filter volume that can be defragmented
- *	@handle: the volume handle
- *	@user_data: not used
- *
- *	This routine is a standard clist_filter_func function type that checks to see
- *	if the given volume can be defragmented.
- */
-int filter_defragable_volume(engine_handle_t handle, void *user_data)
-{
-	return evms_can_defrag(handle);
-}
-
 /**
  *	filter_unmkfsable_volumes - filter volumes that can be unmkfs'd
  *	@handle: the volume handle
@@ -181,11 +168,11 @@ void on_delete_filesys_task_dialog(struct dialog_window *dialog)
 }
 
 /**
- *	show_filesys_task_dialog - present dialog with list of volumes to process a defrag or fsck task
+ *	show_filesys_task_dialog - present dialog with list of volumes to process a fsck task
  *	@title: the dialog window title
- *	@action: the task action code (EVMS_Task_defrag or EVMS_Task_fsck)
+ *	@action: the task action code (EVMS_Task_fsck)
  *
- *	This routine displays volumes to either defrag or fsck and handles the processing
+ *	This routine displays volumes to fsck and handles the processing
  *	necessary for the follow-on task dialogs.
  */
 void show_filesys_task_dialog(char *title, task_action_t action)
@@ -218,7 +205,7 @@ void show_filesys_task_dialog(char *title, task_action_t action)
 	print_clist_column_title(selwin->clist, 2, _("Size"));
 
 	clist_populate(selwin->clist, enum_volumes,
-			action == EVMS_Task_fsck ? filter_fsckable_volume : filter_defragable_volume,
+			filter_fsckable_volume,
 			format_standard_item, NULL, NULL);
 
 	if (g_list_length(selwin->clist->choices) == 1)
@@ -542,39 +529,3 @@ int actions_unmkfs_menuitem_activated(struct menu_item *item)
 	show_unmkfs_dialog(0);
 	return 0;
 }
-
-/**
- *	context_defrag_menuitem_activated - defrag initiated from context menu
- *	@item: the menu item that caused this function to get invoked
- *
- *	This routine is called to defragment a filesystem on the volume selected
- *	and is initiated from the context popup menu.
- */
-int context_defrag_menuitem_activated(struct menu_item *item)
-{
-	struct dialog_window *dialog;
-
-	dialog = create_task_dialog(NULL, GPOINTER_TO_UINT(item->user_data),
-				EVMS_Task_defrag,
-				_("Deframent File System"),
-				_("Defragment"),
-				NULL);
-	if (dialog != NULL) {
-		process_dialog_list_events(dialog->list);
-		delete_dialog_list(dialog->list->list);
-	}
-	return 0; 
-}
-
-/**
- *	actions_defrag_menuitem_activated - defrag initiated from Actions menu
- *	@item: the menu item that caused this function to get invoked
- *
- *	This routine is invoked by the File System->Defragment Actions pulldown
- *	menu item.
- */
-int actions_defrag_menuitem_activated(struct menu_item *item)
-{
-	show_filesys_task_dialog(_("Defragment File System"), EVMS_Task_defrag);
-	return 0; 
-}
