@@ -4,20 +4,20 @@
  *
  *   This program is free software;  you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or 
+ *   the Free Software Foundation; either version 2 of the License, or
  *   (at your option) any later version.
- * 
+ *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  *   the GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program;  if not, write to the Free Software 
+ *   along with this program;  if not, write to the Free Software
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
- */ 
- 
+ */
+
 #include <frontend.h>
 #include <glib.h>
 
@@ -63,7 +63,7 @@ inline gchar *make_data_type_readable_string(data_type_t type)
  *	@use_short_form: TRUE if a condensed version is preferred
  *
  *	This routine takes a value unit type code and returns the
- *	string description of the data type. The caller can request 
+ *	string description of the data type. The caller can request
  *	a short form if available, i.e. "KB" instead of "kilobytes".
  */
 inline gchar *make_unit_readable_string(value_unit_t unit, gboolean use_short_form)
@@ -73,11 +73,11 @@ inline gchar *make_unit_readable_string(value_unit_t unit, gboolean use_short_fo
 	switch (unit) {
 	case EVMS_Unit_Bytes:
 		readable_string = g_strdup(_("bytes"));
-		break;           
+		break;
 	case EVMS_Unit_Kilobytes:
 		if (use_short_form)
 			readable_string = g_strdup("KB");
-		else 
+		else
 			readable_string = g_strdup(_("kilobytes"));
 		break;
 	case EVMS_Unit_Megabytes:
@@ -143,11 +143,11 @@ inline gchar *make_unit_readable_string(value_unit_t unit, gboolean use_short_fo
  *
  *	This routine takes a size in sectors and returns the
  *	size as a string representation in either TB, GB,
- *	MB, KB, or bytes. The dynamically allocated string 
- *	should be freed by the caller with g_free() when no 
+ *	MB, KB, or bytes. The dynamically allocated string
+ *	should be freed by the caller with g_free() when no
  *	longer needed.
  */
-inline gchar *make_sectors_readable_size_string(u_int64_t size_in_sectors, value_unit_t *unit) 
+inline gchar *make_sectors_readable_size_string(u_int64_t size_in_sectors, value_unit_t *unit)
 {
 	gchar *readable_string = NULL;
 	u_int64_t size_in_kilobytes = size_in_sectors / 2;
@@ -202,11 +202,11 @@ inline gchar *make_size_readable_string(gchar *size_string, value_unit_t unit)
  *
  *	This routine takes a size in sectors and returns the
  *	size as a string representation in either TB, GB,
- *	MB, KB, or bytes. The dynamically allocated string 
- *	should be freed by the caller with g_free() when no 
+ *	MB, KB, or bytes. The dynamically allocated string
+ *	should be freed by the caller with g_free() when no
  *	longer needed.
  */
-inline gchar *make_sectors_readable_string(u_int64_t size_in_sectors) 
+inline gchar *make_sectors_readable_string(u_int64_t size_in_sectors)
 {
 	gchar *size_string;
 	gchar *readable_string = NULL;
@@ -229,11 +229,11 @@ inline gchar *make_sectors_readable_string(u_int64_t size_in_sectors)
  *
  *	This routine takes a size in kilobytes and returns the
  *	size as a string representation in either TB, GB,
- *	MB, or KB. The dynamically allocated string 
- *	should be freed by the caller with g_free() when no 
+ *	MB, or KB. The dynamically allocated string
+ *	should be freed by the caller with g_free() when no
  *	longer needed.
  */
-inline gchar *make_kbytes_readable_string(u_int64_t size_in_kbytes) 
+inline gchar *make_kbytes_readable_string(u_int64_t size_in_kbytes)
 {
 	return make_sectors_readable_string(size_in_kbytes * 2);
 }
@@ -343,7 +343,7 @@ inline gchar *get_object_name(object_handle_t handle)
 inline gchar *make_object_type_readable_string(object_type_t type)
 {
 	gchar *readable_string = NULL;
-    
+
 	switch (type) {
 	case VOLUME:
 		readable_string = _("Logical Volume");
@@ -383,8 +383,8 @@ inline gchar *get_volume_fsim_name(logical_volume_info_t *volume)
 	handle_object_info_t *plugin;
 
 	plugin_name = g_strdup("");
-    
-	if (volume->file_system_manager != 0) {    
+
+	if (volume->file_system_manager != 0) {
 		rc = evms_get_info(volume->file_system_manager, &plugin);
     		if (rc != 0) {
 			log_error("%s: evms_get_info() returned error %d.\n", __FUNCTION__, rc);
@@ -410,13 +410,13 @@ inline gchar *get_plugin_name(plugin_handle_t plugin_handle)
 
 	if (plugin_handle != 0) {
 		handle_object_info_t *plugin;
-        
-		rc = evms_get_info(plugin_handle, &plugin); 
+
+		rc = evms_get_info(plugin_handle, &plugin);
 		if (rc == 0) {
 			plugin_name = g_strdup(plugin->info.plugin.short_name);
 			evms_free(plugin);
 		} else {
-			plugin_name = g_strdup(_("Plug-in not available"));            
+			plugin_name = g_strdup(_("Plug-in not available"));
 		}
 	} else {
 		plugin_name = g_strdup(_("Plug-in not available"));
@@ -447,49 +447,49 @@ inline gchar *make_plugin_type_readable_string(evms_plugin_code_t type, gboolean
 		if (lowercase)
 			readable_string = _("segment manager");
 		else
-			readable_string = _("Segment Manager");                
+			readable_string = _("Segment Manager");
 		break;
 	case EVMS_REGION_MANAGER:
 		if (lowercase)
 			readable_string = _("region manager");
 		else
-			readable_string = _("Region Manager");                
+			readable_string = _("Region Manager");
 		break;
 	case EVMS_FEATURE:
 		if (lowercase)
 			readable_string = _("feature plug-in");
 		else
-			readable_string = _("Feature Plug-in");                
+			readable_string = _("Feature Plug-in");
 		break;
 	case EVMS_ASSOCIATIVE_FEATURE:
 		if (lowercase)
 			readable_string = _("associative feature");
 		else
-			readable_string = _("Associative Feature");               
+			readable_string = _("Associative Feature");
 		break;
 	case EVMS_FILESYSTEM_INTERFACE_MODULE:
 		if (lowercase)
 			readable_string = _("file system interface module");
 		else
-			readable_string = _("File System Interface Module");           
+			readable_string = _("File System Interface Module");
 		break;
 	case EVMS_CLUSTER_MANAGER_INTERFACE_MODULE:
 		if (lowercase)
 			readable_string = _("cluster manager interface module");
 		else
-			readable_string = _("Cluster Manager Interface Module");                
+			readable_string = _("Cluster Manager Interface Module");
 		break;
 	case EVMS_DISTRIBUTED_LOCK_MANAGER_INTERFACE_MODULE:
 		if (lowercase)
 			readable_string = _("distributed lock manager interface module");
 		else
-			readable_string = _("Distributed Lock Manager Interface Module");                
+			readable_string = _("Distributed Lock Manager Interface Module");
 		break;
 	default:
 		if (lowercase)
 			readable_string = _("plug-in");
 		else
-			readable_string = _("Plug-in");                
+			readable_string = _("Plug-in");
 	}
 	return readable_string;
 }
@@ -509,8 +509,8 @@ gchar *object_flags_to_string(u_int32_t flags)
 	
 	gstring = g_string_new("");
 
-	if (flags & SOFLAG_ACTIVE) {
-		if (flags & SOFLAG_NEEDS_ACTIVATE)
+	if (flags & SOFLAG_NEEDS_ACTIVATE) {
+		if (flags & SOFLAG_ACTIVE)
 			gstring = g_string_append(gstring, _("Re-activation"));
 	} else if (flags & SOFLAG_NEEDS_ACTIVATE) {
 		gstring = g_string_append(gstring, _("Activation"));
@@ -525,7 +525,7 @@ gchar *object_flags_to_string(u_int32_t flags)
 	if ((flags & SOFLAG_DIRTY) || (flags & SOFLAG_FEATURE_HEADER_DIRTY)) {
 		if (*(gstring->str) != '\0')
 			gstring = g_string_append_c(gstring, ',');
-		gstring = g_string_append(gstring, _("Dirty"));
+		gstring = g_string_append(gstring, _("Modified"));
 	}
 
 	if (flags & SOFLAG_NEW) {
@@ -556,7 +556,7 @@ gchar *container_flags_to_string(u_int32_t flags)
 	gstring = g_string_new("");
 
 	if (flags & SCFLAG_DIRTY) {
-		gstring = g_string_append(gstring, _("Dirty"));
+		gstring = g_string_append(gstring, _("Modified"));
 	}
 
 	if (flags & SCFLAG_NEW) {
@@ -586,8 +586,8 @@ gchar *volume_flags_to_string(u_int32_t flags)
 	
 	gstring = g_string_new("");
 
-	if (flags & VOLFLAG_ACTIVE) {
-		if (flags & VOLFLAG_NEEDS_ACTIVATE)
+	if (flags & VOLFLAG_NEEDS_ACTIVATE) {
+		if (flags & VOLFLAG_ACTIVE)
 			gstring = g_string_append(gstring, _("Re-activation"));
 	} else {
 		gstring = g_string_append(gstring, _("Activation"));
@@ -599,10 +599,10 @@ gchar *volume_flags_to_string(u_int32_t flags)
 		gstring = g_string_append(gstring, _("Deactivation"));
 	}
 
-	if (flags & VOLFLAG_DIRTY) {
+	if (flags & (VOLFLAG_DIRTY | VOLFLAG_FEATURE_HEADER_DIRTY)) {
 		if (*(gstring->str) != '\0')
 			gstring = g_string_append_c(gstring, ',');
-		gstring = g_string_append(gstring, _("Dirty"));
+		gstring = g_string_append(gstring, _("Modified"));
 	}
 	
 	if (flags & VOLFLAG_NEW) {
@@ -629,12 +629,6 @@ gchar *volume_flags_to_string(u_int32_t flags)
 		gstring = g_string_append(gstring, _("Unmkfs"));
 	}
 	
-	if (flags & VOLFLAG_DEFRAG) {
-		if (*(gstring->str) != '\0')
-			gstring = g_string_append_c(gstring, ',');
-		gstring = g_string_append(gstring, _("Defragment"));
-	}
-			
 	if (flags & VOLFLAG_PROBE_FS) {
 		if (*(gstring->str) != '\0')
 			gstring = g_string_append_c(gstring, ',');
@@ -687,13 +681,13 @@ gchar *changes_flag_to_string(u_int32_t flags)
 	if (flags & CHANGE_DIRTY) {
 		if (*(gstring->str) != '\0')
 			gstring = g_string_append_c(gstring, ',');
-		gstring = g_string_append(gstring, _("Dirty"));
+		gstring = g_string_append(gstring, _("Modified"));
 	}
 
 	if (flags & CHANGE_FEATURE_HEADER_DIRTY) {
 		if (*(gstring->str) != '\0')
 			gstring = g_string_append_c(gstring, ',');
-		gstring = g_string_append(gstring, _("Feature Header Dirty"));
+		gstring = g_string_append(gstring, _("Feature Header Modified"));
 	}
 	
 	if (flags & CHANGE_EXPAND) {
@@ -726,12 +720,6 @@ gchar *changes_flag_to_string(u_int32_t flags)
 		gstring = g_string_append(gstring, _("Unmkfs"));
 	}
 	
-	if (flags & CHANGE_DEFRAG) {
-		if (*(gstring->str) != '\0')
-			gstring = g_string_append_c(gstring, ',');
-		gstring = g_string_append(gstring, _("Defragment"));
-	}
-			
 	if (flags & CHANGE_DELETE) {
 		if (*(gstring->str) != '\0')
 			gstring = g_string_append_c(gstring, ',');
