@@ -4,19 +4,19 @@
  *
  *   This program is free software;  you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or 
+ *   the Free Software Foundation; either version 2 of the License, or
  *   (at your option) any later version.
- * 
+ *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  *   the GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program;  if not, write to the Free Software 
+ *   along with this program;  if not, write to the Free Software
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
- */ 
+ */
 
 #include <string.h>
 #include <time.h>
@@ -49,13 +49,14 @@
 int set_selected_object(task_handle_t task, object_handle_t handle)
 {
 	gint rc;
-	handle_array_t ha;
+	gchar mem[sizeof(handle_array_t) + sizeof(object_handle_t)];
+	handle_array_t *ha = (handle_array_t *) mem;
 	declined_handle_array_t *declined_list;
 
-	ha.count = 1;
-	ha.handle[0] = handle;
+	ha->count = 1;
+	ha->handle[0] = handle;
 
-	rc = evms_set_selected_objects(task, &ha, &declined_list, NULL);
+	rc = evms_set_selected_objects(task, ha, &declined_list, NULL);
 	if (rc == 0 && declined_list) {
 		if (declined_list->count > 0)
 			rc = EINVAL;
@@ -97,7 +98,7 @@ gboolean acceptable_objects_available(task_handle_t task)
 	gboolean result = FALSE;
 	handle_array_t *acceptable_list;
 
-	rc = evms_get_acceptable_objects(task, &acceptable_list); 
+	rc = evms_get_acceptable_objects(task, &acceptable_list);
 	if (rc != 0)  {
 		log_error("%s: evms_get_acceptable_objects() returned error code %d.\n",
 				__FUNCTION__, rc);
@@ -115,7 +116,7 @@ gboolean acceptable_objects_available(task_handle_t task)
  *	@action: the task action code
  *
  *	This routine attempts to see if the plugin could attempt the
- *	given task action. We basically make a determination by 
+ *	given task action. We basically make a determination by
  *	creating a task, checking the acceptable object count and the
  *	minimum selection count.
  */
@@ -162,11 +163,9 @@ gchar *get_task_action_string(task_action_t action)
 		action_text = g_strdup(_("Add"));
 		break;
 	case EVMS_Task_Expand:
-	case EVMS_Task_Expand_Container:
 		action_text = g_strdup(_("Expand"));
 		break;
 	case EVMS_Task_Shrink:
-	case EVMS_Task_Shrink_Container:
 		action_text = g_strdup(_("Shrink"));
 		break;
 	case EVMS_Task_Slide:
@@ -181,9 +180,6 @@ gchar *get_task_action_string(task_action_t action)
 	case EVMS_Task_mkfs:
 		action_text = g_strdup(_("Make Filesystem"));
 		break;
-	case EVMS_Task_defrag:
-		action_text = g_strdup(_("Defragment Filesystem"));
-		break;
 	default:
 		log_debug("%s: Unknown task action %d received.\n", __FUNCTION__, action);
 		break;
@@ -257,8 +253,8 @@ gboolean are_handle_arrays_equal(handle_array_t *array1, handle_array_t *array2)
 	if (array1 != NULL && array2 != NULL && array1->count == array2->count) {
 		gint array_size;
 
-		array_size = sizeof(handle_array_t) + ((array1->count - 1)
-				* sizeof(object_handle_t));
+		array_size = sizeof(handle_array_t) +
+			(array1->count * sizeof(object_handle_t));
 		arrays_equal = memcmp(array1, array2, array_size) == 0;
 	}
 	return arrays_equal;
@@ -296,9 +292,9 @@ inline gboolean object_is_only_available(task_handle_t task, engine_handle_t han
  *	to properly invoked. We check option count and the min and max for the
  *	acceptable objects as well. This information is used in order to present
  *	the proper user panels to gather the selections and configuration for the
- *	task. 
+ *	task.
  *
- *	While we are here, we also set the selected object if we find that there 
+ *	While we are here, we also set the selected object if we find that there
  *	is only one and the plugin can accept at least one to continue.
  */
 task_requirements_t determine_task_requirements(task_handle_t task, engine_handle_t handle,
@@ -360,7 +356,7 @@ task_requirements_t determine_task_requirements(task_handle_t task, engine_handl
  *	whether the task will need to present an acceptable objects selection list
  *	and/or an options configuration panel or none at all.
  */
-int create_task(engine_handle_t handle, task_action_t action, 
+int create_task(engine_handle_t handle, task_action_t action,
 		task_requirements_t *requirements, task_handle_t *task,
 		int *objects_min, int *objects_max)
 {
@@ -372,9 +368,9 @@ int create_task(engine_handle_t handle, task_action_t action,
 	if (rc == 0) {
 		rc = evms_get_selected_object_limits(*task, objects_min, objects_max);
 
-		if (rc == 0) {    
+		if (rc == 0) {
 			rc = evms_get_option_count(*task, &option_count);
-            
+
 			if (rc == 0) {
 				*requirements = determine_task_requirements(*task,
 									handle,
@@ -389,7 +385,7 @@ int create_task(engine_handle_t handle, task_action_t action,
 		} else {
 			evms_destroy_task(*task);
 			log_error("%s: evms_get_selected_object_limits() returned error code %d.\n",
-					__FUNCTION__, rc);            
+					__FUNCTION__, rc);
 		}
 	} else {
 		log_error("%s: evms_create_task() returned error code %d.\n", __FUNCTION__, rc);
@@ -775,7 +771,7 @@ int format_declination_item(declined_handle_t *declined, GPtrArray *text)
  *	@declined_objects: the address of an array containing handles and declination reason codes
  *
  *	This routine populates the declination window clist with the objects that have been declined
- *	by a plugin from a evms_set_selected_objects() call. 
+ *	by a plugin from a evms_set_selected_objects() call.
  */
 void populate_declination_clist(struct clist *clist, declined_handle_array_t *declined_objects)
 {
@@ -856,7 +852,7 @@ void mark_selected_rows(struct clist *clist, handle_array_t *selected_objects)
  *	@clist: the clist containing the acceptable objects
  *
  *	This routine is typically scheduled for invocation after processing of other clist events
- *	in order to reload the clist with the current set of acceptable objects and marking 
+ *	in order to reload the clist with the current set of acceptable objects and marking
  *	objects that the engine task believes are currently selected.
  */
 int reload_acceptable_objects(struct clist *clist)
@@ -904,7 +900,7 @@ int reload_acceptable_objects(struct clist *clist)
 		clist_populate(clist, enum_acceptable_objects, clist_allow_all,
 			format_standard_item, NULL, GUINT_TO_POINTER(task));
 
-		rc = evms_get_selected_objects(task, &selected_list); 
+		rc = evms_get_selected_objects(task, &selected_list);
 		if (rc == 0) {
 			int count;
 			GList *focus_item;
@@ -931,7 +927,7 @@ int reload_acceptable_objects(struct clist *clist)
 			log_error("%s: evms_get_selected_objects () returned error code %d.\n",
 					__FUNCTION__, rc);
 		}
-        
+
 		set_clist_select_item_cb(clist, select_cb);
 		set_clist_unselect_item_cb(clist, unselect_cb);
 		evms_free(acceptable_list);
@@ -996,8 +992,8 @@ int update_selected_objects(struct clist *clist, task_handle_t task, int *count)
  *	This routine is invoked for the row_selected event in the acceptable objects
  *	dialog clist. If the min for selected objects is met, we enable the "Next" button
  *	and call update_selected_objects() to give our current selections to the engine.
- *	Otherwise we simply "mark" the object as selected. If there is any 
- *	the new selection to the engine through the 
+ *	Otherwise we simply "mark" the object as selected. If there is any
+ *	the new selection to the engine through the
  */
 int on_select_acceptable_object(struct clist *clist, struct clist_item *item)
 {
@@ -1046,7 +1042,7 @@ int on_select_acceptable_object(struct clist *clist, struct clist_item *item)
  *	@item: the clist item that was unselected
  *
  *	This routine is invoked for the row_unselected event in the acceptable objects
- *	dialog clist. 
+ *	dialog clist.
  */
 int on_unselect_acceptable_object(struct clist *clist, struct clist_item *item)
 {
@@ -1207,13 +1203,13 @@ struct dialog_window *create_task_dialog(struct dialog_list *list, engine_handle
 	if (rc == 0) {
 		g_hash_table_insert(hash_table, "task", GUINT_TO_POINTER(task));
 
-		switch (requirements) {            
+		switch (requirements) {
 		case TASK_REQUIRES_OPTIONS_ONLY:
 			dialog = create_task_options_dialog(task, title, verb, help, hash_table);
 			break;
 		case TASK_REQUIRES_OBJECTS_ONLY:
 		case TASK_REQUIRES_OBJECTS_AND_OPTIONS:
-			dialog = create_acceptable_objects_selection_dialog(task, title, verb, 
+			dialog = create_acceptable_objects_selection_dialog(task, title, verb,
 									requirements, help, hash_table);
 			break;
 		case TASK_REQUIRES_NO_ADDITIONAL_ARGS:
@@ -1249,7 +1245,7 @@ struct dialog_window *create_task_dialog(struct dialog_list *list, engine_handle
  *
  *	This routine is invoked when a menu item (Next button) is activated in a
  *	selection dialog window. We expect that item->user_data points to the
- *	current dialog. In that dialog, dialog->user_data is expected to be 
+ *	current dialog. In that dialog, dialog->user_data is expected to be
  *	hash table containing values accessible through predefined string keys.
  */
 int on_init_task_menuitem_activated(struct menu_item *item)
