@@ -4,19 +4,19 @@
  *
  *   This program is free software;  you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or 
+ *   the Free Software Foundation; either version 2 of the License, or
  *   (at your option) any later version.
- * 
+ *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
  *   the GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program;  if not, write to the Free Software 
+ *   along with this program;  if not, write to the Free Software
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
- */ 
+ */
 
 #include <frontend.h>
 #include <glib.h>
@@ -75,11 +75,14 @@ int format_volume_view_item(object_handle_t handle, void *not_used, GPtrArray *t
 	if (rc == 0) {
 		g_ptr_array_add(text, g_strdup(object->info.volume.name));
 		g_ptr_array_add(text, make_sectors_readable_string(object->info.volume.vol_size));
-		g_ptr_array_add(text, checkmark_if_true(object->info.volume.flags & VOLFLAG_DIRTY));
+		g_ptr_array_add(text, checkmark_if_true(object->info.volume.flags & (VOLFLAG_DIRTY |
+										     VOLFLAG_FEATURE_HEADER_DIRTY |
+										     VOLFLAG_NEEDS_ACTIVATE |
+										     VOLFLAG_NEEDS_DEACTIVATE)));
 		g_ptr_array_add(text, checkmark_if_true(object->info.volume.flags & VOLFLAG_ACTIVE));
 		g_ptr_array_add(text, checkmark_if_true(object->info.volume.flags & VOLFLAG_READ_ONLY));
 		g_ptr_array_add(text, get_volume_fsim_name(&(object->info.volume)));
-		g_ptr_array_add(text, g_strdup(object->info.volume.mount_point ? 
+		g_ptr_array_add(text, g_strdup(object->info.volume.mount_point ?
 					object->info.volume.mount_point : " "));
 		evms_free(object);
 	}
@@ -117,7 +120,9 @@ int format_object_view_item(object_handle_t handle, void *not_used, GPtrArray *t
 	if (rc == 0) {
 		g_ptr_array_add(text, g_strdup(object->info.object.name));
 		g_ptr_array_add(text, make_sectors_readable_string(object->info.object.size));
-		g_ptr_array_add(text, checkmark_if_true(object->info.object.flags & SOFLAG_DIRTY));
+		g_ptr_array_add(text, checkmark_if_true(object->info.object.flags & (SOFLAG_DIRTY |
+										     SOFLAG_NEEDS_ACTIVATE |
+										     SOFLAG_NEEDS_DEACTIVATE)));
 		g_ptr_array_add(text, checkmark_if_true(object->info.object.flags & SOFLAG_ACTIVE));
 		g_ptr_array_add(text, checkmark_if_true(object->info.object.flags & SOFLAG_READ_ONLY));
 		g_ptr_array_add(text, get_plugin_name(object->info.object.plugin));
@@ -158,7 +163,9 @@ int format_feature_view_item(object_handle_t handle, void *not_used, GPtrArray *
 	if (rc == 0) {
 		g_ptr_array_add(text, g_strdup(object->info.object.name));
 		g_ptr_array_add(text, make_sectors_readable_string(object->info.object.size));
-		g_ptr_array_add(text, checkmark_if_true(object->info.object.flags & SOFLAG_DIRTY));
+		g_ptr_array_add(text, checkmark_if_true(object->info.object.flags & (SOFLAG_DIRTY |
+										     SOFLAG_NEEDS_ACTIVATE |
+										     SOFLAG_NEEDS_DEACTIVATE)));
 		g_ptr_array_add(text, checkmark_if_true(object->info.object.flags & SOFLAG_ACTIVE));
 		g_ptr_array_add(text, checkmark_if_true(object->info.object.flags & SOFLAG_READ_ONLY));
 		g_ptr_array_add(text, get_plugin_name(object->info.object.plugin));
@@ -197,7 +204,9 @@ int format_region_view_item(object_handle_t handle, void *not_used, GPtrArray *t
 	if (rc == 0) {
 		g_ptr_array_add(text, g_strdup(object->info.object.name));
 		g_ptr_array_add(text, make_sectors_readable_string(object->info.object.size));
-		g_ptr_array_add(text, checkmark_if_true(object->info.object.flags & SOFLAG_DIRTY));
+		g_ptr_array_add(text, checkmark_if_true(object->info.object.flags & (SOFLAG_DIRTY |
+										     SOFLAG_NEEDS_ACTIVATE |
+										     SOFLAG_NEEDS_DEACTIVATE)));
 		g_ptr_array_add(text, checkmark_if_true(object->info.object.flags & SOFLAG_ACTIVE));
 		g_ptr_array_add(text, checkmark_if_true(object->info.object.flags & SOFLAG_READ_ONLY));
 		g_ptr_array_add(text, checkmark_if_true(object->info.object.flags & SOFLAG_CORRUPT));
@@ -274,7 +283,9 @@ int format_segment_view_item(object_handle_t handle, void *not_used, GPtrArray *
 	if (rc == 0) {
 		g_ptr_array_add(text, g_strdup(object->info.object.name));
 		g_ptr_array_add(text, make_sectors_readable_string(object->info.object.size));
-		g_ptr_array_add(text, checkmark_if_true(object->info.object.flags & SOFLAG_DIRTY));
+		g_ptr_array_add(text, checkmark_if_true(object->info.object.flags & (SOFLAG_DIRTY |
+										     SOFLAG_NEEDS_ACTIVATE |
+										     SOFLAG_NEEDS_DEACTIVATE)));
 		g_ptr_array_add(text, checkmark_if_true(object->info.object.flags & SOFLAG_ACTIVE));
 		g_ptr_array_add(text, checkmark_if_true(object->info.object.flags & SOFLAG_READ_ONLY));
 		g_ptr_array_add(text, make_data_type_readable_string(object->info.object.data_type));
@@ -316,7 +327,9 @@ int format_disk_view_item(object_handle_t handle, void *not_used, GPtrArray *tex
 	if (rc == 0) {
 		g_ptr_array_add(text, g_strdup(object->info.object.name));
 		g_ptr_array_add(text, make_sectors_readable_string(object->info.object.size));
-		g_ptr_array_add(text, checkmark_if_true(object->info.object.flags & SOFLAG_DIRTY));
+		g_ptr_array_add(text, checkmark_if_true(object->info.object.flags & (SOFLAG_DIRTY |
+										     SOFLAG_NEEDS_ACTIVATE |
+										     SOFLAG_NEEDS_DEACTIVATE)));
 		g_ptr_array_add(text, checkmark_if_true(object->info.object.flags & SOFLAG_ACTIVE));
 		g_ptr_array_add(text, checkmark_if_true(object->info.object.flags & SOFLAG_READ_ONLY));
 		g_ptr_array_add(text, g_strdup_printf("%"PRIu64",%u,%u", object->info.object.geometry.cylinders,
@@ -554,12 +567,13 @@ void create_volumes_view(void)
 	mvwaddstr(win, 0, 1, _("0=Logical Volumes"));
 	mvwhline(win, 3, 1, ACS_HLINE, getmaxx(win) - 2);
 
-	set_clist_column_info(clist, 0, calc_clist_column_width(clist, 0.38), 0, 
+	set_clist_column_info(clist, 0, calc_clist_column_width(clist, 0.32),
+					0,
 					CLIST_TEXT_JUSTIFY_LEFT);
 	set_clist_column_info(clist, 1, calc_clist_column_width(clist, 0.12),
 					get_clist_column_end(clist, 0),
 					CLIST_TEXT_JUSTIFY_RIGHT);
-	set_clist_column_info(clist, 2, calc_clist_column_width(clist, 0.08),
+	set_clist_column_info(clist, 2, calc_clist_column_width(clist, 0.12),
 					get_clist_column_end(clist, 1),
 					CLIST_TEXT_JUSTIFY_CENTER);
 	set_clist_column_info(clist, 3, calc_clist_column_width(clist, 0.08),
@@ -571,13 +585,13 @@ void create_volumes_view(void)
 	set_clist_column_info(clist, 5, calc_clist_column_width(clist, 0.12),
 					get_clist_column_end(clist, 4),
 					CLIST_TEXT_JUSTIFY_LEFT);
-	set_clist_column_info(clist, 6, calc_clist_column_width(clist, 0.13),
+	set_clist_column_info(clist, 6, calc_clist_column_width(clist, 0.20),
 					get_clist_column_end(clist, 5),
 					CLIST_TEXT_JUSTIFY_LEFT);
 
 	print_clist_column_title(clist, 0, _("Name"));
 	print_clist_column_title(clist, 1, _("Size"));
-	print_clist_column_title(clist, 2, _("Dirty"));
+	print_clist_column_title(clist, 2, _(" Modified"));
 	print_clist_column_title(clist, 3, _("Active"));
 	print_clist_column_title(clist, 4, _("R/O"));
 	print_clist_column_title(clist, 5, _("Plug-in"));
@@ -604,12 +618,13 @@ void create_objects_view(void)
 	mvwaddstr(win, 0, 1, _("1=Available Objects"));
 	mvwhline(win, 3, 1, ACS_HLINE, getmaxx(win) - 2);
 
-	set_clist_column_info(clist, 0, calc_clist_column_width(clist, 0.38), 0,
+	set_clist_column_info(clist, 0, calc_clist_column_width(clist, 0.25),
+					0,
 					CLIST_TEXT_JUSTIFY_LEFT);
 	set_clist_column_info(clist, 1, calc_clist_column_width(clist, 0.12),
 					get_clist_column_end(clist, 0),
 					CLIST_TEXT_JUSTIFY_RIGHT);
-	set_clist_column_info(clist, 2, calc_clist_column_width(clist, 0.08),
+	set_clist_column_info(clist, 2, calc_clist_column_width(clist, 0.12),
 					get_clist_column_end(clist, 1),
 					CLIST_TEXT_JUSTIFY_CENTER);
 	set_clist_column_info(clist, 3, calc_clist_column_width(clist, 0.08),
@@ -618,13 +633,13 @@ void create_objects_view(void)
 	set_clist_column_info(clist, 4, calc_clist_column_width(clist, 0.04),
 					get_clist_column_end(clist, 3),
 					CLIST_TEXT_JUSTIFY_CENTER);
-	set_clist_column_info(clist, 5, calc_clist_column_width(clist, 0.26),
+	set_clist_column_info(clist, 5, calc_clist_column_width(clist, 0.39),
 					get_clist_column_end(clist, 4),
 					CLIST_TEXT_JUSTIFY_LEFT);
 
 	print_clist_column_title(clist, 0, _("Name"));
 	print_clist_column_title(clist, 1, _("Size"));
-	print_clist_column_title(clist, 2, _("Dirty"));
+	print_clist_column_title(clist, 2, _(" Modified"));
 	print_clist_column_title(clist, 3, _("Active"));
 	print_clist_column_title(clist, 4, _("R/O"));
 	print_clist_column_title(clist, 5, _("Plug-in"));
@@ -650,12 +665,13 @@ void create_features_view(void)
 	mvwaddstr(win, 0, 1, _("2=Feature Objects"));
 	mvwhline(win, 3, 1, ACS_HLINE, getmaxx(win) - 2);
 
-	set_clist_column_info(clist, 0, calc_clist_column_width(clist, 0.38), 0,
+	set_clist_column_info(clist, 0, calc_clist_column_width(clist, 0.25),
+					0,
 					CLIST_TEXT_JUSTIFY_LEFT);
 	set_clist_column_info(clist, 1, calc_clist_column_width(clist, 0.12),
 					get_clist_column_end(clist, 0),
 					CLIST_TEXT_JUSTIFY_RIGHT);
-	set_clist_column_info(clist, 2, calc_clist_column_width(clist, 0.08),
+	set_clist_column_info(clist, 2, calc_clist_column_width(clist, 0.12),
 					get_clist_column_end(clist, 1),
 					CLIST_TEXT_JUSTIFY_CENTER);
 	set_clist_column_info(clist, 3, calc_clist_column_width(clist, 0.08),
@@ -664,13 +680,13 @@ void create_features_view(void)
 	set_clist_column_info(clist, 4, calc_clist_column_width(clist, 0.04),
 					get_clist_column_end(clist, 3),
 					CLIST_TEXT_JUSTIFY_CENTER);
-	set_clist_column_info(clist, 5, calc_clist_column_width(clist, 0.26),
+	set_clist_column_info(clist, 5, calc_clist_column_width(clist, 0.39),
 					get_clist_column_end(clist, 4),
 					CLIST_TEXT_JUSTIFY_LEFT);
 
 	print_clist_column_title(clist, 0, _("Name"));
 	print_clist_column_title(clist, 1, _("Size"));
-	print_clist_column_title(clist, 2, _("Dirty"));
+	print_clist_column_title(clist, 2, _(" Modified"));
 	print_clist_column_title(clist, 3, _("Active"));
 	print_clist_column_title(clist, 4, _("R/O"));
 	print_clist_column_title(clist, 5, _("Plug-in"));
@@ -696,12 +712,13 @@ void create_regions_view(void)
 	mvwaddstr(win, 0, 1, _("3=Storage Regions"));
 	mvwhline(win, 3, 1, ACS_HLINE, getmaxx(win) - 2);
 
-	set_clist_column_info(clist, 0, calc_clist_column_width(clist, 0.38), 0,
+	set_clist_column_info(clist, 0, calc_clist_column_width(clist, 0.25),
+					0,
 					CLIST_TEXT_JUSTIFY_LEFT);
 	set_clist_column_info(clist, 1, calc_clist_column_width(clist, 0.12),
 					get_clist_column_end(clist, 0),
 					CLIST_TEXT_JUSTIFY_RIGHT);
-	set_clist_column_info(clist, 2, calc_clist_column_width(clist, 0.08),
+	set_clist_column_info(clist, 2, calc_clist_column_width(clist, 0.12),
 					get_clist_column_end(clist, 1),
 					CLIST_TEXT_JUSTIFY_CENTER);
 	set_clist_column_info(clist, 3, calc_clist_column_width(clist, 0.08),
@@ -713,13 +730,13 @@ void create_regions_view(void)
 	set_clist_column_info(clist, 5, calc_clist_column_width(clist, 0.09),
 					get_clist_column_end(clist, 4),
 					CLIST_TEXT_JUSTIFY_CENTER);
-	set_clist_column_info(clist, 6, calc_clist_column_width(clist, 0.15),
+	set_clist_column_info(clist, 6, calc_clist_column_width(clist, 0.30),
 					get_clist_column_end(clist, 5),
 					CLIST_TEXT_JUSTIFY_LEFT);
 
 	print_clist_column_title(clist, 0, _("Name"));
 	print_clist_column_title(clist, 1, _("Size"));
-	print_clist_column_title(clist, 2, _("Dirty"));
+	print_clist_column_title(clist, 2, _(" Modified"));
 	print_clist_column_title(clist, 3, _("Active"));
 	print_clist_column_title(clist, 4, _("R/O"));
 	print_clist_column_title(clist, 5, _("Corrupt"));
@@ -746,21 +763,22 @@ void create_containers_view(void)
 	mvwaddstr(win, 0, 1, _("4=Storage Containers"));
 	mvwhline(win, 3, 1, ACS_HLINE, getmaxx(win) - 2);
 
-	set_clist_column_info(clist, 0, calc_clist_column_width(clist, 0.44), 0,
+	set_clist_column_info(clist, 0, calc_clist_column_width(clist, 0.25),
+					0,
 					CLIST_TEXT_JUSTIFY_LEFT);
 	set_clist_column_info(clist, 1, calc_clist_column_width(clist, 0.12),
 					get_clist_column_end(clist, 0),
 					CLIST_TEXT_JUSTIFY_RIGHT);
-	set_clist_column_info(clist, 2, calc_clist_column_width(clist, 0.08),
+	set_clist_column_info(clist, 2, calc_clist_column_width(clist, 0.12),
 					get_clist_column_end(clist, 1),
 					CLIST_TEXT_JUSTIFY_CENTER);
-	set_clist_column_info(clist, 3, calc_clist_column_width(clist, 0.53),
+	set_clist_column_info(clist, 3, calc_clist_column_width(clist, 0.51),
 					get_clist_column_end(clist, 2),
 					CLIST_TEXT_JUSTIFY_LEFT);
 
 	print_clist_column_title(clist, 0, _("Name"));
 	print_clist_column_title(clist, 1, _("Size"));
-	print_clist_column_title(clist, 2, _("Dirty"));
+	print_clist_column_title(clist, 2, _(" Modified"));
 	print_clist_column_title(clist, 3, _("Plug-in"));
 }
 
@@ -784,12 +802,13 @@ void create_segments_view(void)
 	mvwaddstr(win, 0, 1, _("5=Disk Segments"));
 	mvwhline(win, 3, 1, ACS_HLINE, getmaxx(win) - 2);
 
-	set_clist_column_info(clist, 0, calc_clist_column_width(clist, 0.27), 0,
+	set_clist_column_info(clist, 0, calc_clist_column_width(clist, 0.25),
+					0,
 					CLIST_TEXT_JUSTIFY_LEFT);
-	set_clist_column_info(clist, 1, calc_clist_column_width(clist, 0.12), 
+	set_clist_column_info(clist, 1, calc_clist_column_width(clist, 0.12),
 					get_clist_column_end(clist, 0),
 					CLIST_TEXT_JUSTIFY_RIGHT);
-	set_clist_column_info(clist, 2, calc_clist_column_width(clist, 0.08),
+	set_clist_column_info(clist, 2, calc_clist_column_width(clist, 0.12),
 					get_clist_column_end(clist, 1),
 					CLIST_TEXT_JUSTIFY_CENTER);
 	set_clist_column_info(clist, 3, calc_clist_column_width(clist, 0.08),
@@ -804,13 +823,13 @@ void create_segments_view(void)
 	set_clist_column_info(clist, 6, calc_clist_column_width(clist, 0.12),
 					get_clist_column_end(clist, 5),
 					CLIST_TEXT_JUSTIFY_LEFT);
-	set_clist_column_info(clist, 7, calc_clist_column_width(clist, 0.12),
+	set_clist_column_info(clist, 7, calc_clist_column_width(clist, 0.15),
 					get_clist_column_end(clist, 6),
 					CLIST_TEXT_JUSTIFY_LEFT);
 
 	print_clist_column_title(clist, 0, _("Name"));
 	print_clist_column_title(clist, 1, _("Size"));
-	print_clist_column_title(clist, 2, _("Dirty"));
+	print_clist_column_title(clist, 2, _(" Modified"));
 	print_clist_column_title(clist, 3, _("Active"));
 	print_clist_column_title(clist, 4, _("R/O"));
 	print_clist_column_title(clist, 5, _("Type"));
@@ -838,12 +857,13 @@ void create_disks_view(void)
 	mvwaddstr(win, 0, 1, _("6=Logical Disks"));
 	mvwhline(win, 3, 1, ACS_HLINE, getmaxx(win) - 2);
 
-	set_clist_column_info(clist, 0, calc_clist_column_width(clist, 0.31), 0,
+	set_clist_column_info(clist, 0, calc_clist_column_width(clist, 0.25),
+					0,
 					CLIST_TEXT_JUSTIFY_LEFT);
 	set_clist_column_info(clist, 1, calc_clist_column_width(clist, 0.12),
 					get_clist_column_end(clist, 0),
 					CLIST_TEXT_JUSTIFY_RIGHT);
-	set_clist_column_info(clist, 2, calc_clist_column_width(clist, 0.08),
+	set_clist_column_info(clist, 2, calc_clist_column_width(clist, 0.12),
 					get_clist_column_end(clist, 1),
 					CLIST_TEXT_JUSTIFY_CENTER);
 	set_clist_column_info(clist, 3, calc_clist_column_width(clist, 0.08),
@@ -855,13 +875,13 @@ void create_disks_view(void)
 	set_clist_column_info(clist, 5, calc_clist_column_width(clist, 0.15),
 					get_clist_column_end(clist, 4),
 					CLIST_TEXT_JUSTIFY_LEFT);
-	set_clist_column_info(clist, 6, calc_clist_column_width(clist, 0.17),
+	set_clist_column_info(clist, 6, calc_clist_column_width(clist, 0.24),
 					get_clist_column_end(clist, 5),
 					CLIST_TEXT_JUSTIFY_LEFT);
 
 	print_clist_column_title(clist, 0, _("Name"));
 	print_clist_column_title(clist, 1, _("Size"));
-	print_clist_column_title(clist, 2, _("Dirty"));
+	print_clist_column_title(clist, 2, _(" Modified"));
 	print_clist_column_title(clist, 3, _("Active"));
 	print_clist_column_title(clist, 4, _("R/O"));
 	print_clist_column_title(clist, 5, _("Geometry"));
@@ -888,7 +908,8 @@ void create_plugins_view(void)
 	mvwaddstr(win, 0, 1, _("7=Plug-ins"));
 	mvwhline(win, 3, 1, ACS_HLINE, getmaxx(win) - 2);
 
-	set_clist_column_info(clist, 0, calc_clist_column_width(clist, 0.14), 0,
+	set_clist_column_info(clist, 0, calc_clist_column_width(clist, 0.14),
+					0,
 					CLIST_TEXT_JUSTIFY_LEFT);
 	set_clist_column_info(clist, 1, calc_clist_column_width(clist, 0.12),
 					get_clist_column_end(clist, 0),
@@ -899,7 +920,7 @@ void create_plugins_view(void)
 	set_clist_column_info(clist, 3, calc_clist_column_width(clist, 0.12),
 					get_clist_column_end(clist, 2),
 					CLIST_TEXT_JUSTIFY_LEFT);
-	set_clist_column_info(clist, 4, calc_clist_column_width(clist, 0.43),
+	set_clist_column_info(clist, 4, calc_clist_column_width(clist, 0.45),
 					get_clist_column_end(clist, 3),
 					CLIST_TEXT_JUSTIFY_LEFT);
 
@@ -985,7 +1006,7 @@ int process_notebook_events(int key)
  *	This routine takes keyboard input and translates that to some action/event.
  *	The processing order is view events are processed first. If they are not
  *	handled then notebook events are processed second and if those are not
- *	handled then menubar events are processed last. 
+ *	handled then menubar events are processed last.
  */
 int process_current_view_events(void)
 {
@@ -996,7 +1017,7 @@ int process_current_view_events(void)
 	if (event_handled == FALSE) {
 		event_handled = process_notebook_events(key);
 
-		if (event_handled == FALSE) { 
+		if (event_handled == FALSE) {
 			rc = process_menubar_events(key);
 		}
 	}
