@@ -37,6 +37,7 @@
 #include <sys/types.h>
 #include <sys/mman.h>
 #include <common.h>
+#include <locale.h>
 
 #define SLASH_REPLACEMENT	'|'
 #define LOCK_FILE_PREFIX	"/var/lock/evms-mpathd-"
@@ -54,6 +55,7 @@ static int debug = FALSE;
 static char *log_file = NULL;
 static int log_file_fd = 0;
 static char log_buf[LOG_BUF_SIZE];
+static int dm_multipath_version[3];
 
 struct path {
 	char name[128];
@@ -185,11 +187,11 @@ out:
  **/
 static void show_help(void)
 {
-	printf("Usage: %s [options] <multipath_name> <size> <child_name>+\n", prog_name);
-	printf("Monitor a multipath device for path failures.\n\n"
-	       "  [-d, --debug] write debug information to a log file\n"
-	       "  [-l log_file_name, --log-file log_file_name]\n"
-	       "  [-h | -? | --help] display this help\n");
+	printf(_("Usage: %s [options] <multipath_name> <size> <child_name>+\n"), prog_name);
+	printf(_("Monitor a multipath device for path failures.\n\n"
+		 "  [-d, --debug] write debug information to a log file\n"
+		 "  [-l log_file_name, --log-file log_file_name]\n"
+		 "  [-h | -? | --help] display this help\n"));
 }
 
 /**
@@ -224,7 +226,7 @@ static int parse_options(int argc, char **argv)
 			break;
 
 		default:
-			fprintf(stderr, "%s -- unrecognized option \"%c\"\n\n",
+			fprintf(stderr, _("%s -- unrecognized option \"%c\"\n\n"),
 				prog_name, c);
 			/* Display the help. */
 			rc = EINVAL;
@@ -560,13 +562,74 @@ static int alloc_dm_task(struct multipath *mp, struct dm_task **dmt, int type)
 
 	*dmt = dm_task_create(type);
 	if (*dmt) {
-		rc = dm_task_set_name(*dmt, mp->name);
-		rc = (rc) ? 0 : ENOMEM;
+		if (mp) {
+			rc = dm_task_set_name(*dmt, mp->name);
+			rc = (rc) ? 0 : ENOMEM;
+		} else {
+			rc = 0;
+		}
 	}
 
 	return rc;
 }
 
+/**
+ * get_dm_multipath_version
+ *
+ * Get the list of targets from DM and find the version of the
+ * multipath target.
+ **/
+static int get_dm_multipath_version(void)
+{
+	struct dm_versions *version, *old_version;
+	struct dm_task *task;
+	int rc;
+
+	rc = alloc_dm_task(NULL, &task, DM_DEVICE_LIST_VERSIONS);
+	if (rc) {
+		return rc;
+	}
+
+	rc = dm_task_run(task);
+	if (!rc) {
+		log_debug("Error running list-versions command.\n");
+		rc = EINVAL;
+		goto out;
+	}
+
+	rc = ENOENT;
+	version = dm_task_get_versions(task);
+	if (!version) {
+		log_debug("Error getting versions array.\n");
+		goto out;
+	}
+
+	do {
+		if (!strcmp(version->name, "multipath")) {
+			dm_multipath_version[0] = version->version[0];
+			dm_multipath_version[1] = version->version[1];
+			dm_multipath_version[2] = version->version[2];
+			log_debug("Found multipath target: version %d.%d.%d\n",
+				  dm_multipath_version[0],
+				  dm_multipath_version[1],
+				  dm_multipath_version[2]);
+			rc = 0;
+			break;
+		}
+
+		old_version = version;
+		version = (void*)version + version->next;
+	} while (old_version != version);
+
+	if (rc) {
+		log_debug("No multipath target found.\n");
+	}
+
+out:
+	dm_task_destroy(task);
+	return rc;
+}
+
 /**
  * get_info
  *
@@ -760,12 +823,24 @@ static int reload_device_table(struct multipath *mp)
 		return rc;
 	}
 
-	sz += snprintf(params + sz, maxlen - sz,
-		       "1 round-robin %u 0", mp->num_paths);
+	if (dm_multipath_version[0] == 1 &&
+	    dm_multipath_version[1] == 0 &&
+	    dm_multipath_version[2] <= 3) {
+		sz += snprintf(params + sz, maxlen - sz,
+			       "1 round-robin %u 0", mp->num_paths);
 
-	for (i = 0; i < mp->num_paths; i++) {
-		sz += snprintf(params + sz, maxlen - sz, " %u:%u",
-			       mp->paths[i].major, mp->paths[i].minor);
+		for (i = 0; i < mp->num_paths; i++) {
+			sz += snprintf(params + sz, maxlen - sz, " %u:%u",
+				       mp->paths[i].major, mp->paths[i].minor);
+		}
+	} else {
+		sz += snprintf(params + sz, maxlen - sz,
+			       "0 0 1 1 round-robin 0 %u 0", mp->num_paths);
+
+		for (i = 0; i < mp->num_paths; i++) {
+			sz += snprintf(params + sz, maxlen - sz, " %u:%u",
+				       mp->paths[i].major, mp->paths[i].minor);
+		}
 	}
 
 	log_debug("Creating table parameters: %s\n", params);
@@ -854,6 +929,10 @@ int main(int argc, char **argv)
 	struct multipath *mp;
 	int rc;
 
+	setlocale(LC_MESSAGES, "");
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	textdomain(PACKAGE);
+
 	prog_name = basename(argv[0]);
 
 	rc = parse_command_line(argc, argv, &mp);
@@ -893,6 +972,12 @@ int main(int argc, char **argv)
 		return rc;
 	}
 
+	rc = get_dm_multipath_version();
+	if (rc) {
+		free_multipath(mp);
+		return rc;
+	}
+
 	test_multipath(mp);
 
 	/* Should only get here on a bad error. */
